@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -41,7 +41,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: aklog.c,v 1.6 2000/09/04 05:37:38 lha Exp $");
+RCSID("$Id: aklog.c,v 1.6.2.2 2001/10/03 22:52:44 assar Exp $");
 
 #include "appl_locl.h"
 #include "kafs.h"
@@ -61,7 +61,62 @@ main(int argc, char **argv)
 static int
 krb_afslog_uid (const char *cell, const char *realm, uid_t afsid)
 {
-    return k_afslog_uid(cell, realm, afsid);
+    return k_afsklog_uid(cell, realm, afsid);
+}
+#endif
+
+#ifndef HAVE_KRB_GET_DEFAULT_PRINCIPAL
+
+/* stolen from krb4's lib/krb/get_default_principal.c */
+
+static int
+krb_get_default_principal(char *name, char *instance, char *realm)
+{
+  char *file;
+  int ret;
+  char *p;
+
+  file = tkt_string ();
+  
+  ret = krb_get_tf_fullname(file, name, instance, realm);
+  if(ret == KSUCCESS)
+      return 0;
+
+  p = getenv("KRB4PRINCIPAL");
+  if(p && kname_parse(name, instance, realm, p) == KSUCCESS)
+      return 1;
+
+#ifdef HAVE_PWD_H
+  {
+    struct passwd *pw;
+    pw = getpwuid(getuid());
+    if(pw == NULL){
+      return -1;
+    }
+
+    strlcpy (name, pw->pw_name, ANAME_SZ);
+    strlcpy (instance, "", INST_SZ);
+    krb_get_lrealm(realm, 1);
+
+    if(strcmp(name, "root") == 0) {
+      p = NULL;
+#if defined(HAVE_GETLOGIN) && !defined(POSIX_GETLOGIN)
+      p = getlogin();
+#endif
+      if(p == NULL)
+	p = getenv("USER");
+      if(p == NULL)
+	p = getenv("LOGNAME");
+      if(p){
+	  strlcpy (name, p, ANAME_SZ);
+	  strlcpy (instance, "root", INST_SZ);
+      }
+    }
+    return 1;
+  }
+#else
+  return -1;
+#endif
 }
 #endif
 
@@ -262,7 +317,7 @@ main (int argc, char **argv)
      * get the realm and name of the user
      */
 
-    if (krb_get_default_principal(krb_name, krb_instance, krb_realm)) {
+    if (krb_get_default_principal(krb_name, krb_instance, krb_realm) < 0) {
 	if (!arg_quiet)
 	    warnx ("Could not even figure out who you are");
 	exit (1);
