@@ -51,7 +51,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: klog.c,v 1.29 2000/10/08 17:50:44 assar Exp $");
+RCSID("$Id: klog.c,v 1.29.2.1 2001/10/03 22:52:01 assar Exp $");
 
 #include "appl_locl.h"
 #include "klog.h"
@@ -242,7 +242,7 @@ int get_afs_token(void)
 #ifdef HAVE_KRB_AFSLOG_UID
 	rc = krb_afslog_uid(arg_cell, arg_realm, afsid);
 #else
-	rc = k_afslog_uid(arg_cell, arg_realm, afsid);
+	rc = k_afsklog_uid(arg_cell, arg_realm, afsid);
 #endif
 	if(rc)
 	    warnx("Unable to get an AFS token: %s", krb_get_err_text(rc));
