@@ -33,7 +33,7 @@
 
 #include "appl_locl.h"
 
-RCSID("$Id: arlalib.c,v 1.41 2001/01/07 12:14:44 lha Exp $");
+RCSID("$Id: arlalib.c,v 1.41.2.2 2001/10/02 16:13:00 jimmy Exp $");
 
 
 static struct rx_securityClass *secureobj = NULL ; 
@@ -593,7 +593,7 @@ int
 arlalib_try_next_db (int error)
 {
     switch (error) {
-    case RX_CALL_DEAD:
+    case ARLA_CALL_DEAD:
     case UNOTSYNC :
     case ENETDOWN :
 	return TRUE;
@@ -620,3 +620,26 @@ free_db_server_context(struct db_server_context *context)
   free(context->conn);
 }   
 
+/*
+ * give a name for the server `addr'
+ */
+
+void
+arlalib_host_to_name (u_int32_t addr, char *str, size_t str_sz)
+{
+    struct sockaddr_in sock;
+    int error;
+
+    memset (&sock, 0, sizeof(sock));
+#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
+    sock.sin_len = sizeof(sock);
+#endif
+    sock.sin_family = AF_INET;
+    sock.sin_port = 0;
+    sock.sin_addr.s_addr = addr;
+
+    error = getnameinfo((struct sockaddr *)&sock, sizeof(sock),
+			str, str_sz, NULL, 0, 0);
+    if (error)
+	strlcpy (str, "<unknown>", str_sz);
+}
