@@ -40,7 +40,7 @@
 #include <config.h>
 #endif
 
-RCSID("$Id: ka-procs.c,v 1.4 2000/10/15 21:37:59 assar Exp $");
+RCSID("$Id: ka-procs.c,v 1.4.2.2 2001/10/03 22:53:44 assar Exp $");
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -218,7 +218,7 @@ get_password (const char *realm, const char *password, des_cblock *key)
     strlcpy (cell, realm, sizeof(cell));
     strlwr (cell);
 
-    afs_string_to_key (buf, realm, key);
+    afs_string_to_key (password, realm, key);
     memset (buf, 0, sizeof(buf));
 
     return 0;
@@ -305,7 +305,11 @@ ka_authenticate (const char *user, const char *instance, const char *cell,
 	    goto out;
 
 	if (k_hasafs()) {
+#ifdef HAVE_KRB_AFSLOG_UID
 	    ret = krb_afslog (cell, cell);
+#else
+	    ret = k_afsklog_uid (cell, cell);
+#endif
 	    if (ret) {
 		warnx ("failed inserting tokens for cell %s", cell);
 	    }
