@@ -33,7 +33,7 @@
 
 #include "appl_locl.h"
 
-RCSID("$Id: tokens.c,v 1.1 2001/01/07 12:12:03 lha Exp $");
+RCSID("$Id: tokens.c,v 1.1.2.2 2001/04/18 13:53:36 lha Exp $");
 
 /*
  * Iterate over all tokens and apply `func' on all of them, passing
@@ -51,6 +51,7 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
     unsigned char secret[128];
     size_t secret_sz;
     struct ClearToken ct;
+    int ret;
 
     if (!k_hasafs())
 	return ENOSYS;
@@ -90,19 +91,23 @@ arlalib_token_iter (const char *cell, arlalib_token_iter_func func, void *arg)
 	strlcpy(token_cell, t, sizeof(token_cell));
 	
 	if (cell) {
-	    if (strcasecmp(token_cell, cell) == 0)
-		return (*func) (secret, secret_sz, &ct, cell, arg);
-	} else {
-	    int ret;
-
-	    ret = (*func) (secret, secret_sz, &ct, token_cell, arg);
-	    if (ret)
+	    if (strcasecmp(token_cell, cell) == 0) {
+		ret = (*func) (secret, secret_sz, &ct, cell, arg);
+		memset (data, 0, sizeof(data));
 		return ret;
+	    }
+	} else {
+	    ret = (*func) (secret, secret_sz, &ct, token_cell, arg);
+	    if (ret) {
+		memset (data, 0, sizeof(data));
+		return ret;
+	    }
 	}
 	    
     next_token:
 	i++;
     }
+    memset (data, 0, sizeof(data));
     if (cell)
 	return ENOENT;
     return 0;
