@@ -36,7 +36,7 @@
 #include <sl.h>
 #include "appl_locl.h"
 
-RCSID("$Id: pts.c,v 1.50 2001/01/07 12:30:33 lha Exp $");
+RCSID("$Id: pts.c,v 1.50.2.2 2001/07/06 01:40:11 lha Exp $");
 
 static int help_cmd (int argc, char **argv);
 
@@ -679,7 +679,10 @@ pts_name_to_id(struct rx_connection *conn, const char *name, int32_t *id)
 	errx(1, "Out of memory");
     strlcpy(nlist.val[0], name, sizeof(prname));
     res = PR_NameToID(conn, &nlist, &ilist);
-    *id = ilist.val[0];
+    if (res == 0)
+	*id = ilist.val[0];
+    if (*id == PR_ANONYMOUSID)
+	res = PRNOENT;
     free(ilist.val);
     free(nlist.val);
     return res;
@@ -695,8 +698,6 @@ listowned_1 (struct rx_connection *conn, const char *user)
     prlist pr;
     int i;
 
-    printf("%s\n", user);
-
     error = pts_name_to_id (conn, user, &id);
     if(error != 0)
         return error;
@@ -706,14 +707,10 @@ listowned_1 (struct rx_connection *conn, const char *user)
     if(pr.val == NULL)
 	errx(1, "Out of memory");
 
-    printf("%d\n", id);
     error = PR_ListOwned(conn, id, &pr, &over);
     if(error != 0)
 	return error;
 
-    printf("id = %d\n", id);
-    printf("pr.len = %d\n", pr.len);
-
     i = 0;
     name = malloc(PR_MAXNAMELEN);
     if(name == NULL)
@@ -1338,41 +1335,35 @@ listmembershipbyname(struct rx_connection *conn, const char *name)
 {
     int32_t res = 0;
     int32_t id = 0;
+    int32_t over;
+    int i;
+    prlist elist;
 
     res = pts_name_to_id(conn, name, &id);
-    if(res) {
-	fprintf(stderr, "membership: error %s (%d)\n",
-		koerr_gettext(res), res);
+    if(res)
         return res;
-    } else {
-	int32_t over;
-	int i;
-	prlist elist;
-	elist.len = PR_MAXGROUPS; /* XXX this will allocate 5000 ints, should
-				  check how many groups first. That will take
-				  a lot of code though. */
-	elist.val = malloc(sizeof(int32_t) * elist.len);
-	if(elist.val == NULL)
-	    errx(1, "Out of memory");
-	res = PR_ListElements(conn, id, &elist, &over);
-	if(res != 0) {
-	    fprintf(stderr, "membership: error %s (%d)\n",
-		    koerr_gettext(res), res);
-	    return res;
-	} else {
-	    if(id>=0)
-		printf("Groups %s (id: %d) is a member of:\n", name, id);
-	    else
-		printf("Members of %s (id: %d) are:\n", name, id);
-	    for(i = 0; i < elist.len; i++) {
-		prname pr;
-		res = pts_id_to_name(conn, elist.val[i], &pr);
-		printf("  %s\n", (char *)&pr);
-	    }
-	    free(elist.val);
-	    return 0;
-	}
+
+    elist.len = PR_MAXGROUPS; /* XXX this will allocate 5000 ints, should
+				 check how many groups first. That will take
+				 a lot of code though. */
+    elist.val = malloc(sizeof(int32_t) * elist.len);
+    if(elist.val == NULL)
+	errx(1, "Out of memory");
+    res = PR_ListElements(conn, id, &elist, &over);
+    if(res != 0)
+	return res;
+    
+    if(id>=0)
+	printf("Groups %s (id: %d) is a member of:\n", name, id);
+    else
+	printf("Members of %s (id: %d) are:\n", name, id);
+    for(i = 0; i < elist.len; i++) {
+	prname pr;
+	res = pts_id_to_name(conn, elist.val[i], &pr);
+	printf("  %s\n", (char *)&pr);
     }
+    free(elist.val);
+    return 0;
 }
 
 static int
@@ -1409,9 +1400,6 @@ member_cmd (int argc, char **argv)
 	return 0;
     }
 
-    for(i=0;i<users.num_strings;i++)
-	printf("%s\n", users.strings[i]);
-    
     cell = cell_expand_cell (cell);
 
     memset(&conn_context, '\0', sizeof(struct db_server_context));
@@ -1429,16 +1417,13 @@ member_cmd (int argc, char **argv)
 		error = tmp;
 	}
     }
+    if (error)
+	fprintf(stderr, "membership: %s (%d)\n",
+		koerr_gettext(error), error);
     free_db_server_context(&conn_context);
     return 0;
 }
 
-static void
-listowned_usage(void)
-{
-    printf("Usage: pts [-id] <user or group> [-cell <cell>] [-noauth] [-help]\n");
-}
-
 static int
 listowned_cmd(int argc, char **argv)
 {
@@ -1465,7 +1450,7 @@ listowned_cmd(int argc, char **argv)
     arg->value = &noauth; arg++;
 
     if (agetarg (listowned_args, argc, argv, &optind, AARG_AFSSTYLE)) {
-	listowned_usage();
+	aarg_printusage (listowned_args, "pts listowned", NULL, AARG_AFSSTYLE);
 	return 0;
     }
 
@@ -1541,6 +1526,7 @@ main(int argc, char **argv)
     int i;
 
     set_progname(argv[0]);
+    tzset();
 
     method = log_open (get_progname(), "/dev/stderr:notime");
     if (method == NULL)
