@@ -35,7 +35,7 @@
 #include <sl.h>
 #include "vos_local.h"
 
-RCSID("$Id: vos.c,v 1.61 2000/10/03 00:08:18 lha Exp $");
+RCSID("$Id: vos.c,v 1.61.2.2 2001/09/18 00:24:17 mattiasa Exp $");
 
 int vos_interactive = 0;
 
@@ -52,8 +52,8 @@ static int apropos_cmd(int argc, char **argv);
 static SL_cmd cmds[] = {
     {"addsite",    empty_cmd,       "not yet implemented"},
     {"apropos",    apropos_cmd,     "apropos"},
-    {"backup",     empty_cmd,       "not yet implemented"},
-    {"backupsys",  empty_cmd,       "not yet implemented"},
+    {"backup",     vos_backup,      "Make a backup copy of a volume"},
+    {"backupsys",  vos_backupsys,   "Make backup copies of multiple volumes"},
     {"changeaddr", empty_cmd,       "not yet implemented"},
     {"create",     vos_create,      "create a volume"},
     {"createentry",vos_createentry, "create a vldb entry"},
@@ -148,6 +148,7 @@ main(int argc, char **argv)
     int ret = 0;
     
     set_progname(argv[0]);
+    tzset();
 
     method = log_open (get_progname(), "/dev/stderr:notime");
     if (method == NULL)
