@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -32,7 +32,7 @@
  */
 
 /*
- * RCSID("$Id: vos_local.h,v 1.12 2000/10/03 00:09:12 lha Exp $");
+ * RCSID("$Id: vos_local.h,v 1.12.2.2 2001/09/17 23:59:02 mattiasa Exp $");
  */
 
 #define LISTVOL_PART      0x1
@@ -61,8 +61,11 @@ int vos_syncsite (int, char **);
 int vos_dump (int, char **);
 int vos_create(int argc, char **argv);
 int vos_endtrans(int argc, char **argv);
+int vos_backup(int argc, char **argv);
+int vos_backupsys(int argc, char **argv);
 
-int vos_listvldb_iter (const char *host, const char *cell,
+int vos_listvldb_iter (const char *host, const char *cell, const char *volname,
+		       const char *fileserver, const char *part,
 		       arlalib_authflags_t auth, 
 		       int (*proc)(void *data, struct vldbentry *),
 		       void *data);
@@ -87,3 +90,4 @@ int vos_endtransaction (const char *cell, const char *host,
 			int32_t trans, arlalib_authflags_t auth, int verbose);
 
 void find_db_cell_and_host (const char **cell, const char **host);
+void get_servername (u_int32_t addr, char *str, size_t str_sz);
