@@ -35,7 +35,7 @@
 #include <sl.h>
 #include "vos_local.h"
 
-RCSID("$Id: vos_status.c,v 1.7 2000/10/03 00:09:28 lha Exp $");
+RCSID("$Id: vos_status.c,v 1.7.2.1 2001/03/04 04:16:20 lha Exp $");
 
 static int
 printstatus (const char *cell, const char *host,
@@ -70,11 +70,17 @@ printstatus (const char *cell, const char *host,
 	printf ("No active transactions on %s\n", host);
     else {
 	for (i = 0; i < entries_len; i--) {
+	    struct tm tm;
+	    char timestr[20];
 	    char part[100];
 
 	    printf("--------------------------------------\n");
-	    printf("transaction: %d  created: %s", entries->tid, 
-		   ctime((time_t *) &entries->creationTime));
+	    
+	    memset (&tm, 0, sizeof(tm));
+	    strftime (timestr, sizeof(timestr), "%Y-%m-%d %H:%M:%S %Z",
+		      localtime_r((time_t*) &entries->creationTime, &tm));
+	    printf("transaction: %d  created: %s\n", entries->tid, 
+		   timestr);
 	    printf("attachFlags:  ");
 
 	    if ((entries->iflags & ITOffline) == ITOffline)
