@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -35,7 +35,7 @@
 #include <sl.h>
 #include "vos_local.h"
 
-RCSID("$Id: vos_vldbexamine.c,v 1.11 2000/12/18 13:42:52 assar Exp $");
+RCSID("$Id: vos_vldbexamine.c,v 1.11.2.1 2001/03/04 04:16:23 lha Exp $");
 
 static void
 print_volume (const nvldbentry *nvlentry, const char *server_name)
@@ -103,23 +103,8 @@ printvolstat(const char *volname, const char *cell, const char *host,
     if (verbose)
 	fprintf (stderr, "done\n");
 
-    {
-	struct sockaddr_in sock;
-
-	memset (&sock, 0, sizeof(sock));
-#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
-	sock.sin_len = sizeof(sock);
-#endif
-	sock.sin_family = AF_INET;
-	sock.sin_port = 0;
-	sock.sin_addr.s_addr = htonl(nvlentry.serverNumber[0]);
-	
-	error = getnameinfo((struct sockaddr *)&sock, sizeof(sock),
-			    server_name, sizeof(server_name), NULL, 0, 0);
-	if (error)
-	    strlcpy (server_name, "<unknown>", sizeof(server_name));
-    }
-
+    get_servername (htonl(nvlentry.serverNumber[0]),
+		    server_name, sizeof(server_name));
     print_volume (&nvlentry, server_name);
 
     printf("    ");
@@ -132,22 +117,8 @@ printvolstat(const char *volname, const char *cell, const char *host,
      for (i = 0; i < nvlentry.nServers; i++) {
 	 printf("       ");
 	 
-	 {
-	     struct sockaddr_in sock;
-
-	     memset(&sock, 0, sizeof(sock));
-#if HAVE_STRUCT_SOCKADDR_IN_SIN_LEN
-	     sock.sin_len = sizeof(sock);
-#endif
-	     sock.sin_family = AF_INET;
-	     sock.sin_port = 0;
-	     sock.sin_addr.s_addr = htonl(nvlentry.serverNumber[i]);
-	     
-	     error = getnameinfo((struct sockaddr *)&sock, sizeof(sock),
-				 server_name, sizeof(server_name), NULL, 0, 0);
-	     if (error)
-		 strlcpy (server_name, "<unknown>", sizeof(server_name));
-	 }
+	 get_servername (htonl(nvlentry.serverNumber[i]),
+			 server_name, sizeof(server_name));
 
 	 partition_num2name (nvlentry.serverPartition[i],
 			     part_name, sizeof(part_name));
