@@ -37,7 +37,7 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: adir.c,v 1.66 2000/10/20 16:24:54 assar Exp $") ;
+RCSID("$Id: adir.c,v 1.66.2.1 2001/06/04 22:16:34 ahltorp Exp $") ;
 
 /*
  *
@@ -76,11 +76,7 @@ get_fbuf_from_fid (VenusFid *fid, CredCacheEntry **ce,
  */
 
 int
-adir_lookup_fcacheentry (FCacheEntry *centry,
-			 VenusFid dir,
-			 const char *name,
-			 VenusFid *file,
-			 CredCacheEntry *ce)
+adir_lookup (FCacheEntry *centry, const char *name, VenusFid *file)
 {
      int ret;
      int fd;
@@ -91,52 +87,12 @@ adir_lookup_fcacheentry (FCacheEntry *centry,
      if (ret)
 	 return ret;
 
-     ret = fdir_lookup (&the_fbuf, &dir, name, file);
+     ret = fdir_lookup (&the_fbuf, &centry->fid, name, file);
      fbuf_end (&the_fbuf);
      close (fd);
      return ret;
 }
 
-/*
- * Lookup ``name'' in the AFS directory identified by ``dir'' and
- * return the Fid in ``file''.  All operations are done as ``cred''
- * and return value is 0 or error code. If ``dentry'' is set, its the
- * entry of the ``dir'' if returnvalue is 0.
- */
-
-int
-adir_lookup (VenusFid *dir,
-	     const char *name,
-	     VenusFid *file,
-	     FCacheEntry **dentry, 
-	     CredCacheEntry **ce)
-{
-    int fd;
-    fbuf the_fbuf;
-    FCacheEntry *centry;
-    int ret;
-
-    ret = get_fbuf_from_fid (dir, ce, &fd, &the_fbuf, &centry, O_RDONLY, 
-			     FBUF_READ|FBUF_PRIVATE);
-    if (ret)
-	return ret;
-
-    ret = fdir_lookup (&the_fbuf, dir, name, file);
-    fbuf_end (&the_fbuf);
-    close (fd);
-    if (dentry) {
-	if (ret) {
-	    *dentry = NULL;
-	    fcache_release (centry);
-	} else {
-	    *dentry = centry;
-	}
-    } else {
-	fcache_release (centry);
-    }
-    return ret;
-}
-
 /*
  * Lookup `name' in the AFS directory identified by `dir' and change the
  * fid to `fid'.
