@@ -35,7 +35,7 @@
  * Interface to directory handling routines
  */
 
-/* $Id: adir.h,v 1.14 2000/10/02 22:30:01 lha Exp $ */
+/* $Id: adir.h,v 1.14.2.1 2001/06/04 22:16:35 ahltorp Exp $ */
 
 #ifndef _ADIR_H_
 #define _ADIR_H_
@@ -45,13 +45,8 @@
 #include "fcache.h"
 #include "fdir.h"
 
-int adir_lookup (VenusFid *dir, const char *name, VenusFid *file,
-		 FCacheEntry **dentry, CredCacheEntry **ce);
-
 int
-adir_lookup_fcacheentry (FCacheEntry *centry, VenusFid dir,
-			 const char *name,
-			 VenusFid *file, CredCacheEntry *ce);
+adir_lookup (FCacheEntry *centry, const char *name, VenusFid *file);
 
 int
 adir_changefid (VenusFid *dir,
