@@ -38,7 +38,7 @@
 #include "arla_local.h"
 #include <getarg.h>
 
-RCSID("$Id: afsdir_check.c,v 1.21 2000/10/10 16:27:06 lha Exp $");
+RCSID("$Id: afsdir_check.c,v 1.21.2.1 2001/10/03 23:36:00 assar Exp $");
 
 static int help_flag;
 static int verbose = 0;
@@ -80,7 +80,7 @@ getentry (DirPage0 *page0,
 }
 
 static int
-check (const char *filename)
+check_dir (const char *filename)
 {
     struct stat statbuf;
     int fd;
@@ -324,5 +324,5 @@ main(int argc, char **argv)
     if (argc != 1)
 	usage (1);
 
-    return check (argv[0]);
+    return check_dir (argv[0]);
 }
