@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,30 +31,26 @@
  * SUCH DAMAGE.
  */
 
+#define _KERNEL
+
 #include "arla_local.h"
-RCSID("$Id: aix-subr.c,v 1.6 1999/07/24 19:48:30 lha Exp $");
+RCSID("$Id: aix-subr.c,v 1.14 2000/10/14 19:58:04 map Exp $");
 
 static long blocksize = 1024;
 
-struct args {
-     int fd;
-     off_t off;
-     char *buf;
-     char *ptr;
-     struct dirent *last;
-};
-
 static void
-flushbuf (struct args *args)
+flushbuf (void *vargs)
 {
-     unsigned inc = blocksize - (args->ptr - args->buf);
+    struct write_dirent_args *args = (struct write_dirent_args *)vargs;
+    struct dirent *last = (struct dirent *)args->last;
+    unsigned inc = blocksize - (args->ptr - args->buf);
 
-     args->last->d_reclen += inc;
-     args->last->d_offset += inc;
-     if (write (args->fd, args->buf, blocksize) != blocksize)
-	  arla_warn (ADEBWARN, errno, "write");
-     args->ptr = args->buf;
-     args->last = NULL;
+    last->d_reclen += inc;
+    last->d_offset += inc;
+    if (write (args->fd, args->buf, blocksize) != blocksize)
+	arla_warn (ADEBWARN, errno, "write");
+    args->ptr = args->buf;
+    args->last = NULL;
 }
 
 #undef DIRSIZ
@@ -71,7 +62,7 @@ static void
 write_dirent(VenusFid *fid, const char *name, void *arg)
 {
      struct dirent *real;
-     struct args *args = (struct args *)arg;
+     struct write_dirent_args *args = (struct write_dirent_args *)arg;
      u_short reclen;
 
      reclen = DIRSIZ(name);
@@ -81,7 +72,7 @@ write_dirent(VenusFid *fid, const char *name, void *arg)
      real = (struct dirent *)args->ptr;
 
      real->d_reclen = reclen;
-     real->d_ino = afsfid2inode (fid);
+     real->d_ino = dentry2ino (name, fid, args->e);
      strcpy (real->d_name, name);
      args->ptr += real->d_reclen;
      args->off += real->d_reclen;
@@ -91,59 +82,11 @@ write_dirent(VenusFid *fid, const char *name, void *arg)
 
 Result
 conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
-	  xfs_cache_handle *cache_handle,
+	  fcache_cache_handle *cache_handle,
 	  char *cache_name, size_t cache_name_sz)
 {
-     struct args args;
-     Result res;
-     fbuf the_fbuf;
-     int ret;
-
-     e->flags.extradirp = TRUE;
-     fcache_extra_file_name (e, cache_name, cache_name_sz);
-     res.tokens = e->tokens |= XFS_DATA_R | XFS_OPEN_NR;
-
-     args.fd = open (cache_name,
-		     O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
-     if (args.fd == -1) {
-	  arla_warn (ADEBWARN, errno, "open %s", cache_name);
-	  res.res = -1;
-	  res.error = errno;
-	  return res;
-     }
-     fcache_fhget (cache_name, cache_handle);
-
-     args.off  = 0;
-     args.buf  = (char *)malloc (blocksize);
-     if (args.buf == NULL) {
-	 arla_warn (ADEBWARN, errno, "malloc %lu", blocksize);
-	 res.res = -1;
-	 res.error = errno;
-	 close (args.fd);
-	 return res;
-     }
-     ret = fcache_get_fbuf (e, &the_fbuf, O_RDONLY, FBUF_READ);
-     if (ret) {
-	 res.res = -1;
-	 res.error = ret;
-	 close (args.fd);
-	 free (args.buf);
-	 return res;
-     }
-     
-     args.ptr  = args.buf;
-     args.last = NULL;
-     
-     fdir_readdir (&the_fbuf, write_dirent, (void *)&args, &e->fid);
-
-     fbuf_end (&the_fbuf);
-
-     if (args.last)
-	  flushbuf (&args);
-     free (args.buf);
-     if ((res.res = close (args.fd)) != 0)
-	  res.error = errno;
-     return res;
+    return conv_dir_sub (e, ce, tokens, cache_handle, cache_name,
+			 cache_name_sz, write_dirent, flushbuf, blocksize);
 }
 
 /*
@@ -152,7 +95,7 @@ conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
 
 int
 dir_remove_name (FCacheEntry *e, const char *filename,
-		 xfs_cache_handle *cache_handle,
+		 fcache_cache_handle *cache_handle,
 		 char *cache_name, size_t cache_name_sz)
 {
     int ret;
@@ -177,22 +120,31 @@ dir_remove_name (FCacheEntry *e, const char *filename,
     }
     len = sb.st_size;
 
-    ret = fbuf_create (&fb, fd, len, FBUF_WRITE);
-    if (ret)
+    ret = fbuf_create (&fb, fd, len, FBUF_READ|FBUF_WRITE|FBUF_SHARED);
+    if (ret) {
+	close (fd);
 	return ret;
+    }
     last_dp = NULL;
+    ret = ENOENT;
     for (p = buf = fbuf_buf (&fb); p < buf + len; p += dp->d_reclen) {
 	dp = (struct dirent *)p;
 
 	if (strcmp (filename, dp->d_name) == 0) {
-	    if (last_dp != NULL)
-		last_dp->d_reclen += dp->d_reclen;
+	    if (last_dp != NULL) {
+		struct dirent new_last;
+
+		new_last.d_reclen = last_dp->d_reclen + dp->d_reclen;
+		if (new_last.d_reclen >= last_dp->d_reclen)
+		    last_dp->d_reclen = new_last.d_reclen;
+	    }
 	    dp->d_ino = 0;
-	    fbuf_end (&fb);
-	    return 0;
+	    ret = 0;
+	    break;
 	}
 	last_dp = dp;
     }
     fbuf_end (&fb);
-    return ENOENT;
+    close (fd);
+    return ret;
 }
