@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -32,7 +32,7 @@
  */
 
 #include <arla_local.h>
-RCSID("$Id: arladeb.c,v 1.24 2000/10/02 22:30:28 lha Exp $");
+RCSID("$Id: arladeb.c,v 1.24.2.2 2001/04/30 00:01:59 lha Exp $");
 
 Log_method* arla_log_method = NULL;
 Log_unit* arla_log_unit = NULL;
@@ -51,7 +51,7 @@ arla_log(unsigned level, char *fmt, ...)
 
 
 void
-arla_loginit(char *log)
+arla_loginit(char *log, log_flags flags)
 {
     assert (log);
     
@@ -62,6 +62,7 @@ arla_loginit(char *log)
 				   ARLA_DEFAULT_LOG);
     if (arla_log_unit == NULL)
 	errx (1, "arla_loginit: log_unit_init failed");
+    log_setflags (arla_log_method, flags);
 }
 
 int
@@ -193,14 +194,14 @@ arla_vwarnx_with_fid (unsigned level, const VenusFid *fid, const char *fmt,
 {
     char *s;
     const char *cellname;
-    const char *volname;
+    char volname[VLDB_MAXNAMELEN];
 
     cellname = cell_num2name (fid->Cell);
     if (cellname == NULL)
 	cellname = "<unknown>";
-    volname = volcache_getname (fid->fid.Volume, fid->Cell);
-    if (volname == NULL)
-	volname = "<unknown>";
+    if (volcache_getname (fid->fid.Volume, fid->Cell,
+			  volname, sizeof(volname)) != 0)
+	strlcpy(volname, "<unknown>", sizeof(volname));
 
     vasprintf (&s, fmt, args);
     if (s == NULL) {
