@@ -32,7 +32,7 @@
  */
 
 /*
- * $Id: arladeb.h,v 1.24 2000/10/02 22:30:33 lha Exp $
+ * $Id: arladeb.h,v 1.24.2.1 2001/04/30 00:02:15 lha Exp $
  */
 
 #ifndef _arladeb_h
@@ -71,7 +71,7 @@ extern struct units arla_deb_units[];
 extern struct units arla_deb_units[];
 
 void arla_log(unsigned level, char *fmt, ...);
-void arla_loginit(char *log);
+void arla_loginit(char *log, log_flags flags);
 int arla_log_set_level (const char *s);
 void arla_log_set_level_num (unsigned level);
 void arla_log_get_level (char *s, size_t len);
