@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,49 +35,37 @@
 #ifdef __osf__
 #define _OSF_SOURCE
 #define _BSD
+/* Might want to define _KERNEL for osf and use KDIRSIZ */
 #endif
+
 #include "arla_local.h"
-RCSID("$Id: bsd-subr.c,v 1.35 1999/07/24 19:48:30 lha Exp $");
+RCSID("$Id: bsd-subr.c,v 1.55.2.1 2001/01/29 02:02:17 lha Exp $");
 
 #ifdef __linux__
 #include <xfs/xfs_dirent.h>
 #else
-#define XFS_DIRENT_BLOCKSIZE 1024
+#define XFS_DIRENT_BLOCKSIZE 512
 #define xfs_dirent dirent
 #endif
-#if defined(GENERIC_DIRSIZ_IN_SYS_DIRENT_H)
-#include <sys/dirent.h>
-#elif defined(DIRSIZ_IN_DIRENT_H)
-#include <dirent.h>
-#elif defined(DIRSIZ_IN_SYS_DIR_H)
-#include <sys/dir.h>
-#endif
 
 static long blocksize = XFS_DIRENT_BLOCKSIZE;	/* XXX */
 
-struct args {
-    int fd;
-    off_t off;
-    char *buf;
-    char *ptr;
-    struct xfs_dirent *last;
-    FCacheEntry *e; 
-};
-
 /*
  * Write out all remaining data in `args'
  */
 
 static void
-flushbuf (struct args *args)
+flushbuf (void *vargs)
 {
-     unsigned inc = blocksize - (args->ptr - args->buf);
+    struct write_dirent_args *args = (struct write_dirent_args *)vargs;
+    unsigned inc = blocksize - (args->ptr - args->buf);
+    struct xfs_dirent *last = (struct xfs_dirent *)args->last;
 
-     args->last->d_reclen += inc;
-     if (write (args->fd, args->buf, blocksize) != blocksize)
-	  arla_warn (ADEBWARN, errno, "write");
-     args->ptr = args->buf;
-     args->last = NULL;
+    last->d_reclen += inc;
+    if (write (args->fd, args->buf, blocksize) != blocksize)
+	arla_warn (ADEBWARN, errno, "write");
+    args->ptr = args->buf;
+    args->last = NULL;
 }
 
 /*
@@ -93,11 +76,13 @@ static void
 write_dirent(VenusFid *fid, const char *name, void *arg)
 {
      struct xfs_dirent dirent, *real;
-     struct args *args = (struct args *)arg;
+     struct write_dirent_args *args = (struct write_dirent_args *)arg;
 
      dirent.d_namlen = strlen (name);
 #ifdef _GENERIC_DIRSIZ
      dirent.d_reclen = _GENERIC_DIRSIZ(&dirent);
+#elif defined(DIRENT_SIZE)
+     dirent.d_reclen = DIRENT_SIZE(&dirent);
 #else
      dirent.d_reclen = DIRSIZ(&dirent);
 #endif
@@ -112,102 +97,33 @@ write_dirent(VenusFid *fid, const char *name, void *arg)
      real->d_type   = DT_UNKNOWN;
 #endif
      
-     if (dirent.d_namlen == 2
-	 && strcmp(name, "..") == 0
-	 && args->e->flags.mountp) {
-	 real->d_fileno = afsfid2inode (&args->e->parent);
-     } else if (dirent.d_namlen == 1 && 
-		strcmp(name, ".") == 0 &&
-		args->e->flags.mountp) {
-	 real->d_fileno = afsfid2inode (&args->e->realfid); 
-     } else
-	 real->d_fileno = afsfid2inode (fid);
-     strcpy (real->d_name, name);
+     real->d_fileno = dentry2ino (name, fid, args->e);
+     strlcpy (real->d_name, name, sizeof(real->d_name));
      args->ptr += real->d_reclen;
      args->off += real->d_reclen;
-#if 0
-     real->d_off = args->off;
-#endif
      args->last = real;
 }
 
-/*
- * Assume `e' has valid data.
- */
-
 Result
 conv_dir (FCacheEntry *e, CredCacheEntry *ce, u_int tokens,
-	  xfs_cache_handle *cache_handle,
+	  fcache_cache_handle *cache_handle,
 	  char *cache_name, size_t cache_name_sz)
 {
-     struct args args;
-     Result res;
-     int ret;
-     fbuf the_fbuf;
-
-     e->flags.extradirp = TRUE;
-     fcache_extra_file_name (e, cache_name, cache_name_sz);
-     res.tokens = e->tokens |= XFS_DATA_R | XFS_OPEN_NR;
-
-     args.fd = open (cache_name, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
-     if (args.fd == -1) {
-	  res.res = -1;
-	  res.error = errno;
-	  arla_warn (ADEBWARN, errno, "open %s", cache_name);
-	  return res;
-     }
-     ret = fcache_fhget (cache_name, cache_handle);
-     if(ret) {
-	 res.res = -1;
-	 res.error = errno;
-	 close (args.fd);
-	 arla_warn (ADEBWARN, res.error, "fcache_fhget(%s)", cache_name);
-	 return res;
-     }
-
-     args.off  = 0;
-     args.buf  = (char *)malloc (blocksize);
-     if (args.buf == NULL) {
-	 arla_warn (ADEBWARN, errno, "malloc %u", (unsigned)blocksize);
-	 res.res = -1;
-	 res.error = errno;
-	 close (args.fd);
-	 return res;
-     }
-
-     ret = fcache_get_fbuf (e, &the_fbuf, O_RDONLY, FBUF_READ);
-     if (ret) {
-	 res.res = -1;
-	 res.error = ret;
-	 close (args.fd);
-	 free (args.buf);
-	 return res;
-     }
-     
-     args.ptr  = args.buf;
-     args.last = NULL;
-     args.e = e;
-     
-     fdir_readdir (&the_fbuf, write_dirent, (void *)&args, &e->fid);
-
-     fbuf_end (&the_fbuf);
-
-     if (args.last)
-	  flushbuf (&args);
-     free (args.buf);
-     res.res = close (args.fd);
-     if (res.res)
-	  res.error = errno;
-     return res;
+    return conv_dir_sub (e, ce, tokens, cache_handle, cache_name,
+			 cache_name_sz, write_dirent, flushbuf, blocksize);
 }
 
 /*
  * remove `filename` from the converted directory for `e'
  */
 
+#ifndef DIRBLKSIZ
+#define DIRBLKSIZ 1024
+#endif
+
 int
 dir_remove_name (FCacheEntry *e, const char *filename,
-		 xfs_cache_handle *cache_handle,
+		 fcache_cache_handle *cache_handle,
 		 char *cache_name, size_t cache_name_sz)
 {
     int ret;
@@ -232,35 +148,39 @@ dir_remove_name (FCacheEntry *e, const char *filename,
     }
     len = sb.st_size;
 
-    ret = fbuf_create (&fb, fd, len, FBUF_WRITE);
-    if (ret)
+    ret = fbuf_create (&fb, fd, len, FBUF_READ|FBUF_WRITE|FBUF_SHARED);
+    if (ret) {
+	close (fd);
 	return ret;
+    }
     last_dp = NULL;
+    ret = ENOENT;
     for (p = buf = fbuf_buf (&fb); p < buf + len; p += dp->d_reclen) {
 
 	dp = (struct xfs_dirent *)p;
 
-	assert (dp->d_reclen != 0 &&
-		p >= buf &&
-		p < buf + len);
-	
+	assert (dp->d_reclen > 0);
 
 	if (strcmp (filename, dp->d_name) == 0) {
-	    assert (last_dp != NULL);
+	    if (last_dp != NULL) {
+		unsigned len;
 
-	    /*
-	     * Make sure we dont overrun the d_reclen
-	     */
-
-	    if (sizeof (last_dp->d_reclen) - dp->d_reclen < last_dp->d_reclen)
-		last_dp->d_reclen += dp->d_reclen;
+		/*
+		 * It's not totally clear how large we can make
+		 * d_reclen without loosing.  Not making it larger
+		 * than DIRBLKSIZ seems safe.
+		 */
+		len = last_dp->d_reclen + dp->d_reclen;
+		if (len < DIRBLKSIZ)
+		    last_dp->d_reclen = len;
+	    }
 	    dp->d_fileno = 0;
-
-	    fbuf_end (&fb);
-	    return 0;
+	    ret = 0;
+	    break;
 	}
 	last_dp = dp;
     }
     fbuf_end (&fb);
-    return ENOENT;
+    close (fd);
+    return ret;
 }
