@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -41,7 +36,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: cred.c,v 1.28 1999/04/20 20:58:07 map Exp $");
+RCSID("$Id: cred.c,v 1.33.2.2 2001/04/21 14:55:34 lha Exp $");
 
 #define CREDCACHESIZE 101
 
@@ -121,7 +116,7 @@ cred_init (unsigned nentries)
 }
 
 static CredCacheEntry *
-internal_get (long cell, pag_t cred, int type)
+internal_get (long cell, xfs_pag_t cred, int type)
 {
     CredCacheEntry *e;
     CredCacheEntry key;
@@ -143,7 +138,7 @@ internal_get (long cell, pag_t cred, int type)
 }
 
 CredCacheEntry *
-cred_get (long cell, pag_t cred, int type)
+cred_get (long cell, xfs_pag_t cred, int type)
 {
     if (type == CRED_ANY) {
 	CredCacheEntry *e;
@@ -163,7 +158,8 @@ cred_get (long cell, pag_t cred, int type)
 static void
 recycle_entry (CredCacheEntry *ce)
 {
-    hashtabdel (credhtab, ce);
+    if (!ce->flags.killme)
+	hashtabdel (credhtab, ce);
     if (ce->cred_data != NULL)
 	free (ce->cred_data);
     memset (ce, 0, sizeof(*ce));
@@ -199,7 +195,7 @@ get_free_cred (void)
 }
 
 CredCacheEntry *
-cred_add (pag_t cred, int type, int securityindex, long cell,
+cred_add (xfs_pag_t cred, int type, int securityindex, long cell,
 	  time_t expire, void *cred_data, size_t cred_data_sz,
 	  uid_t uid)
 {
@@ -236,7 +232,7 @@ cred_add (pag_t cred, int type, int securityindex, long cell,
 
 #if KERBEROS
 CredCacheEntry *
-cred_add_krb4 (pag_t cred, uid_t uid, CREDENTIALS *c)
+cred_add_krb4 (xfs_pag_t cred, uid_t uid, CREDENTIALS *c)
 {
     CredCacheEntry *ce;
     char *cellname;
@@ -246,10 +242,13 @@ cred_add_krb4 (pag_t cred, uid_t uid, CREDENTIALS *c)
 	cellname = strdup (c->realm);
     else
 	cellname = strdup (c->instance);
+    if (cellname == NULL)
+	return NULL;
     strlwr (cellname);
     cellnum = cell_name2num(cellname);
     free (cellname);
-    assert (cellnum != -1);
+    if (cellnum == -1)
+	return NULL;
 
     ce = cred_add (cred, CRED_KRB4, 2, cellnum, -1, c, sizeof(*c), uid);
     return ce;
@@ -263,9 +262,10 @@ cred_add_krb4 (pag_t cred, uid_t uid, CREDENTIALS *c)
 void
 cred_delete (CredCacheEntry *ce)
 {
-    if (ce->refcount > 0)
+    if (ce->refcount > 0) {
 	ce->flags.killme = 1;
-    else
+	hashtabdel (credhtab, ce);
+    } else
 	recycle_entry (ce);
 }
 
@@ -277,22 +277,15 @@ void
 cred_expire (CredCacheEntry *ce)
 {
     const char *cell_name = cell_num2name (ce->cell);
-    struct passwd *pwd = getpwuid (ce->uid);
-    const char *user_name;
-
-    if (pwd != NULL)
-	user_name = pwd->pw_name;
-    else
-	user_name = "<who-are-you?>";
 
     if (cell_name != NULL)
 	arla_warnx (ADEBWARN,
-		    "Credentials for %s (%u) in cell %s has expired",
-		    user_name, (unsigned)ce->uid, cell_name);
+		    "Credentials for UID %u in cell %s has expired",
+		    (unsigned)ce->uid, cell_name);
     else
 	arla_warnx (ADEBWARN,
-		    "Credentials for %s (%u) in cell unknown %ld has expired",
-		    user_name, (unsigned)ce->uid, ce->cell);
+		    "Credentials for UID %u in cell unknown %ld has expired",
+		    (unsigned)ce->uid, ce->cell);
 
     cred_delete (ce);
 }
@@ -301,7 +294,7 @@ static Bool
 remove_entry (void *ptr, void *arg)
 {
     CredCacheEntry *ce = (CredCacheEntry *)ptr;
-    pag_t *cred = (pag_t *)arg;
+    xfs_pag_t *cred = (xfs_pag_t *)arg;
 
     if (ce->cred == *cred)
 	cred_delete (ce);
@@ -309,7 +302,7 @@ remove_entry (void *ptr, void *arg)
 }
 
 void
-cred_remove (pag_t cred)
+cred_remove (xfs_pag_t cred)
 {
     hashtabforeach (credhtab, remove_entry, &cred);
 }
@@ -319,8 +312,8 @@ print_cred (void *ptr, void *arg)
 {
     CredCacheEntry *e = (CredCacheEntry *)ptr;
 
-    arla_log(ADEBVLOG, "cred = %u, type = %d, securityindex = %d\n"
-	     "cell = %ld, refcount = %u, killme = %d, uid = %lu\n\n",
+    arla_log(ADEBVLOG, "cred = %u, type = %d, securityindex = %d, "
+	     "cell = %ld, refcount = %u, killme = %d, uid = %lu",
 	     e->cred, e->type, e->securityindex, e->cell, e->refcount,
 	     e->flags.killme, (unsigned long)e->uid);
     return FALSE;
@@ -329,7 +322,7 @@ print_cred (void *ptr, void *arg)
 void
 cred_status (void)
 {
-    arla_log(ADEBVLOG, "%u(%u) credentials\n",
+    arla_log(ADEBVLOG, "%u(%u) credentials",
 	     nactive_credentials, ncredentials);
     hashtabforeach (credhtab, print_cred, NULL);
 }
