@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
  * Header for credetial cache
  */
 
-/* $Id: cred.h,v 1.23 1999/04/14 15:27:34 map Exp $ */
+/* $Id: cred.h,v 1.26 2000/10/02 22:31:15 lha Exp $ */
 
 #ifndef _CRED_H_
 #define _CRED_H_
@@ -69,7 +64,7 @@ typedef struct {
 #endif
 
 typedef struct {
-    pag_t cred;
+    xfs_pag_t cred;
     uid_t uid;
     int type;
     int securityindex;
@@ -85,13 +80,13 @@ typedef struct {
 void cred_init (unsigned nentries);
 
 CredCacheEntry *
-cred_get (long cell, pag_t cred, int type);
+cred_get (long cell, xfs_pag_t cred, int type);
 
 void
 cred_free (CredCacheEntry *ce);
 
 CredCacheEntry *
-cred_add (pag_t cred, int type, int securityindex, long cell,
+cred_add (xfs_pag_t cred, int type, int securityindex, long cell,
 	  time_t expire, void *cred_data, size_t cred_data_sz,
 	  uid_t uid);
 
@@ -102,11 +97,11 @@ void
 cred_expire (CredCacheEntry *ce);
 
 #ifdef KERBEROS
-CredCacheEntry * cred_add_krb4 (pag_t cred, uid_t uid, CREDENTIALS *c);
+CredCacheEntry * cred_add_krb4 (xfs_pag_t cred, uid_t uid, CREDENTIALS *c);
 #endif
 
 void cred_status (void);
 
-void cred_remove (pag_t cred);
+void cred_remove (xfs_pag_t cred);
 
 #endif /* _CRED_H_ */
