@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -44,7 +39,7 @@
 
 #include <arla_local.h>
 
-RCSID("$Id: dynroot.c,v 1.1 1999/07/06 17:31:46 lha Exp $");
+RCSID("$Id: dynroot.c,v 1.13.2.2 2001/05/28 15:19:20 map Exp $");
 
 struct create_entry {
     fbuf *thedir;	/* pointer to the fbuf that contains the dir */
@@ -81,12 +76,15 @@ afs2cellnum (int32_t afsvnode)
  * each `cell' creates a entry in the root directory.
  */
 
-int
+static int
 create_entry_func (const cell_entry *cell, void *arg)
 {
     struct create_entry *entry = (struct create_entry *) arg;
     int ret;
 
+    if (!cell_dynroot(cell))
+	return 0;
+
     entry->fid.Vnode = cellnum2afs (cell->id);
 
     ret = fdir_creat (entry->thedir, cell->name, entry->fid);
@@ -154,8 +152,7 @@ dynroot_create_symlink (fbuf *fbuf, int32_t vnode)
     if (ret)
 	return ret;
 
-    strncpy (fbuf_buf(fbuf), name, len);
-
+    memmove (fbuf_buf(fbuf), name, len);
     return 0;
 }
 
@@ -168,7 +165,7 @@ Bool
 dynroot_isvolumep (int cell, const char *volume)
 {
     assert (volume);
-
+    
     if (cell == 0 && strcmp (volume, "1") == 0)
 	return TRUE;
 
@@ -184,7 +181,7 @@ dynroot_fetch_vldbN (nvldbentry *entry)
 {
     memset (entry, 0, sizeof(*entry));
 
-    strncpy(entry->name, "root.cell", sizeof(entry->name));
+    strlcpy(entry->name, "root.cell", sizeof(entry->name));
     entry->nServers = 0;
     entry->volumeId[RWVOL] = DYNROOT_ROOTVOLUME;
     entry->flags = VLF_RWEXISTS;
@@ -197,10 +194,12 @@ dynroot_fetch_vldbN (nvldbentry *entry)
  * Note: doesn't update status.Length and status.LinkCount
  */
 
-void
-dynroot_update_entry (FCacheEntry *entry, int32_t filetype)
+static void
+dynroot_update_entry (FCacheEntry *entry, int32_t filetype,
+		      xfs_pag_t cred)
 {
     struct timeval tv;
+    AccessEntry *ae;
 
     assert (entry);
     entry->status.InterfaceVersion = 1;
@@ -210,10 +209,16 @@ dynroot_update_entry (FCacheEntry *entry, int32_t filetype)
     entry->status.Owner		= 0;
     entry->status.CallerAccess 	= ALIST | AREAD;
     entry->status.AnonymousAccess = ALIST | AREAD;
-    entry->status.UnixModeBits	= 
-	S_IRUSR|S_IXUSR|
-	S_IRGRP|S_IXGRP|
-	S_IROTH|S_IXOTH;
+    switch (filetype) {
+    case TYPE_DIR: 
+	entry->status.UnixModeBits = 0755;
+	break;
+    case TYPE_LINK:
+	entry->status.UnixModeBits = 0644;
+	break;
+    default:
+	abort();
+    }
     entry->status.ParentVnode	= DYNROOT_ROOTDIR;
     entry->status.ParentUnique	= DYNROOT_UNIQUE;
     entry->status.SegSize	= 64*1024;
@@ -236,17 +241,20 @@ dynroot_update_entry (FCacheEntry *entry, int32_t filetype)
 
     entry->anonaccess = entry->status.AnonymousAccess;
 
-    /* XXX CallerAccess */
+    findaccess(cred, entry->acccache, &ae);
+    ae->cred = cred;
+    ae->access = entry->status.CallerAccess;
 }
 
 /*
  * Fetch data and attr for `entry'
  */
 
-int
-dynroot_get_node (FCacheEntry *entry)
+static int
+dynroot_get_node (FCacheEntry *entry, CredCacheEntry *ce)
 {
-    int ret, fd, len, rootnode;
+    int ret, fd, rootnode;
+    size_t len;
     fbuf dir;
 
     rootnode = entry->fid.fid.Vnode == DYNROOT_ROOTDIR ? 1 : 0;
@@ -257,10 +265,10 @@ dynroot_get_node (FCacheEntry *entry)
 	return 0;
 
     fd = fcache_open_file (entry, O_RDWR);
-    if (ret < 0)
+    if (fd < 0)
 	return errno; 
 
-    ret = fbuf_create (&dir, fd, 0, FBUF_WRITE);
+    ret = fbuf_create (&dir, fd, 0, FBUF_READ | FBUF_WRITE | FBUF_SHARED);
     if (ret) {
 	close (fd);
 	return ret;
@@ -272,20 +280,25 @@ dynroot_get_node (FCacheEntry *entry)
     } else {
 	ret = dynroot_create_symlink (&dir, entry->fid.fid.Vnode);
 	entry->status.LinkCount = 1;
+	fcache_mark_as_mountpoint (entry);
     }
 
     if (ret) {
 	fbuf_end (&dir);
+	close(fd);
 	return ret;
     }
 
     entry->status.Length 	= dir.len;
+    entry->length	 	= dir.len;
 
     ret = fbuf_end (&dir);
+    close(fd);
     if (ret)
 	return ret;
 
-    dynroot_update_entry (entry, rootnode ? TYPE_DIR : TYPE_LINK);
+    dynroot_update_entry (entry, rootnode ? TYPE_DIR : TYPE_LINK,
+			  ce->cred);
 
     entry->flags.attrp = TRUE;
     entry->flags.datap = TRUE;
@@ -300,9 +313,9 @@ dynroot_get_node (FCacheEntry *entry)
  */
 
 int
-dynroot_get_attr (FCacheEntry *entry)
+dynroot_get_attr (FCacheEntry *entry, CredCacheEntry *ce)
 {
-    return dynroot_get_node (entry);
+    return dynroot_get_node (entry, ce);
 }
 
 
@@ -311,9 +324,9 @@ dynroot_get_attr (FCacheEntry *entry)
  */
 
 int
-dynroot_get_data (FCacheEntry *entry)
+dynroot_get_data (FCacheEntry *entry, CredCacheEntry *ce)
 {
-    return dynroot_get_node (entry);
+    return dynroot_get_node (entry, ce);
 }
 
 /*
