@@ -35,7 +35,7 @@
  * The interface for the file-cache.
  */
 
-/* $Id: fcache.h,v 1.72 2000/11/23 08:58:08 lha Exp $ */
+/* $Id: fcache.h,v 1.72.2.2 2001/10/02 16:13:07 jimmy Exp $ */
 
 #ifndef _FCACHE_H_
 #define _FCACHE_H_
@@ -296,12 +296,18 @@ fcache_highbytes(void);
 u_long
 fcache_usedbytes(void);
 
+u_long
+fcache_lowbytes(void);
+
 u_long
 fcache_highvnodes(void);
 
 u_long
 fcache_usedvnodes(void);
 
+u_long
+fcache_lowvnodes(void);
+
 int
 fcache_need_bytes(u_long needed);
 
@@ -359,4 +365,12 @@ fcache_realfid (const FCacheEntry *entry);
 void
 fcache_mark_as_mountpoint (FCacheEntry *entry);
 
+int
+collectstats_hostpart(u_int32_t *host, u_int32_t *part, int *n);
+
+int
+collectstats_getentry(u_int32_t host, u_int32_t part, u_int32_t type,
+		      u_int32_t items_slot, u_int32_t *count,
+		      int64_t *items_total, int64_t *total_time);
+
 #endif /* _FCACHE_H_ */
