@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -42,7 +37,7 @@
 
 #include "arla_local.h"
 #include <kafs.h>
-RCSID("$Id: fprio.c,v 1.6 1999/04/20 20:58:08 map Exp $");
+RCSID("$Id: fprio.c,v 1.9 2000/10/02 22:31:43 lha Exp $");
 
 /* Hashtable of entries by name */
 static Hashtab *fpriohashtab;
@@ -159,7 +154,7 @@ fprio_remove(VenusFid fid)
  */
 
 void
-fprio_set(VenusFid fid, unsigned prio)
+fprio_set(VenusFid fid, Bool prio)
 {
     struct fpriorityentry *e; 
     struct fpriorityentry key; 
@@ -229,7 +224,7 @@ fprio_readin(char *file)
 	}
 
 	fid.Cell = cellnum;
-	fprio_set(fid, prio);
+	fprio_set(fid, prio ? TRUE : FALSE);
     }
     return 0;
 }
@@ -238,7 +233,7 @@ fprio_readin(char *file)
  * Find the priority of a fid
  */
 
-int 
+Bool
 fprio_get(VenusFid fid)
 {
     struct fpriorityentry a;
@@ -249,7 +244,7 @@ fprio_get(VenusFid fid)
     b = hashtabsearch(fpriohashtab, &a);
     if (b)
 	return b->priority;
-    return 0;
+    return FALSE;
 }
 
 /*
@@ -269,9 +264,8 @@ fprio_print_entry (void *ptr, void *arg)
 	comment = "";
 
     arla_log(ADEBVLOG, "%s%d:%s:%d:%d:%d", 
-	     comment, n->priority, cell?cell:"unknowncell", n->fid.fid.Volume, 
-	     n->fid.fid.Vnode, n->fid.fid.Unique);
-
+	     comment, n->priority == TRUE ? 1 : 0, cell?cell:"unknowncell", 
+	     n->fid.fid.Volume, n->fid.fid.Vnode, n->fid.fid.Unique);
     return FALSE;
 }
 
@@ -284,11 +278,10 @@ fprio_status (void)
 {
     time_t the_time = time(NULL);
 
-    arla_log(ADEBVLOG, "#fprio entries\n#\n#  Date: %s\n#\n"
-	     "#priority range from %d to %d\n#\n"
+    arla_log(ADEBVLOG, "#fprio entries\n#\n#  Date: %s\n#"
 	     "#Syntax: (# means comment)\n"
 	     "#priority:cell:volume:vnode:unique\n",
-	     ctime(&the_time), FPRIO_MIN, FPRIO_MAX);
+	     ctime(&the_time));
     hashtabforeach (fpriohashtab, fprio_print_entry, NULL);
 }
 
