@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -41,17 +36,19 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: inter.c,v 1.75 1999/07/20 19:40:44 lha Exp $") ;
+RCSID("$Id: inter.c,v 1.110.2.3 2001/06/05 01:27:05 ahltorp Exp $") ;
 
 #include <xfs/xfs_message.h>
 
+Bool cm_consistencyp = FALSE;
+
 /*
  * Return the rights for user cred and entry e.
  * If the rights are not existant fill in the entry.
  * The locking of e is up to the caller.
  */
 
-u_long
+static u_long
 getrights (FCacheEntry *e, CredCacheEntry *ce)
 {
      AccessEntry *ae;
@@ -116,13 +113,17 @@ cm_store_state (void)
     fclose (log_fp);
 }
 
+/*
+ *
+ */
+
 static void
 log_operation (const char *fmt, ...)
 {
     va_list args;
     struct timeval now;
 
-    if(connected_mode == CONNECTED)
+    if(connected_mode == CONNECTED && cm_consistencyp == FALSE)
 	return;
 
     va_start (args, fmt);
@@ -138,6 +139,54 @@ log_operation (const char *fmt, ...)
     va_end (args);
 }
 
+/*
+ *
+ *
+ */
+
+void
+cm_turn_on_consistency_check(void)
+{
+    cm_consistencyp = TRUE;
+}
+
+/*
+ * Check consistency of the fcache.
+ * Will break the log-file.
+ */
+
+void
+cm_check_consistency (void)
+{
+    static unsigned int log_times = 0;
+    static unsigned int file_times = 0;
+    u_long calc_size;
+    u_long real_size;
+    char newname[MAXPATHLEN];
+
+    if (cm_consistencyp == FALSE)
+	return;
+    
+    calc_size = fcache_calculate_usage();
+    real_size = fcache_usedbytes ();
+
+    if (calc_size != real_size) {
+	    log_operation ("consistency check not guaranteed "
+			   "(calc: %d, real: %d), aborting\n", 
+			   (int) calc_size, (int) real_size);
+	    cm_store_state ();
+	    abort();
+    }
+    if (log_times % 100000 == 0) {
+	log_operation ("consistency check ok, rotating logs\n");
+	cm_store_state ();
+	snprintf (newname, sizeof(newname), "log.%d", file_times++);
+	rename ("log", newname);
+	cm_init ();	
+	log_operation ("brave new world\n");
+    }
+    log_times++;
+}
 
 /*
  * These functions often take a FID as an argument to be general, but
@@ -153,10 +202,10 @@ log_operation (const char *fmt, ...)
  */
 
 Result
-cm_open (VenusFid fid,
-	 CredCacheEntry* ce,
+cm_open (VenusFid *fid,
+	 CredCacheEntry **ce,
 	 u_int tokens,
-	 xfs_cache_handle *cache_handle,
+	 fcache_cache_handle *cache_handle,
 	 char *cache_name,
 	 size_t cache_name_sz)
 {
@@ -165,23 +214,13 @@ cm_open (VenusFid fid,
      u_long mask;
      int error;
 
-     error = fcache_get (&entry, fid, ce);
+     error = fcache_get_data (&entry, fid, ce);
      if (error) {
 	 ret.res = -1;
 	 ret.error = error;
 	 return ret;
      }
 
-     assert (CheckLock (&entry->lock) == -1);
-
-     error = fcache_get_data (entry, ce);
-     if(error) {
-	 fcache_release(entry);
-	 ret.res = -1;
-	 ret.error = error;
-	 return ret;
-     }
-
      switch(tokens) {
      case XFS_DATA_R:
        /*     case XFS_OPEN_NR:
@@ -201,7 +240,10 @@ cm_open (VenusFid fid,
 /*	  assert(FALSE); */
      }
 
-     if (checkright (entry, mask, ce)) {
+     if (checkright (entry, mask, *ce)) {
+#if 0
+	  assert(entry->flags.attrusedp);
+#endif
 	  entry->flags.datausedp = TRUE;
 	  entry->tokens |= tokens;
 	  ret.res    = 0;
@@ -212,8 +254,8 @@ cm_open (VenusFid fid,
 	  fcache_file_name (entry, cache_name, cache_name_sz);
 
 	  log_operation ("open (%ld,%lu,%lu,%lu) %u\n",
-			 fid.Cell,
-			 fid.fid.Volume, fid.fid.Vnode, fid.fid.Unique,
+			 fid->Cell,
+			 fid->fid.Volume, fid->fid.Vnode, fid->fid.Unique,
 			 mask);
      } else {
 	  ret.res = -1;
@@ -221,6 +263,8 @@ cm_open (VenusFid fid,
      }
      fcache_release(entry);
 
+     cm_check_consistency();
+ 
      return ret;
 }
 
@@ -244,6 +288,9 @@ cm_close (VenusFid fid, int flag, AFSStoreStatus *status, CredCacheEntry* ce)
     }
 
     if (flag & XFS_WRITE) {
+	if (flag & XFS_FSYNC)
+	    status->Mask |= SS_FSYNC;
+
 	error = write_data (entry, status, ce);
 
 	if (error) {
@@ -258,10 +305,12 @@ cm_close (VenusFid fid, int flag, AFSStoreStatus *status, CredCacheEntry* ce)
     log_operation ("close (%ld,%lu,%lu,%lu) %d\n",
 	     fid.Cell, fid.fid.Volume, fid.fid.Vnode, fid.fid.Unique,
 	     flag);
-
     fcache_release(entry);
     ret.res   = 0;
     ret.error = 0;
+
+    cm_check_consistency();
+
     return ret;
 }
 
@@ -273,21 +322,27 @@ Result
 cm_getattr (VenusFid fid,
 	    AFSFetchStatus *attr,
 	    VenusFid *realfid,
-	    CredCacheEntry* ce,
+	    CredCacheEntry *ce,
 	    AccessEntry **ae)
 {
      FCacheEntry *entry;
      Result ret;
      int error;
 
+     arla_warnx (ADEBCM, "cm_getattr");
+
+     assert (ae);
+
      error = fcache_get (&entry, fid, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 return ret;
      }
+     
+     AssertExclLocked(&entry->lock);
 
-     error = fcache_get_attr (entry, ce);
+     error = fcache_verify_attr (entry, NULL, NULL, ce);
      if (error) {
 	 fcache_release(entry);
 	 ret.res   = -1;
@@ -295,27 +350,37 @@ cm_getattr (VenusFid fid,
 	 return ret;
      }
 
+     arla_warnx (ADEBCM, "cm_getattr: done get attr");
+
      if (checkright (entry,
-		     entry->status.FileType == TYPE_FILE ? AREAD : ALIST,
+		     entry->status.FileType == TYPE_FILE ? AREAD : 0,
 		     ce)) {
-	  *attr = entry->status;
-	  ret.res   = 0;
-	  ret.error = 0;
-	  entry->flags.attrusedp = TRUE;
-	  entry->flags.kernelp   = TRUE;
-	  if (ae != NULL)
-	      *ae = entry->acccache;
-
-	  log_operation ("getattr (%ld,%lu,%lu,%lu)\n",
-		   fid.Cell, fid.fid.Volume, fid.fid.Vnode, fid.fid.Unique);
-
+	 *attr = entry->status;
+	 ret.res   = 0;
+	 ret.error = 0;
+	 entry->flags.attrusedp = TRUE;
+	 entry->flags.kernelp   = TRUE;
+	 *ae = entry->acccache;
+	 
+	 log_operation ("getattr (%ld,%lu,%lu,%lu)\n",
+			fid.Cell, fid.fid.Volume, fid.fid.Vnode, fid.fid.Unique);
+	 
      } else {
-	  ret.res   = -1;
-	  ret.error = EACCES;
+	 *ae = NULL;
+	 ret.res   = -1;
+	 ret.error = EACCES;
      }
-     *realfid = entry->realfid;
-     fcache_release(entry);
+     *realfid = *fcache_realfid (entry);
      ret.tokens = entry->tokens;
+     if (!entry->flags.datausedp)
+	 ret.tokens &= ~(XFS_DATA_MASK | XFS_OPEN_MASK);
+     fcache_release(entry);
+     
+     arla_warnx (ADEBCM, "cm_getattr: return: %d.%d", ret.res, ret.error);
+     cm_check_consistency();
+
+     AssertNotExclLocked(&entry->lock);
+
      return ret;
 }
 
@@ -338,7 +403,7 @@ cm_setattr (VenusFid fid, AFSStoreStatus *attr, CredCacheEntry* ce)
 	 return ret;
      }
 
-     error = fcache_get_attr (entry, ce);
+     error = fcache_verify_attr (entry, NULL, NULL, ce);
      if (error) {
 	 fcache_release(entry);
 	 ret.res   = -1;
@@ -358,6 +423,7 @@ cm_setattr (VenusFid fid, AFSStoreStatus *attr, CredCacheEntry* ce)
 	  ret.error = EACCES;
      }
      fcache_release(entry);
+     cm_check_consistency();
      return ret;
 }
 
@@ -379,7 +445,7 @@ cm_ftruncate (VenusFid fid, off_t size, CredCacheEntry* ce)
 	 return ret;
      }
 
-     error = fcache_get_attr (entry, ce);
+     error = fcache_verify_attr (entry, NULL, NULL, ce);
      if (error) {
 	 fcache_release(entry);
 	 ret.res   = -1;
@@ -387,6 +453,16 @@ cm_ftruncate (VenusFid fid, off_t size, CredCacheEntry* ce)
 	 return ret;
      }
 
+     if (size) {
+	 error = fcache_verify_data (entry, ce);
+	 if (error) {
+	     fcache_release (entry);
+	     ret.res   = -1;
+	     ret.error = error;
+	     return ret;
+	 }
+     }
+
      if (checkright (entry, AWRITE, ce)) {
 	  ret.res   = truncate_file (entry, size, ce);
 	  ret.error = ret.res;
@@ -399,6 +475,7 @@ cm_ftruncate (VenusFid fid, off_t size, CredCacheEntry* ce)
 	  ret.error = EACCES;
      }
      fcache_release(entry);
+     cm_check_consistency();
      return ret;
 }
 
@@ -420,7 +497,7 @@ cm_access (VenusFid fid, int mode, CredCacheEntry* ce)
 	 return ret;
      }
 
-     error = fcache_get_attr (entry, ce);
+     error = fcache_verify_attr (entry, NULL, NULL, ce);
      if (error) {
 	 fcache_release(entry);
 	 ret.res   = -1;
@@ -440,6 +517,7 @@ cm_access (VenusFid fid, int mode, CredCacheEntry* ce)
 	      fid.Cell, fid.fid.Volume, fid.fid.Vnode, fid.fid.Unique);
 
      fcache_release(entry);
+     cm_check_consistency();
      return ret;
 }
 
@@ -492,13 +570,14 @@ expand_sys (char *dest, size_t dst_sz, const char *src,
  */
 
 Result
-cm_lookup (VenusFid dir_fid,
+cm_lookup (VenusFid *dir_fid,
 	   const char *name,
 	   VenusFid *res,
 	   CredCacheEntry** ce,
 	   int follow_mount_point)
 {
      char tmp_name[MAXPATHLEN];
+     FCacheEntry *entry;
      Result ret;
      int error;
 
@@ -512,63 +591,74 @@ cm_lookup (VenusFid dir_fid,
 	 name = tmp_name;
      }
 
-     error = adir_lookup (dir_fid, name, res, *ce);
+     error = fcache_get_data(&entry, dir_fid, ce);
      if (error) {
-	  ret.res   = -1;
-	  ret.error = error;
-	  return ret;
-     }
-     if (follow_mount_point &&
-	 (error = followmountpoint (res, &dir_fid, ce) != 0)) {
 	 ret.res   = -1;
 	 ret.error = error;
-     } else {
-	 ret.res    = 0;
-	 ret.tokens = 0;
-	 ret.error  = 0;
+	 return ret;
      }
 
-     /* Assume that this means a bad .. */
-     if (   strcmp("..", name) == 0 
-	 && dir_fid.Cell == res->Cell 
-	 && dir_fid.fid.Volume == res->fid.Volume
-	 && dir_fid.fid.Vnode == res->fid.Vnode 
-	 && dir_fid.fid.Unique == res->fid.Unique) {
-	  FCacheEntry *e;
-	  int error;
-
-	  error = fcache_get (&e, dir_fid, *ce);
-	  if (error) {
-	      ret.res   = -1;
-	      ret.error = error;
-	      return ret;
-	  }
-
-	  error = fcache_get_attr (e, *ce);
-	  if (error) {
-	      fcache_release(e);
-	      ret.res   = -1;
-	      ret.error = error;
-	      return ret;
-	  }
-
-	  assert (e->flags.mountp);
-
-#if 0
-	  *res = e->mp_traversal->parent;
-#endif
-	  *res = e->parent;
-	  ret.res    = 0;
-	  ret.error  = 0;
-	  ret.tokens = e->tokens;
-	  fcache_release(e);
+     error = adir_lookup (entry, name, res);
+     if (error) {
+	 fcache_release(entry);
+	 ret.res   = -1;
+	 ret.error = error;
+	 return ret;
      }
-     
+
+     ret.res    = 0;
+     ret.error  = 0; 
+     ret.tokens = 0;
+
+     /* 
+      * The ".." at the top of a volume just points to the volume root,
+      * so get the real ".." from the volume cache instead.
+      *
+      * Or if we are looking up "." we don't want to follow the
+      * mountpoint
+      */
+
+     if (strcmp(".", name) == 0) {
+
+	 error = fcache_verify_attr (entry, NULL, NULL, *ce);
+	 if (error) {
+	     ret.res   = -1;
+	     ret.error = error;
+	     goto out;
+	 }
+
+	 *res = *dir_fid;
+	 ret.tokens = entry->tokens;
+     } else if ((strcmp("..", name) == 0 && VenusFid_cmp(dir_fid, res) == 0)) {
+
+	 error = fcache_verify_attr (entry, NULL, NULL, *ce);
+	 if (error) {
+	     ret.res   = -1;
+	     ret.error = error;
+	     goto out;
+	 }
+
+	 *res = entry->volume->parent_fid; /* entry->parent */
+	 ret.tokens = entry->tokens;
+     } else if (follow_mount_point) {
+	 error = followmountpoint (res, dir_fid, entry, ce);
+	 if (error) {
+	     ret.res   = -1;
+	     ret.error = error;
+	     goto out;
+	 }
+     }
+out:
+     fcache_release(entry);
+
      log_operation ("lookup (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
-	      name);
+		    dir_fid->Cell,
+		    dir_fid->fid.Volume,
+		    dir_fid->fid.Vnode,
+		    dir_fid->fid.Unique,
+		    name);
 
+     cm_check_consistency();
      return ret;
 }
 
@@ -577,31 +667,24 @@ cm_lookup (VenusFid dir_fid,
  */
 
 Result
-cm_create (VenusFid dir_fid, const char *name, AFSStoreStatus *store_attr,
+cm_create (VenusFid *dir_fid, const char *name, AFSStoreStatus *store_attr,
 	   VenusFid *res, AFSFetchStatus *fetch_attr,
-	   CredCacheEntry* ce)
+	   CredCacheEntry **ce)
 {
      FCacheEntry *dire;
      Result ret;
      int error;
 
-     error = fcache_get (&dire, dir_fid, ce);
+     error = fcache_get_data (&dire, dir_fid, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 return ret;
      }
 
-     error = fcache_get_data (dire, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
-
-     if (checkright (dire, AINSERT, ce)) {
+     if (checkright (dire, AINSERT, *ce)) {
 	  error = create_file (dire, name, store_attr,
-			       res, fetch_attr, ce);
+			       res, fetch_attr, *ce);
 	  if (error) {
 	      ret.res   = -1;
 	      ret.error = error;
@@ -619,13 +702,13 @@ cm_create (VenusFid dir_fid, const char *name, AFSStoreStatus *store_attr,
 	  ret.res   = -1;
 	  ret.error = EACCES;
      }
-out:
      log_operation ("create (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
+	      dir_fid->Cell,
+	      dir_fid->fid.Volume, dir_fid->fid.Vnode, dir_fid->fid.Unique,
 	      name);
 
      fcache_release(dire);
+     cm_check_consistency();
      return ret;
 }
 
@@ -634,32 +717,25 @@ out:
  */
 
 Result
-cm_mkdir (VenusFid dir_fid, const char *name,
+cm_mkdir (VenusFid *dir_fid, const char *name,
 	  AFSStoreStatus *store_attr,
 	  VenusFid *res, AFSFetchStatus *fetch_attr,
-	  CredCacheEntry* ce)
+	  CredCacheEntry **ce)
 {
      FCacheEntry *dire;
      Result ret;
      int error;
 
-     error = fcache_get (&dire, dir_fid, ce);
+     error = fcache_get_data (&dire, dir_fid, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 return ret;
      }
 
-     error = fcache_get_data (dire, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
-
-     if (checkright (dire, AINSERT, ce)) {
+     if (checkright (dire, AINSERT, *ce)) {
 	  error = create_directory (dire, name, store_attr,
-				    res, fetch_attr, ce);
+				    res, fetch_attr, *ce);
 	  if (error) {
 	      ret.res   = -1;
 	      ret.error = error;
@@ -679,12 +755,14 @@ cm_mkdir (VenusFid dir_fid, const char *name,
      }
 
      log_operation ("mkdir (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
+		    dir_fid->Cell,
+		    dir_fid->fid.Volume,
+		    dir_fid->fid.Vnode,
+		    dir_fid->fid.Unique,
 	      name);
 
-out:
      ReleaseWriteLock (&dire->lock);
+     cm_check_consistency();
      return ret;
 }
 
@@ -693,39 +771,42 @@ out:
  */
 
 Result
-cm_symlink (VenusFid dir_fid,
+cm_symlink (VenusFid *dir_fid,
 	    const char *name, AFSStoreStatus *store_attr,
-	    VenusFid *res, AFSFetchStatus *fetch_attr,
+	    VenusFid *res, VenusFid *realfid,
+	    AFSFetchStatus *fetch_attr,
 	    const char *contents,
-	    CredCacheEntry* ce)
+	    CredCacheEntry **ce)
 {
      FCacheEntry *dire, *symlink_entry;
      Result ret;
      int error;
 
-     error = fcache_get (&dire, dir_fid, ce);
+     error = fcache_get_data (&dire, dir_fid, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 return ret;
      }
 
-     error = fcache_get_data (dire, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
-
-     if (!checkright (dire, AINSERT, ce)) {
+     if (!checkright (dire, AINSERT, *ce)) {
 	 ret.res   = -1;
 	 ret.error = EACCES;
 	 goto out;
      }
 
+     /* It seems Transarc insists on mount points having mode bits 0644 */
+
+     if (contents[0] == '%' || contents[0] == '#') {
+	 store_attr->UnixModeBits = 0644;
+	 store_attr->Mask |= SS_MODEBITS;
+     } else if (store_attr->Mask & SS_MODEBITS
+		&& store_attr->UnixModeBits == 0644)
+	 store_attr->UnixModeBits = 0755;
+
      error = create_symlink (dire, name, store_attr,
 			     res, fetch_attr,
-			     contents, ce);
+			     contents, *ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
@@ -739,43 +820,60 @@ cm_symlink (VenusFid dir_fid,
 	 goto out;
      }
 
-     error = followmountpoint(res, &dir_fid, &ce);
+     error = followmountpoint(res, dir_fid, NULL, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 goto out;
      }
      
-     error = fcache_get (&symlink_entry, *res, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
+     /*
+      * If the new symlink is a mountpoint and it points
+      * to dir_fid we will deadlock if we look it up.
+      */
+
+     if (VenusFid_cmp (res, dir_fid) != 0) {
+
+	 error = fcache_get (&symlink_entry, *res, *ce);
+	 if (error) {
+	     ret.res   = -1;
+	     ret.error = error;
+	     goto out;
+	 }
+	 
+	 error = fcache_verify_attr (symlink_entry, NULL, NULL, *ce);
+	 if (error) {
+	     fcache_release (symlink_entry);
+	     ret.res   = -1;
+	     ret.error = error;
+	     goto out;
+	 }
+	 
+	 symlink_entry->flags.kernelp = TRUE;
+
+	 *fetch_attr = symlink_entry->status;
+	 *realfid = *fcache_realfid (symlink_entry);
 
-     error = fcache_get_attr (symlink_entry, ce);
-     if (error) {
 	 fcache_release (symlink_entry);
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
+     } else {
+	 *fetch_attr = dire->status;
+	 *realfid = *fcache_realfid (dire);
      }
-
-     symlink_entry->flags.kernelp = TRUE;
      
-     *fetch_attr = symlink_entry->status;
-     fcache_release (symlink_entry);
      ret.res   = 0;
      ret.error = 0;
-
+     
      log_operation ("symlink (%ld,%lu,%lu,%lu) %s %s\n",
-		    dir_fid.Cell,
-		    dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
+		    dir_fid->Cell,
+		    dir_fid->fid.Volume,
+		    dir_fid->fid.Vnode,
+		    dir_fid->fid.Unique,
 		    name,
 		    contents);
-
-out:
+     
+ out:
      fcache_release(dire);
+     cm_check_consistency();
      return ret;
 }
 
@@ -784,24 +882,24 @@ out:
  */
 
 Result
-cm_link (VenusFid dir_fid,
+cm_link (VenusFid *dir_fid,
 	 const char *name,
 	 VenusFid existing_fid,
 	 AFSFetchStatus *existing_status,
-	 CredCacheEntry* ce)
+	 CredCacheEntry **ce)
 {
      FCacheEntry *dire, *file;
      Result ret;
      int error;
 
-     error = fcache_get (&dire, dir_fid, ce);
+     error = fcache_get_data (&dire, dir_fid, ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 return ret;
      }
 
-     error = fcache_get_data (dire, ce);
+     error = fcache_get (&file, existing_fid, *ce);
      if (error) {
 	 fcache_release(dire);
 	 ret.res   = -1;
@@ -809,23 +907,15 @@ cm_link (VenusFid dir_fid,
 	 return ret;
      }
 
-     error = fcache_get (&file, existing_fid, ce);
-     if (error) {
-	 fcache_release(dire);
-	 ret.res   = -1;
-	 ret.error = error;
-	 return ret;
-     }
-
-     error = fcache_get_attr (file, ce);
+     error = fcache_verify_attr (file, NULL, NULL, *ce);
      if (error) {
 	 ret.res   = -1;
 	 ret.error = error;
 	 goto out;
      }
 
-     if (checkright (dire, AINSERT, ce)) {
-	  error = create_link (dire, name, file, ce);
+     if (checkright (dire, AINSERT, *ce)) {
+	  error = create_link (dire, name, file, *ce);
 	  if (error) {
 	      ret.res   = -1;
 	      ret.error = error;
@@ -845,8 +935,8 @@ cm_link (VenusFid dir_fid,
 	  ret.error = EACCES;
      }
      log_operation ("link (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
+	      dir_fid->Cell,
+	      dir_fid->fid.Volume, dir_fid->fid.Vnode, dir_fid->fid.Unique,
 	      existing_fid.Cell,
 	      existing_fid.fid.Volume,
 	      existing_fid.fid.Vnode,
@@ -856,6 +946,61 @@ cm_link (VenusFid dir_fid,
 out:
      fcache_release(dire);
      fcache_release(file);
+     cm_check_consistency();
+     return ret;
+}
+
+/*
+ * generic function for both remove and rmdir
+ */
+
+static Result
+sub_remove (VenusFid *dir_fid, const char *name, CredCacheEntry **ce,
+	    const char *operation,
+	    int (*func)(FCacheEntry *fe,
+			const char *name,
+			CredCacheEntry *ce))
+{
+     FCacheEntry *dire;
+     Result ret;
+     int error;
+
+     error = fcache_get_data (&dire, dir_fid, ce);
+     if (error) {
+	 ret.res   = -1;
+	 ret.error = error;
+	 return ret;
+     }
+
+     if (checkright (dire, ADELETE, *ce)) {
+	  error = (*func) (dire, name, *ce);
+	  if (error) {
+	      ret.res   = -1;
+	      ret.error = error;
+	  } else {
+	      error = adir_remove (dire, name);
+	      if (error) {
+		  ret.res   = -1;
+		  ret.error = error;
+	      } else {
+		  ret.res   = 0;
+		  ret.error = 0;
+	      }
+	  }
+     } else {
+	  ret.res   = -1;
+	  ret.error = EACCES;
+     }
+     log_operation ("%s (%ld,%lu,%lu,%lu) %s\n",
+		    operation,
+		    dir_fid->Cell,
+		    dir_fid->fid.Volume,
+		    dir_fid->fid.Vnode,
+		    dir_fid->fid.Unique,
+		    name);
+
+     fcache_release(dire);
+     cm_check_consistency();
      return ret;
 }
 
@@ -864,54 +1009,10 @@ out:
  */
 
 Result
-cm_remove(VenusFid dir_fid,
-	  const char *name, CredCacheEntry* ce)
+cm_remove(VenusFid *dir_fid,
+	  const char *name, CredCacheEntry **ce)
 {
-     FCacheEntry *dire;
-     Result ret;
-     int error;
-
-     error = fcache_get (&dire, dir_fid, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 return ret;
-     }
-
-     error = fcache_get_data (dire, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
-
-     if (checkright (dire, ADELETE, ce)) {
-	  error = remove_file (dire, name, ce);
-	  if (error) {
-	      ret.res   = -1;
-	      ret.error = error;
-	  } else {
-	      error = adir_remove (dire, name);
-	      if (error) {
-		  ret.res   = -1;
-		  ret.error = error;
-	      } else {
-		  ret.res   = 0;
-		  ret.error = 0;
-	      }
-	  }
-     } else {
-	  ret.res   = -1;
-	  ret.error = EACCES;
-     }
-     log_operation ("remove (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
-	      name);
-
-out:
-     fcache_release(dire);
-     return ret;
+    return sub_remove (dir_fid, name, ce, "remove", remove_file);
 }
 
 /*
@@ -919,185 +1020,195 @@ out:
  */
 
 Result
-cm_rmdir(VenusFid dir_fid,
-	 const char *name, CredCacheEntry* ce)
+cm_rmdir(VenusFid *dir_fid,
+	 const char *name, CredCacheEntry **ce)
 {
-     FCacheEntry *dire;
-     Result ret;
-     int error;
+    return sub_remove (dir_fid, name, ce, "rmdir", remove_directory);
+}
 
-     error = fcache_get (&dire, dir_fid, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 return ret;
-     }
+/*
+ * Called when the object is being moved to a new directory, to be
+ * able to update .. when required.
+ */
 
-     error = fcache_get_data (dire, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 goto out;
-     }
+static int
+potential_update_dir(FCacheEntry *child_entry,
+		     const VenusFid *new_parent_fid,
+		     FCacheEntry *parent_entry,
+		     int *update_child,
+		     CredCacheEntry *ce)
+{
+    int error;
 
-     if (checkright (dire, ADELETE, ce)) {
-	  error = remove_directory (dire, name, ce);
-	  if (error) {
-	      ret.res   = -1;
-	      ret.error = error;
-	  } else {
-	      error = adir_remove (dire, name);
-	      if (error) {
-		  ret.res   = -1;
-		  ret.error = error;
-	      } else {
-		  ret.res   = 0;
-		  ret.error = 0;
-	      }
-	  }
-     } else {
-	  ret.res   = -1;
-	  ret.error = EACCES;
-     }
+    error = fcache_verify_attr (child_entry, parent_entry, NULL, ce);
+    if (error) 
+	return error;
 
-     log_operation ("rmdir (%ld,%lu,%lu,%lu) %s\n",
-	      dir_fid.Cell,
-	      dir_fid.fid.Volume, dir_fid.fid.Vnode, dir_fid.fid.Unique,
-	      name);
+    /*
+     * if we're moving a directory.
+     */
 
-out:
-     fcache_release(dire);
-     return ret;
+    if (child_entry->status.FileType == TYPE_DIR) {
+	int fd;
+	fbuf the_fbuf;
+
+	error = fcache_verify_data (child_entry, ce); /* XXX - check fake_mp */
+	if (error)
+	    return error;
+
+	error = fcache_get_fbuf (child_entry, &fd, &the_fbuf, O_RDWR,
+				 FBUF_READ|FBUF_WRITE|FBUF_SHARED);
+	if (error)
+	    return error;
+
+	error = fdir_changefid (&the_fbuf, "..", new_parent_fid);
+	fbuf_end (&the_fbuf);
+	close (fd);
+	if (error)
+	    return error;
+
+	*update_child = 1;
+    }
+    return 0;
 }
 
 /*
  * Rename (old_parent_fid, old_name) -> (new_parent_fid, new_name)
+ * update the `child' in the new directory if update_child.
+ * set child_fid to the fid of the moved object.
  */
 
 Result
-cm_rename(VenusFid old_parent_fid, const char *old_name,
-	  VenusFid new_parent_fid, const char *new_name,
-	  CredCacheEntry* ce)
+cm_rename(VenusFid *old_parent_fid, const char *old_name,
+	  VenusFid *new_parent_fid, const char *new_name,
+	  VenusFid *child_fid,
+	  int *update_child,
+	  CredCacheEntry **ce)
 {
-     FCacheEntry *old_dir;
-     FCacheEntry *new_dir;
-     Result ret;
-     int error;
+    FCacheEntry *old_dir;
+    FCacheEntry *new_dir;
+    Result ret;
+    int error;
+    int diff_dir;
 
-     /* old parent dir */
+    *update_child = 0;
 
-     error = fcache_get (&old_dir, old_parent_fid, ce);
-     if (error) {
-	 ret.res   = -1;
-	 ret.error = error;
-	 return ret;
-     }
+    /* old parent dir */
 
-     error = fcache_get_data (old_dir, ce);
-     if (error) {
-	 fcache_release(old_dir);
-	 ret.res   = -1;
-	 ret.error = error;
-	 return ret;
-     }
+    error = fcache_get_data (&old_dir, old_parent_fid, ce);
+    if (error) {
+	ret.res   = -1;
+	ret.error = error;
+	return ret;
+    }
+
+    diff_dir = VenusFid_cmp (old_parent_fid, new_parent_fid);
 
      /* new parent dir */
 
-     if (old_parent_fid.fid.Vnode != new_parent_fid.fid.Vnode
-	 || old_parent_fid.fid.Unique != new_parent_fid.fid.Unique) {
+    if (diff_dir) {
+	error = fcache_get_data (&new_dir, new_parent_fid, ce);
+	if (error) {
+	    fcache_release(old_dir);
+	    ret.res   = -1;
+	    ret.error = error;
+	    return ret;
+	}
+    } else {
+	new_dir = old_dir;
+    }
 
-	 error = fcache_get (&new_dir, new_parent_fid, ce);
-	 if (error) {
-	     fcache_release(old_dir);
-	     ret.res   = -1;
-	     ret.error = error;
-	     return ret;
-	 }
+    if (checkright (old_dir, ADELETE, *ce)
+	&& checkright (new_dir, AINSERT, *ce)) {
+	
+	error = rename_file (old_dir, old_name, new_dir, new_name, *ce);
+	if (error) {
+	    ret.res   = -1;
+	    ret.error = error;
+	} else {
+	    VenusFid new_fid, old_fid;
+	    
+	    /*
+	     * Lookup the old name (to get the fid of the new name)
+	     */
+	    
+	    error = adir_lookup (old_dir, old_name, &new_fid);
+	    
+	    if (error) {
+		ret.res   = -1;
+		ret.error = error;
+		goto out;
+	    }
+	    
+	    *child_fid = new_fid;
+	    
+	    if (diff_dir) {
+		FCacheEntry *child_entry;
+		
+		error = fcache_get (&child_entry, *child_fid, *ce);
+		if (error) {
+		    ret.res   = -1;
+		    ret.error = error;
+		    goto out;
+		}
+		
+		child_entry->parent = *new_parent_fid;
 
-	 error = fcache_get_data (new_dir, ce);
-	 if (error) {
-	     ret.res   = -1;
-	     ret.error = error;
-	     goto out;
-	 }
-
-     } else {
-	 new_dir = old_dir;
-     }
-
-     if (checkright (old_dir, ADELETE, ce)
-	 && checkright (new_dir, AINSERT, ce)) {
-
-	 error = rename_file (old_dir, old_name,
-			      new_dir, new_name,
-			      ce);
-	  if (error) {
-	      ret.res   = -1;
-	      ret.error = error;
-	  } else {
-	      VenusFid new_fid, old_fid;
-
-	      /*
-	       * Lookup the old name (to get the fid of the new name)
-	       */
-
-	      error = adir_lookup_fcacheentry (old_dir,
-					       old_parent_fid,
-					       old_name, &new_fid, ce);
-
-	      if (error) {
-		  ret.res   = -1;
-		  ret.error = error;
-		  goto out;
-	      }
-
-	      /*
-	       * Lookup the new name, if it exists we need to clear it out.
-	       * XXX Should we check the lnkcount and clear it from fcache ?
-	       */
-
-	      error = adir_lookup_fcacheentry (new_dir,
-					       new_parent_fid,
-					       new_name, &old_fid, ce);
-	      if (error == 0)
-		  adir_remove (new_dir, new_name);
-
-	      /*
-	       * Now do the rename, ie create the new name and remove
-	       * the old name.
-	       */
-
-	      error = adir_creat (new_dir, new_name,  new_fid.fid)
-		  || adir_remove (old_dir, old_name);
-
-	      if (error) {
-		  ret.res   = -1;
-		  ret.error = error;
-	      } else {
-		  ret.res   = 0;
-		  ret.error = 0;
-	      }
-	  }
-     } else {
-	  ret.res   = -1;
-	  ret.error = EACCES;
-     }
-
-     log_operation ("rename (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s %s\n",
-	      old_parent_fid.Cell,
-	      old_parent_fid.fid.Volume,
-	      old_parent_fid.fid.Vnode,
-	      old_parent_fid.fid.Unique,
-	      new_parent_fid.Cell,
-	      new_parent_fid.fid.Volume,
-	      new_parent_fid.fid.Vnode,
-	      new_parent_fid.fid.Unique,
-	      old_name, new_name);
-
-out:
-     fcache_release(old_dir);
-     if (old_parent_fid.fid.Vnode != new_parent_fid.fid.Vnode
-	 || old_parent_fid.fid.Unique != new_parent_fid.fid.Unique)
-	 fcache_release(new_dir);
-     return ret;
+		error = potential_update_dir (child_entry, new_parent_fid,
+					      new_dir, update_child, *ce);
+		fcache_release (child_entry);
+		if (error) {
+		    ret.res   = -1;
+		    ret.error = error;
+		    goto out;
+		}
+	    }
+	    
+	    /*
+	     * Lookup the new name, if it exists we need to clear it out.
+	     * XXX Should we check the lnkcount and clear it from fcache ?
+	     */
+	    
+	    error = adir_lookup (new_dir, new_name, &old_fid);
+	    if (error == 0)
+		adir_remove (new_dir, new_name);
+	    
+	    /*
+	     * Now do the rename, ie create the new name and remove
+	     * the old name.
+	     */
+	    
+	    error = adir_creat (new_dir, new_name,  new_fid.fid)
+		|| adir_remove (old_dir, old_name);
+	    
+	    if (error) {
+		ret.res   = -1;
+		ret.error = error;
+	    } else {
+		ret.res   = 0;
+		ret.error = 0;
+	    }
+	}
+    } else {
+	ret.res   = -1;
+	ret.error = EACCES;
+    }
+    
+    log_operation ("rename (%ld,%lu,%lu,%lu) (%ld,%lu,%lu,%lu) %s %s\n",
+		   old_parent_fid->Cell,
+		   old_parent_fid->fid.Volume,
+		   old_parent_fid->fid.Vnode,
+		   old_parent_fid->fid.Unique,
+		   new_parent_fid->Cell,
+		   new_parent_fid->fid.Volume,
+		   new_parent_fid->fid.Vnode,
+		   new_parent_fid->fid.Unique,
+		   old_name, new_name);
+    
+ out:
+    fcache_release(old_dir);
+    if (diff_dir)
+	fcache_release(new_dir);
+    cm_check_consistency();
+    return ret;
 }
