@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,18 +31,20 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kernel.h,v 1.4 1998/12/06 20:49:11 lha Exp $ */
+/* $Id: kernel.h,v 1.8 2000/10/02 22:32:26 lha Exp $ */
 
 #ifndef _KERNEL_H_
 #define _KERNEL_H_
 
 struct kernel_args {
-    const char *device;
     unsigned num_workers;
 };
 
 void kernel_interface (struct kernel_args *args);
 
+void
+kernel_opendevice (const char *dev);
+
 extern int kernel_fd;
 
 unsigned long
@@ -56,5 +53,10 @@ kernel_highworkers(void);
 unsigned long
 kernel_usedworkers(void);
 
+ssize_t
+kern_read (int fd, void *data, size_t len);
+
+ssize_t
+kern_write (int fd, const void *data, size_t len);
 
 #endif /* _KERNEL_H_ */
