@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995-2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995-2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -32,7 +32,7 @@
  */
 
 #include "arla_local.h"
-RCSID("$Id: messages.c,v 1.231.2.1 2001/01/25 12:43:51 lha Exp $");
+RCSID("$Id: messages.c,v 1.231.2.10 2001/10/02 16:13:08 jimmy Exp $");
 
 #include <xfs/xfs_message.h>
 
@@ -292,7 +292,8 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, const VenusFid *fid)
 {
     switch (*ret) {
 #ifdef KERBEROS
-    case RXKADEXPIRED : {
+    case RXKADEXPIRED : 
+    case RXKADUNKNOWNKEY: {
 	int32_t cell = (*ce)->cell;
 
 	conn_clearcred (CONN_CS_CRED|CONN_CS_SECIDX, 0, cred->pag, 2);
@@ -307,11 +308,6 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, const VenusFid *fid)
 			     "seal error");
 	*ret = EINVAL;
 	return FALSE;
-    case RXKADUNKNOWNKEY :
-	arla_warnx_with_fid (ADEBWARN, fid,
-			     "unknown key");
-	*ret = EINVAL;
-	return FALSE;
 #endif	 
     case ARLA_VSALVAGE :
 	*ret = EIO;
@@ -322,7 +318,6 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, const VenusFid *fid)
     case ARLA_VMOVED :
     case ARLA_VNOVOL :
 	if (fid && !volcache_reliable (fid->fid.Volume, fid->Cell)) {
-	    volcache_invalidate (fid->fid.Volume, fid->Cell);
 	    return TRUE;
 	} else {
 	    *ret = ENOENT;
@@ -344,12 +339,12 @@ try_again (int *ret, CredCacheEntry **ce, xfs_cred *cred, const VenusFid *fid)
     case ARLA_VBUSY :
 	arla_warnx_with_fid (ADEBWARN, fid,
 			     "Waiting for busy volume...");
-	IOMGR_Sleep (1);
+	IOMGR_Sleep (afs_BusyWaitPeriod);
 	return TRUE;
     case ARLA_VRESTARTING:
 	arla_warnx_with_fid (ADEBWARN, fid,
 			     "Waiting for fileserver to restart...");
-	IOMGR_Sleep (1);
+	IOMGR_Sleep (afs_BusyWaitPeriod);
 	return TRUE;
     case ARLA_VIO :
 	*ret = EIO;
@@ -718,7 +713,11 @@ xfs_message_create (int fd, struct xfs_message_create *h, u_int size)
 	     msg3.flag |= XFS_ID_HANDLE_VALID;
 
 	 child_entry->flags.kernelp = TRUE;
+	 child_entry->flags.attrusedp = TRUE;
 	 child_entry->flags.datausedp = TRUE;
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
 
 	 msg1.header.opcode = XFS_MSG_INSTALLDATA;
@@ -803,7 +802,11 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
      FCacheEntry parent_entry;
      AFSStoreStatus log_store_status;
 
+#if 0
      parent_fid = fid_translate((VenusFid *)&h->parent_handle);
+#else
+     parent_fid = (VenusFid *)&h->parent_handle;
+#endif
      arla_warnx (ADEBMSG, "mkdir (%ld.%lu.%lu.%lu) \"%s\"",
 		 (long)parent_fid->Cell, (unsigned long)parent_fid->fid.Volume,
 		 (unsigned long)parent_fid->fid.Vnode,
@@ -884,7 +887,11 @@ xfs_message_mkdir (int fd, struct xfs_message_mkdir *h, u_int size)
 	 if (cache_handle.valid)
 	     msg3.flag |= XFS_ID_HANDLE_VALID;
 
+	 child_entry->flags.attrusedp = TRUE;
 	 child_entry->flags.datausedp = TRUE;
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
 
 	 msg2.node.tokens = res.tokens;
@@ -998,6 +1005,9 @@ xfs_message_link (int fd, struct xfs_message_link *h, u_int size)
 	 if (cache_handle.valid)
 	     msg1.flag |= XFS_ID_HANDLE_VALID;
 	 msg1.node.tokens = res.tokens;
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
 
 	 fcacheentry2xfsnode (parent_fid,
@@ -1098,6 +1108,9 @@ xfs_message_symlink (int fd, struct xfs_message_symlink *h, u_int size)
 	 if (cache_handle.valid)
 	     msg1.flag |= XFS_ID_HANDLE_VALID;
 	 msg1.node.tokens = res.tokens;
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
 
 	 fcacheentry2xfsnode (parent_fid,
@@ -1250,6 +1263,9 @@ xfs_message_remove (int fd, struct xfs_message_remove *h, u_int size)
 	  * Set datausedp since we push data to kernel in out:
 	  */
 
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
 
 	 /*
@@ -1448,6 +1464,9 @@ xfs_message_rmdir (int fd, struct xfs_message_rmdir *h, u_int size)
 	     h1 = (struct xfs_message_header *)&msg1;
 	     h1_len = sizeof(msg1);
 	 }
+#if 0
+	 assert(dir_entry->flags.attrusedp);
+#endif
 	 dir_entry->flags.datausedp = TRUE;
      }
 
@@ -1599,8 +1618,15 @@ xfs_message_rename (int fd, struct xfs_message_rename *h, u_int size)
 	 h1 = (struct xfs_message_header *)&msg2;
 	 h1_len = sizeof(msg2);
 
-	 if (old_entry)
+	 if (old_entry) {
+#if 0
+	     assert(old_entry->flags.attrusedp);
+#endif
 	     old_entry->flags.datausedp = TRUE;
+	 }
+#if 0
+	 assert(new_entry->flags.attrusedp);
+#endif
 	 new_entry->flags.datausedp = TRUE;
 
 	 if (update_child) {
@@ -1762,7 +1788,8 @@ xfs_message_getdata (int fd, struct xfs_message_getdata *h, u_int size)
 		   msg.flag = XFS_ID_INVALID_DNLC;
 		   if (cache_handle.valid)
 		       msg.flag |= XFS_ID_HANDLE_VALID;
-		   
+
+		   entry->flags.attrusedp = TRUE;
 		   entry->flags.datausedp = TRUE;
 	       }
 	       fcache_release(entry);
@@ -1820,10 +1847,6 @@ break_callback (FCacheEntry *entry)
 
      assert (entry->flags.kernelp);
 
-     entry->flags.kernelp   = FALSE;
-     entry->flags.attrusedp = FALSE;
-     entry->flags.datausedp = FALSE;
-
      msg.header.opcode = XFS_MSG_INVALIDNODE;
      memcpy (&msg.handle, &entry->fid, sizeof(entry->fid));
      ret = xfs_message_send (kernel_fd, (struct xfs_message_header *)&msg, 
@@ -2217,13 +2240,19 @@ get_mount_point (VenusFid fid,
 		 FCacheEntry **ret_mp_entry)
 {
     FCacheEntry *mp_entry;
+    FCacheEntry *dentry;
     VenusFid mp_fid;
     int error;
 
     if (fid.fid.Volume == 0 && fid.fid.Vnode == 0 && fid.fid.Unique == 0)
 	return EINVAL;
 
-    error = adir_lookup(&fid, filename, &mp_fid, NULL, ce);
+    error = fcache_get_data(&dentry, &fid, ce);
+    if (error)
+	return error;
+
+    error = adir_lookup(dentry, filename, &mp_fid);
+    fcache_release(dentry);
     if (error)
 	return error;
 
@@ -2573,10 +2602,13 @@ vioc_new_cell(int fd, struct xfs_message_pioctl *h, u_int size)
     if (dbs == NULL)
 	return xfs_send_message_wakeup (fd, h->header.sequence_num, ENOMEM);
 	
+    memset(dbs, 0, count * sizeof(*dbs));
+
     hp = (u_int32_t *)h->msg;
     for (i = 0; i < count; ++i) {
 	dbs[i].name = NULL;
 	dbs[i].addr.s_addr = hp[i];
+	dbs[i].timeout = 0;
     }
 
     cellname = h->msg + 8 * sizeof(u_int32_t);
@@ -2836,7 +2868,9 @@ viocconnect(int fd, struct xfs_message_pioctl *h, u_int size)
 		DARLA_Close(&log_data);
 		Log_is_open = 0;
 
+#if 0
 		do_replay("discon_log", log_data.log_entries, 0);
+#endif
 	    }
 	    if (connected_mode == DISCONNECTED) {
 		connected_mode = CONNECTED ;
@@ -3164,6 +3198,8 @@ viocgetcacheparms (int fd, struct xfs_message_pioctl *h, u_int size)
     parms[3] = fcache_usedvnodes();
     parms[4] = fcache_highbytes();
     parms[5] = fcache_usedbytes();
+    parms[6] = fcache_lowbytes();
+    parms[7] = fcache_lowvnodes();
 
     h->outsize = sizeof(parms);
     return xfs_send_message_wakeup_data(fd, h->header.sequence_num, 0,
@@ -3299,6 +3335,118 @@ vioc_breakcallback(int fd, struct xfs_message_pioctl *h, u_int size)
     return 0;
 }
 
+/*
+ * check volume mappings
+ */
+
+static int
+statistics_hostpart(int fd, struct xfs_message_pioctl *h, u_int size)
+{
+    u_int32_t host[100];
+    u_int32_t part[100];
+    u_int32_t outparms[512];
+    int n;
+    int outsize;
+    int maxslots;
+    int i;
+
+    if (h->outsize < sizeof(u_int32_t))
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EINVAL);
+    
+    n = 100;
+    collectstats_hostpart(host, part, &n);
+    maxslots = (h->outsize / sizeof(u_int32_t) - 1) / 2;
+    if (n > maxslots)
+	n = maxslots;
+    
+    outsize = (n * 2 + 1) * sizeof(u_int32_t);
+    
+    outparms[0] = n;
+    for (i = 0; i < n; i++) {
+	outparms[i*2 + 1] = host[i];
+	outparms[i*2 + 2] = part[i];
+    }
+    
+    return xfs_send_message_wakeup_data (fd, h->header.sequence_num, 0,
+					 (char *) &outparms, outsize);
+}
+
+static int
+statistics_entry(int fd, struct xfs_message_pioctl *h, u_int size)
+{
+    u_int32_t *request = (u_int32_t *) h->msg;
+    u_int32_t host;
+    u_int32_t part;
+    u_int32_t type;
+    u_int32_t items_slot;
+    u_int32_t count[32];
+    int64_t items_total[32];
+    int64_t total_time[32];
+    u_int32_t outparms[160];
+    int i;
+    int j;
+
+    if (h->insize < sizeof(u_int32_t) * 5) {
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EINVAL);
+    }
+
+    if (h->outsize < sizeof(u_int32_t) * 160) {
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EINVAL);
+    }
+
+    host = request[1];
+    part = request[2];
+    type = request[3];
+    items_slot = request[4];
+
+    collectstats_getentry(host, part, type, items_slot,
+			  count, items_total, total_time);
+
+    j = 0;
+    for (i = 0; i < 32; i++) {
+	outparms[j++] = count[i];
+    }
+    for (i = 0; i < 32; i++) {
+	memcpy(&outparms[j], &items_total[i], 8);
+	j+=2;
+    }
+    for (i = 0; i < 32; i++) {
+	memcpy(&outparms[j], &total_time[i], 8);
+	j+=2;
+    }
+    return xfs_send_message_wakeup_data (fd, h->header.sequence_num, 0,
+					 (char *) &outparms, sizeof(outparms));
+}
+
+static int
+aioc_statistics(int fd, struct xfs_message_pioctl *h, u_int size)
+{
+    u_int32_t opcode;
+
+    if (!all_powerful_p (&h->cred))
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EPERM);
+
+    if (h->insize < sizeof(u_int32_t))
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EPERM);
+
+    opcode = *((int32_t *)h->msg);
+
+    if (opcode == 0) {
+	return statistics_hostpart(fd, h, size);
+    } else if (opcode == 1) {
+	return statistics_entry(fd, h, size);
+    } else {
+	return xfs_send_message_wakeup (fd, h->header.sequence_num,
+					EINVAL);
+    }
+}
+
+
 /*
  * Handle a pioctl message in `h'
  */
@@ -3567,6 +3715,22 @@ xfs_message_pioctl (int fd, struct xfs_message_pioctl *h, u_int size)
 #endif	
 	error = vioc_breakcallback (fd, h, size);
 	break;
+#ifdef VIOCCKBACK_32
+    case VIOCCKBACK_32 :
+    case VIOCCKBACK_64 :
+#else
+    case VIOCCKBACK :
+#endif
+	break;
+
+#ifdef VIOCCKBACK_32
+    case AIOC_STATISTICS_32:
+    case AIOC_STATISTICS_64:
+#else
+    case AIOC_STATISTICS:
+#endif
+	return aioc_statistics (fd, h, size);
+
     default:
 	arla_warnx (ADEBMSG, "unknown pioctl call %d", h->opcode);
 	error = EINVAL ;
