@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -35,7 +35,7 @@
  * Our cache of volume information.
  */
 
-/* $Id: volcache.h,v 1.28 2000/10/02 22:33:20 lha Exp $ */
+/* $Id: volcache.h,v 1.28.2.2 2001/02/12 12:53:02 assar Exp $ */
 
 #ifndef _VOLCACHE_
 #define _VOLCACHE_
@@ -45,9 +45,6 @@
 #include <list.h>
 #include "vldb.h"
 
-#define BACKSUFFIX ".backup"
-#define ROSUFFIX   ".readonly"
-
 /*
  * index for number into a VolCacheEntry
  */
@@ -67,7 +64,6 @@ struct name_ptr {
     int32_t cell;
     char name[VLDB_MAXNAMELEN];
     struct volcacheentry *ptr;
-    int32_t type;
 };
 
 struct volcacheentry {
@@ -85,7 +81,7 @@ struct volcacheentry {
 	unsigned validp  : 1;
 	unsigned stablep : 1;
     } flags;
-    struct name_ptr name_ptr[MAXTYPES];
+    struct name_ptr name_ptr;
     struct num_ptr num_ptr[MAXTYPES];
 };
 
@@ -103,7 +99,7 @@ typedef struct volcacheentry VolCacheEntry;
  * current version number of the dump file
  */
 
-#define VOLCACHE_VERSION	0x2
+#define VOLCACHE_VERSION	0x3
 
 const char *volcache_get_rootvolume (void);
 
@@ -127,6 +123,8 @@ void volcache_update_volsync (VolCacheEntry *e, AFSVolSync volsync);
 
 void volcache_free (VolCacheEntry *e);
 
+void volcache_ref (VolCacheEntry *e);
+
 void volcache_volref (VolCacheEntry *e, VolCacheEntry *parent);
 
 void volcache_volfree (VolCacheEntry *e);
@@ -139,7 +137,7 @@ int volume_make_uptodate (VolCacheEntry *e, CredCacheEntry *ce);
 
 Bool volcache_reliable (u_int32_t id, int32_t cell);
 
-const char *volcache_getname (u_int32_t id, int32_t cell);
+int volcache_getname (u_int32_t id, int32_t cell, char *, size_t);
 
 void volcache_status (void);
 
