@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "arla_local.h"
 
-RCSID("$Id: xfs.c,v 1.3 1999/07/25 03:13:35 map Exp $");
+RCSID("$Id: xfs.c,v 1.13 2000/10/02 22:33:25 lha Exp $");
 
 /*
  * Begining of breakout of xfs releated junk
@@ -56,10 +51,6 @@ static unsigned sent_stat[XFS_MSG_COUNT];
 
 static unsigned recv_stat[XFS_MSG_COUNT];
 
-/* count of the number of messages in a write */
-
-static unsigned send_count[9];	/* 8 is the max the multiple stuff handles */
-
 static char *rcvfuncs_name[] = 
 {
   "version",
@@ -109,144 +100,35 @@ xfs_message_rpc (int fd, struct xfs_message_header *h, u_int size)
 }
 
 /*
- * List of capabilities of the xfs-module
- */
-
-xfs_version_t xfsv_version;
-xfs_version_t xfsv_fhget;
-xfs_version_t xfsv_fhget_native;
-xfs_version_t xfsv_gc_nodes;
-
-struct {
-    unsigned	capability;
-    xfs_version_t *status;
-    char	*desc;
-} probe_pattern[] =  {
-    { XFS_VERSION_VERSION,
-      &xfsv_version,
-      "version" },
-    { XFS_VERSION_FHGET,
-      &xfsv_fhget,
-      "xfs fhget" },
-    { XFS_VERSION_FHGET_NATIVE,
-      &xfsv_fhget_native,
-      "native fhget" },
-    { XFS_VERSION_GC_NODES,
-      &xfsv_gc_nodes,
-      "gc nodes" },
-    { 0, NULL, NULL}
-};
-    
-
-/*
- * Try to probe if `fd' has capability `probe', return
- * whatever status in `status', will not touch `status'
- * if no versioning is there.
+ * Try to probe the version on `fd', returning the version.
  */
 
 static int
-xfs_send_message_version (int fd, int probe, xfs_version_t *status)
+xfs_send_message_version (int fd)
 {
      struct xfs_message_version msg;
      int ret;
 
-     if (xfsv_version == XFSV_NO)
-	 return 1;
-     
      msg.header.opcode = XFS_MSG_VERSION;
-     msg.probe = probe;
-     arla_warnx (ADEBMSG, "sending version: probe = %d", probe);
+     arla_warnx (ADEBMSG, "sending version");
      ret = xfs_message_rpc (fd, (struct xfs_message_header *)&msg, 
-			     sizeof(msg));
-     
-     if (ret == XFS_VERSION_YES)
-	 *status = XFSV_YES;
-     else if (ret == XFS_VERSION_NO)
-	 *status = XFSV_NO;
-     else {
-	 assert (probe == XFS_VERSION_VERSION);
-	 xfsv_version = XFSV_NO;
-	 return 1;
-     }
-     return 0;
+			    sizeof(msg));
+     return ret;
 }
 
 /*
- * Probe for capabilities on `fd'.
+ * Probe for version on `fd'.  Fail if != version
  */
 
 void
-xfs_probe_capability (int fd)
+xfs_probe_version (int fd, int version)
 {
-    int i;
+    int ret = xfs_send_message_version (fd);
 
-    arla_warnx (ADEBMISC, "xfs_probe_capability");
-
-    for (i = 0; 
-	 probe_pattern[i].status != NULL;
-	 i++) {
-
-	*probe_pattern[i].status = XFSV_UNKNOWN;
-
-	if (xfs_send_message_version (fd, 
-				      probe_pattern[i].capability,
-				      probe_pattern[i].status))
-	    break;
-
-    }
-}
-
-
-/*
- * Disable capability `cap'
- */
-
-void
-xfs_probe_disable (xfs_version_t *cap)
-{
-    *cap = XFSV_NO;
-}
-
-/*
- * Print list of capablities
- */
-
-static const char *
-cap2str (xfs_version_t cap)
-{
-    switch (cap) {
-    case XFSV_NO: return "no";
-    case XFSV_YES: return "yes";
-    case XFSV_UNKNOWN: return "unknown";
-    default: abort();
-    }
-    return "foo";
-}
-
-/*
- * Print all probe info on `f' or with arladeb_warn if f == NULL
- */
-
-void
-xfs_probe_print (FILE *f)
-{
-    int i;
-    
-    for (i = 0; 
-	 probe_pattern[i].status != NULL;
-	 i++) {
-
-	if (f == NULL)
-	    arla_warnx (ADEBMISC, "support %-20s : %s", 
-			probe_pattern[i].desc,
-			cap2str(*probe_pattern[i].status));
-	else
-	    fprintf (f,
-		     "support %-20s : %s",
-		     probe_pattern[i].desc,
-		     cap2str(*probe_pattern[i].status));
-	
-    }
+    if (ret != version)
+	arla_errx (1, ADEBERROR,
+		   "Wrong version of xfs.  Please {up,down}grade to %d",
+		   version);
 }
 
 /*
@@ -262,10 +144,8 @@ xfs_send_message_gc_nodes (int fd, int num, VenusFid *fids)
     struct xfs_message_gc_nodes msg;
     int i;
     
-    if (!xfs_probe_have (xfsv_gc_nodes))
-	return;
-
-    arla_warnx (ADEBMSG, "sending gc");
+    arla_warnx (ADEBMSG, 
+		"xfs_send_message_gc_nodes sending gc: num = %d", num);
     
     if (num > XFS_GC_NODES_MAX_HANDLE)
 	num = XFS_GC_NODES_MAX_HANDLE;
@@ -330,8 +210,8 @@ xfs_message_receive (int fd, struct xfs_message_header *h, u_int size)
 int
 xfs_message_send (int fd, struct xfs_message_header *h, u_int size)
 {
-     int res;
      unsigned opcode = h->opcode;
+     int ret;
 
      h->size = size;
      h->sequence_num = seqnums[fd]++;
@@ -342,14 +222,14 @@ xfs_message_send (int fd, struct xfs_message_header *h, u_int size)
      }
 
      ++sent_stat[opcode];
-     ++send_count[1];
 
      arla_warnx (ADEBMSG, "Send message: opcode = %u (%s), size = %u",
 		 opcode, rcvfuncs_name[opcode], h->size);
 
-     if ((res = write (fd, h, size)) < 0) {
+     ret = kern_write (fd, h, size);
+     if (ret != size) {
 	 arla_warn (ADEBMSG, errno, "xfs_message_send: write");
-	 return -1;
+	 return errno;
      } else
 	 return 0;
 }
@@ -364,7 +244,7 @@ xfs_message_wakeup (int fd, struct xfs_message_wakeup *h, u_int size)
      Listitem *i, *next;
      struct xfs_message_wakeup *w;
 
-     assert (sizeof(*h) <= size);
+     assert (sizeof(*w) >= size);
 
      for (i = listhead (sleepers); i; i = next) {
 	  next = listnext (sleepers, i);
@@ -373,8 +253,11 @@ xfs_message_wakeup (int fd, struct xfs_message_wakeup *h, u_int size)
 	       listdel (sleepers, i);
 	       memcpy (w, h, size);
 	       LWP_SignalProcess ((char *)w);
+	       break;
 	  }
      }
+     if (i == NULL)
+	 arla_warnx (ADEBWARN, "xfs_message_wakeup: no message to wakeup!");
      return 0;
 }
 
@@ -441,6 +324,162 @@ xfs_send_message_wakeup_data (int fd, u_int seqnum, int error,
 			      sizeof(msg));
 }
 
+/*
+ *
+ */
+
+struct write_buf {
+    unsigned char buf[MAX_XMSG_SIZE];
+    size_t len;
+};
+
+/*
+ * Return 1 it buf is full, 0 if it's not.
+ */
+
+static int
+add_new_msg (int fd, 
+	     struct xfs_message_header *h, size_t size,
+	     struct write_buf *buf)
+{
+    /* align on 8 byte boundery */
+
+    if (size > sizeof (buf->buf) - buf->len)
+	return 1;
+
+    h->sequence_num 	= seqnums[fd]++;
+    h->size		= (size + 8) & ~ 7;
+
+    assert (h->opcode >= 0 && h->opcode < XFS_MSG_COUNT);
+    ++sent_stat[h->opcode];
+
+    arla_warnx (ADEBMSG, "Multi-send: opcode = %u (%s), size = %u",
+		h->opcode, rcvfuncs_name[h->opcode], h->size);
+    
+    memcpy (buf->buf + buf->len, h, size);
+    memset (buf->buf + buf->len + size, 0, h->size - size);
+    buf->len += h->size;
+    return 0;
+}
+
+/*
+ * Blast of a `buf' to `fd'.
+ */
+
+static int
+send_msg (int fd, struct write_buf *buf)
+{
+    int ret;
+
+    if (buf->len == 0)
+	return 0;
+
+    ret = kern_write (fd, buf->buf, buf->len);
+    if (ret != buf->len) {
+	arla_warn (ADEBMSG, errno,
+		   "send_msg: write");
+	buf->len = 0;
+	return errno;
+    }
+    buf->len = 0;
+    return 0;
+}
+
+/*
+ *
+ */
+
+int
+xfs_send_message_vmultiple (int fd,
+			    va_list args)
+{
+    struct xfs_message_header *h;
+    struct write_buf *buf;
+    size_t size;
+    int ret;
+
+    buf = malloc (sizeof (*buf));
+    if (buf == NULL)
+	return ENOMEM;
+
+    h = va_arg (args, struct xfs_message_header *);
+    size = va_arg (args, size_t);
+    buf->len = 0;
+    while (h != NULL) {
+	if (add_new_msg (fd, h, size, buf)) {
+	    ret = send_msg (fd, buf);
+	    if (ret) {
+		free (buf);
+		return ret;
+	    }
+	    if (add_new_msg (fd, h, size, buf))
+		arla_warnx (ADEBERROR, 
+			    "xfs_send_message_vmultiple: "
+			    "add_new_msg failed");
+	}
+	    
+	h = va_arg (args, struct xfs_message_header *);
+	size = va_arg (args, size_t);
+    }
+    ret = send_msg (fd, buf);
+    free (buf);
+    return ret;
+}
+
+/*
+ * Same as above but diffrent.
+ */
+
+int
+xfs_send_message_multiple (int fd,
+			   ...)
+{
+    va_list args;
+    int ret;
+
+    va_start (args, fd);
+    ret = xfs_send_message_vmultiple (fd, args);
+    va_end (args);
+    return ret;
+}
+			   
+/*
+ * Almost same as above but diffrent.
+ */
+
+int
+xfs_send_message_multiple_list (int fd,
+				struct xfs_message_header *h,
+				size_t size,
+				u_int num)
+{
+    struct write_buf *buf;
+    int ret = 0;
+
+    buf = malloc (sizeof (*buf));
+    if (buf == NULL)
+	return ENOMEM;
+    buf->len = 0;
+    while (num && ret == 0) {
+	if (add_new_msg (fd, h, size, buf)) {
+	    ret = send_msg (fd, buf);
+	    if (add_new_msg (fd, h, size, buf))
+		arla_warnx (ADEBERROR, 
+			    "xfs_send_message_multiple_list: "
+			    "add_new_msg failed");
+	}
+	h = (struct xfs_message_header *) (((unsigned char *)h) + size);
+	num--;
+    }
+    if (ret) {
+	free (buf);
+	return ret;
+    }
+    ret = send_msg (fd, buf);
+    free (buf);
+    return ret;
+}
+
 /*
  * Send multiple message to the kernel (for performace/simple resons)
  */
@@ -451,51 +490,27 @@ xfs_send_message_wakeup_vmultiple (int fd,
 				   int error,
 				   va_list args)
 {
-    struct iovec iovec[8];
-    struct xfs_message_header *h;
     struct xfs_message_wakeup msg;
-    size_t size;
-    int i = 0;
     int ret;
 
-    h = va_arg (args, struct xfs_message_header *);
-    size = va_arg (args, size_t);
-    while (h != NULL) {
-	h->size = size;
-	h->sequence_num = seqnums[fd]++;
-	assert (h->opcode >= 0 && h->opcode < XFS_MSG_COUNT);
-	assert (i < 8);
-	iovec[i].iov_base = (char *)h;
-	iovec[i].iov_len  = size;
+    ret = xfs_send_message_vmultiple (fd, args);
+    if (ret)
+	arla_warnx (ADEBERROR, "xfs_send_message_wakeup_vmultiple: "
+		    "failed sending messages with error %d", ret);
 
-	++sent_stat[h->opcode];
-
-	arla_warnx (ADEBMSG, "Multi-send: opcode = %u (%s), size = %u",
-		    h->opcode, rcvfuncs_name[h->opcode], h->size);
-
-	h = va_arg (args, struct xfs_message_header *);
-	size = va_arg (args, size_t);
-	++i;
-    }
     msg.header.opcode = XFS_MSG_WAKEUP;
     msg.header.size  = sizeof(msg);
     msg.header.sequence_num = seqnums[fd]++;
     msg.sleepers_sequence_num = seqnum;
     msg.error = error;
-    iovec[i].iov_base = (char *)&msg;
-    iovec[i].iov_len  = sizeof(msg);
 
     ++sent_stat[XFS_MSG_WAKEUP];
 
     arla_warnx (ADEBMSG, "multi-sending wakeup: seq = %u, error = %d",
 		seqnum, error);
 
-    ++i;
-    
-    ++send_count[i];
-
-    ret = writev (fd, iovec, i);
-    if (ret < 0) {
+    ret = kern_write (fd, &msg, sizeof(msg));
+    if (ret != sizeof(msg)) {
 	arla_warn (ADEBMSG, errno,
 		   "xfs_send_message_wakeup_vmultiple: writev");
 	return -1;
@@ -504,7 +519,7 @@ xfs_send_message_wakeup_vmultiple (int fd,
 }
 
 /*
- * Save as above but diffrent.
+ * Same as above but diffrent.
  */
 
 int
