@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,20 +31,14 @@
  * SUCH DAMAGE.
  */
 
+/*
+ * $Id: xfs.h,v 1.8 2000/10/02 22:33:30 lha Exp $
+ */
+
 #ifndef __XFS_H_V
 #define __XFS_H_V 1
 
-typedef enum { XFSV_UNKNOWN, XFSV_YES, XFSV_NO } xfs_version_t;
-
-extern xfs_version_t xfsv_version;
-extern xfs_version_t xfsv_fhget;
-extern xfs_version_t xfsv_fhget_native;
-extern xfs_version_t xfsv_gc_nodes;
-
-void xfs_probe_capability (int fd);
-void xfs_probe_disable (xfs_version_t *cap);
-void xfs_probe_print (FILE *f);
-
+void xfs_probe_version (int fd, int version);
 
 void xfs_message_init (void);
 int  xfs_message_receive (int fd, struct xfs_message_header *h, u_int size);
@@ -64,31 +53,17 @@ int  xfs_send_message_wakeup_multiple (int fd, u_int seqnum,
 				       int error, ...);
 int  xfs_send_message_wakeup_data (int fd, u_int seqnum, int error,
 				   void *data, int size);
-int  xfs_message_send (int fd, struct xfs_message_header *h, u_int size);
+int  xfs_send_message_multiple_list (int fd, struct xfs_message_header *h,
+				     size_t size, u_int num);
+int  xfs_send_message_multiple (int fd, ...);
+int  xfs_send_message_vmultiple (int fd, va_list args);
 
+int  xfs_message_send (int fd, struct xfs_message_header *h, u_int size);
+int  xfs_message_rpc (int fd, struct xfs_message_header *h, u_int size);
 
 typedef int 
 (*xfs_message_function) (int, struct xfs_message_header*, u_int);
 
-xfs_message_function rcvfuncs[XFS_MSG_COUNT];
-
-
-static inline Bool __attribute__ ((unused))
-xfs_probe_status(xfs_version_t cap)
-{
-    if (cap == XFSV_YES &&
-	(xfsv_version == XFSV_NO && cap == XFSV_UNKNOWN))
-	return TRUE;
-    return FALSE;
-}
-
-static inline Bool __attribute__ ((unused))
-xfs_probe_have(xfs_version_t cap)
-{
-    if (cap == XFSV_YES)
-	return TRUE;
-    return FALSE;
-}
-
+extern xfs_message_function rcvfuncs[XFS_MSG_COUNT];
 
 #endif /* __XFS_H_V */
