@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: kafs.h,v 1.24 1999/08/04 13:44:48 assar Exp $ */
+/* $Id: kafs.h,v 1.32.2.2 2001/10/02 16:13:12 jimmy Exp $ */
 
 #ifndef __KAFS_H
 #define __KAFS_H
@@ -54,6 +49,10 @@
 #define _VICEIOCTL(id)  ((unsigned int ) _IOW('V', id, struct ViceIoctl))
 #endif /* _VICEIOCTL */
 
+#ifndef _ARLAIOCTL
+#define _ARLAIOCTL(id)  ((unsigned int ) _IOW('A', id, struct ViceIoctl))
+#endif /* _ARLAIOCTL */
+
 /* 
  * this is for operating systems that support a 32-bit API while being
  * 64-bit (such as solaris 7 in 64 bit mode).  The ioctls will get
@@ -67,6 +66,19 @@
 #define _VICEIOCTL32(id)  ((unsigned int ) _IOW('V', id, struct ViceIoctl32))
 #endif /* _VICEIOCTL32 */
 
+#ifndef _ARLAIOCTL32
+#define _ARLAIOCTL32(id)  ((unsigned int ) _IOW('A', id, struct ViceIoctl32))
+#endif /* _ARLAIOCTL32 */
+
+#ifndef _VICEIOCTL64
+#define _VICEIOCTL64(id)  ((unsigned int ) _IOW('V', id, struct ViceIoctl64))
+#endif /* _VICEIOCTL64 */
+
+#ifndef _ARLAIOCTL64
+#define _ARLAIOCTL64(id)  ((unsigned int ) _IOW('A', id, struct ViceIoctl64))
+#endif /* _ARLAIOCTL64 */
+
+
 #endif /* NEED_VICEIOCTL32 */
 
 
@@ -135,21 +147,29 @@
 #define VIOC_SETSPREFS		_VICEIOCTL(46)
 #define VIOC_STORBEHIND		_VICEIOCTL(47)
 #define VIOC_GCPAGS             _VICEIOCTL(48)
-#define VIOC_GETRXKCRYPT	_VICEIOCTL(49) /* 48 in some implementations */
-#define VIOC_SETRXKCRYPT	_VICEIOCTL(50) /* with cryptosupport in afs */
-#define VIOC_FPRIOSTATUS	_VICEIOCTL(51)
 
-#define VIOC_FHGET		_VICEIOCTL(52)
-#define VIOC_FHOPEN		_VICEIOCTL(53)
 
-#define VIOC_XFSDEBUG           _VICEIOCTL(54)
-#define VIOC_ARLADEBUG		_VICEIOCTL(55)
+#define VIOC_GETRXKCRYPT	_VICEIOCTL(55) /* 48 in some implementations */
+#define VIOC_SETRXKCRYPT	_VICEIOCTL(56) /* with cryptosupport in afs */
 
-#define VIOC_AVIATOR            _VICEIOCTL(56)
+/* arla specific */
 
-#define VIOC_XFSDEBUG_PRINT	_VICEIOCTL(57)
+#define VIOC_FPRIOSTATUS	_VICEIOCTL(57) /* arla: set file prio */
+#define VIOC_FHGET		_VICEIOCTL(58) /* arla: fallback getfh */
+#define VIOC_FHOPEN		_VICEIOCTL(59) /* arla: fallback fhopen */
 
-#define VIOC_CALCULATE_CACHE	_VICEIOCTL(58)
+#define VIOC_XFSDEBUG           _VICEIOCTL(60) /* arla: controls xfsdebug */
+#define VIOC_ARLADEBUG		_VICEIOCTL(61) /* arla: controls arla debug */
+
+#define VIOC_AVIATOR            _VICEIOCTL(62) /* arla: debug interface */
+
+#define VIOC_XFSDEBUG_PRINT	_VICEIOCTL(63) /* arla: print xfs status */
+
+#define VIOC_CALCULATE_CACHE	_VICEIOCTL(64) /* arla: force cache check */
+
+#define VIOC_BREAKCALLBACK	_VICEIOCTL(65) /* arla: break callback */
+
+#define AIOC_STATISTICS         _ARLAIOCTL(1)   /* arla: fetch statistics */
 
 #ifdef NEED_VICEIOCTL32
 
@@ -204,21 +224,102 @@
 #define VIOC_SETSPREFS_32		_VICEIOCTL32(46)
 #define VIOC_STORBEHIND_32		_VICEIOCTL32(47)
 #define VIOC_GCPAGS_32			_VICEIOCTL32(48)
-#define VIOC_GETRXKCRYPT_32		_VICEIOCTL32(49) /* 48 in some implementations */
-#define VIOC_SETRXKCRYPT_32		_VICEIOCTL32(50) /* with cryptosupport in afs */
-#define VIOC_FPRIOSTATUS_32		_VICEIOCTL32(51)
 
-#define VIOC_FHGET_32			_VICEIOCTL32(52)
-#define VIOC_FHOPEN_32			_VICEIOCTL32(53)
+#define VIOC_GETRXKCRYPT_32		_VICEIOCTL32(55) /* 48 in some implementations */
+#define VIOC_SETRXKCRYPT_32		_VICEIOCTL32(56) /* with cryptosupport in afs */
+#define VIOC_FPRIOSTATUS_32		_VICEIOCTL32(57)
 
-#define VIOC_XFSDEBUG_32		_VICEIOCTL32(54)
-#define VIOC_ARLADEBUG_32		_VICEIOCTL32(55)
+#define VIOC_FHGET_32			_VICEIOCTL32(58)
+#define VIOC_FHOPEN_32			_VICEIOCTL32(59)
 
-#define VIOC_AVIATOR_32			_VICEIOCTL32(56)
+#define VIOC_XFSDEBUG_32		_VICEIOCTL32(60)
+#define VIOC_ARLADEBUG_32		_VICEIOCTL32(61)
 
-#define VIOC_XFSDEBUG_PRINT_32		_VICEIOCTL32(57)
+#define VIOC_AVIATOR_32			_VICEIOCTL32(62)
 
-#define VIOC_CALCULATE_CACHE_32		_VICEIOCTL32(58)
+#define VIOC_XFSDEBUG_PRINT_32		_VICEIOCTL32(63)
+
+#define VIOC_CALCULATE_CACHE_32		_VICEIOCTL32(64)
+
+#define VIOC_BREAKCALLBACK_32		_VICEIOCTL32(65)
+
+/* and now for the 64-bit versions */
+
+#define VIOCSETAL_64			_VICEIOCTL64(1)
+#define VIOCGETAL_64			_VICEIOCTL64(2)
+#define VIOCSETTOK_64			_VICEIOCTL64(3)
+#define VIOCGETVOLSTAT_64		_VICEIOCTL64(4)
+#define VIOCSETVOLSTAT_64		_VICEIOCTL64(5)
+#define VIOCFLUSH_64			_VICEIOCTL64(6)
+#define VIOCSTAT_64			_VICEIOCTL64(7)
+#define VIOCGETTOK_64			_VICEIOCTL64(8)
+#define VIOCUNLOG_64			_VICEIOCTL64(9)
+#define VIOCCKSERV_64			_VICEIOCTL64(10)
+#define VIOCCKBACK_64			_VICEIOCTL64(11)
+#define VIOCCKCONN_64			_VICEIOCTL64(12)
+#define VIOCGETTIME_64			_VICEIOCTL64(13)
+#define VIOCWHEREIS_64			_VICEIOCTL64(14)
+#define VIOCPREFETCH_64			_VICEIOCTL64(15)
+#define VIOCNOP_64			_VICEIOCTL64(16)
+#define VIOCENGROUP_64			_VICEIOCTL64(17)
+#define VIOCDISGROUP_64			_VICEIOCTL64(18)
+#define VIOCLISTGROUPS_64		_VICEIOCTL64(19)
+#define VIOCACCESS_64			_VICEIOCTL64(20)
+#define VIOCUNPAG_64			_VICEIOCTL64(21)
+#define VIOCGETFID_64			_VICEIOCTL64(22)
+#define VIOCWAITFOREVER_64		_VICEIOCTL64(23)
+#define VIOCSETCACHESIZE_64		_VICEIOCTL64(24)
+#define VIOCFLUSHCB_64			_VICEIOCTL64(25)
+#define VIOCNEWCELL_64			_VICEIOCTL64(26)
+#define VIOCGETCELL_64			_VICEIOCTL64(27)
+#define VIOC_AFS_DELETE_MT_PT_64	_VICEIOCTL64(28)
+#define VIOC_AFS_STAT_MT_PT_64		_VICEIOCTL64(29)
+#define VIOC_FILE_CELL_NAME_64		_VICEIOCTL64(30)
+#define VIOC_GET_WS_CELL_64		_VICEIOCTL64(31)
+#define VIOC_AFS_MARINER_HOST_64	_VICEIOCTL64(32)
+#define VIOC_GET_PRIMARY_CELL_64	_VICEIOCTL64(33)
+#define VIOC_VENUSLOG_64		_VICEIOCTL64(34)
+#define VIOC_GETCELLSTATUS_64		_VICEIOCTL64(35)
+#define VIOC_SETCELLSTATUS_64		_VICEIOCTL64(36)
+#define VIOC_FLUSHVOLUME_64		_VICEIOCTL64(37)
+#define VIOC_AFS_SYSNAME_64		_VICEIOCTL64(38)
+#define VIOC_EXPORTAFS_64		_VICEIOCTL64(39)
+#define VIOCGETCACHEPARAMS_64		_VICEIOCTL64(40)
+#define VIOCCONNECTMODE_64	        _VICEIOCTL64(41)
+#define VIOCGETVCXSTATUS_64		_VICEIOCTL64(41)
+#define VIOC_SETSPREFS_6433		_VICEIOCTL64(42)
+#define VIOC_GETSPREFS_64		_VICEIOCTL64(43)
+#define VIOC_GAG_64			_VICEIOCTL64(44)
+#define VIOC_TWIDDLE_64			_VICEIOCTL64(45)
+#define VIOC_SETSPREFS_64		_VICEIOCTL64(46)
+#define VIOC_STORBEHIND_64		_VICEIOCTL64(47)
+#define VIOC_GCPAGS_64			_VICEIOCTL64(48)
+
+
+#define VIOC_GETRXKCRYPT_64		_VICEIOCTL64(55) /* 48 in some implementations */
+#define VIOC_SETRXKCRYPT_64		_VICEIOCTL64(56) /* with cryptosupport in afs */
+#define VIOC_FPRIOSTATUS_64		_VICEIOCTL64(57)
+
+#define VIOC_FHGET_64			_VICEIOCTL64(58)
+#define VIOC_FHOPEN_64			_VICEIOCTL64(59)
+
+#define VIOC_XFSDEBUG_64		_VICEIOCTL64(60)
+#define VIOC_ARLADEBUG_64		_VICEIOCTL64(61)
+
+#define VIOC_AVIATOR_64			_VICEIOCTL64(62)
+
+#define VIOC_XFSDEBUG_PRINT_64		_VICEIOCTL64(63)
+
+#define VIOC_CALCULATE_CACHE_64		_VICEIOCTL64(64)
+
+#define VIOC_BREAKCALLBACK_64		_VICEIOCTL64(65)
+
+/*
+ * Arla implementationspecific IOCTL'S
+ */
+
+#define AIOC_STATISTICS_32	_ARLAIOCTL32(1) /* arla: fetch statistics */
+#define AIOC_STATISTICS_64	_ARLAIOCTL64(1) /* arla: fetch statistics */
 
 #endif /* NEED_VICEIOCTL32 */
 
@@ -273,6 +374,17 @@ struct vioc_fprio {
 #define CKSERV_MAXSERVERS   16 /* limitation of VIOCCKSERV number of 
 				  returned servers */
 
+/* 
+ *  for AIOC_STATISTICS
+ */
+
+#define STATISTICS_OPCODE_LIST 0
+#define STATISTICS_OPCODE_GETENTRY 1
+
+#define STATISTICS_REQTYPE_FETCHSTATUS 1
+#define STATISTICS_REQTYPE_FETCHDATA 2
+#define STATISTICS_REQTYPE_BULKSTATUS 3
+
 
 #if !defined(HAVE_STRUCT_VICEIOCTL_IN) || !defined(__KERNEL__)
 struct ViceIoctl {
@@ -280,13 +392,25 @@ struct ViceIoctl {
   short in_size;
   short out_size;
 };
+#endif
 
 struct ViceIoctl32 {
   u_int32_t in, out;		/* really caddr_t in 32 bits */
   short in_size;
   short out_size;
 };
+
+#if NEED_VICEIOCTL32
+struct ViceIoctl64 {
+#ifdef HAVE_U_INT64_T
+  u_int64_t in, out;		/* really caddr_t in 64 bits */
+#else
+  caddr_t in, out;
 #endif
+  short in_size;
+  short out_size;
+};
+#endif /* NEED_VICEIOCTL32 */
 
 struct ClearToken {
   int32_t AuthHandle;
@@ -307,9 +431,6 @@ int k_hasafs __P((void));
 
 int krb_afslog __P((const char *cell, const char *realm));
 int krb_afslog_uid __P((const char *cell, const char *realm, uid_t uid));
-/* compat */
-#define k_afsklog krb_afslog
-#define k_afsklog_uid krb_afslog_uid
 
 int k_pioctl __P((char *a_path,
 		  int o_opcode,
@@ -345,4 +466,9 @@ krb5_error_code krb5_afslog __P((krb5_context, krb5_ccache,
 #define _PATH_CELLSERVDB 	_PATH_VICE "CellServDB"
 #define _PATH_THESECELLS	_PATH_VICE "TheseCells"
 
+#define _PATH_ARLA_VICE		"/usr/arla/etc/"
+#define _PATH_ARLA_THISCELL	_PATH_ARLA_VICE "ThisCell"
+#define _PATH_ARLA_CELLSERVDB 	_PATH_ARLA_VICE "CellServDB"
+#define _PATH_ARLA_THESECELLS	_PATH_ARLA_VICE "TheseCells"
+
 #endif /* __KAFS_H */
