@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: stds.h,v 1.3 1998/03/19 21:28:28 assar Exp $ */
+/* $Id: stds.h,v 1.6 2000/10/03 00:34:21 lha Exp $ */
 
 #ifndef __STDS_H
 #define __STDS_H 1
@@ -49,8 +44,16 @@ typedef int32_t int32;
 typedef u_int32_t u_int32;
 #endif
 
+#ifndef HAVE_INT16
+typedef int16_t int16;
+#endif
+
+#ifndef HAVE_U_INT16
+typedef u_int16_t u_int16;
+#endif
+
 /* for compat resons with other kerberos distributions */
-#ifndef HAVE_KRB_PRINCIPAL 
+#if defined(KERBEROS) && !defined(HAVE_KRB_PRINCIPAL)
 typedef struct krb_principal{
     char name[ANAME_SZ];
     char instance[INST_SZ];
