@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,14 +33,17 @@
 
 #include <config.h>
 
-RCSID("$Id: fbuf.c,v 1.7 1999/07/24 13:25:39 lha Exp $") ;
+RCSID("$Id: fbuf.c,v 1.13 2000/10/02 22:59:33 lha Exp $") ;
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
+#ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
+#endif
 #include <unistd.h>
 
 #include <fs_errors.h>
@@ -69,13 +67,32 @@ mmap_flags (fbuf_flags flags)
 {
     int ret = 0;
 
-    if (flags & FBUF_READ)
+    assert ((flags & (FBUF_PRIVATE | FBUF_SHARED)) == FBUF_PRIVATE
+	|| (flags & (FBUF_PRIVATE | FBUF_SHARED)) == FBUF_SHARED);
+
+    if (flags & FBUF_PRIVATE)
 	ret = MAP_PRIVATE;
-    if (flags & FBUF_WRITE)
+    else if (flags & FBUF_SHARED)
 	ret = MAP_SHARED;
     return ret;
 }
 
+/*
+ * Map fbuf_flags in `flags' to mmap prot
+ */
+
+static int
+mmap_prot (fbuf_flags flags)
+{
+    int ret = 0;
+
+    if (flags & FBUF_READ)
+	ret |= PROT_READ;
+    if (flags & FBUF_WRITE)
+	ret |= PROT_WRITE;
+    return ret;
+}
+
 /*
  * Create a fbuf with (fd, len, flags).
  * Returns 0 or error.
@@ -87,11 +104,9 @@ mmap_create (fbuf *f, int fd, size_t len, fbuf_flags flags)
     void *buf;
 
     if (len != 0) {
-	buf = mmap (0, len, PROT_READ | PROT_WRITE, mmap_flags(flags), fd, 0);
-	if (buf == (void *) MAP_FAILED) {
-	    close (fd);
+	buf = mmap (0, len, mmap_prot(flags), mmap_flags(flags), fd, 0);
+	if (buf == (void *) MAP_FAILED)
 	    return errno;
-	}
     } else
 	buf = NULL;
 
@@ -115,16 +130,12 @@ mmap_truncate (fbuf *f, size_t new_len)
 
     if (f->buf != NULL) {
 	ret = munmap (f->buf, f->len);
-	if (ret < 0) {
-	    close (f->fd);
+	if (ret < 0)
 	    return errno;
-	}
     }
     ret = ftruncate (f->fd, new_len);
-    if (ret < 0) {
-	close (f->fd);
+    if (ret < 0)
 	return errno;
-    }
     return mmap_create (f, f->fd, new_len, f->flags);
 }
 
@@ -141,7 +152,6 @@ mmap_end (fbuf *f)
     ret = munmap (f->buf, f->len);
     if (ret < 0)
 	ret = errno;
-    close (f->fd);
     return ret;
 }
 
@@ -208,14 +218,11 @@ malloc_create (fbuf *f, int fd, size_t len, fbuf_flags flags)
     void *buf;
 
     buf = malloc (len);
-    if (buf == NULL) {
-	close(fd);
+    if (buf == NULL)
 	return ENOMEM;
-    }
     if (lseek(fd, 0, SEEK_SET) == -1 ||
 	read (fd, buf, len) != len) {
 	free(buf);
-	close(fd);
 	return errno;
     }
     f->buf   = buf;
@@ -233,8 +240,8 @@ malloc_create (fbuf *f, int fd, size_t len, fbuf_flags flags)
 static int
 malloc_flush (fbuf *f)
 {
-    if (f->flags & FBUF_WRITE) {
-	lseek (f->fd, SEEK_SET, 0);
+    if ((f->flags & FBUF_WRITE) && (f->flags & FBUF_SHARED)) {
+	lseek (f->fd, 0, SEEK_SET);
 	if (write (f->fd, f->buf, f->len) != f->len)
 	    return errno;
     }
@@ -253,7 +260,6 @@ malloc_end (fbuf *f)
 
     ret = malloc_flush (f);
     free (f->buf);
-    close (f->fd);
     return ret;
 }
 
@@ -291,7 +297,6 @@ malloc_truncate (fbuf *f, size_t new_len)
 
 fail:
     free (f->buf);
-    close (f->fd);
     return ret;
 }
 
