@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,14 +31,19 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: fbuf.h,v 1.4 1999/06/24 02:06:20 assar Exp $ */
+/* $Id: fbuf.h,v 1.6 2000/10/02 22:59:38 lha Exp $ */
 
 #ifndef _FBUF_H_
 #define _FBUF_H_
 
 #include <rx/rx.h>
 
-typedef enum { FBUF_READ = 1, FBUF_WRITE = 2 } fbuf_flags;
+typedef enum {
+    FBUF_READ    = 0x01,
+    FBUF_WRITE   = 0x02,
+    FBUF_PRIVATE = 0x04,
+    FBUF_SHARED  = 0x08
+} fbuf_flags;
 
 struct fbuf {
     void *buf;
