@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
  * Interface to directory handling routines
  */
 
-/* $Id: fdir.h,v 1.1 1999/05/17 14:26:34 lha Exp $ */
+/* $Id: fdir.h,v 1.5 2000/10/02 22:59:49 lha Exp $ */
 
 #ifndef _FDIR_H_
 #define _FDIR_H_
@@ -50,7 +45,7 @@
 
 int
 fdir_lookup (fbuf *the_fbuf,
-	     VenusFid *dir,
+	     const VenusFid *dir,
 	     const char *name,
 	     VenusFid *file);
 
@@ -58,17 +53,18 @@ fdir_lookup (fbuf *the_fbuf,
 int
 fdir_changefid (fbuf *the_fbuf,
 		const char *name,
-		VenusFid *file,
-		VenusFid *dir);
+		const VenusFid *file);
 
 int
 fdir_emptyp (fbuf *dir);
 
+typedef void (*fdir_readdir_func)(VenusFid *, const char *, void *);
+
 int
 fdir_readdir (fbuf *the_fbuf,
-	      void (*func)(VenusFid *, const char *, void *), 
+	      fdir_readdir_func func,
 	      void *arg,
-	      VenusFid *dir);
+	      const VenusFid *dir);
 
 int
 fdir_creat (fbuf *dir,
