@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: agetarg.c,v 1.8.2.1 2001/01/09 02:18:55 lha Exp $");
+RCSID("$Id: agetarg.c,v 1.8.2.3 2001/05/17 07:38:30 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -186,10 +186,14 @@ aarg_printusage (struct agetargs *args,
 
 	if (arg->long_name) {
 	    if (style & AARG_TRANSLONG) {
-		if (arg->mandatoryp)
+		switch (arg->mandatoryp) {
+		case aarg_mandatory:
 		    fprintf (stderr, " -");
-		else
+		    break;
+		default:
 		    fprintf (stderr, " [-");
+		    break;
+		}
 	    } else
 		fprintf (stderr, " [--");
 
@@ -200,10 +204,10 @@ aarg_printusage (struct agetargs *args,
 	    fprintf (stderr, "%s", arg->long_name);
 	    len += 2 + strlen(arg->long_name);
 	    len += print_arg (stderr, 0, 1, arg, style);
-	    if(!(style & AARG_TRANSLONG) || !arg->mandatoryp)
-		putc (']', stderr);
 	    if(arg->type == aarg_strings)
 		fprintf (stderr, "...");
+	    if(!(style & AARG_TRANSLONG) || arg->mandatoryp != aarg_mandatory)
+		putc (']', stderr);
 	}
 	if (arg->short_name) {
 	    len += 2;
@@ -453,7 +457,10 @@ agetarg(struct agetargs *args,
     srand (time(NULL));
     (*optind)++;
     for(i = *optind; i < argc; i++) {
-	if(argv[i][0] != '-' && swcount != -1 && (args[swcount].mandatoryp == aarg_mandatory) ) {
+	if(argv[i][0] != '-'
+	   && swcount != -1
+	   && (args[swcount].mandatoryp == aarg_mandatory
+	       || args[swcount].mandatoryp == aarg_optional_swless)) {
 	    /* the mandatory junk up there is to prevent agetarg() from
 	       automatically matching options even when not specified with
 	       their flagged name
@@ -471,8 +478,9 @@ agetarg(struct agetargs *args,
 	    usedargs[swcount] = 1;
 	    i += j;
 	    swcount++;
-	} else if(argv[i][1] == '-' || 
-		  ((style & AARG_TRANSLONG) && argv[i][1] != 0)) {
+	} else if(argv[i][0] == '-' &&
+		  (argv[i][1] == '-' || 
+		   ((style & AARG_TRANSLONG) && argv[i][1] != 0))) {
 	    int k;
 
 	    if(argv[i][2] == 0 && !(style & AARG_TRANSLONG)){
@@ -550,7 +558,8 @@ agetarg(struct agetargs *args,
 		}
 	    }
 	out:;
-	}
+	} else
+	    break;
     }
     *optind = i;
 
