@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: ko.h,v 1.27 2000/10/08 07:44:38 lha Exp $ */
+/* $Id: ko.h,v 1.27.2.2 2001/05/06 22:40:49 ahltorp Exp $ */
 
 #ifndef __KO_H
 #define __KO_H 1
@@ -67,6 +67,11 @@ typedef struct {
     time_t timeout;		/* timeout of address */
 } cell_db_entry;
 
+enum { SUID_CELL 	= 0x1,	/* if this is a suid cell */
+       DYNROOT_CELL	= 0x2	/* cell should show up in dynroot */
+};
+
+
 typedef struct {
     int32_t id;			/* Cell-ID */
     const char *name;		/* Domain-style name */
@@ -74,7 +79,7 @@ typedef struct {
     unsigned ndbservers;	/* # of database servers */
     unsigned active_hosts;	/* # of active db servers */
     cell_db_entry *dbservers;	/* Database servers */
-    enum { NOSUID_CELL, SUID_CELL } suid_cell ; /* if this is a suid cell */
+    unsigned flags;		/* Various flags, like SUID_CELL */
     time_t timeout;		/* when this entry expire */
 } cell_entry;
 
@@ -91,7 +96,8 @@ cell_entry    *cell_get_by_name (const char *cellname);
 cell_entry    *cell_get_by_id (int32_t cell);
 cell_entry    *cell_new (const char *name);
 cell_entry    *cell_new_dynamic (const char *name);
-Bool           cell_issuid (cell_entry *c);
+Bool	       cell_dynroot (const cell_entry *c);
+Bool           cell_issuid (const cell_entry *c);
 Bool           cell_issuid_by_num (int32_t cell);
 Bool           cell_issuid_by_name (const char *cell);
 Bool	       cell_setsuid_by_num (int32_t cell);
@@ -115,4 +121,9 @@ void 	       cell_print_cell (cell_entry *c, FILE *out);
 void vldb2vldbN (const vldbentry *old, nvldbentry *new);
 void volintInfo2xvolintInfo (const volintInfo *old, xvolintInfo *new);
 
+int volname_canonicalize (char *volname);
+size_t volname_specific (const char *volname, int type,
+			 char *buf, size_t buf_sz);
+const char *volname_suffix (int type);
+
 #endif /* __KO_H */
