@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 /*
  *  Include file for whole arlad
- *  $Id: ko_locl.h,v 1.5 1999/06/13 05:27:56 assar Exp $
+ *  $Id: ko_locl.h,v 1.12 2000/10/02 22:43:03 lha Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,8 +53,13 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/uio.h>
+#ifdef HAVE_SYS_MMAN_H
 #include <sys/mman.h>
+#endif
 #include <sys/stat.h>
+#ifdef HAVE_SYS_IOCCOM_H
+#include <sys/ioccom.h>
+#endif
 #ifdef HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
 #endif
@@ -70,14 +70,13 @@
 #include <err.h>
 #include <strings.h>
 #include <roken.h>
-#include <getarg.h>
+#include <parse_units.h>
+#include <agetarg.h>
 
 #include <hash.h>
 #include <bool.h>
+#include <log.h>
 
 #include "ko.h"
-#include "kodebug.h"
-
-#include "strmatch.h"
 
 
