@@ -50,13 +50,14 @@
 
 #include "resolve.h"
 
-RCSID("$Id: kocell.c,v 1.47 2000/12/21 22:07:43 lha Exp $");
+RCSID("$Id: kocell.c,v 1.47.2.3 2001/07/09 03:27:48 lha Exp $");
 
 #define TRANSARCSYSCONFDIR "/usr/vice/etc"
 #define CELLFILENAME "CellServDB"
 #define THISCELLFILENAME "ThisCell"
 #define SUIDCELLSFILENAME "SuidCells"
 #define THESECELLFILENAME "TheseCells"
+#define DYNROOTDBFILENAME "DynRootDB"
 #define DEFCELLCACHESIZE 499
 
 /*
@@ -72,6 +73,7 @@ static Hashtab *cellnamehtab, *cellnumhtab;
 static char *thiscell = NULL;
 static char **thesecells = NULL;
 static int numthesecells = 1;
+static int dynrootdb_in_use;
 
 enum { CELL_INVALID_HOST = 1 };
 
@@ -219,7 +221,7 @@ host_sort (const void *p1, const void *p2)
 	return -1;
     if (b->timeout == CELL_INVALID_HOST)
 	return 1;
-    return strcmp(a->name, b->name);
+    return a->addr.s_addr - b->addr.s_addr;
 }
 
 /*
@@ -244,13 +246,14 @@ updatehosts (cell_entry *c, int dbnum, cell_db_entry *dbservers)
 
     c->ndbservers = dbnum;
     c->dbservers  = malloc (dbnum * sizeof(cell_db_entry));
-    if (c->dbservers == NULL)
+    if (c->dbservers == NULL && dbnum != 0)
 	err (1, "malloc %lu", (unsigned long)dbnum * sizeof(cell_db_entry));
     memcpy (c->dbservers, dbservers, dbnum * sizeof (cell_db_entry));
     free (old_servers);
 
-    qsort (c->dbservers, c->ndbservers, sizeof(c->dbservers[0]),
-	   host_sort);
+    if (c->ndbservers)
+	qsort (c->dbservers, c->ndbservers, sizeof(c->dbservers[0]),
+	       host_sort);
 
     c->active_hosts = 0;
     for (i = 0; i < c->ndbservers; i++) {
@@ -410,7 +413,7 @@ cell_new (const char *name)
     c->ndbservers   = 0;
     c->active_hosts = 0;
     c->dbservers    = NULL;
-    c->suid_cell    = NOSUID_CELL;
+    c->flags	    = 0;
     hashtabadd (cellnamehtab, c);
     hashtabadd (cellnumhtab, c);
     c->timeout      = 0;
@@ -540,6 +543,29 @@ readcellservdb (const char *filename)
      return 0;
 }
 
+/*
+ * Read single line confilefiles in `filename' and send the 
+ * to `func'.
+ */
+
+static int
+parse_simple_file (const char *filename, void (*func)(const char *))
+{
+    FILE *f;
+    char line[256];
+    
+    f = fopen (filename, "r");
+    if (f == NULL)
+	return 1;
+    
+    while ((fgets (line, sizeof(line), f) != NULL)) {
+	line[strlen(line) - 1] = '\0';
+	(*func) (line);
+    }
+    fclose (f);
+    return 0;
+}
+
 /*
  * Add expanded `cellname' to Thesecells list, filter out duplicates
  */
@@ -560,24 +586,13 @@ addthesecell (const char *cellname)
 }
 
 /*
- * Read cells in these cell's
+ * Read cells in TheseCells
  */
 
 static int
 readthesecells (const char *filename)
 {
-     FILE *f;
-     char cell[256];
-
-     f = fopen (filename, "r");
-     if (f == NULL)
-	 return 1;
-
-     while ((fgets (cell, sizeof cell, f) != NULL)) {
-	 cell[strlen(cell) - 1] = '\0';
-	 addthesecell (cell);
-     }
-     fclose (f);
+     parse_simple_file (filename, addthesecell);
      if (numthesecells == 1)
 	 return 1;
      return 0;
@@ -615,34 +630,49 @@ readthiscell (const char *filename)
  * Read suidcells file and set suidcell flag
  */
 
+static void
+addsuidcell (const char *cellname)
+{
+    cell_entry *e;
+
+    e = cell_get_by_name (cellname);
+    if (e == NULL) {
+	log_log (cell_log, CDEBWARN,
+		 "suidcell: cell %s doesn't exist in the db\n", cellname);
+    } else {
+	e->flags |= SUID_CELL;
+    }
+}
+
 static int
 readsuidcell (const char *filename)
 {
-     FILE *f;
-     char cell[256];
+    return parse_simple_file (filename, addsuidcell);
+}
 
-     f = fopen (filename, "r");
-     if (f == NULL)
-	 return 1;
+/*
+ *
+ */
 
-     while (fgets (cell, sizeof(cell), f) != NULL) {
-	 int i;
-	 cell_entry *e;
+static void
+add_dynroot(const char *cellname)
+{
+    cell_entry *e; 
 
-	 i = strlen (cell);
-	 if (cell[i - 1] == '\n')
-	     cell[i - 1] = '\0';
+    e = cell_get_by_name (cellname);
+    if (e == NULL) {
+	log_log (cell_log, CDEBWARN,
+		 "dynroot: cell %s doesn't exist in the db\n", cellname);
+    } else {
+	e->flags |= DYNROOT_CELL;
+	dynrootdb_in_use = 1;
+    }
+}
 
-	 e = cell_get_by_name (cell);
-	 if (e == NULL) {
-	     	log_log (cell_log, CDEBWARN,
-			 "cell %s doesn't exist in the db\n", cell);
-	 } else {
-	     e->suid_cell = SUID_CELL;
-	 }
-     }
-     fclose (f);
-     return 0;
+static int
+readdynrootdb (const char *filename)
+{
+    return parse_simple_file (filename, add_dynroot);
 }
 
 /*
@@ -717,6 +747,9 @@ cell_init (int cellcachesize, Log_method *log)
     addthesecell (thiscell);
     if (readsuidcell (SYSCONFDIR "/" SUIDCELLSFILENAME))
 	readsuidcell (TRANSARCSYSCONFDIR "/" SUIDCELLSFILENAME);
+    if (readdynrootdb (SYSCONFDIR "/" DYNROOTDBFILENAME))
+	readdynrootdb (TRANSARCSYSCONFDIR "/" DYNROOTDBFILENAME);
+
 }
 
 /*
@@ -859,15 +892,28 @@ cell_setthiscell (const char *cell)
     return 0;
 }
 
+/*
+ * Return if this should be in dynroot
+ */
+
+Bool
+cell_dynroot (const cell_entry *c)
+{
+    assert (c);
+    if (!dynrootdb_in_use)
+	return TRUE;
+    return (c->flags & DYNROOT_CELL) != 0;
+}
+
 /*
  * Return if this is a suid cell
  */
 
 Bool
-cell_issuid (cell_entry *c)
+cell_issuid (const cell_entry *c)
 {
     assert (c);
-    return c->suid_cell == SUID_CELL;
+    return (c->flags & SUID_CELL) != 0;
 }
 
 Bool
@@ -903,7 +949,7 @@ cell_setsuid_by_num (int32_t cell)
     if (c == NULL)
 	return FALSE;
     
-    c->suid_cell = SUID_CELL;
+    c->flags |= SUID_CELL;
 
     return 0;
 }
@@ -1060,7 +1106,7 @@ add_special_dynroot_cell (void)
     c->expl       = "The special dynroot cell";
     c->ndbservers = 0;
     c->dbservers  = NULL;
-    c->suid_cell  = NOSUID_CELL;
+    c->flags	  = 0;
     hashtabadd (cellnamehtab, c);
     hashtabadd (cellnumhtab, c);
     return 0;
