@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: koerror.c,v 1.13 1999/05/30 00:50:19 lha Exp $");
+RCSID("$Id: koerror.c,v 1.20 2001/01/08 16:48:03 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -50,11 +45,13 @@ RCSID("$Id: koerror.c,v 1.13 1999/05/30 00:50:19 lha Exp $");
 #include <volumeserver.h>
 #include <pts.h>
 #include <bos.h>
+#include <ubik.h>
+#include <ka.h>
+#include <rx/rx.h>
+#include <rx/rxgencon.h>
 #ifdef KERBEROS
 #include <krb.h>
 #include <des.h>
-#include <rx/rx.h>
-#include <rx/rxgencon.h>
 #include <rxkad.h>
 #endif
 #include <ko.h>
@@ -191,6 +188,91 @@ static struct koerr koerrmsg[] = {
     {BZNET,		      "bos - Network error"},
     {BZBADTYPE,		      "bos - Bad type"},
 
+    /* ubik errors */
+
+    {UNOQUORUM,		      "no quorum elected"},
+    {UNOTSYNC,		      "not synchronization site (should work on sync site)"},
+    {UNHOSTS,		      "too many hosts"},
+    {UIOERROR,		      "I/O error writing dbase or log"},
+    {UINTERNAL,		      "mysterious internal error"},
+    {USYNC,		      "major synchronization error"},
+    {UNOENT,		      "file not found when processing dbase"},
+    {UBADLOCK,		      "bad lock range size (must be 1)"},
+    {UBADLOG,		      "read error reprocessing log"},
+    {UBADHOST,		      "problems with host name"},
+    {UBADTYPE,		      "bad operation for this transaction type"},
+    {UTWOENDS,		      "two commits or aborts done to transaction"},
+    {UDONE,		      "operation done after abort (or commmit)"},
+    {UNOSERVERS,	      "no servers appear to be up"},
+    {UEOF,		      "premature EOF"},
+    {ULOGIO,		      "error writing log file"},
+    {UBADFAM,		      "UBADFAM"},
+    {UBADCELL,		      "UBADCELL"},
+    {UBADSECGRP,	      "UBADSECGRP"},
+    {UBADGROUP,	              "UBADGROUP"},
+    {UBADUUID,	              "UBADUUID"},
+    {UNOMEM,	              "UNOMEM"},
+    {UNOTMEMBER,	      "UNOTMEMBER"},
+    {UNBINDINGS,	      "UNBINDINGS"},
+    {UBADPRINNAME,	      "UBADPRINNAME"},
+    {UPIPE,	              "UPIPE"},
+    {UDEADLOCK,	              "UDEADLOCK"},
+    {UEXCEPTION,	      "UEXCEPTION"},
+    {UTPQFAIL,	              "UTPQFAIL"},
+    {USKEWED,	              "USKEWED"},
+    {UNOLOCK,	              "UNOLOCK"},
+    {UNOACCESS,	              "UNOACCESS"},
+    {UNOSPC,	              "UNOSPC"},
+    {UBADPATH,	              "UBADPATH"},
+    {UBADF,	              "UBADF"},
+    {UREINITIALIZE,	      "UREINITIALIZE"},
+
+    /* ka errors */
+
+    {KADATABASEINCONSISTENT,	"ka - database inconsistent"},
+    {KAEXIST,			"ka - already exists"},
+    {KAIO,			"ka - io error"},
+    {KACREATEFAIL,		"ka - creation failed"},
+    {KANOENT,			"ka - no such entry"},
+    {KAEMPTY,			"ka - empty"},
+    {KABADNAME,			"ka - bad name"},
+    {KABADINDEX,		"ka - bad index"},
+    {KANOAUTH,			"ka - no authorization"},
+    {KAANSWERTOOLONG,		"ka - answer too long"},
+    {KABADREQUEST,		"ka - bad request"},
+    {KAOLDINTERFACE,		"ka - old interface"},
+    {KABADARGUMENT,		"ka - bad argument"},
+    {KABADCMD,			"ka - bad command"},
+    {KANOKEYS,			"ka - no keys"},
+    {KAREADPW,			"ka - error reading password"},
+    {KABADKEY,			"ka - bad key"},
+    {KAUBIKINIT,		"ka - error initialing ubik"},
+    {KAUBIKCALL,		"ka - error in ubik call"},
+    {KABADPROTOCOL,		"ka - bad protocol"},
+    {KANOCELLS,			"ka - no cells"},
+    {KANOCELL,			"ka - no cell"},
+    {KATOOMANYUBIKS,		"ka - too many ubiks"},
+    {KATOOMANYKEYS,		"ka - too many keys"},
+    {KABADTICKET,		"ka - bad ticket"},
+    {KAUNKNOWNKEY,		"ka - unknown key"},
+    {KAKEYCACHEINVALID,		"ka - key cache invalid"},
+    {KABADSERVER,		"ka - bad server"},
+    {KABADUSER,			"ka - bad user"},
+    {KABADCPW,			"ka - bad change password"},
+    {KABADCREATE,		"ka - bad creation"},
+    {KANOTICKET,		"ka - no ticket"},
+    {KAASSOCUSER,		"ka - associated user"},
+    {KANOTSPECIAL,		"ka - not special"},
+    {KACLOCKSKEW,		"ka - clock skew"},
+    {KANORECURSE,		"ka - no recurse"},
+    {KARXFAIL,			"ka - rx failed"},
+    {KANULLPASSWORD,		"ka - null password"},
+    {KAINTERNALERROR,		"ka - internal error"},
+    {KAPWEXPIRED,		"ka - password expired"},
+    {KAREUSED,			"ka - password reused"},
+    {KATOOSOON,			"ka - password changed too soon"},
+    {KALOCKED,			"ka - account locked"},
+
     /* Not a known error */
 
     { 0L,              "Unknown error"}
@@ -201,6 +283,7 @@ static struct koerr koerrmsg[] = {
 const char *
 koerr_gettext(koerr_t err) 
 {
+    const char *ret = NULL;
     struct koerr *koerror = koerrmsg;
 
     while (koerror->code != 0) {
@@ -209,8 +292,10 @@ koerr_gettext(koerr_t err)
 	++koerror;
     }
 
-    if (koerror->code)
-	return koerror->msg;
-    else
-	return strerror(err);
+    if (koerror->code == 0)
+	ret = strerror(err);
+    if (!ret)
+	ret = koerror->msg;
+
+    return ret;
 }
