@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,9 @@
 
 #include "ko_locl.h"
 
-RCSID("$Id: part.c,v 1.1 1999/07/12 08:40:02 map Exp $");
+#include "part.h"
+
+RCSID("$Id: part.c,v 1.3 2000/10/02 22:43:24 lha Exp $");
 
 /*
  * partition_num2name
