@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include "ko_locl.h"
 
-RCSID("$Id: vlmisc.c,v 1.3 2000/10/02 22:43:44 lha Exp $");
+RCSID("$Id: vlmisc.c,v 1.3.2.1 2001/03/04 04:07:15 lha Exp $");
 
 /*
  * Convert old style vldbentry `old` to newer vldbNentry style `new'
@@ -83,3 +83,59 @@ volintInfo2xvolintInfo (const volintInfo *old, xvolintInfo *new)
     new->maxquota	= old->maxquota;
     new->size		= old->size;
 }
+
+/*
+ * Suffixes for volume names.
+ */
+
+static char *volsuffixes[] = {
+"",				/* RWVOL */
+".readonly",			/* ROVOL */
+".backup"			/* BACKVOL */
+};
+
+/*
+ * canonicalize `volname' and return type of volume
+ */
+
+int
+volname_canonicalize (char *volname)
+{
+    int i;
+    int type = RWVOL;
+
+    for (i = 0; i < MAXTYPES; ++i) {
+	char *p = strrchr (volname, '.');
+	if (p != NULL && strcmp(p, volsuffixes[i]) == 0) {
+	    *p = '\0';
+	    type = i;
+	    break;
+	}
+    }
+    return type;
+}
+
+/*
+ * make specific volume name
+ */
+
+size_t
+volname_specific (const char *volname, int type, char *buf, size_t buf_sz)
+{
+    assert (type >= 0 && type < MAXTYPES);
+
+    return snprintf (buf, buf_sz, "%s%s", volname, volsuffixes[type]);
+}
+
+
+/*
+ * get volume suffix
+ */
+
+const char *
+volname_suffix (int type)
+{
+    assert (type >= 0 && type < MAXTYPES);
+
+    return volsuffixes[type];
+}
