@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,16 +33,13 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: flock.c,v 1.1 1998/01/13 16:25:19 lha Exp $");
 #endif
 
+#ifndef HAVE_FLOCK
+RCSID("$Id: flock.c,v 1.2 2000/08/16 01:23:37 lha Exp $");
+
 #include "roken.h"
-#ifdef HAVE_FCNTL_H
-#include <fcntl.h>
-#endif
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
-#endif
+
 
 #define OP_MASK (LOCK_SH | LOCK_EX | LOCK_UN)
 
@@ -90,3 +82,6 @@ flock(int fd, int operation)
   return -1;
 #endif
 }
+
+#endif
+
