@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -36,33 +31,28 @@
  * SUCH DAMAGE. 
  */
 
-/* $Id: getarg.h,v 1.6 1998/12/08 04:00:08 lha Exp $ */
+/* $Id: getarg.h,v 1.9 2000/09/01 21:25:55 lha Exp $ */
 
 #ifndef __GETARG_H__
 #define __GETARG_H__
 
 #include <stddef.h>
 
-#define ARG_DEFAULT     0x0    /* ARG_GNUSTYLE */
-#define ARG_LONGARG     0x1    /* --foo=bar */
-#define ARG_SHORTARG    0x2    /* -abc   a, b and c are all three flags */
-#define ARG_TRANSLONG   0x4    /* Incompatible with {SHORT,LONG}ARG */
-#define ARG_SWITCHLESS  0x8    /* No switches */
-#define ARG_SUBOPTION   0xF    /* For manpage generation */
-
-#define ARG_GNUSTYLE (ARG_LONGARG|ARG_SHORTARG)
-#define ARG_AFSSTYLE (ARG_TRANSLONG|ARG_SWITCHLESS)
-
 struct getargs{
     const char *long_name;
     char short_name;
-    enum { arg_end = 0, arg_integer, arg_string, 
-	   arg_flag, arg_negative_flag, arg_strings,
-           arg_generic } type;
+    enum { arg_integer, 
+	   arg_string, 
+	   arg_flag, 
+	   arg_negative_flag, 
+	   arg_strings,
+	   arg_double,
+	   arg_collect,
+	   arg_counter
+    } type;
     void *value;
     const char *help;
     const char *arg_help;
-    enum { arg_optional = 0, arg_mandatory } mandatoryp;
 };
 
 enum {
@@ -76,12 +66,24 @@ typedef struct getarg_strings {
     char **strings;
 } getarg_strings;
 
-int getarg(struct getargs *args,
-	   int argc, char **argv, int *optind, int style);
+typedef int (*getarg_collect_func)(int short_opt,
+				   int argc,
+				   char **argv,
+				   int *optind,
+				   int *optarg,
+				   void *data);
+
+typedef struct getarg_collect_info {
+    getarg_collect_func func;
+    void *data;
+} getarg_collect_info;
+
+int getarg(struct getargs *args, size_t num_args, 
+	   int argc, char **argv, int *optind);
 
 void arg_printusage (struct getargs *args,
+		     size_t num_args,
 		     const char *progname,
-		     const char *extra_string,
-		     int style);
+		     const char *extra_string);
 
 #endif /* __GETARG_H__ */
