@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: k_getpwuid.c,v 1.1 1998/01/13 16:25:27 lha Exp $");
+RCSID("$Id: k_getpwuid.c,v 1.3 2000/08/16 01:23:41 lha Exp $");
 #endif /* HAVE_CONFIG_H */
 
 #include "roken.h"
@@ -52,12 +47,12 @@ k_getpwuid (uid_t uid)
      struct passwd *p;
 
      p = getpwuid (uid);
-#ifdef HAVE_GETSPUID
+#if defined(HAVE_GETSPNAM) && defined(HAVE_STRUCT_SPWD)
      if (p)
      {
 	  struct spwd *spwd;
 
-	  spwd = getspuid (uid);
+	  spwd = getspnam (p->pw_name);
 	  if (spwd)
 	       p->pw_passwd = spwd->sp_pwdp;
 	  endspent ();
