@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden). 
  * All rights reserved. 
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the 
  *    documentation and/or other materials provided with the distribution. 
  *
- * 3. All advertising materials mentioning features or use of this software 
- *    must display the following acknowledgement: 
- *      This product includes software developed by Kungliga Tekniska 
- *      Hgskolan and its contributors. 
- *
- * 4. Neither the name of the Institute nor the names of its contributors 
+ * 3. Neither the name of the Institute nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software 
  *    without specific prior written permission. 
  *
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: parse_units.c,v 1.5 1998/12/20 15:52:37 assar Exp $");
+RCSID("$Id: parse_units.c,v 1.6 2000/08/16 01:23:43 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -83,6 +78,7 @@ parse_something (const char *s, const struct units *units,
 	const struct units *u, *partial_unit;
 	size_t u_len;
 	unsigned partial;
+	int no_val_p = 0;
 
 	while(isspace((unsigned char)*p) || *p == ',')
 	    ++p;
@@ -91,6 +87,7 @@ parse_something (const char *s, const struct units *units,
 	if (val == 0 && p == next) {
 	    if(!accept_no_val_p)
 		return -1;
+	    no_val_p = 1;
 	}
 	p = next;
 	while (isspace((unsigned char)*p))
@@ -107,6 +104,8 @@ parse_something (const char *s, const struct units *units,
 	    ++p;
 	    val = -1;
 	}
+	if (no_val_p && val == 0)
+	    val = 1;
 	u_len = strcspn (p, ", \t");
 	partial = 0;
 	partial_unit = NULL;
@@ -149,9 +148,6 @@ parse_something (const char *s, const struct units *units,
 static int
 acc_units(int res, int val, unsigned mult)
 {
-    if (val == 0)
-	val = 1;
-
     return res + val * mult;
 }
 
