@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: recvmsg.c,v 1.2 1998/09/08 03:17:51 assar Exp $");
+RCSID("$Id: recvmsg.c,v 1.3 2000/08/16 01:23:44 lha Exp $");
 #endif
 
 #include "roken.h"
@@ -55,7 +50,7 @@ recvmsg(int s, struct msghdr *msg, int flags)
     for(i = 0; i < msg->msg_iovlen; ++i)
 	tot += iov[i].iov_len;
     buf = malloc(tot);
-    if (buf == NULL) {
+    if (tot != 0 && buf == NULL) {
 	errno = ENOMEM;
 	return -1;
     }
