@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: verrx.c,v 1.2 1998/12/20 15:52:38 assar Exp $");
+RCSID("$Id: verrx.c,v 1.3 2000/08/16 01:23:50 lha Exp $");
 #endif
 
 #include "err.h"
@@ -46,6 +41,6 @@ RCSID("$Id: verrx.c,v 1.2 1998/12/20 15:52:38 assar Exp $");
 void
 verrx(int eval, const char *fmt, va_list ap)
 {
-    warnerr(1, eval, 0, fmt, ap);
+    warnerr(0, fmt, ap);
     exit(eval);
 }
