@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: vwarnx.c,v 1.1 1998/01/13 16:25:48 lha Exp $");
+RCSID("$Id: vwarnx.c,v 1.2 2000/08/16 01:23:51 lha Exp $");
 #endif
 
 #include "err.h"
@@ -46,6 +41,6 @@ RCSID("$Id: vwarnx.c,v 1.1 1998/01/13 16:25:48 lha Exp $");
 void
 vwarnx(const char *fmt, va_list ap)
 {
-    warnerr(0, 0, 0, fmt, ap);
+    warnerr(0, fmt, ap);
 }
 
