@@ -35,7 +35,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: iomgr.c,v 1.21 1998/10/01 16:53:39 assar Exp $");
+RCSID("$Id: iomgr.c,v 1.24 2000/02/15 23:53:40 assar Exp $");
 #endif
 
 #include <stdio.h>
@@ -97,8 +97,6 @@ struct IoRequest {
 #define badsig(signo)		(((signo) <= 0) || ((signo) >= NSIG))
 #define mysigmask(signo)		(1 << ((signo)-1))
 
-extern int errno;
-
 static long openMask;		/* mask of open files on an IOMGR abort */
 static long sigsHandled;        /* sigmask(signo) is on if we handle signo */
 static int anySigsDelivered;		/* true if any have been delivered. */
@@ -130,7 +128,7 @@ static int iomgr_errno;
 static struct timeval iomgr_badtv;
 static PROCESS iomgr_badpid;
 
-void
+static void
 FreeRequest(struct IoRequest *req)
 {
     req->next = iorFreeList; 
@@ -243,10 +241,6 @@ static void
 IOMGR(char *dummy)
 {
     for (;;) {
- /*
-  * commented out to match use - kazar	
-  * static struct timeval Poll = { 0, 0 }; 
-  */
 	int fds;
 	int nfds;
 	fd_set readfds, writefds, exceptfds;
@@ -381,7 +375,8 @@ IOMGR(char *dummy)
 	    /*
 	     * For SGI and SVR4 - poll & select can return EAGAIN ...
 	     */
-	    if (fds < 0 && errno != EINTR && errno != EAGAIN) {
+	    if (fds < 0
+		&& errno != EINTR && errno != EAGAIN && errno != ENOMEM) {
 		iomgr_errno = errno;
 		for(fds = 0; fds < FD_SETSIZE; fds++) {
 		    if (fcntl(fds, F_GETFD, 0) < 0 && errno == EBADF) 
@@ -548,7 +543,6 @@ IOMGR_SoftSig(void (*aproc)(), char *arock)
 int
 IOMGR_Initialize(void)
 {
-    extern int TM_Init();
     PROCESS pid;
 
     /* If lready initialized, just return */
