@@ -37,7 +37,7 @@
 
 #include "lwp.h"
 
-RCSID("$Id: lwp_asm.c,v 1.16 2000/10/14 16:39:30 lha Exp $");
+RCSID("$Id: lwp_asm.c,v 1.16.2.1 2001/02/21 17:53:17 lha Exp $");
 
 #ifdef	AFS_AIX32_ENV
 #include <ulimit.h>
@@ -79,6 +79,8 @@ extern char PRE_Block;	/* from preempt.c */
 
 #ifdef __hp9000s800
 #define STACK_HEADROOM 16
+#elif defined(__s390__)
+#define STACK_HEADROOM 24
 #else
 #define STACK_HEADROOM 5		
 #endif
@@ -286,7 +288,7 @@ LWP_CreateProcess(void (*ep)(), int stacksize, int priority,
     /* Throw away all dead process control blocks */
     purge_dead_pcbs();
     if (lwp_init) {
-	temp = (PROCESS) malloc (sizeof (struct lwp_pcb));
+      temp = (PROCESS) malloc(sizeof(struct lwp_pcb));
 	if (temp == NULL) {
 	    Set_LWP_RC();
 	    return LWP_ENOMEM;
