@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,7 +34,7 @@
 #include <config.h>
 #include "roken.h"
 
-RCSID("$Id: sked.c,v 1.6 1999/07/18 19:51:12 map Exp $");
+RCSID("$Id: sked.c,v 1.26 2000/12/29 20:21:30 tol Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -51,7 +46,7 @@ RCSID("$Id: sked.c,v 1.6 1999/07/18 19:51:12 map Exp $");
 #include <assert.h>
 
 #include <sl.h>
-#include <getarg.h>
+#include <agetarg.h>
 
 #include <atypes.h>
 #include <rx/rx.h>
@@ -62,15 +57,22 @@ RCSID("$Id: sked.c,v 1.6 1999/07/18 19:51:12 map Exp $");
 #include <vstatus.h>
 #include <voldb.h>
 #include <vld.h>
+#include <mnode.h>
 #include <vldb.h>
 #include <mdir.h>
+#include <salvage.h>
+#include <mlog.h>
+#include <mdebug.h>
 
+static int interactivep = 0;
+
+#define INTER_RETURN(ret) if (interactivep) return 0; else exit (ret);
 
 static int
 create_volume (u_int32_t part, u_int32_t num, char *name)
 {
     int ret;
-    int32_t voltype = VLD_SVOL;
+    int32_t backstoretype = VLD_SVOL;
     struct dp_part *dp;
 
 
@@ -78,14 +80,13 @@ create_volume (u_int32_t part, u_int32_t num, char *name)
     if (ret)
 	return ret;
 
-    ret = vld_create_volume (dp, num, name, voltype);
+    ret = vld_create_volume (dp, num, name, backstoretype, RWVOL, 0);
     if (ret)
 	return ret;
 
     return 0;
 }
 
-
 /*
  *
  */
@@ -93,40 +94,57 @@ create_volume (u_int32_t part, u_int32_t num, char *name)
 static int
 show_volume (u_int32_t part, u_int32_t num)
 {
-    volume_handle *vh;
+    volume_handle *vh = NULL;
     int ret;
-    struct dp_part *dp;
+    struct dp_part *dp = NULL;
 
     ret = dp_create (part, &dp);
     if (ret) {
 	printf ("show_volume: dp_create: %d\n", ret);
-	return 0;
+	goto out;
     }
 
     ret = vld_open_volume_by_num (dp, num, &vh);
     if (ret) {
 	printf ("show_volume: vld_open_volume_ny_num: %d\n", ret);
-	dp_free (dp);
-	return 0;
+	goto out;
     }
     
     ret = vld_info_uptodatep (vh);
     if (ret) {
 	printf ("show_volume: vld_info_uptodatep: %d\n", ret);
-	vld_free (vh);
-	dp_free (dp);
-	return 0;
+	goto out;
     }
 
     printf ("volume-type:\t%s\n", vld_backstoretype_name(vh->type));
     printf ("------------------------\n");
     vol_pretty_print_info (stdout, &vh->info);
-    printf ("info: %d dir: %d file: %d\n", vh->sino, vh->dino, vh->fino);
+    
+    ret = vld_db_uptodate (vh);
+    if (ret) {
+	printf ("show_volume: vld_db_uptodate: %d\n", ret);
+	goto out;
+    }
 
-    vld_free (vh);
-    dp_free (dp);
+    printf ("voldb header:\n dir:\n");
+    voldb_pretty_print_header(vh->db[0]);
+    printf ("file:\n");
+    voldb_pretty_print_header(vh->db[1]);
 
-    return 0;
+    printf ("info: ");
+    vstatus_print_onode (stdout, &vh->sino);
+    printf ("dir: ");
+    vstatus_print_onode (stdout, &vh->dino);
+    printf ("file: ");
+    vstatus_print_onode (stdout, &vh->fino);
+
+ out:
+    if (vh)
+	vld_free (vh);
+    if (dp)
+	dp_free (dp);
+
+    INTER_RETURN(ret ? 1 : 0);
 }
 
 /*
@@ -156,7 +174,7 @@ showvols_cb (void *data, int fd)
     if (vh->vol != vh->info.volid)
 	printf ("PANIC: vh->vol != vh->info.volid\n");
     
-    printf ("%-32s %-4s %-4s %-8d %-8d %-8d %-8d\n",
+    printf ("%-32s %-4s %-4s %-8u %-8u %-8u %-8u\n",
 	    vh->info.name, vld_backstoretype_name(vh->type), 
 	    vol_voltype2name(vh->info.type), vh->vol,
 	    vh->info.backupID, vh->info.parentID, 
@@ -169,7 +187,7 @@ showvols_cb (void *data, int fd)
  * List volumes on partition `part'.
  */
 
-int
+static int
 show_vols (u_int32_t part)
 {
     struct dp_part *p;
@@ -201,23 +219,29 @@ volcreate_cmd (int argc, char **argv)
     u_int32_t num, part;
     int ret;
 
-    if (argc != 4) {
-	printf ("usage: volcreate part num name\n");
-	return 0;
+    if (argc < 4 || argc > 5) {
+	printf ("usage: volcreate part num name [partdir]\n");
+	INTER_RETURN(1);
     }
 
-    part = atoi (argv[1]);
-    
+    dpart_root = argv[4];
+
+    ret = dp_parse (argv[1], &part);
+    if (ret) {
+	printf ("volcreate: `%s' is a invalid argument for partition\n", argv[1]);
+	INTER_RETURN(1);
+    }
+	
     num = atoi (argv[2]);
     if (num == 0) {
 	printf ("volcreate: `%s' is a invalid argument for volnum\n", argv[2]);
-	return 0;
+	INTER_RETURN(1);
     }
 
     ret = create_volume (part, num, argv[3]);
     if (ret) {
 	printf ("volcreate: create_volume returned: %d\n", ret);
-	return 0;
+	INTER_RETURN(ret);
     }
 
     printf ("volume %u created successfully\n", num);
@@ -235,9 +259,9 @@ volshow_cmd (int argc, char **argv)
     u_int32_t num, part;
     int ret;
 
-    if (argc != 3) {
-	printf ("usage: volshow part num\n");
-	return 0;
+    if (argc < 3 || argc > 4) {
+	printf ("usage: volshow part num [partdir]\n");
+	INTER_RETURN(1);
     }
 
     part = atoi (argv[1]);
@@ -245,13 +269,15 @@ volshow_cmd (int argc, char **argv)
     num = atoi (argv[2]);
     if (num == 0) {
 	printf ("volcreate: `%s' is a invalid argument for volnum\n", argv[2]);
-	return 0;
+	INTER_RETURN(1);
     }
 
+    dpart_root = argv[4];
+
     ret = show_volume (part, num);
     if (ret) {
 	printf ("volshow: show_volume returned: %d\n", ret);
-	return 0;
+	INTER_RETURN(ret);
     }
 
     return 0;
@@ -269,7 +295,7 @@ vollist_cmd (int argc, char **argv)
 
     if (argc != 2) {
 	printf ("usage: vollist part\n");
-	return 0;
+	INTER_RETURN(1);
     }
 
     part = atoi (argv[1]);
@@ -277,7 +303,7 @@ vollist_cmd (int argc, char **argv)
     ret = show_vols (part);
     if (ret) {
 	printf ("vollist: show_vols returned: %d\n", ret);
-	return 0;
+	INTER_RETURN(ret);
     }
 
     return 0;
@@ -306,34 +332,38 @@ volls_cmd (int argc, char **argv)
     struct dp_part *dp;
     int ret;
     
-    if (argc != 4) {
-	printf ("volls part volume# vnode#\n");
-	return 0;
+    if (argc < 4 || argc > 5) {
+	printf ("volls part volume# vnode# [partdir]\n");
+	INTER_RETURN(1);
     }
 
     part = atoi(argv[1]);
     vol = atoi(argv[2]);
     if (vol == 0) {
 	printf ("erronous volume number\n");
-	return 0;
+	INTER_RETURN(1);
     }
 
     vnode = atoi(argv[3]);
     if (vnode == 0) {
 	printf ("erronous vnode number\n");
-	return 0;
+	INTER_RETURN(1);
     }
 
+    dpart_root = argv[4];
+
     ret = dp_create (part, &dp);
     if (ret) {
 	printf ("volls: dp_create: %d\n", ret);
-	return 0;
+	INTER_RETURN(1);
     }
 
     {
 	struct volume_handle *volh;
-	int fd;
-	AFSFetchStatus fs;
+	struct mnode n;
+	struct msec m;
+
+	memset (&n, 0, sizeof(n));
 
 	ret = vld_open_volume_by_num (dp, vol, &volh);
 	if (ret)
@@ -343,22 +373,31 @@ volls_cmd (int argc, char **argv)
 	if (ret)
 	    errx (1, "volls: vld_db_uptodate: %d", ret);
 
-	ret = vld_open_vnode (volh, vnode, &fs, O_RDONLY, &fd, NULL);
+#if 1
+	n.fid.Vnode = vnode;
+	m.flags = VOLOP_GETSTATUS;
+
+	ret = vld_open_vnode (volh, &n, &m);
 	if (ret)
 	    errx (1, "volls: vld_open_vnode failed with %d\n", ret);
+#else
+	abort();
+#endif
 
-
-	if (vnode & 1) {
+	printf ("mode=%o ino=%x dev=%x rdev=%x\n",
+		n.sb.st_mode, n.sb.st_ino, n.sb.st_dev, n.sb.st_rdev);
+	
+	printf ("size: %d\nparent fid: %d.%d\n",
+		n.fs.Length, n.fs.ParentVnode, n.fs.ParentUnique);
+	
+	if (afs_dir_p (vnode)) {
 	    VenusFid fid;
 	    fid.Cell = 0;
 	    fid.fid.Volume = volh->vol;
 	    fid.fid.Vnode = vnode;
 	    fid.fid.Unique = 0;
 
-	    printf ("size: %d\nparent fid: %d.%d\n",
-		    fs.Length, fs.ParentVnode, fs.ParentUnique);
-
-	    ret = mdir_readdir (fd, volls_dir_cb, NULL, &fid);
+	    ret = mdir_readdir (&n, volls_dir_cb, NULL, &fid);
 	    if (ret)
 		errx (1, "volls: mdir_readdir failed with %d", ret);
 
@@ -366,7 +405,7 @@ volls_cmd (int argc, char **argv)
 	    printf ("this is a file.... XXX\n");
 	}
 
-	close (fd);
+	close (n.fd);
 
 	vld_free(volh);
 
@@ -378,7 +417,6 @@ volls_cmd (int argc, char **argv)
  *
  */
 
-
 static int
 volvnode_cmd (int argc, char **argv)
 {
@@ -390,14 +428,14 @@ volvnode_cmd (int argc, char **argv)
     int do_list = 0;
     int ret, optind = 0;
 
-    struct getargs args[] = {
-	{"part",	0, arg_string, NULL,
-	 "what part to use", NULL, arg_mandatory},
-	{"vol",	        0, arg_string, NULL,
-	 "what part to use", NULL, arg_mandatory},
-	{"list",      'l', arg_flag, NULL,
+    struct agetargs args[] = {
+	{"part",	0, aarg_string, NULL,
+	 "what part to use", NULL, aarg_mandatory},
+	{"vol",	        0, aarg_string, NULL,
+	 "what vol to use", NULL, aarg_mandatory},
+	{"list",      'l', aarg_flag, NULL,
 	 "list vnodes" },
-        { NULL, 0, arg_end, NULL }
+        { NULL, 0, aarg_end, NULL }
     }, *arg;
 
     arg = args;
@@ -405,27 +443,27 @@ volvnode_cmd (int argc, char **argv)
     arg->value = &vol_str;     arg++;
     arg->value = &do_list;   arg++;
 
-    if (getarg (args, argc, argv, &optind, ARG_AFSSTYLE)) {
-	arg_printusage(args, "volvnode", NULL, ARG_AFSSTYLE);
-	return 0;
+    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
+	aarg_printusage(args, "volvnode", NULL, AARG_AFSSTYLE);
+	INTER_RETURN(1);
     }
     part = atoi(part_str);
     vol = atoi(vol_str);
     if (vol == 0) {
 	printf ("erronous volume number\n");
-	return 0;
+	INTER_RETURN(1);
     }
 
     ret = dp_create (part, &dp);
     if (ret) {
 	printf ("volls: dp_create: %d\n", ret);
-	return 0;
+	INTER_RETURN(1);
     }
 
     {
 	struct volume_handle *volh;
 	int i;
-	u_int32_t num, freeptr, flags;
+	u_int32_t num, flags;
 
 
 	ret = vld_open_volume_by_num (dp, vol, &volh);
@@ -454,25 +492,25 @@ volvnode_cmd (int argc, char **argv)
 	 * Print dir nodes
 	 */
 
-	ret = voldb_header_info (volh->db[0], &num, &freeptr, &flags);
+	ret = voldb_header_info (volh->db[0], &num, &flags);
 	if (ret)
 	    errx (1, "voldb_header_info: %d", ret);
 
 	printf ("---------------------------------\n");
 	printf ("dir nodes contain:\n");
-	printf (" num: %d\tfreeptr: %d flags 0x%x\n", num, freeptr, flags);
+	printf (" num: %d\tflags 0x%x\n", num, flags);
 	printf ("---------------------------------\n");
 
 	if (do_list) {
 	    for (i = 0; i < num; i++) {
-		struct voldb_dir_entry e;
+		struct voldb_entry e;
 		
 		printf ("dnode #%d\n", i);
 		
-		ret = voldb_get_dir (volh->db[0], i, &e);
+		ret = voldb_get_entry (volh->db[0], i, &e);
 		if (ret)
 		    errx (1, "voldb_get_dir (%d) returned: %d", i, ret);
-		voldb_pretty_print_dir (&e);
+		voldb_pretty_print_dir (&e.u.dir);
 	    }
 	}	    
 
@@ -480,25 +518,25 @@ volvnode_cmd (int argc, char **argv)
 	 * Print file nodes
 	 */
 
-	ret = voldb_header_info (volh->db[1], &num, &freeptr, &flags);
+	ret = voldb_header_info (volh->db[1], &num, &flags);
 	if (ret)
 	    errx (1, "voldb_header_info: %d", ret);
 
 	printf ("---------------------------------\n");
 	printf ("file nodes contain:\n");
-	printf (" num: %d\tfreeptr: %d flags 0x%x\n", num, freeptr, flags);
+	printf (" num: %d\tflags 0x%x\n", num, flags);
 	printf ("---------------------------------\n");
 
 	if (do_list) {
 	    for (i = 0; i < num; i++) {
-		struct voldb_file_entry e;
+		struct voldb_entry e;
 		
 		printf ("fnode #%d\n", i);
 		
-		ret = voldb_get_file (volh->db[1], i, &e);
+		ret = voldb_get_entry (volh->db[1], i, &e);
 		if (ret)
 		    errx (1, "voldb_get_dir (%d) returned: %d", i, ret);
-		voldb_pretty_print_file (&e);
+		voldb_pretty_print_file (&e.u.file);
 	    }
 	}
 
@@ -508,6 +546,61 @@ volvnode_cmd (int argc, char **argv)
     return 0;
 }
 
+/*
+ *
+ */
+
+static int
+fvolcreate_cmd (int argc, char **argv)
+{
+    u_int32_t part;
+    char *part_str = NULL;
+    char *path_str = NULL;
+    int vol_int = 0;
+    struct dp_part *dp;
+    int ret, optind = 0;
+
+    struct agetargs args[] = {
+	{"part",	0, aarg_string, NULL,
+	 "what part to use", NULL, aarg_mandatory},
+	{"vol",	        0, aarg_integer, NULL,
+	 "what vol-number to use", NULL, aarg_mandatory},
+	{"path",        0, aarg_string, NULL,
+	 "what path to volume-ify", NULL, aarg_mandatory},
+        { NULL, 0, aarg_end, NULL }
+    }, *arg;
+
+    arg = args;
+    arg->value = &part_str;   arg++;
+    arg->value = &vol_int;    arg++;
+    arg->value = &path_str;   arg++;
+
+    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
+	aarg_printusage(args, "volvnode", NULL, AARG_AFSSTYLE);
+	INTER_RETURN(1);
+    }
+
+    part = atoi(part_str);
+    if (vol_int == 0) {
+	printf ("erronous volume number\n");
+	INTER_RETURN(1);
+    }
+
+    ret = dp_create (part, &dp);
+    if (ret) {
+	printf ("volls: dp_create: %d\n", ret);
+	INTER_RETURN(1);
+    }
+
+    ret = vld_fvol_create_volume_ondisk (dp, vol_int, path_str);
+    if (ret) {
+	printf ("volls: fvol_create_volume_ondisk: %d\n", ret);
+	INTER_RETURN(1);
+    }
+
+    return 0;
+}
+
 /*
  *
  */
@@ -529,8 +622,10 @@ showvolname_cmd (int argc, char **argv)
 	    errx (1, "showvolname: `%s' is a invalid argument", argv[1]);
 	
 	ret = vol_getname (num, name, sizeof (name));
-	if (ret)
-	    errx (1, "vol_getname returned %d", ret);
+	if (ret) {
+	    printf ("vol_getname returned %d", ret);
+	    INTER_RETURN(ret);
+	}
 	printf ("volume name is: %s\n", name);
     } else {
 	part = atoi (argv[1]); /* XXX 0 is valid volume */
@@ -552,7 +647,68 @@ showvolname_cmd (int argc, char **argv)
  *
  */
 
-int
+static int
+salvage_cmd (int argc, char **argv)
+{
+    char *part_str = NULL;
+    int vol = 0;
+    struct dp_part *dp;
+    int ret, optind = 0;
+	struct volume_handle *volh;
+
+    struct agetargs args[] = {
+	{"part",	0, aarg_string, NULL,
+	 "what part to use", NULL, aarg_mandatory},
+	{"vol",	        0, aarg_integer, NULL,
+	 "what vol-number to use", NULL, aarg_mandatory},
+	{"partdir",	0, aarg_string, NULL,
+	 "where to find vicep*", NULL, aarg_optional},
+        { NULL, 0, aarg_end, NULL }
+    }, *arg;
+
+    arg = args;
+    arg->value = &part_str;   arg++;
+    arg->value = &vol;    arg++;
+    arg->value = &dpart_root;    arg++;
+
+    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
+	aarg_printusage(args, "volvnode", NULL, AARG_AFSSTYLE);
+	INTER_RETURN(1);
+    }
+
+    if (part_str == NULL || vol == 0) {
+	warnx ("missing argument");
+	INTER_RETURN(1);
+    }
+    
+    dp = dp_getpart (part_str);
+    if (dp == NULL) {
+	warnx ("erronous volume number");
+	INTER_RETURN(1);
+    }
+
+    ret = vld_open_volume_by_num (dp, vol, &volh);
+    if (ret) {
+	warnx ("salvage: vld_open_volume_by_num: %d", ret);
+	INTER_RETURN(1);
+    }
+
+    ret = salvage_volume (volh);
+    if (ret)
+	warnx ("salvage returned %d", ret);
+    else
+	printf ("volume %s (%u) is ok\n", volh->info.name, vol);
+
+    vld_free(volh);
+
+    return 0;
+}
+
+/*
+ *
+ */
+
+static int
 version_cmd (int argc, char **argv)
 {
     printf ("sked - maintaing volumes the hard way\n");
@@ -578,7 +734,9 @@ static SL_cmd cmds[] = {
     {"vollist",        vollist_cmd,	"list volumes on part"},
     {"volls",          volls_cmd,	"list files in volume,vnode pair"},
     {"volvnode",       volvnode_cmd,    "list all vnodes"},
+    {"fvolcreate",     fvolcreate_cmd,  "create a fvolume"},
     {"name",           showvolname_cmd, "name of a volume"},
+    {"salvage",        salvage_cmd,	"salvage a volume"},
     {"version",        version_cmd,     "show version of sked"},
     { NULL, NULL, NULL}
 };
@@ -616,17 +774,29 @@ int
 main (int argc, char **argv)
 {
     int ret;
+    Log_method *method;
+    char *log_file = "/dev/stderr";
+    
+    set_progname (argv[0]);
+
+    method = log_open (get_progname(), log_file);
+    if (method == NULL)
+	errx (1, "log_open failed");
     
    /*
     * We only boot, not init since we dont want to read in all volumes
     */
-    vld_boot();
+    vld_boot ();
+    mnode_init (173);
+    
+    mlog_loginit (method, milko_deb_units, MDEFAULT_LOG /* MDEBALL */);
 
     /*
      * Command loop or if command, eval.
      */
 
     if (argc < 2) {
+	interactivep = 1;
 	sl_loop (cmds, "sked (cmd): ");
     } else {
 	ret = sl_command(cmds, argc - 1, argv + 1);
