@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -37,7 +32,7 @@
  */
 
 /*
- * $Id: fsrv_locl.h,v 1.3 1999/07/12 08:56:13 map Exp $
+ * $Id: fsrv_locl.h,v 1.11 2000/10/03 00:17:31 lha Exp $
  */
 
 
@@ -68,12 +63,15 @@
 #include <service.h>
 #include <part.h>
 
+#include <agetarg.h>
+
 #include <fs.ss.h>
 #include <volumeserver.ss.h>
 
 #include "fs_def.h"
 
 #include <pts.h>
+#include "connsec.h"
 
 #include <dpart.h>
 #include <voldb.h>
@@ -81,5 +79,13 @@
 
 #include <fbuf.h>
 #include <fdir.h>
+#include <mdir.h>
+
+#include <ropa.h>
+
+#include <mlog.h>
+#include <mdebug.h>
+
+#include <salvage.h>
 
 #endif /* __FILBUNKE_FSRV_H */
