@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,40 @@
 
 #include "fsrv_locl.h"
 
-RCSID("$Id: volprocs.c,v 1.1 1999/07/12 08:56:14 map Exp $");
+RCSID("$Id: volprocs.c,v 1.9 2000/10/03 00:17:36 lha Exp $");
+
+/*
+ * Helper function
+ */
+
+static int
+volser_fetch_vh (int32_t partition, int32_t volid, 
+		 struct dp_part **dp, volume_handle **vh)
+{
+    int ret;
+
+    ret = dp_create (partition, dp);
+    if (ret) {
+	fprintf (stderr, "volser_fetch_vh: dp_create: %d\n", ret);
+	return VOLSERFAILEDOP; /* XXX */
+    }
+
+    ret = vld_open_volume_by_num (*dp, volid, vh);
+    if (ret) {
+	fprintf (stderr, "volser_fetch_vh: vld_open_volume_by_num: %d\n", ret);
+	dp_free (*dp);
+	return VOLSERFAILEDOP; /* XXX */
+    }
+    
+    ret = vld_info_uptodatep (*vh);
+    if (ret) {
+	printf ("volser_fetch_vh: vld_info_uptodatep: %d\n", ret);
+	vld_free (*vh);
+	dp_free (*dp);
+	return VOLSERFAILEDOP; /* XXX */
+    }
+    return 0;
+}
 
 /*
  *
@@ -53,7 +81,38 @@ VOLSER_AFSVolCreateVolume(struct rx_call *call,
 			  int32_t *volid,
 			  int32_t *trans)
 {
-    return VOLSERFAILEDOP;
+    int ret;
+    int32_t backstoretype = VLD_SVOL;
+    struct dp_part *dp;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolCreateVolume "
+	     "part %d name %s type %d parent %d volid %u",
+	     partition, name, type, parent, *volid);
+
+    /* XXX parent should be used */
+
+    ret = vld_create_trans (partition, *volid, trans);
+
+    if (ret)
+	return ret;
+
+    ret = vld_trans_set_iflags (*trans, ITOffline);
+    if (ret)
+	return ret;
+
+    ret = dp_create (partition, &dp);
+    if (ret) {
+	vld_end_trans (*trans, NULL);
+	return ret;
+    }
+
+    ret = vld_create_volume (dp, *volid, name, backstoretype, type, 0);
+    if (ret) {
+	vld_end_trans (*trans, NULL);
+	return ret;
+    }
+
+    return 0;
 }
 
 /*
@@ -64,6 +123,12 @@ int
 VOLSER_AFSVolDeleteVolume(struct rx_call *call,
 			  const int32_t trans)
 {
+    int ret;
+
+    ret = vld_verify_trans(trans);
+    if (ret)
+	return ret;
+
     return VOLSERFAILEDOP;
 }
 
@@ -88,6 +153,37 @@ VOLSER_AFSVolDump(struct rx_call *call,
 		  const int32_t fromTrans,
 		  const int32_t fromDate)
 {
+    int ret;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log (MDEBVOLDB, 
+	      "VOLSER_AFSVolDump trans %d fromdate %d", 
+	      fromTrans, fromDate);
+    
+    ret = vld_verify_trans (fromTrans);
+    if (ret)
+	return ret;
+
+    ret = vld_get_trans (fromTrans, &trans);
+    if (ret)
+	return ret;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	return ret;
+    }
+
+    if (fromDate != 0) {
+	vld_put_trans(trans);
+	return VOLSERFAILEDOP;
+    }
+
+
+
+    vld_put_trans (trans);
     return VOLSERFAILEDOP;
 }
 
@@ -183,7 +279,19 @@ VOLSER_AFSVolTransCreate(struct rx_call *call,
 			 const int32_t flags,
 			 int32_t *trans)
 {
-    return VOLSERFAILEDOP;
+    int ret;
+
+    ret = vld_create_trans(partition, volume, trans);
+    if (ret)
+	return ret;
+
+    ret = vld_trans_set_iflags(*trans, flags);
+    if (ret) {
+	vld_end_trans (*trans, NULL);
+	return ret;
+    }
+
+    return 0;
 }
 
 /*
@@ -195,7 +303,17 @@ VOLSER_AFSVolEndTrans(struct rx_call *call,
 		      const int32_t trans,
 		      int32_t *rcode)
 {
-    return VOLSERFAILEDOP;
+    int ret;
+
+    ret = vld_verify_trans(trans);
+    if (ret)
+	return ret;
+
+    ret = vld_end_trans(trans, rcode);
+    if (ret)
+	return ret;
+
+    return 0;
 }
 
 /*
@@ -207,7 +325,17 @@ VOLSER_AFSVolGetFlags(struct rx_call *call,
 		      const int32_t trans,
 		      int32_t *flags)
 {
-    return VOLSERFAILEDOP;
+    int ret;
+
+    ret = vld_verify_trans(trans);
+    if (ret)
+	return ret;
+
+    ret = vld_trans_get_vflags(trans, flags);
+    if (ret)
+	return ret;
+
+    return 0;
 }
 
 /*
@@ -216,10 +344,50 @@ VOLSER_AFSVolGetFlags(struct rx_call *call,
 
 int
 VOLSER_AFSVolSetFlags(struct rx_call *call,
-		      const int32_t trans,
+		      const int32_t transid,
 		      const int32_t flags)
 {
-    return VOLSERFAILEDOP;
+    int ret;
+    struct trans *trans;
+    struct dp_part *dp;
+    volume_handle *vh;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetFlags "
+	     "trans %d flags %d", transid, flags);
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	return ret;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	return ret;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	return ret;
+    }
+
+    if (flags & VTDeleteOnSalvage)
+	vh->info.destroyMe = 's';
+    else
+	vh->info.destroyMe = 0;
+
+    assert ((flags & VTOutOfService) == 0); /* XXX */
+
+    vld_info_write(vh);
+
+    vld_free (vh);
+    dp_free (dp);
+
+    vld_put_trans(trans);
+
+    ret = vld_trans_set_vflags(transid, flags);
+    if (ret)
+	return ret;
+
+    return 0;
 }
 
 /*
@@ -325,7 +493,7 @@ VOLSER_AFSVolPartitionInfo(struct rx_call *call,
 	return ret;
 
     memset(partition, 0, sizeof(*partition));
-    strncpy(partition->name, dp->part, 32);    
+    strlcpy(partition->name, dp->part, 32);    
     
     dp_free(dp);
     
@@ -355,12 +523,51 @@ VOLSER_AFSVolListOneVolume(struct rx_call *call,
 			   const int32_t volid,
 			   volEntries *resultEntries)
 {
+    volume_handle *vh;
+    int ret;
+    struct dp_part *dp;
+    struct volintInfo *volinfo;
+
     fprintf(stderr, 
-	    "VOLSER_AFSVolListOneVolume partid: %d volid: %d\n",
+	    "VOLSER_AFSVolListOneVolume partid: %d volid: %u\n",
 	    partID, volid);
+
+    ret = volser_fetch_vh (partID, volid, &dp, &vh);
+    if (ret)
+	return ret;
+
+    resultEntries->len = 1;
     
+    volinfo = calloc(sizeof(struct volintInfo) * resultEntries->len, 1);
+    resultEntries->val = volinfo;
+    strlcpy(volinfo->name, vh->info.name, VNAMESIZE);
+    volinfo->volid = vh->info.volid;
+    volinfo->type = vh->info.type;
+    volinfo->backupID = vh->info.backupID;
+    volinfo->parentID = vh->info.parentID;
+    volinfo->cloneID = vh->info.cloneID;
+    if (vld_check_busy(volid, partID))
+	volinfo->status = VBUSY;
+    else
+	volinfo->status = VOK;
+    volinfo->copyDate = vh->info.copyDate;
+    volinfo->inUse = vh->info.inUse;
+    volinfo->creationDate = vh->info.creationDate;
+    volinfo->accessDate = vh->info.accessDate;
+    volinfo->updateDate = vh->info.updateDate;
+    volinfo->backupDate = vh->info.backupDate;
+    volinfo->dayUse = vh->info.dayUse;
+    volinfo->filecount = vh->info.filecount;
+    volinfo->maxquota = vh->info.maxquota;
+    volinfo->size = vh->info.size;
+    volinfo->needsSalvaged = 0;
+    volinfo->destroyMe = 0;
+
+    vld_free (vh);
+    dp_free (dp);
+
+
     return 0;
-    return VOLSERFAILEDOP;
 }
 
 /*
@@ -416,48 +623,27 @@ VOLSER_AFSVolXListOneVolume(struct rx_call *call,
     struct xvolintInfo *volinfo;
 
     fprintf(stderr, 
-	    "VOLSER_AFSVolXListOneVolume partid: %d volid: %d\n",
+	    "VOLSER_AFSVolXListOneVolume partid: %d volid: %u\n",
 	    partID, volid);
 
-    ret = dp_create (partID, &dp);
-    if (ret) {
-	fprintf (stderr, "XListOneVolume: dp_create: %d\n", ret);
-	return 0; /* XXX */
-    }
-
-    ret = vld_open_volume_by_num (dp, volid, &vh);
-    if (ret) {
-	fprintf (stderr, "XListOneVolume: vld_open_volume_ny_num: %d\n", ret);
-	dp_free (dp);
-	return 0; /* XXX */
-    }
-    
-    ret = vld_info_uptodatep (vh);
-    if (ret) {
-	printf ("XListOneVolume: vld_info_uptodatep: %d\n", ret);
-	vld_free (vh);
-	dp_free (dp);
-	return 0; /* XXX */
-    }
-
-#if 0
-    printf ("volume-type:\t%s\n", vld_voltype_name(vh->type));
-    printf ("------------------------\n");
-    vol_pretty_print_info (stdout, &vh->info);
-    printf ("info: %d dir: %d file: %d\n", vh->sino, vh->dino, vh->fino);
-#endif
+    ret = volser_fetch_vh (partID, volid, &dp, &vh);
+    if (ret)
+	return ret;
 
     resultEntries->len = 1;
     
     volinfo = calloc(sizeof(struct xvolintInfo) * resultEntries->len, 1);
     resultEntries->val = volinfo;
-    strncpy(volinfo->name, vh->info.name, VNAMESIZE);
+    strlcpy(volinfo->name, vh->info.name, VNAMESIZE);
     volinfo->volid = vh->info.volid;
     volinfo->type = vh->info.type;
     volinfo->backupID = vh->info.backupID;
     volinfo->parentID = vh->info.parentID;
     volinfo->cloneID = vh->info.cloneID;
-    volinfo->status = vh->info.status;
+    if (vld_check_busy(volid, partID))
+	volinfo->status = VBUSY;
+    else
+	volinfo->status = VOK;
     volinfo->copyDate = vh->info.copyDate;
     volinfo->inUse = vh->info.inUse;
     volinfo->creationDate = vh->info.creationDate;
@@ -485,7 +671,69 @@ VOLSER_AFSVolSetInfo(struct rx_call *call,
 		     const int32_t transid,
 		     const struct volintInfo *volinfo)
 {
-    return VOLSERFAILEDOP;
+    volume_handle *vh;
+    int ret;
+    struct dp_part *dp;
+    struct trans *trans;
+
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolSetInfo "
+	     "trans %d name %s type %d parent %d volid %d backup %d",
+	     transid, volinfo->name, volinfo->type, volinfo->parentID,
+	     volinfo->volid, volinfo->backupID);
+
+    ret = vld_verify_trans(transid);
+    if (ret)
+	return ret;
+
+    ret = vld_get_trans(transid, &trans);
+    if (ret)
+	return ret;
+    
+    ret = volser_fetch_vh (trans->partition, trans->volid, &dp, &vh);
+    if (ret) {
+	vld_put_trans(trans);
+	return ret;
+    }
+
+    if (volinfo->name[0])
+	strlcpy(vh->info.name, volinfo->name, VNAMESIZE);
+
+    if (volinfo->volid)
+	vh->info.volid = volinfo->volid;
+    if (volinfo->type)
+	vh->info.type = volinfo->type;
+    if (volinfo->backupID)
+	vh->info.backupID = volinfo->backupID;
+    if (volinfo->parentID)
+	vh->info.parentID = volinfo->parentID;
+    if (volinfo->cloneID)
+	vh->info.cloneID = volinfo->cloneID;
+    if (volinfo->status)
+	vh->info.status = volinfo->status;
+    if (volinfo->copyDate)
+	vh->info.copyDate = volinfo->copyDate;
+    if (volinfo->inUse)
+	vh->info.inUse = volinfo->inUse;
+    if (volinfo->creationDate)
+	vh->info.creationDate = volinfo->creationDate;
+    if (volinfo->backupDate)
+	vh->info.backupDate = volinfo->backupDate;
+    if (volinfo->dayUse != -1)
+	vh->info.dayUse = volinfo->dayUse;
+    if (volinfo->filecount)
+	vh->info.filecount = volinfo->filecount;
+    if (volinfo->maxquota != -1)
+	vh->info.maxquota = volinfo->maxquota;
+    if (volinfo->size)
+	vh->info.size = volinfo->size;
+
+    vld_info_write(vh);
+
+    vld_free (vh);
+    dp_free (dp);
+    vld_put_trans(trans);
+
+    return 0;
 }
 
 /*
@@ -501,6 +749,8 @@ VOLSER_AFSVolXListPartitions(struct rx_call *call,
     int ret;
     int32_t partIDs[26]; /* XXX */
 
+    mlog_log(MDEBVOLDB, "VOLSER_AFSVolXListPartitions");
+
     i = 0;
     do {
 	ret = dp_find(&dp);
