@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: dpart.h,v 1.1 1999/05/17 14:20:45 lha Exp $ */
+/* $Id: dpart.h,v 1.6 2000/12/29 20:12:36 tol Exp $ */
 
 #ifndef __FILBULKE_DPART_H
 #define __FILBULKE_DPART_H 1
@@ -47,7 +42,10 @@ struct dp_part {
     int ref;
 };
 
+extern char *dpart_root;
+
 #define DP_NUMBER(dp) ((dp)->num)
+#define DP_NAME(dp)   ((dp)->part)
 
 int
 dp_create (u_int32_t num, struct dp_part **dp);
@@ -64,4 +62,10 @@ dp_find (struct dp_part **dp);
 int
 dp_findvol (struct dp_part *d, void (*cb)(void *, int), void *data);
 
+int
+dp_parse (const char *str, u_int32_t *num);
+
+struct dp_part *
+dp_getpart (const char *str);
+
 #endif /* __FILBULKE_DPART_H */
