@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,10 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: msecurity.h,v 1.2 1999/05/18 10:51:04 lha Exp $ */
+/* $Id: msecurity.h,v 1.5 2000/10/03 00:18:14 lha Exp $ */
+
+void
+sec_disable_superuser_check (void);
 
 int
 sec_is_superuser(struct rx_call *call);
@@ -48,4 +46,5 @@ int
 sec_del_superuser(char *user);
 
 int
-sec_getname(struct rx_call *call, char *name, char *instance, char *realm);
+sec_getname(struct rx_connection *conn,
+	    char *name, char *instance, char *realm);
