@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -53,12 +48,13 @@
 #include <string.h>
 #include <assert.h>
 #include <err.h>
+#include <netinit.h>
 
 #ifndef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-RCSID("$Id: netinit.c,v 1.5 1999/07/12 08:19:29 map Exp $");
+RCSID("$Id: netinit.c,v 1.14 2000/10/03 00:18:19 lha Exp $");
 
 /*
  * Network functions
@@ -98,14 +94,60 @@ server_get_key(void *appl, int kvno, des_cblock *key)
 	ret = read_service_key("afs", cell_name, realm_name, 
 			       0, srvtab_filename,
 			       (char *)&afskey);
-	if (ret)
-	    return ret;
+	if (ret) {
+	    /*
+	     * Try also afs.realm@REALM
+	     */
+
+	    strlcpy(cell_name, realm_name, sizeof(realm_name));
+	    strlwr (cell_name);
+	    ret = read_service_key("afs", cell_name, realm_name, 
+				   0, srvtab_filename,
+				   (char *)&afskey);
+	    if (ret)
+		return ret;
+	}
 	read_key = 1;
     }
 
     memcpy (key, &afskey, sizeof (afskey));
     return 0;
 }
+
+struct rx_securityClass *
+netinit_client_getcred(void)
+{
+    int k_errno;
+    CREDENTIALS c;
+    struct rx_securityClass *sec;
+    struct timeval tv;
+
+    gettimeofday (&tv, NULL);
+
+    k_errno = krb_get_cred("afs", cell_name, realm_name, &c);
+    if (krb_life_to_time (c.issue_date, c.lifetime) < tv.tv_sec - 100) {
+	dest_tkt();
+	k_errno = 4711;
+    }
+    if(k_errno != KSUCCESS) {
+	k_errno = krb_get_svc_in_tkt("afs", cell_name, realm_name,
+				     "afs", cell_name, 0xFF /* XXX */,
+				     srvtab_filename);
+	if (k_errno == KSUCCESS)
+	    k_errno = krb_get_cred("afs", cell_name, realm_name, &c);
+    }
+
+    if (k_errno != KSUCCESS)
+	return NULL;
+
+    sec = rxkad_NewClientSecurityObject(rxkad_auth,
+					&c.session,
+					c.kvno,
+					c.ticket_st.length,
+					c.ticket_st.dat);
+
+    return sec;
+}
 #endif
 
 #define N_SECURITY_OBJECTS 3
@@ -114,13 +156,14 @@ int
 network_init (int serviceport, 
 	      char *servicename, 
 	      int serviceid,
-	      int (*request) (struct rx_call *),
+	      int32_t (*request) (struct rx_call *),
 	      struct rx_service **service,
 	      const char *realm)
 {
     struct rx_service *serv;
-    int ret, maxsec = 1;
+    int maxsec = 1;
     struct rx_securityClass **secObjs;
+    static char krbtkfile[MaxPathLen];
 
     secObjs = malloc(sizeof(*secObjs) * N_SECURITY_OBJECTS);
     if (secObjs == NULL)
@@ -133,14 +176,20 @@ network_init (int serviceport,
     if (secObjs[0] == NULL ) 
 	errx(1, "cant create security object") ;
 
-    strncpy (cell_name, cell_getthiscell(), sizeof (cell_name));
-    cell_name[sizeof(cell_name)-1] = '\0';
-
 #ifdef KERBEROS
+    strlcpy (cell_name, cell_getthiscell(), sizeof (cell_name));
+
+    if (krbtkfile[0] == '\0') { 
+	snprintf (krbtkfile, sizeof(krbtkfile),
+		  "%sfileserver_%d", TKT_ROOT, (unsigned int)(getpid()*time(0)));
+	krb_set_tkt_string (krbtkfile);
+    }
+
     if (realm) {
-	strncpy(realm_name, realm, sizeof(realm_name));
-	realm_name[sizeof(realm_name)-1] = '\0';
-    } else {
+	strlcpy(realm_name, realm, sizeof(realm_name));
+    } else if (*realm_name == '\0') {
+	int ret;
+
 	ret = krb_get_lrealm(realm_name, 1);
 	if (ret)
 	    return ret;
@@ -157,16 +206,16 @@ network_init (int serviceport,
     maxsec = 3;
     secObjs[1] = NULL;
     secObjs[2] = rxkad_NewServerSecurityObject(rxkad_clear,
-						       NULL,
-						       server_get_key,
-						       NULL);
+					       NULL,
+					       server_get_key,
+					       NULL);
     if (secObjs[2] == NULL)
 	errx(1, "init_network: can't init rxkad server-security-object");
 #endif
 
-    serv = rx_NewService(serviceport, serviceid, servicename, 
-			 secObjs,  maxsec, 
-			 request);
+    serv = rx_NewService (serviceport, serviceid, servicename, 
+			  secObjs,  maxsec, 
+			  request);
     
     if (serv == NULL) 
 	errx(1, "Cant create %s service", servicename);
@@ -176,3 +225,8 @@ network_init (int serviceport,
     
     return 0;
 }
+
+char *
+netinit_getrealm(void) {
+    return realm_name;
+}
