@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -42,7 +37,7 @@
 
 #include "voldb_locl.h"
 
-RCSID("$Id: vol.c,v 1.1 1999/05/17 14:20:54 lha Exp $");
+RCSID("$Id: vol.c,v 1.6 2000/12/29 20:16:01 tol Exp $");
 
 /*
  * get the partial name of the volume
@@ -76,10 +71,11 @@ vol_getfullname (u_int32_t part, u_int32_t num, char *str, size_t sz)
 	return ret;
 
     if (part <= 'z' - 'a')
-	ret = snprintf (str, sz, "/vicep%c/%s", 'a' + part, volname);
+	ret = snprintf (str, sz, "%s/vicep%c/%s",
+			dpart_root, 'a' + part, volname);
 #if 0
     else if (part <= ('z' - 'a') * ('z' - 'a'))
-	ret = snprintf (str, sz, "/vicep%c%c/%s", 
+	ret = snprintf (str, sz, "%s/vicep%c%c/%s", dpart_root,
 		      'a' + part / ('z' - 'a'),
 		      'a' + part % ('z' - 'a'),
 		      volname);
@@ -188,14 +184,13 @@ int
 vol_create (int fd, u_int32_t num, const char *name,
 	    u_int32_t type, u_int32_t parent)
 {
-    char volname[MAXPATHLEN];
     int ret;
     struct timeval tv;
     volintInfo info;
 
     gettimeofday (&tv, NULL);
 
-    strncpy (info.name, name, sizeof (info.name));
+    strlcpy (info.name, name, sizeof (info.name));
     info.name[sizeof (info.name)-1] = '\0';
     info.type = type;
     info.volid = num;
@@ -204,9 +199,9 @@ vol_create (int fd, u_int32_t num, const char *name,
     info.cloneID = 0;
     info.status = 0;
     info.copyDate = 0;
-    info.inUse = 0;
+    info.inUse = 1;
     info.needsSalvaged = 0;
-    info.destroyMe = 0;
+    info.destroyMe = 'c';
     info.creationDate = tv.tv_sec;
     info.accessDate = tv.tv_sec;
     info.updateDate = tv.tv_sec;
@@ -222,13 +217,7 @@ vol_create (int fd, u_int32_t num, const char *name,
     info.spare3 = 0;
 
     ret = vol_write_header (fd, &info);
-
     close (fd);
-    if (ret) {
-	unlink (volname);
-	return ret;
-    }
-
     return ret;
 }
 
