@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,13 +31,22 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: voldb.h,v 1.3 1999/06/23 09:27:49 lha Exp $ */
+/* $Id: voldb.h,v 1.17 2000/10/03 00:20:19 lha Exp $ */
 
 #ifndef FILBUNKE_VOLDB_H
 #define FILBUNKE_VOLDB_H 1
 
 #include <volumeserver.h>
 #include <fs_def.h>
+#include <vstatus.h>
+
+/*
+ * Of this space half is shared of the voldb and the underlaying
+ * layer.
+ */
+
+#define VOLDB_HEADER_SIZE (10*4)
+#define VOLDB_HEADER_HALF (VOLDB_HEADER_SIZE/2) 
 
 /*
  * Data structure for a file (link is same)
@@ -51,7 +55,7 @@
 struct voldb_file_entry {    
     u_int32_t nextptr;
     u_int32_t unique;
-    u_int32_t ino;
+    onode_opaque ino;
     u_int32_t spare2;		/* Must be zero */
     u_int32_t InterfaceVersion;
     u_int32_t FileType;
@@ -71,6 +75,15 @@ struct voldb_file_entry {
     u_int32_t spare5;		/* Must be zero */
 };
 
+/*
+ * Data structure for an acl_entry
+ */
+
+struct acl_entry {
+    u_int32_t owner;
+    u_int32_t flags;
+};
+
 /*
  * Data structure for a dir
  */
@@ -78,7 +91,7 @@ struct voldb_file_entry {
 struct voldb_dir_entry {    
     u_int32_t nextptr;
     u_int32_t unique;
-    u_int32_t ino;
+    onode_opaque ino;
     u_int32_t spare2;		/* Must be zero */
     u_int32_t InterfaceVersion;
     u_int32_t FileType;
@@ -96,10 +109,16 @@ struct voldb_dir_entry {
     u_int32_t spare4;		/* Must be zero */
     u_int32_t Group;
     u_int32_t spare5;		/* Must be zero */
-    struct {
-	u_int32_t owner;
-	u_int32_t flags;
-    } acl[FS_MAX_ACL];
+    struct acl_entry negacl[FS_MAX_ACL];
+    struct acl_entry acl[FS_MAX_ACL];
+};
+
+struct voldb_entry {
+    int32_t type;
+    union {
+	struct voldb_dir_entry dir;
+	struct voldb_file_entry file;
+    } u;
 };
 
 /* defines */
@@ -107,6 +126,8 @@ struct voldb_dir_entry {
 #define VOLDB_FILE 0x1
 #define VOLDB_DIR 0x2
 
+#define VOLDB_DEFAULT_TYPE 0
+
 /* forward declarations */
 
 struct voldb;
@@ -114,26 +135,23 @@ struct voldb;
 /* prototypes */
 
 int
-voldb_init (int fd, struct voldb **db);
+voldb_init (int fd, int32_t type, int32_t volume, struct voldb **db);
 
 int
 voldb_close (struct voldb *db);
 
 int
-voldb_get_dir (struct voldb *db, u_int32_t num, struct voldb_dir_entry *e);
+voldb_put_entry (struct voldb *db, int32_t num, struct voldb_entry *e);
 
 int
-voldb_put_dir (struct voldb *db, u_int32_t num, struct voldb_dir_entry *e);
+voldb_get_size (struct voldb *db, int32_t *total_size, int32_t *num_entries);
+
+int
+voldb_get_entry (struct voldb *db, int32_t num, struct voldb_entry *e);
 
 int
 voldb_put_acl (struct voldb *db, u_int32_t num, struct voldb_dir_entry *e);
 
-int
-voldb_get_file (struct voldb *db, u_int32_t num, struct voldb_file_entry *e);
-
-int
-voldb_put_file (struct voldb *db, u_int32_t num, struct voldb_file_entry *e);
-
 int
 voldb_pretty_print_file (struct voldb_file_entry *e);
 
@@ -141,23 +159,33 @@ int
 voldb_pretty_print_dir (struct voldb_dir_entry *e);
 
 int
-voldb_create_header (int fd, int flags);
+voldb_pretty_print_header (struct voldb *db);
+
+int
+voldb_create_header (int fd, int type, int flags);
 
 int
 voldb_flush (struct voldb *db);
 
-int
-voldb_extend_db (struct voldb *db, unsigned int num);
-
 int
 voldb_new_entry (struct voldb *db, u_int32_t *num, u_int32_t *unique);
 
 int
-voldb_del_entry (struct voldb *db, u_int32_t num, int32_t *ino);
+voldb_del_entry (struct voldb *db, u_int32_t num, onode_opaque *ino);
 
 int
-voldb_header_info (struct voldb *db, u_int32_t *num,
-		   u_int32_t *freeptr, u_int32_t *flags);
+voldb_header_info (struct voldb *db, 
+		   u_int32_t *num,
+		   u_int32_t *flags);
+
+u_int32_t
+voldb_get_volume (struct voldb *db);
+
+int
+voldb_write_hdr (int fd, struct voldb *db);
+
+void
+voldb_update_time(struct voldb_entry *e, time_t t);
 
 /* vol.c */
 
@@ -180,9 +208,25 @@ vol_create (int fd, u_int32_t num, const char *name,
 void
 vol_pretty_print_info (FILE *out, volintInfo *info);
 
-#define VOLDB_ENTEND_NUM 10
+enum voldb_newnum_sizes {  VOLDB_ENTEND_NUM = 10 } ;
 
 const char *
 vol_voltype2name (int32_t type);
 
+
+int
+afs_dir_p (int32_t vno);
+
+int32_t
+dir_afs2local (int32_t vno);
+
+int32_t
+dir_local2afs (int32_t vno);
+
+int32_t
+file_afs2local (int32_t vno);
+
+int32_t
+file_local2afs (int32_t vno);
+
 #endif /*  FILBUNKE_VOLDB_H 1 */
