@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,21 +34,12 @@
 /*
  * This module contains the code that parse the file /vice.+/vol\d{8}
  * file. The file is have the following fields:
- *	int32_t version;
- *	int32_t magic;
- *	int32_t voltype;
- *	int32_t volid;
- *      int32_t spare1;		64 bit compat
- *	int32_t volinfoinode;
- *      int32_t spare2;		64 bit compat
- *	int32_t dirinode;
- *      int32_t spare3;		64 bit compat
- *	int32_t fileinode;
+ *	see vstat.xg file.
  */
 
 #include <config.h>
 
-RCSID("$Id: vstatus.c,v 1.1 1999/05/17 14:20:57 lha Exp $");
+RCSID("$Id: vstatus.c,v 1.5 2000/10/03 00:20:35 lha Exp $");
 
 #include <sys/types.h>
 #include <sys/uio.h>
@@ -170,3 +156,17 @@ vstatus_write (int fd, vstatus *v)
 
     return ret;
 }
+
+void
+vstatus_print_onode (FILE *out, onode_opaque *o)
+{
+    int i;
+    assert (o);
+
+    fprintf (out, "%d:0x", o->size);
+    for (i = 0; i < o->size; i++) {
+	fprintf (out, "%02x", (int) o->data[i] & 0xff);
+    }
+    fprintf (out, "\n");
+
+}
