@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,12 +31,16 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: vstatus.h,v 1.1 1999/05/17 14:20:58 lha Exp $ */
+/* $Id: vstatus.h,v 1.3 2000/10/03 00:20:40 lha Exp $ */
+
+#ifndef __FILBUNKE_VSTATUS_H
+#define __FILBUNKE_VSTATUS_H 1
 
 #include <vstat.h>
 
-int vstatus_init (void) ;
+int  vstatus_init (void) ;
+int  vstatus_read (int fd, vstatus *v);
+int  vstatus_write (int fd, vstatus *v);
+void vstatus_print_onode (FILE *out, onode_opaque *o);
 
-int vstatus_read (int fd, vstatus *v);
-
-int vstatus_write (int fd, vstatus *v);
+#endif /* __FILBUNKE_VSTATUS_H */
