@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -37,11 +32,14 @@
  */
 
 #include "ubik.ss.h"
-#include "netinet/in.h"
-#include "arpa/inet.h"
-#include "krb.h"
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+#ifdef KERBEROS
+#include <krb.h>
+#endif
 
-RCSID("$Id: ubikprocs.c,v 1.1 1999/07/18 19:47:09 map Exp $");
+RCSID("$Id: ubikprocs.c,v 1.3 2000/10/03 00:21:02 lha Exp $");
 
 int Ubik_Beacon(struct rx_call *call, 
 		const int32_t state, 
@@ -55,14 +53,32 @@ int Ubik_Beacon(struct rx_call *call,
 int Ubik_Debug(struct rx_call *call, 
 	       struct ubik_debug *db)
 {
-    struct in_addr *ina;
-    int ret;
+#ifdef KERBEROS
+    {
+	struct in_addr *ina;
+	int ret;
 
-    ret = k_get_all_addrs(&ina);
-    if (ret < 1)
-	return -1;
+	ret = k_get_all_addrs(&ina);
+	if (ret < 1)
+	    return -1;
+	db->syncHost = ntohl(ina[0].s_addr); /* XXX */
+    }
+#else
+    {
+	char name[MAXHOSTNAMELEN];
+	struct hostent *he;
+	struct in_addr tmp;
+
+	if (gethostname (name, sizeof(name) < 0))
+	    return -1;
+	he = gethostbyname (name);
+	if (he == NULL)
+	    return -1;
+	memcpy (&tmp, he->h_addr_list[0], sizeof(struct in_addr));
+	db->syncHost = ntohl(tmp.s_addr);
+    }
+#endif
 
-    db->syncHost = ntohl(ina[0].s_addr); /* XXX */
     return 0;
 }
 
