@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,293 +31,19 @@
  * SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "vldb_locl.h"
 
-#include <sys/types.h>
-#include <rx/rx.h>
-#include <rx/rx_null.h>
+RCSID("$Id: vldbserver.c,v 1.33 2001/01/01 20:42:58 lha Exp $");
 
-#include <ports.h>
-#include <ko.h>
-
-#ifdef KERBEROS
-#include <des.h>
-#include <krb.h>
-#include <rxkad.h>
-#endif
-
-#include <err.h>
-#include <assert.h>
-#include <ctype.h>
-
-#ifndef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <service.h>
-#include <msecurity.h>
-#include <netinit.h>
-
-#include <getarg.h>
-
-#include "rx/rxgencon.h"
-#include "vldb.h"
-#include "vldb.ss.h"
-#include "ubik.ss.h"
-
-RCSID("$Id: vldbserver.c,v 1.18 1999/07/18 19:47:09 map Exp $");
-
-int vl_database;
-vlheader vl_header;
-off_t file_length;
-
-char vl_header_ydr[VLHEADER_SIZE];
-
-static int debug = 0;
+static void make_vldb_from_vl(struct vldbentry *vldb_entry,
+			      struct disk_vlentry *vl_entry);
 
 static void
-vlservdebug (char *fmt, ...)
-    __attribute__ ((format (printf, 1, 2)));
-
-static void
-vlservdebug (char *fmt, ...)
-{
-    va_list args;
-    if (!debug)
-	return ;
-
-    va_start (args, fmt);
-    vfprintf (stderr, fmt, args);
-    va_end(args);
-}
-
-void
-write_header()
-{
-    off_t pos;
-    int length = VLHEADER_SIZE;
-
-    if (ydr_encode_vlheader(&vl_header, vl_header_ydr, &length) == NULL)
-        err(1, "write_header");
-
-    pos = lseek(vl_database, 0, SEEK_SET);
-    assert(pos == 0);
-
-    length = write(vl_database, vl_header_ydr, VLHEADER_SIZE);
-    assert (length == VLHEADER_SIZE);
-}
-
-void
-read_header()
-{
-    char vl_header_ydr[VLHEADER_SIZE];
-    int length = VLHEADER_SIZE;
-
-    if (lseek(vl_database, 0, SEEK_SET) == -1)
-        err(1, "lseek");
-
-    length = read(vl_database, vl_header_ydr, VLHEADER_SIZE);
-    if (length == -1)
-        err(1, "read");
-    if (length != VLHEADER_SIZE)
-        errx(1, "read_header read failed");
-
-    if (ydr_decode_vlheader(&vl_header, vl_header_ydr, &length) == NULL)
-        err(1, "read_header");
-}
-
-void
-get_file_length()
-{
-    file_length = lseek(vl_database, 0, SEEK_END);
-    if (file_length == -1) {
-        err(1, "lseek");
-    }
-}
-
-off_t
-find_first_free()
-{
-    off_t pos;
-
-    if (vl_header.vital_header.freePtr == 0) { /* if there are no free entries */
-	pos = lseek(vl_database, 0, SEEK_END);
-	if (pos == -1)
-	    err(1, "lseek");
-	if (ftruncate(vl_database, pos + DISK_VLENTRY_SIZE) == -1)
-	    err(1, "ftruncate");
-	return pos;
-    } else { /* there are free entries */
-	/* Not implemented yet */
-	assert(0);
-    }
-    return 0;
-}
-
-int
-write_entry(off_t offset, disk_vlentry *vl_entry)
-{
-    off_t pos;
-    char vl_entry_ydr[DISK_VLENTRY_SIZE];
-    int length = DISK_VLENTRY_SIZE;
-
-    if (ydr_encode_disk_vlentry((disk_vlentry *) vl_entry, vl_entry_ydr, &length) == NULL)
-	err(1, "write_entry");
-
-    pos = lseek(vl_database, offset, SEEK_SET);
-    assert(pos == offset);
-
-    length = write(vl_database, vl_entry_ydr, DISK_VLENTRY_SIZE);
-    assert (length == DISK_VLENTRY_SIZE);
-
-    return 0;
-}
-
-int
-read_entry(off_t offset, disk_vlentry *vl_entry)
-{
-    off_t pos;
-    char vl_entry_ydr[DISK_VLENTRY_SIZE];
-    int length = DISK_VLENTRY_SIZE;
-
-    pos = lseek(vl_database, offset, SEEK_SET);
-    assert(pos == offset);
-
-    length = read(vl_database, vl_entry_ydr, DISK_VLENTRY_SIZE);
-    assert (length == DISK_VLENTRY_SIZE);
-
-    if (ydr_decode_disk_vlentry((disk_vlentry *) vl_entry, vl_entry_ydr, &length) == NULL)
-	err(1, "write_entry");
-
-    return 0;
-}
-
-void
-create_database()
-{
-    int i;
-
-    vl_header.vital_header.vldbversion = 0;
-    vl_header.vital_header.headersize = VLHEADER_SIZE;
-    vl_header.vital_header.freePtr = 0;
-    vl_header.vital_header.eofPtr = VLHEADER_SIZE;
-    vl_header.vital_header.allocs = 0;
-    vl_header.vital_header.frees = 0;
-    vl_header.vital_header.MaxVolumeId = 0xA0000000 - 1;
-    for (i = 0; i < MAXTYPES; i++)
-	vl_header.vital_header.totalEntries[i] = 0;
-
-    for (i = 0; i < MAXSERVERID+1; i++)
-	vl_header.IpMappedAddr[i] = 0;
-
-    memset(vl_header.VolnameHash, 0, HASHSIZE * sizeof(int32_t));
-    memset(vl_header.VolidHash, 0, HASHSIZE * MAXTYPES * sizeof(int32_t));
-    write_header();
-    get_file_length();
-}
-
-unsigned long
-get_id_hash(long id)
-{
-    return ((unsigned long) id) % HASHSIZE;
-}
-
-unsigned long
-get_name_hash(const char *name)
-{
-    int i;
-    unsigned long hash = 0x47114711;
-
-    for (i = 0; name[i] && i < 32; i++)
-	hash *= name[i];
-
-    return hash % HASHSIZE;
-}
-
-int
-get_first_id_entry(unsigned long hash_id, long type, disk_vlentry *vl_entry)
-{
-    off_t offset = vl_header.VolidHash[type][hash_id];
-    int status;
-
-    vlservdebug ("  get_first_id_entry hash_id: %lu type: %ld offset: %d\n", 
-		 hash_id, type, (int) offset);
-
-    if (offset == 0)
-	return VL_NOENT;
-
-    status = read_entry(offset, vl_entry);
-
-    return status;
-}
-
-int
-get_first_name_entry(unsigned long hash_name, disk_vlentry *vl_entry)
-{
-    off_t offset = vl_header.VolnameHash[hash_name];
-    int status;
-
-    vlservdebug ("  get_first_name_entry hash_name: %lu offset: %d\n", 
-		 hash_name, (int) offset);
-    if (offset == 0)
-	return VL_NOENT;
-
-    status = read_entry(offset, vl_entry);
-
-    return status;
-}
-
-int
-insert_entry(disk_vlentry *vl_entry)
-{
-    off_t offset;
-    int status;
-    unsigned long hash_id, hash_name;
-    disk_vlentry first_id_entry;
-    disk_vlentry first_name_entry;
-    
-    /* Allokera plats i filen */
-    offset = find_first_free();
-
-    /* Allocate new volume id? */
-    /*id = vl_header.vital_header.MaxVolumeId++;*/
-
-    /* Hitta plats i hashtabellerna */
-    /* XXX At present, only RW is handled */
-    hash_id = get_id_hash(vl_entry->volumeId[RWVOL]);
-    hash_name = get_name_hash(vl_entry->name);
-
-    status = get_first_id_entry(hash_id, vl_entry->volumeType, &first_id_entry);
-
-/* XXX    vl_entry->nextIDHash[vldb_entry->type] = status ? 0 : first_id_entry.nextID;*/
-    vl_entry->nextIdHash[vl_entry->volumeType] = status ? 0 : vl_header.VolidHash[vl_entry->volumeType][hash_id];
-
-    status = get_first_name_entry(hash_name, &first_name_entry);
-/* XXX    pr_entry->nextName = status ? 0 : first_name_entry.nextName;*/
-    vl_entry->nextNameHash = status ? 0 : vl_header.VolnameHash[hash_name];
-
-    /* XXX: uppdatera owned och nextOwned */
-
-    /* Lgg in entryt i filen */
-    status = write_entry(offset, vl_entry);
-    if (status)
-	return status;
-    
-    /* Uppdatera hashtabell */
-    vl_header.VolidHash[vl_entry->volumeType][hash_id] = offset;
-    vl_header.VolnameHash[hash_name] = offset;
-    write_header();
-    return 0;
-}
-
-void
 make_vldb_from_vl(struct vldbentry *vldb_entry, struct disk_vlentry *vl_entry)
 {
     int i;
 
-    strncpy(vldb_entry->name, vl_entry->name, VLDB_MAXNAMELEN);
+    strlcpy(vldb_entry->name, vl_entry->name, VLDB_MAXNAMELEN);
     vldb_entry->volumeType = vl_entry->volumeType;
     vldb_entry->nServers = 1; /* XXX What is this supposed to be? */
 
@@ -348,28 +69,33 @@ VL_CreateEntry(struct rx_call *call,
 	       const vldbentry *newentry) 
 {
     struct disk_vlentry vl_entry;
+    int32_t nServers;
     int i;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
-    vlservdebug ("VL_CreateEntry (name=%s, type=%d, ids=%d,%d,%d flags=%d)\n",
-		 newentry->name, newentry->volumeType,
-		 newentry->volumeId[RWVOL],
-		 newentry->volumeId[ROVOL],
-		 newentry->volumeId[BACKVOL],
-		 newentry->flags);
+    memset (&vl_entry, 0, sizeof(vl_entry));
 
-    /* XXX Should see to it that name is null terminated too */
-    strncpy(vl_entry.name, newentry->name, VLDB_MAXNAMELEN);
+    vldb_debug ("VL_CreateEntry (name=%s, type=%d, ids=%d,%d,%d flags=%d)\n",
+		newentry->name, newentry->volumeType,
+		newentry->volumeId[RWVOL],
+		newentry->volumeId[ROVOL],
+		newentry->volumeId[BACKVOL],
+		newentry->flags);
+
+    strlcpy(vl_entry.name, newentry->name, VLDB_MAXNAMELEN);
     vl_entry.volumeType = newentry->volumeType;
 
     /* XXX All fields mustn't be set */
-    for (i = 0; i < MAXNSERVERS; i++) {
+    nServers = newentry->nServers;
+    if (nServers > MAXNSERVERS)
+	nServers = MAXNSERVERS;
+    for (i = nServers - 1 ; i >= 0 ; i--) {
 	vl_entry.serverNumber[i] = newentry->serverNumber[i];
 	vl_entry.serverPartition[i] = newentry->serverPartition[i];
 	vl_entry.serverFlags[i] = newentry->serverFlags[i];
-    }
+    }    
 
     for (i = 0; i < MAXTYPES; i++)
 	vl_entry.volumeId[i] = newentry->volumeId[i];
@@ -377,7 +103,7 @@ VL_CreateEntry(struct rx_call *call,
     vl_entry.cloneId = newentry->cloneId;
     vl_entry.flags = newentry->flags;
 
-    insert_entry(&vl_entry);
+    vldb_insert_entry(&vl_entry);
     return 0;
 }
 
@@ -386,7 +112,7 @@ VL_DeleteEntry(struct rx_call *call,
 	       const int32_t Volid, 
 	       const int32_t voltype)
 {
-    vlservdebug ("VL_DeleteEntry\n") ;
+    vldb_debug ("VL_DeleteEntry\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -406,10 +132,11 @@ VL_GetEntryByID(struct rx_call *call,
 {
     struct disk_vlentry vl_entry;
 
-    vlservdebug ("VL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
+    vldb_debug ("VL_GetEntryByID (Volid=%d,Voltype=%d)\n", 
 		  Volid, voltype);
     
-    if (get_first_id_entry(get_id_hash(Volid), voltype, &vl_entry) != 0)
+    if (vldb_get_first_id_entry(vldb_get_id_hash(Volid),
+				voltype, &vl_entry) != 0)
 	return VL_NOENT;
 
     while (1) {
@@ -422,7 +149,7 @@ VL_GetEntryByID(struct rx_call *call,
 	if (vl_entry.nextIdHash[voltype] == 0)
 	    break;
 	
-	read_entry(vl_entry.nextIdHash[voltype], &vl_entry);
+	vldb_read_entry(vl_entry.nextIdHash[voltype], &vl_entry);
     }
     return VL_NOENT;
 }
@@ -438,13 +165,14 @@ VL_GetEntryByName(struct rx_call *call,
 {
     struct disk_vlentry vl_entry;
 
-    vlservdebug ("VL_GetEntryByName %s\n", volumename) ;
+    vldb_debug ("VL_GetEntryByName %s\n", volumename) ;
 
     if (isdigit(volumename[0])) {
 	return VL_GetEntryByID(call, atol(volumename), 0 /* XXX */, entry);
     }
 
-    if (get_first_name_entry(get_name_hash(volumename), &vl_entry) == 0) {
+    if (vldb_get_first_name_entry(vldb_get_name_hash(volumename),
+				  &vl_entry) == 0) {
 	while (1) {
 	    /* Return entry if match found */
 	    if (strcmp(vl_entry.name, volumename) == 0) {
@@ -455,7 +183,7 @@ VL_GetEntryByName(struct rx_call *call,
 	    if (vl_entry.nextNameHash == 0)
 		break;
 	    
-	    read_entry(vl_entry.nextNameHash, &vl_entry);
+	    vldb_read_entry(vl_entry.nextNameHash, &vl_entry);
 	}
     } 
 
@@ -471,15 +199,15 @@ VL_GetNewVolumeId (struct rx_call *call,
 		   const int32_t bumpcount,
 		   int32_t *newvolumid)
 {
-    vlservdebug ("VL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
+    vldb_debug ("VL_GetNewVolumeId(bumpcount=%d)\n", bumpcount) ;
     
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
     *newvolumid = vl_header.vital_header.MaxVolumeId;
-    vlservdebug ("   returning low volume id = %d\n", *newvolumid);
+    vldb_debug ("   returning low volume id = %d\n", *newvolumid);
     vl_header.vital_header.MaxVolumeId += bumpcount;
-    write_header();
+    vldb_write_header();
 
     return 0;
 }
@@ -495,7 +223,7 @@ VL_ReplaceEntry (struct rx_call *call,
 		 const vldbentry *newentry,
 		 const int32_t ReleaseType) 
 {
-    vlservdebug ("VL_ReplaceEntry\n") ;
+    vldb_debug ("VL_ReplaceEntry\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -514,7 +242,7 @@ VL_UpdateEntry (struct rx_call *call,
 		const VldbUpdateEntry *UpdateEntry,
 		const int32_t ReleaseType)
 {
-    vlservdebug ("VL_UpdateEntry\n") ;
+    vldb_debug ("VL_UpdateEntry\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -532,7 +260,7 @@ VL_SetLock (struct rx_call *call,
 	    const int32_t voltype,
 	    const int32_t voloper) 
 {
-    vlservdebug ("VL_SetLock\n") ;
+    vldb_debug ("VL_SetLock\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -550,7 +278,7 @@ VL_ReleaseLock (struct rx_call *call,
 		const int32_t voltype, 
 		const int32_t ReleaseType) 
 {
-    vlservdebug ("VL_ReleaseLock\n") ;
+    vldb_debug ("VL_ReleaseLock\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -570,7 +298,7 @@ VL_ListEntry (struct rx_call *call,
 	      int32_t *next_index, 
 	      vldbentry *entry) 
 {
-    vlservdebug ("VL_ListEntry\n") ;
+    vldb_debug ("VL_ListEntry\n") ;
     return VL_PERM ;
 }
 
@@ -584,7 +312,7 @@ VL_ListAttributes (struct rx_call *call,
 		   int32_t *nentries,
 		   bulkentries *blkentries) 
 {
-    vlservdebug ("VL_ListAttributes\n") ;
+    vldb_debug ("VL_ListAttributes\n") ;
     return VL_PERM ;
 }
 
@@ -597,7 +325,7 @@ VL_GetStats (struct rx_call *call,
 	     vldstats *stats,
 	     vital_vlheader *vital_header) 
 {
-    vlservdebug ("VL_GetStats") ;
+    vldb_debug ("VL_GetStats") ;
     return VL_PERM ;
 }
 
@@ -608,7 +336,7 @@ VL_GetStats (struct rx_call *call,
 int 
 VL_Probe(struct rx_call *call)
 {
-    vlservdebug ("VL_Probe\n") ;
+    vldb_debug ("VL_Probe\n") ;
     return 0;
 }
 
@@ -623,11 +351,13 @@ VL_CreateEntryN(struct rx_call *call,
     int i;
     struct vldbentry vldb_entry;
 
-    vlservdebug ("VL_CreateEntryN\n") ;
+    vldb_debug ("VL_CreateEntryN\n") ;
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
 
+    memset (&vldb_entry, 0, sizeof (vldb_entry));
+
     strncpy(vldb_entry.name, entry->name, VLDB_MAXNAMELEN);
     vldb_entry.volumeType = RWVOL;
     vldb_entry.nServers = entry->nServers;
@@ -653,24 +383,27 @@ VL_CreateEntryN(struct rx_call *call,
 
 int
 VL_GetEntryByIDN(struct rx_call *call,
-		 int32_t Volid,
-		 int32_t voltype,
+		 const int32_t Volid,
+		 const int32_t voltype,
 		 nvldbentry *entry)
 {
     struct vldbentry vldb_entry;
     int status, i;
+    int32_t type = voltype;
 
-    vlservdebug ("VL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", Volid, voltype);
+    vldb_debug ("VL_GetEntryByIDN (Volid=%d,Voltype=%d)\n", Volid, type);
 
-    if (voltype == -1)
-	voltype = RWVOL;
+    memset (&vldb_entry, 0, sizeof(vldb_entry));
 
-    status = VL_GetEntryByID(call, Volid, voltype, &vldb_entry);
+    if (type == -1)
+	type = RWVOL;
+
+    status = VL_GetEntryByID(call, Volid, type, &vldb_entry);
 
     if (status)
 	return status;
 
-    strncpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
+    strlcpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
     entry->nServers = vldb_entry.nServers;
     for (i = 0; i < MAXNSERVERS; i++) {
 	entry->serverNumber[i] = vldb_entry.serverNumber[i];
@@ -699,14 +432,16 @@ VL_GetEntryByNameN(struct rx_call *call,
     struct vldbentry vldb_entry;
     int status, i;
 
-    vlservdebug ("VL_GetEntryByNameN(volumename=%s)\n", volumename) ;
+    memset (&vldb_entry, 0, sizeof(vldb_entry));
+
+    vldb_debug ("VL_GetEntryByNameN(volumename=%s)\n", volumename) ;
     status = VL_GetEntryByName(call, volumename, &vldb_entry);
 
     if (status)
 	return status;
 
     memset (entry, 0, sizeof(*entry));
-    strncpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
+    strlcpy(entry->name, vldb_entry.name, VLDB_MAXNAMELEN);
     entry->nServers = vldb_entry.nServers;
     for (i = 0; i < MAXNSERVERS; i++) {
 	entry->serverNumber[i] = vldb_entry.serverNumber[i];
@@ -732,7 +467,7 @@ VL_GetEntryByNameU(struct rx_call *call,
 		   const char *volumename, 
 		   uvldbentry *entry) 
 {
-    vlservdebug ("VL_GetEntryByNameU %s\n", volumename);
+    vldb_debug ("VL_GetEntryByNameU %s\n", volumename);
     memset(entry, 0, sizeof(*entry));
     
     return RXGEN_OPCODE;
@@ -748,21 +483,21 @@ VL_ListAttributesN (struct rx_call *call,
 		   int32_t *nentries,
 		   nbulkentries *blkentries) 
 {
-    vlservdebug ("VL_ListAttributesN\n");
-    vlservdebug ("  attributes: Mask=(%d=", attributes->Mask);
+    vldb_debug ("VL_ListAttributesN\n");
+    vldb_debug ("  attributes: Mask=(%d=", attributes->Mask);
 
     if (attributes->Mask & VLLIST_SERVER)
-	vlservdebug  ("SERVER ");
+	vldb_debug  ("SERVER ");
     if (attributes->Mask & VLLIST_PARTITION)
-	vlservdebug  ("PARTITION ");
+	vldb_debug  ("PARTITION ");
     if (attributes->Mask & VLLIST_VOLUMETYPE)
-	vlservdebug  ("VOLUMETYPE ");
+	vldb_debug  ("VOLUMETYPE ");
     if (attributes->Mask & VLLIST_VOLUMEID)
-	vlservdebug  ("VOLUMEID ");
+	vldb_debug  ("VOLUMEID ");
     if (attributes->Mask & VLLIST_FLAG)
-	vlservdebug  ("FLAG");
+	vldb_debug  ("FLAG");
 
-    vlservdebug (") server=%d partition=%d volumetype=%d volumeid=%d flag=%d\n",
+    vldb_debug (") server=%d partition=%d volumetype=%d volumeid=%d flag=%d\n",
 	   attributes->server,
 	   attributes->partition,
 	   attributes->volumetype,
@@ -787,7 +522,7 @@ VL_ListAttributesU(struct rx_call *call,
 		   int32_t *nentries,
 		   ubulkentries *blkentries) 
 {
-    vlservdebug ("VL_ListAttributesU\n") ;
+    vldb_debug ("VL_ListAttributesU\n") ;
     *nentries = 0;
     blkentries->len = 0;
     blkentries->val = NULL;
@@ -804,7 +539,7 @@ VL_UpdateEntryByName(struct rx_call *call,
 		     const struct VldbUpdateEntry *UpdateEntry,
 		     const int32_t ReleaseType)
 {
-    vlservdebug ("VL_UpdateEntryByName (not implemented)\n");
+    vldb_debug ("VL_UpdateEntryByName (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -824,7 +559,7 @@ VL_GetAddrsU(struct rx_call *call,
 	     int32_t *nentries,
 	     bulkaddrs *addrs)
 {
-    vlservdebug ("VL_GetAddrsU (not implemented)\n");
+    vldb_debug ("VL_GetAddrsU (not implemented)\n");
     return VL_PERM;
 }
 
@@ -838,7 +573,7 @@ VL_RegisterAddrs(struct rx_call *call,
 		 const int32_t spare,
 		 const bulkaddrs *addrs)
 {
-    vlservdebug ("VL_RegistersAddrs (not implemented)\n");
+    vldb_debug ("VL_RegistersAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -854,7 +589,7 @@ int
 VL_CreateEntryU(struct rx_call *call,
 		const struct uvldbentry *newentry)
 {
-    vlservdebug ("VL_CreateEntryU (not implemented)\n");
+    vldb_debug ("VL_CreateEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -869,7 +604,7 @@ VL_CreateEntryU(struct rx_call *call,
 int
 VL_ReplaceEntryU(struct rx_call *call)
 {
-    vlservdebug ("VL_ReplaceEntryU (not implemented)\n");
+    vldb_debug ("VL_ReplaceEntryU (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -888,7 +623,7 @@ VL_ReplaceEntryN(struct rx_call *call,
 		 const struct vldbentry *newentry,
 		 const int32_t ReleaseType)
 {
-    vlservdebug ("VL_ReplaceEntryN (not implemented)\n");
+    vldb_debug ("VL_ReplaceEntryN (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -905,7 +640,7 @@ VL_ChangeAddrs(struct rx_call *call,
 	       const int32_t old_ip,
 	       const int32_t new_ip)
 {
-    vlservdebug ("VL_ChangeAddrs (not implemented)\n");
+    vldb_debug ("VL_ChangeAddrs (not implemented)\n");
 
     if (!sec_is_superuser(call))
 	return VL_PERM;
@@ -920,7 +655,7 @@ VL_ChangeAddrs(struct rx_call *call,
 int
 VL_GetEntryByIDU(struct rx_call *call)
 {
-    vlservdebug ("VL_GetEntryByIDU (not implemented)\n");
+    vldb_debug ("VL_GetEntryByIDU (not implemented)\n");
     return VL_PERM;
 }
 
@@ -931,7 +666,7 @@ VL_GetEntryByIDU(struct rx_call *call)
 int
 VL_ListEntryN(struct rx_call *call)
 {
-    vlservdebug ("VL_ListEntryN (not implemented)\n");
+    vldb_debug ("VL_ListEntryN (not implemented)\n");
     return VL_PERM;
 }
 
@@ -942,7 +677,7 @@ VL_ListEntryN(struct rx_call *call)
 int
 VL_ListEntryU(struct rx_call *call)
 {
-    vlservdebug ("VL_ListEntryU (not implemented)\n");
+    vldb_debug ("VL_ListEntryU (not implemented)\n");
     return VL_PERM;
 }
 
@@ -958,7 +693,7 @@ VL_GetAddrs(struct rx_call *call,
 	    const int32_t *nentries,
 	    bulkaddrs *blkaddr)
 {
-    vlservdebug ("VL_GetAddrs (not implemented)\n");
+    vldb_debug ("VL_GetAddrs (not implemented)\n");
     return VL_PERM;
 }
 
@@ -969,7 +704,7 @@ VL_GetAddrs(struct rx_call *call,
 int
 VL_LinkedListN(struct rx_call *call)
 {
-    vlservdebug ("VL_LinkedListN (not implemented)\n");
+    vldb_debug ("VL_LinkedListN (not implemented)\n");
     return VL_PERM;
 }
 
@@ -980,48 +715,10 @@ VL_LinkedListN(struct rx_call *call)
 int
 VL_LinkedListU(struct rx_call *call)
 {
-    vlservdebug ("VL_LinkedListU (not implemented)\n");
+    vldb_debug ("VL_LinkedListU (not implemented)\n");
     return VL_PERM;
 }
- 
-int
-vldbserver_init(char *databaseprefix)
-{
-    char database[MAXPATHLEN];
-    int status;
 
-    if (databaseprefix == NULL)
-	databaseprefix = MILKO_SYSCONFDIR;
-
-    snprintf (database, sizeof(database), "%s/vl_database", 
-	      databaseprefix);
-
-    vlservdebug ("Loading db from file %s\n", database);
-
-    vl_database = open(database,
-                       O_RDWR | O_CREAT,
-                       S_IRWXU);
-
-    if (vl_database == -1)
-        err(1, "open");
-
-    get_file_length();
-    if (file_length == 0) {
-        fprintf (stderr, "Database not found, creating a new.\n");
-	create_database();
-	/* XXX <insert code for making new root.afs or whatever here> */
-
-	status = 0;
-        if (status)
-            return status;
-    } else {
-        fprintf(stderr, "Loading database.\n");
-        read_header();
-        get_file_length();
-    }
-
-    return 0;
-}
 
 /*
  *
@@ -1034,30 +731,41 @@ static char *cell = NULL;
 static char *realm = NULL;
 static char *databasedir = NULL;
 static char *srvtab_file = NULL;
+static char *log_file = "syslog";
+static int no_auth = 0;
+static int do_create = 0;
+static int vlsrv_debug = 0;
 
-static struct getargs args[] = {
-    {"cell",	0, arg_string,    &cell, "what cell to use"},
-    {"realm",	0, arg_string,	  &realm, "what realm to use"},
-    {"prefix",'p', arg_string,    &databasedir, "what dir to store the db"},
-    {"debug", 'd', arg_flag,      &debug, "output debugging"},
-    {"srvtab",'s', arg_string,    &srvtab_file, "what srvtab to use"},
-    { NULL, 0, arg_end, NULL }
+static struct agetargs args[] = {
+    {"create",  0, aarg_flag,      &do_create, "create new database"},
+    {"cell",	0, aarg_string,    &cell, "what cell to use"},
+    {"realm",	0, aarg_string,	  &realm, "what realm to use"},
+    {"prefix",'p', aarg_string,    &databasedir, "what dir to store the db"},
+    {"noauth", 0,  aarg_flag,	  &no_auth, "disable authentication checks"},
+    {"debug", 'd', aarg_flag,      &vlsrv_debug, "output debugging"},
+    {"log",   'd', aarg_string,    &log_file, "log file"},
+    {"srvtab",'s', aarg_string,    &srvtab_file, "what srvtab to use"},
+    { NULL, 0, aarg_end, NULL }
 };
 
 static void
 usage(void)
 {
-    arg_printusage(args, NULL, "", ARG_AFSSTYLE);
+    aarg_printusage(args, NULL, "", AARG_AFSSTYLE);
 }
 
 int
 main(int argc, char **argv) 
 {
-    int ret;
+    Log_method *method;
     int optind = 0;
+    int ret;
     
-    if (getarg (args, argc, argv, &optind, ARG_AFSSTYLE)) {
+    set_progname (argv[0]);
+
+    if (agetarg (args, argc, argv, &optind, AARG_AFSSTYLE)) {
 	usage ();
+	return 1;
     }
 
     argc -= optind;
@@ -1065,20 +773,33 @@ main(int argc, char **argv)
 
     if (argc) {
 	printf("unknown option %s\n", *argv);
-	return 0;
+	return 1;
     }
 
+    if (vlsrv_debug)
+	vldb_setdebug (vlsrv_debug);
+
+    if (no_auth)
+	sec_disable_superuser_check ();
+
+    if (do_create) {
+	vldb_create (databasedir);
+	return 0;
+    }
+	
+    method = log_open (get_progname(), log_file);
+    if (method == NULL)
+	errx (1, "log_open failed");
+    cell_init(0, method);
     ports_init();
-    cell_init(0);
+
 
     if (cell)
 	cell_setthiscell (cell);
 
     network_kerberos_init (srvtab_file);
     
-    ret = vldbserver_init(databasedir);
-    if (ret)
-	errx (1, "vldbserver_init: error %d\n", ret);
+    vldb_init(databasedir);
 
     ret = network_init(htons(afsvldbport), "vl", VLDB_SERVICE_ID, 
 		       VL_ExecuteRequest, &vldbservice, realm);
@@ -1086,7 +807,7 @@ main(int argc, char **argv)
 	errx (1, "network_init failed with %d", ret);
     
     ret = network_init(htons(afsvldbport), "ubik", VOTE_SERVICE_ID, 
-		       Ubik_ExecuteRequest, &ubikservice, NULL);
+		       Ubik_ExecuteRequest, &ubikservice, realm);
     if (ret)
 	errx (1, "network_init failed with %d", ret);
 
