@@ -1,4 +1,4 @@
-/* $Id: rx.h,v 1.11 1999/05/15 16:06:56 assar Exp $ */
+/* $Id: rx.h,v 1.19 2001/01/06 21:54:20 lha Exp $ */
 
 /*
 ****************************************************************************
@@ -31,6 +31,7 @@
 #include "../rx/rx_event.h"
 #include "../rx/rx_queue.h"
 #include "../rx/rx_packet.h"
+#include "../rx/rxgencon.h"
 #else				       /* KERNEL */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -43,6 +44,7 @@
 #include "rx_clock.h"
 #include "rx_event.h"
 #include "rx_pkt.h"
+#include "rxgencon.h"
 #endif				       /* KERNEL */
 
 
@@ -60,30 +62,32 @@
 #define MAXBACKOFF 0x1F
 
 struct rx_securityClass;
+struct rx_peer;
 
 /* Exported interfaces XXXX clean this up:  not all of these are exported */
 int rx_Init(u_short);
 struct rx_service *rx_NewService(u_short, u_short, char *, 
-				 struct rx_securityClass **, int, long (*)());
+				 struct rx_securityClass **, int, int32_t (*)());
 struct rx_connection *rx_NewConnection(register u_long, u_short, u_short,
 				       register struct rx_securityClass *,
 				       int);
-struct rx_call *rx_NewCall();
-struct rx_call *rx_GetCall();	 /* Not normally used, but not obsolete */
-long rx_EndCall();
+struct rx_call *rx_NewCall(struct rx_connection *);
+struct rx_call *rx_GetCall(void); /* Not normally used, but not obsolete */
+long rx_EndCall(struct rx_call * call, int32_t rc);
 int rx_AllocPackets();
 void rx_FreePackets();
-int rx_WriteProc();
-int rx_ReadProc();
-void rx_FlushWrite();
+int rx_WriteProc(struct rx_call *call, const void *buf, int nbytes);
+int rx_ReadProc(struct rx_call *call, void *buf, int nbytes);
+void rx_FlushWrite(struct rx_call *call);
 #ifdef RXDEBUG
-void rx_PrintStats();
+void rx_PrintStats(FILE *);
 #else
 #define rx_PrintStats(a)
 #endif
-void rx_PrintPeerStats();
-void rx_SetArrivalProc();
-void rx_Finalize();
+void rx_PrintPeerStats(FILE *file, struct rx_peer *peer);
+void rx_SetArrivalProc(struct rx_call * call, void (*proc) (),
+		       void *handle, void *arg);
+void rx_Finalize(void);
 void rx_StartServer(int);
 void rx_DestroyConnection(struct rx_connection *);
 void rxi_Free(void *, register int);
@@ -91,6 +95,10 @@ int rxi_GetCallNumberVector(const struct rx_connection *, int32_t *);
 int rxi_SetCallNumberVector(struct rx_connection *, int32_t *);
 void rx_SetEpoch(u_long);
 
+void shutdown_rx(void);
+
+
+
 #define	RX_WAIT	    1
 #define	RX_DONTWAIT 0
 
@@ -180,7 +188,7 @@ void rx_SetEpoch(u_long);
 
 /* Set connection dead time, for a specific client or server connection */
 #define rx_SetConnDeadTime(conn, seconds) (rxi_SetConnDeadTime(conn, seconds))
-extern void rxi_SetConnDeadTime();
+extern void rxi_SetConnDeadTime(struct rx_connection * conn, int seconds);
 
 /* Set connection hard timeout for a connection */
 #define rx_SetConnHardDeadTime(conn, seconds) ((conn)->hardDeadTime = (seconds))
@@ -319,6 +327,9 @@ struct rx_securityClass {
 #define RXS_GetStats(obj,conn,stats) RXS_OP(obj,GetStats,\
 					    (obj,conn,stats))
 
+int 
+rxs_Release(struct rx_securityClass *aobj);
+
 /*
  * A service is installed by rx_NewService, and specifies a service type that
  * is exported by this process.  Incoming calls are stamped with the service
@@ -353,7 +364,8 @@ struct rx_service {
 						 * Array of security class
 						 * objects
 						 */
-    long (*executeRequestProc) ();     /* Routine to call when an rpc request
+    int32_t (*executeRequestProc) (struct rx_call *);
+				       /* Routine to call when an rpc request
 				        * is received */
     void (*destroyConnProc) ();	       /* Routine to call when a server
 				        * connection is destroyed */
@@ -410,6 +422,7 @@ struct rx_serverQueueEntry {
  */
 struct rx_peer {
     struct rx_peer *next;	       /* Next in hash conflict or free list */
+    struct rx_queue connQueue;	       /* a list of all conn use this peer */
     u_long host;		       /* Remote IP address, in net byte
 				        * order */
     u_short port;		       /* Remote UDP port, in net byte order */
@@ -477,6 +490,7 @@ struct rx_peer {
  * limited multiple asynchronous conversations.
  */
 struct rx_connection {
+    struct rx_queue queue_item;        /* conns on same peer */
     struct rx_connection *next;	       /* on hash chain _or_ free list */
     struct rx_peer *peer;
 #ifdef	RX_ENABLE_LOCKS
@@ -610,7 +624,7 @@ struct rx_call {
     u_char localStatus;		       /* Local user status sent out of band */
     u_char remoteStatus;	       /* Remote user status received out of
 				        * band */
-    long error;			       /* Error condition for this call */
+    int32_t error;		       /* Error condition for this call */
     u_long timeout;		       /* High level timeout for this call */
     u_long rnext;		       /*
 			                * Next sequence number expected to be
@@ -855,6 +869,9 @@ struct rx_ackPacket {
  * codes
  */
 
+/* Min rx error */
+#define RX_MIN_ERROR		    (-1)
+
 /* Something bad happened to the connection; temporary loss of communication */
 #define	RX_CALL_DEAD		    (-1)
 
@@ -886,6 +903,9 @@ struct rx_ackPacket {
 /* EMSGSIZE returned from network.  Packet too big, must fragment */
 #define RX_MSGSIZE		    (-8)
 
+/* Max rx error */
+#define RX_MAX_ERROR		    (-64)
+
 /*
  * Structure for keeping rx statistics.  Note that this structure is returned
  * by rxdebug, so, for compatibility reasons, new fields should be appended (or
@@ -979,6 +999,8 @@ struct rx_stats {		       /* General rx statistics */
     u_int32_t spares[8];
 };
 
+void rx_PrintTheseStats(FILE *file, struct rx_stats *s, int size);
+
 /* structures for debug input and output packets */
 
 /* debug input types */
