@@ -21,7 +21,7 @@
 
 #include "rx_locl.h"
 
-RCSID("$Id: rx_event.c,v 1.3 1998/02/22 19:43:59 joda Exp $");
+RCSID("$Id: rx_event.c,v 1.5 2000/09/04 03:59:54 lha Exp $");
 
 /*
  * All event processing is relative to the apparent current time
@@ -212,8 +212,9 @@ rxevent_RaiseEvents(struct clock * next)
     return 0;
 }
 
-void
-shutdown_rxevent()
+#if 0
+static void
+shutdown_rxevent(void)
 {
     struct xfreelist *xp, *nxp;
 
@@ -227,3 +228,4 @@ shutdown_rxevent()
     }
 #endif
 }
+#endif
