@@ -1,6 +1,6 @@
 #include "rx_locl.h"
 
-RCSID("$Id: rx_pkt.c,v 1.15 2000/10/15 21:29:57 assar Exp $");
+RCSID("$Id: rx_pkt.c,v 1.15.2.1 2001/07/05 23:16:53 lha Exp $");
 
 struct rx_packet *rx_mallocedP = 0;
 struct rx_cbuf *rx_mallocedC = 0;
@@ -1036,17 +1036,39 @@ rxi_SendDebugPacket(struct rx_packet *apacket, osi_socket asocket,
 		    long ahost, short aport)
 {
     struct sockaddr_in taddr;
+    int i = 0;
+    int savelen = 0;
+    int saven = 0;
+    int nbytes;
 
     taddr.sin_family = AF_INET;
     taddr.sin_port = aport;
     taddr.sin_addr.s_addr = ahost;
 
+    nbytes = apacket->length;
+
+    for (i = 1; i < apacket->niovecs; i++) {
+	if (nbytes <= apacket->wirevec[i].iov_len) {
+	    savelen = apacket->wirevec[i].iov_len;
+	    saven = apacket->niovecs;
+	    apacket->wirevec[i].iov_len = nbytes;
+	    apacket->niovecs = i + 1;	      
+	    /* so condition fails because i == niovecs */
+	} else
+	    nbytes -= apacket->wirevec[i].iov_len;
+    }
+
     GLOBAL_UNLOCK();
     /* debug packets are not reliably delivered, hence the cast below. */
     /* MTUXXX need to adjust lengths as in sendSpecial */
     (void) osi_NetSend(asocket, (char *)&taddr, apacket->wirevec,
 		       apacket->niovecs, apacket->length + RX_HEADER_SIZE);
     GLOBAL_LOCK();
+
+    if (saven) {
+	apacket->wirevec[i - 1].iov_len = savelen;
+	apacket->niovecs = saven;
+    }
 }
 
 /*
