@@ -1,8 +1,8 @@
 /*
- * Common defintions for cb.xg and fs.xg
+ * Common defintions used by several *.xg files
  */
 
-/* $Id: common.h,v 1.9 1998/10/31 02:29:18 lha Exp $ */
+/* $Id: common.h,v 1.12 2000/11/03 10:09:48 lha Exp $ */
 
 %#ifndef _COMMON_
 %#define _COMMON_
@@ -33,6 +33,8 @@ struct AFSCallBack {
 
 enum CallBackType { CBEXCLUSIVE = 1, CBSHARED = 2, CBDROPPED = 3};
 
+const CALLBACK_VERSION = 1;
+
 struct AFSVolSync {
      unsigned spare1;
      unsigned spare2;
@@ -79,6 +81,8 @@ const SS_OWNER       = 0x02 ;
 const SS_GROUP       = 0x04 ;
 const SS_MODEBITS    = 0x08 ;
 const SS_SEGSIZE     = 0x10 ;
+const SS_FSYNC       = 0x400; /* 1024 */
+
 
 struct AFSStoreStatus {
      unsigned long Mask;
@@ -159,7 +163,13 @@ struct AFSVolumeInfo {
     unsigned short Port7;
 };
 
-
-
+struct afsUUID {
+     u_long time_low;
+     u_short time_mid;
+     u_short time_hi_and_version;
+     char clock_seq_hi_and_reserved;
+     char clock_seq_low;
+     char node[6];
+};
 
 %#endif /* _COMMON_ */
