@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "rxkad_locl.h"
 
-RCSID("$Id: rxk_clnt.c,v 1.5 1999/03/12 17:09:44 assar Exp $");
+RCSID("$Id: rxk_clnt.c,v 1.7 2000/10/03 00:38:22 lha Exp $");
 
 /* This code also links into the kernel so we need to use osi_Alloc()
  * to avoid calling malloc(). Similar trick with memcpy() */
@@ -307,6 +302,7 @@ rxkad_NewClientSecurityObject(/*rxkad_level*/ int level,
 	des_cblock k;
       } u;
       int32 sched[ROUNDS];
+      u_long next_epoch;
 
       u.rnd[0] = rx_nextCid;
       u.rnd[1] = rx_epoch;
@@ -325,7 +321,9 @@ rxkad_NewClientSecurityObject(/*rxkad_level*/ int level,
 
       /* Set new cid and epoch generator */
       rx_nextCid = u.rnd[0] << RX_CIDSHIFT;
-      rx_SetEpoch(u.rnd[0] ^ u.rnd[1]);
+      next_epoch = u.rnd[0] ^ u.rnd[1];
+      next_epoch &= 0x7FFFFFFF;
+      rx_SetEpoch(next_epoch);
       rxkad_EpochWasSet = 1;
       inited = 1;
     }
