@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,25 +33,27 @@
 
 #include "rxkad_locl.h"
 
-#ifdef HAVE_MACHINE_ENDIAN_H
 
-#undef WORDS_BIGENDIAN
+/*
+ * this assumes that KRB_C_BIGENDIAN is used.
+ * if we can find out endianess at compile-time, do so,
+ * otherwise WORDS_BIGENDIAN should already have been defined
+ */
 
-#include <machine/endian.h>
-
-#if BYTE_ORDER == BIG_ENDIAN
-#define WORDS_BIGENDIAN 1
-#elif BYTE_ORDER == LITTLE_ENDIAN
-/* nothing */
-#elif BYTE_ORDER == PDP_ENDIAN
-#error sorry, PDP endian not supported (yet)
-#else
-#error sorry, unknown endianess in <machine/endian.h>
+#if ENDIANESS_IN_SYS_PARAM_H
+#  undef WORDS_BIGENDIAN
+#  include <sys/types.h>
+#  include <sys/param.h>
+#  if BYTE_ORDER == BIG_ENDIAN
+#  define WORDS_BIGENDIAN 1
+#  elif BYTE_ORDER == LITTLE_ENDIAN
+   /* ok */
+#  else
+#  error where do you cut your eggs?
+#  endif
 #endif
- 
-#endif /* HAVE_MACHINE_ENDIAN_H */
 
-RCSID("$Id: rxk_crpt.c,v 1.9 1999/07/20 13:59:32 art Exp $");
+RCSID("$Id: rxk_crpt.c,v 1.12 2000/10/03 00:38:27 lha Exp $");
 
 /*
  * Unrolling of the inner loops helps the most on pentium chips
