@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -53,13 +48,14 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: create-files.c,v 1.1 1999/07/14 11:59:39 assar Exp $");
+RCSID("$Id: create-files.c,v 1.6 2000/10/03 00:33:23 lha Exp $");
 #endif
 
 static int
-creat_files (int count)
+creat_files (int count, long startsize)
 {
     int i;
+    long size = 0;
     
     for (i = 0; i < count; ++i) {
 	char num[17];
@@ -67,9 +63,24 @@ creat_files (int count)
 
 	snprintf (num, sizeof(num), "%d", i);
 	
-	fd = open (num, O_CREAT | O_EXCL, 0777);
+	fd = open (num, O_WRONLY | O_CREAT | O_EXCL, 0777);
 	if (fd < 0)
 	    err (1, "open %s", num);
+	size = startsize;
+	while (size > 0) {
+	    char buf[8192];
+	    size_t len;
+	    ssize_t ret;
+
+	    len = min(sizeof(buf), size);
+
+	    ret = write (fd, buf, len);
+	    if (ret < 0)
+		err (1, "write to %s", num);
+	    if (ret != len)
+		errx (1, "short write to %s", num);
+	    size -= ret;
+	}
 	if (close (fd) < 0)
 	    err (1, "close %s", num);
     }
@@ -79,7 +90,7 @@ creat_files (int count)
 static void
 usage (int ret)
 {
-    fprintf (stderr, "%s number-of-files\n", __progname);
+    fprintf (stderr, "%s number-of-files size-of-files\n", __progname);
     exit (ret);
 }
 
@@ -88,15 +99,20 @@ main(int argc, char **argv)
 {
     char *ptr;
     int count;
+    long size;
 
     set_progname (argv[0]);
 
-    if (argc != 2)
+    if (argc != 3)
 	usage (1);
 
     count = strtol (argv[1], &ptr, 0);
     if (count == 0 && ptr == argv[1])
 	errx (1, "'%s' not a number", argv[1]);
 
-    return creat_files (count);
+    size = strtol (argv[2], &ptr, 0);
+    if (size == 0 && ptr == argv[2])
+	errx (1, "`%s' not a number", argv[2]);
+
+    return creat_files (count, size);
 }
