@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -53,7 +48,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: large-dir2.c,v 1.1 1999/06/20 00:36:25 assar Exp $");
+RCSID("$Id: large-dir2.c,v 1.3 2000/10/03 00:34:37 lha Exp $");
 #endif
 
 static int
@@ -73,8 +68,10 @@ creat_files (const char *dirname, int count)
 	err (1, "chdir %s", dirname);
     if (stat (".", &sb) < 0)
 	err (1, "stat .");
+#if 0
     if (sb.st_size != 2048)
 	errx (1, "size != 2048");
+#endif
     for (i = 0; i < count; ++i) {
 	char fname[256];
 	int len;
