@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -50,7 +45,7 @@
 #include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: mkdir2.c,v 1.1 1999/07/04 09:29:32 assar Exp $");
+RCSID("$Id: mkdir2.c,v 1.3 2000/10/03 00:34:47 lha Exp $");
 #endif
 
 int
@@ -59,6 +54,8 @@ main(int argc, char *argv[])
     int ret;
     struct stat dot_sb, sb;
 
+    set_progname (argv[0]);
+
     ret = mkdir ("foo", 0777);
     if (ret < 0)
 	err (1, "mkdir foo");
