@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -43,20 +38,23 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <time.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <err.h>
+#include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: read-vs-mmap.c,v 1.4 1999/07/14 11:58:14 assar Exp $");
+RCSID("$Id: read-vs-mmap.c,v 1.9 2000/10/03 00:35:18 lha Exp $");
 #endif
 
+static int debug = 0;
+
 static void
-generate_random_file (const char *filename, size_t sz)
+generate_file (const char *filename, int randomp, size_t sz)
 {
     int fd;
     char *buf;
@@ -71,7 +69,10 @@ generate_random_file (const char *filename, size_t sz)
 	err (1, "open %s", filename);
 
     for (i = 0; i < sz; ++i)
-	buf[i] = rand();
+	if (randomp)
+	    buf[i] = rand();
+	else
+	    buf[0] = 0;
 
     if (write (fd, buf, sz) != sz)
 	err (1, "write");
@@ -108,19 +109,25 @@ mmap_file (int fd, size_t sz)
     return ret;
 }
 
-int
-main (int argc, char **argv)
+static void __attribute__ ((__unused__))
+print_area (unsigned char *ptr,  size_t len)
 {
-    const char *file = "foo";
-    const size_t sz  = 16384;
+    while (len--) {
+	printf ("%x", *ptr);
+	ptr++;
+    }
+}
 
+static int
+do_test (int randomp)
+{
     char *malloc_buf;
     void *mmap_buf;
     int fd;
+    const char *file = "foo";
+    const size_t sz  = 16384;
 
-    srand (time(NULL));
-
-    generate_random_file (file, sz);
+    generate_file (file, randomp, sz);
 
     fd = open (file, O_RDONLY, 0);
     if (fd < 0)
@@ -130,7 +137,34 @@ main (int argc, char **argv)
     mmap_buf   = mmap_file (fd, sz);
     close (fd);
     unlink (file);
-    if (memcmp (malloc_buf, mmap_buf, sz) != 0)
+    if (memcmp (malloc_buf, mmap_buf, sz) != 0) {
+	if (debug) {
+	    printf ("type: %s\n", randomp ? "random" : "allzero");
+	    printf ("read: ");
+	    print_area (malloc_buf, sz);
+	    printf ("\nmmap: ");
+	    print_area (mmap_buf, sz);
+	    printf ("\n");
+	}
 	return 1;
+    }
+    return 0;
+}
+
+int
+main (int argc, char **argv)
+{
+    set_progname (argv[0]);
+
+    if (argc != 1)
+	debug = 1;
+
+    srand (time(NULL));
+
+    if (do_test (0))
+	return 1;
+    if (do_test (1))
+	return 2;
+
     return 0;
 }
