@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -43,16 +38,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <sys/fcntl.h>
+#include <fcntl.h>
 #include <time.h>
 #include <unistd.h>
 #include <string.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <err.h>
+#include <roken.h>
 
 #ifdef RCSID
-RCSID("$Id: read-vs-mmap2.c,v 1.4 1999/07/14 11:58:40 assar Exp $");
+RCSID("$Id: read-vs-mmap2.c,v 1.8 2000/10/03 00:35:24 lha Exp $");
 #endif
 
 static void
@@ -113,11 +109,12 @@ main (int argc, char **argv)
 {
     const char *file = "foo";
     const size_t sz  = 16384;
-
     char *malloc_buf;
     void *mmap_buf;
     int fd;
 
+    set_progname (argv[0]);
+
     srand (time(NULL));
 
     generate_random_file (file, sz);
