@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -51,12 +46,15 @@
 #include <err.h>
 #include <roken.h>
 
+#include <atypes.h>
 #include <kafs.h>
 
 #ifdef RCSID
-RCSID("$Id: test-setpag.c,v 1.2 1999/06/20 02:15:53 assar Exp $");
+RCSID("$Id: test-setpag.c,v 1.5 2000/10/03 00:36:21 lha Exp $");
 #endif
 
+#ifdef KERBEROS
+
 #if !defined(NGROUPS) && defined(NGROUPS_MAX)
 #define NGROUPS NGROUPS_MAX
 #endif
@@ -127,3 +125,15 @@ main(int argc, char **argv)
 	    return 1;
     }
 }
+
+#else /* !KERBEROS */
+
+int
+main (int argc, char **argv)
+{
+    set_progname (argv[0]);
+
+    errx (1, "no kafs");
+}
+
+#endif /* !KERBEROS */
