@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -44,8 +39,11 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include <err.h>
+#include <roken.h>
+
 #ifdef RCSID
-RCSID("$Id: write-ro-file.c,v 1.3 1999/05/28 08:46:51 assar Exp $");
+RCSID("$Id: write-ro-file.c,v 1.6 2000/10/03 00:36:48 lha Exp $");
 #endif
 
 int
@@ -54,22 +52,20 @@ main(int argc, char *argv[])
   int fd;
   int ret;
 
+  set_progname (argv[0]);
+
   fd = open("foo", O_RDWR|O_CREAT, 0);
-  if (fd == -1) {
-    perror("open");
-    exit(1);
-  }
+  if (fd < 0)
+      err (1, "open foo");
   ret = write (fd, "foo", 3);
-  if (ret == -1) {
+  if (ret < 0) {
       unlink("foo");
-      perror("write");
-      exit(1);
+      err (1, "write foo");
   }
   ret = close (fd);
-  if (ret == -1) {
+  if (ret < 0) {
       unlink("foo");
-      perror("close");
-      exit(1);
+      err (1, "close foo");
   }
   unlink("foo");
   return 0;
