@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,21 +35,34 @@
  * Boolean
  */
 
-/* $Id: bool.h,v 1.2 1997/11/09 23:48:45 assar Exp $ */
+/* $Id: bool.h,v 1.4 2000/10/03 00:30:38 lha Exp $ */
 
 #ifndef _BOOL_
 #define _BOOL_
 
 #ifndef FALSE
 
+#ifndef Bool
+
 typedef enum {
   FALSE = 0, TRUE = 1
 } Bool;
 
+#else /* Bool */
+
+#define FALSE	0
+#define TRUE	1
+
+#endif /* Bool */
+
 #else /* FALSE */
 
+#ifndef Bool
+
 typedef int Bool;
 
-#endif
+#endif /* Bool */
+
+#endif /* FALSE */
 
 #endif /* _BOOL_ */
