@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -37,7 +37,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: heap.c,v 1.2 2000/10/03 00:30:53 lha Exp $");
+RCSID("$Id: heap.c,v 1.2.2.1 2001/03/12 16:49:07 lha Exp $");
 #endif
 
 #include <assert.h>
@@ -63,7 +63,7 @@ heap_new (unsigned sz, heap_cmp_fn cmp)
     ret->max_sz = sz;
     ret->sz     = 0;
     ret->data   = malloc (sz * sizeof(*ret->data));
-    if (ret->data == NULL) {
+    if (ret->sz != 0 && ret->data == NULL) {
 	free (ret);
 	return NULL;
     }
