@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: log.h,v 1.8 2000/10/03 00:31:25 lha Exp $ */
+/* $Id: log.h,v 1.8.2.1 2001/04/29 23:53:31 lha Exp $ */
 
 #ifndef _LOG_
 #define _LOG_
@@ -39,9 +39,9 @@
 #include <stdarg.h>
 #include <parse_units.h>
 
-#if HAVE_SYSLOG
-#include <syslog.h>
-#endif /* HAVE_SYSLOG */
+typedef enum {
+    LOG_CPU_USAGE = 1
+} log_flags;
 
 typedef struct log_method Log_method;
 typedef struct log_unit Log_unit;
@@ -62,6 +62,9 @@ Log_unit *log_unit_init (Log_method *method, const char *name,
 
 void log_unit_free (Log_method *method, Log_unit *log);
 
+log_flags log_setflags(Log_method *log, log_flags flags);
+log_flags log_getflags(Log_method *log);
+
 void log_log (Log_unit *log, unsigned level, const char *fmt, ...)
 __attribute__((format (printf, 3, 4)))
 ;
