@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_fs.h,v 1.2 1998/12/22 13:16:09 lha Exp $ */
+/* $Id: xfs_fs.h,v 1.5.2.1 2001/01/30 02:07:40 lha Exp $ */
 
 #ifndef _xfs_h
 #define _xfs_h
@@ -88,11 +83,11 @@ void
 xfs_dnlc_remove(vnode_t *dvp, char *name);
 
 void
-xfs_attr2vattr (const struct xfs_attr *xa, struct vattr *va);
+xfs_attr2vattr (const struct xfs_attr *xa, struct vattr *va, int clear_node);
 
 void
 vattr2xfs_attr (const struct vattr *va, struct xfs_attr *xa);
 
-int xfs_has_pag(const struct xfs_node *xn, pag_t);
+int xfs_has_pag(const struct xfs_node *xn, xfs_pag_t);
 
 #endif /* _xfs_h */
