@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_node.h,v 1.2 1998/12/22 13:16:09 lha Exp $ */
+/* $Id: xfs_node.h,v 1.5 2000/10/02 23:27:30 lha Exp $ */
 
 #ifndef _xfs_xnode_h
 #define _xfs_xnode_h
@@ -56,7 +51,7 @@ struct xfs_node {
   u_int flags;
   u_int tokens;
   xfs_handle handle;
-  pag_t id[MAXRIGHTS];
+  xfs_pag_t id[MAXRIGHTS];
   u_char rights[MAXRIGHTS];
   u_char anonrights;
 
