@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_syscalls.h,v 1.2 1998/12/22 13:16:10 lha Exp $ */
+/* $Id: xfs_syscalls.h,v 1.4 2000/10/02 23:27:35 lha Exp $ */
 
 #ifndef  __xfs_syscalls
 #define  __xfs_syscalls
@@ -44,6 +39,6 @@
 #include <xfs/xfs_common.h>
 #include <xfs/xfs_message.h>
 
-pag_t xfs_get_pag(struct ucred *);
+xfs_pag_t xfs_get_pag(struct ucred *);
 
 #endif /*  __xfs_syscalls */
