@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1998 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -44,7 +39,7 @@
 #include <xfs/xfs_deb.h>
 #include <xfs/nxfs.h>
 
-RCSID("$Id: xfs_dev.c,v 1.4 1998/12/22 13:16:02 lha Exp $");
+RCSID("$Id: xfs_dev.c,v 1.7 2000/10/02 23:20:00 lha Exp $");
 
 /*
  * Queues of xfs_links hold outbound messages and processes sleeping
@@ -242,6 +237,9 @@ xfs_devread(dev_t dev, struct uio *uiop, chan_t foo_chan, int ext)
 
       ASSERT(first->message->size != 0);
 
+      if (first->message->size > uiop->uio_resid)
+	  break;
+
       error = uiomove((caddr_t) first->message, first->message->size,
 		      UIO_READ, uiop);
       if (error)
@@ -537,6 +535,12 @@ xfs_message_receive(int fd, struct xfs_message_header *message, u_int size)
     return xfs_message_installdata(fd, (struct xfs_message_installdata *) message, message->size);
   case XFS_MSG_INVALIDNODE:
     return xfs_message_invalidnode(fd, (struct xfs_message_invalidnode *) message, message->size);
+  case XFS_MSG_UPDATEFID:
+    return xfs_message_updatefid(fd, (struct xfs_message_updatefid *)message, message->size);
+  case XFS_MSG_GC_NODES:
+    return xfs_message_gc_nodes(fd, (struct xfs_message_gc_nodes *)message, message->size);
+  case XFS_MSG_VERSION:
+    return xfs_message_version(fd, (struct xfs_message_version *)message, message->size);
   default:
     printf("XFS PANIC Warning xfs_dev: Unknown message opcode == %d\n", message->opcode);
     return EINVAL;
