@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,21 +31,27 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_common.h,v 1.8.6.1 1999/08/08 16:32:13 assar Exp $ */
+/* $Id: xfs_common.h,v 1.14 2000/10/17 18:11:56 lha Exp $ */
 
 #ifndef _xfs_common_h
 #define _xfs_common_h
 
+#if defined(MALLOC_DECLARE)
+MALLOC_DECLARE(M_XFS);
+#elif !defined(M_XFS)
+#define M_XFS M_TEMP
+#endif
+
 #ifdef XFS_DEBUG
 void *xfs_alloc(u_int size);
 void xfs_free(void *, u_int size);
 #else
 #ifdef __osf__
-#define xfs_alloc(a) malloc((a), BUCKETINDEX(a), M_TEMP, M_WAITOK)
+#define xfs_alloc(a) malloc((a), BUCKETINDEX(a), M_XFS, M_WAITOK)
 #else
-#define xfs_alloc(a) malloc((a), M_TEMP, M_WAITOK)
+#define xfs_alloc(a) malloc((a), M_XFS, M_WAITOK)
 #endif
-#define xfs_free(a, size) free(a, M_TEMP)
+#define xfs_free(a, size) free(a, M_XFS)
 #endif /* XFS_DEBUG */
 
 int xfs_suser(struct proc *p);
@@ -60,4 +61,12 @@ void *
 memcpy (void *s1, const void *s2, size_t n);
 #endif
 
+const char *
+xfs_devtoname_r (dev_t dev, char *buf, size_t sz);
+
+#ifndef HAVE_KERNEL_STRLCPY
+size_t
+strlcpy (char *dst, const char *src, size_t dst_sz);
+#endif
+
 #endif /* _xfs_common_h */
