@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_dev.h,v 1.11 1999/06/03 02:08:41 assar Exp $ */
+/* $Id: xfs_dev.h,v 1.15.2.1 2001/09/30 16:14:31 ahltorp Exp $ */
 
 #ifndef _xfs_dev_h
 #define _xfs_dev_h
@@ -68,6 +63,7 @@ struct xfs_channel {
     int status;
 #define CHANNEL_OPENED	0x1
 #define CHANNEL_WAITING 0x2
+    struct proc *proc;
 };
 
 extern struct xfs_channel xfs_channel[NXFS];
@@ -96,10 +92,10 @@ void
 xfs_initq(struct xfs_link *q);
 
 int
-xfs_emptyq(struct xfs_link *q);
+xfs_emptyq(const struct xfs_link *q);
 
 int
-xfs_onq(struct xfs_link *link);
+xfs_onq(const struct xfs_link *link);
 
 void
 xfs_appendq(struct xfs_link *q, struct xfs_link *p);
@@ -108,19 +104,18 @@ void
 xfs_outq(struct xfs_link *p);
 
 int
-xfs_devopen_common(dev_t dev);
+xfs_devopen_common(dev_t dev, struct proc *);
 
 #ifndef __osf__ /* XXX - we should do the same for osf */
 int xfs_devopen(dev_t dev, int flag, int devtype, struct proc *proc);
 int xfs_devclose(dev_t dev, int flag, int devtype, struct proc *proc);
-int xfs_devioctl(dev_t dev,
-#if  defined(__FreeBSD__) && __FreeBSD_version < 400005
-		 int cmd,
-#else /* {net,open}bsd */
-		 u_long cmd,
-#endif
-		 caddr_t data, int flags, struct proc *p);
+int xfs_devioctl(dev_t dev, u_long cmd, caddr_t data, int flags,
+		 struct proc *p);
+#ifdef HAVE_THREE_ARGUMENT_SELRECORD
+int xfs_devselect(dev_t dev, int which, void *wql, struct proc *p);
+#else
 int xfs_devselect(dev_t dev, int which, struct proc *p);
+#endif
 #endif /* ! __osf__ */
 
 int
@@ -136,7 +131,8 @@ int
 xfs_message_send(int fd, struct xfs_message_header * message, u_int size);
 
 int
-xfs_message_rpc(int fd, struct xfs_message_header * message, u_int size);
+xfs_message_rpc(int fd, struct xfs_message_header * message, u_int size,
+		struct proc *p);
 
 int
 xfs_message_receive(int fd,
