@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_fs.h,v 1.14 1999/02/26 00:44:54 lha Exp $ */
+/* $Id: xfs_fs.h,v 1.18.2.1 2001/01/25 08:42:11 lha Exp $ */
 
 #ifndef _xfs_h
 #define _xfs_h
@@ -62,7 +57,11 @@ struct xfs {
 };
 
 #ifdef __osf__
+#ifdef HAVE_STRUCT_MOUNT_M_INFO
+#define VFS_TO_XFS(v)      ((struct xfs *) ((v)->m_info))
+#else
 #define VFS_TO_XFS(v)      ((struct xfs *) ((v)->m_data))
+#endif
 #else
 #define VFS_TO_XFS(v)      ((struct xfs *) ((v)->mnt_data))
 #endif
@@ -88,18 +87,9 @@ void xfs_dnlc_purge(struct vnode *);
 int xfs_dnlc_lookup(struct vnode *, xfs_componentname *, struct vnode **);
 int xfs_dnlc_lookup_name(struct vnode *, const char *, struct vnode **);
 
-void
-xfs_cnp_init (xfs_componentname *ndp,
-	      const char *name,
-	      struct vnode *vp,
-	      struct vnode *dvp,
-	      struct proc *proc,
-	      struct ucred *cred,
-	      int nameiop);
-
 void vattr2xfs_attr(const struct vattr *, struct xfs_attr *);
-void xfs_attr2vattr(const struct xfs_attr *, struct vattr *);
+void xfs_attr2vattr(const struct xfs_attr *, struct vattr *, int);
 
-int xfs_has_pag(const struct xfs_node *, pag_t);
+int xfs_has_pag(const struct xfs_node *, xfs_pag_t);
 
 #endif				       /* _xfs_h */
