@@ -31,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_locl.h,v 1.42 2000/10/04 00:02:54 lha Exp $ */
+/* $Id: xfs_locl.h,v 1.42.2.3 2001/05/21 23:50:38 map Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -106,7 +106,7 @@ typedef struct nameidata xfs_componentname;
 #define xfs_uio_to_proc(uiop) (u.u_procp)
 #define xfs_cnp_to_proc(cnp) (u.u_procp)
 #define xfs_proc_to_cred(p) ((p)->p_rcred)
-#define xfs_proc_to_ruid(p) ((p)->p_ruid)
+#define xfs_proc_to_euid(p) ((p)->p_rcred->cr_uid)
 
 #define xfs_curproc() (u.u_procp)
 
@@ -117,6 +117,8 @@ typedef struct nameidata xfs_componentname;
 
 struct vop_generic_args;
 
+typedef u_long va_size_t;
+
 #else /* !__osf__ */
 
 typedef struct componentname xfs_componentname;
@@ -203,7 +205,7 @@ void cache_purgevfs(struct mount *);
 #define xfs_uio_to_proc(uiop) ((uiop)->uio_procp)
 #define xfs_cnp_to_proc(cnp) ((cnp)->cn_proc)
 #define xfs_proc_to_cred(p) ((p)->p_ucred)
-#define xfs_proc_to_ruid(p) ((p)->p_cred->p_ruid)
+#define xfs_proc_to_euid(p) ((p)->p_ucred->cr_uid)
 
 #ifdef __APPLE__
 #define xfs_curproc() (current_proc())
@@ -216,6 +218,8 @@ void cache_purgevfs(struct mount *);
 #define xfs_vop_getattr(t, attr, cred, proc, error) (error) = VOP_GETATTR((t), (attr), (cred), (proc))
 #define xfs_vop_access(dvp, mode, cred, proc, error) (error) = VOP_ACCESS((dvp), (mode), (cred), (proc))
 
+typedef u_quad_t va_size_t;
+
 #endif /* !__osf__ */
 
 /*
