@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* 	$Id: xfs_node.h,v 1.13 1999/04/30 01:25:52 art Exp $	 */
+/* 	$Id: xfs_node.h,v 1.22 2000/10/02 23:34:44 lha Exp $	 */
 
 #ifndef _xfs_xnode_h
 #define _xfs_xnode_h
@@ -53,6 +48,12 @@
 #include <xfs/xfs_attr.h>
 #include <xfs/xfs_message.h>
 
+#ifdef __APPLE__
+typedef struct lock__bsd__ xfs_vnode_lock;
+#else
+typedef struct lock xfs_vnode_lock;
+#endif
+
 struct xfs_node {
     struct vnode *vn;
     struct vnode *data;
@@ -60,13 +61,18 @@ struct xfs_node {
     u_int flags;
     u_int tokens;
     xfs_handle handle;
-    pag_t id[MAXRIGHTS];
+    xfs_pag_t id[MAXRIGHTS];
     u_char rights[MAXRIGHTS];
     u_char anonrights;
+#if defined(HAVE_KERNEL_LOCKMGR) || defined(HAVE_KERNEL_DEBUGLOCKMGR)
+    xfs_vnode_lock lock;
+#else
     int vnlocks;
+#endif
 #ifdef __NetBSD__
     struct   lockf *i_lockf;
 #endif
+    struct ucred *cred;
 };
 
 int xfs_getnewvnode(struct mount *mp, struct vnode **vpp,
@@ -80,12 +86,14 @@ int xfs_getnewvnode(struct mount *mp, struct vnode **vpp,
 #define XNODE_TO_VNODE(xp) ((xp)->vn)
 #define VNODE_TO_XNODE(vp) ((struct xfs_node *) (vp)->v_data)
 
-#if defined(HAVE_THREE_ARGUMENT_VGET)
-#define xfs_do_vget(vp, lockflag, proc) vget((vp), (lockflag), (proc))
-#elif !defined(__osf__)
-#define xfs_do_vget(vp, lockflag, proc) vget((vp), (lockflag))
-#else
+#if defined(HAVE_ONE_ARGUMENT_VGET)
 #define xfs_do_vget(vp, lockflag, proc) vget((vp))
+#elif defined(HAVE_TWO_ARGUMENT_VGET)
+#define xfs_do_vget(vp, lockflag, proc) vget((vp), (lockflag))
+#elif defined(HAVE_THREE_ARGUMENT_VGET)
+#define xfs_do_vget(vp, lockflag, proc) vget((vp), (lockflag), (proc))
+#else
+#error what kind of vget
 #endif
 
 #ifndef HAVE_VOP_T
