@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,33 +31,15 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_syscalls.h,v 1.18 1999/08/01 10:36:50 lha Exp $ */
+/* $Id: xfs_syscalls.h,v 1.25 2000/10/02 23:34:49 lha Exp $ */
 
 #ifndef  __xfs_syscalls
 #define  __xfs_syscalls
 
 #include <xfs/xfs_common.h>
 #include <xfs/xfs_message.h>
-#ifdef HAVE_SYS_SYSCALLARGS_H
-#include <sys/syscallargs.h>
-#endif
 
-/*
- * XXX
- */
-
-#ifndef SCARG
-#define SCARG(a, b) ((a)->b.datum)
-#define syscallarg(x)   union { x datum; register_t pad; }
-#endif
-
-#ifndef syscallarg
-#define syscallarg(x)   x
-#endif
-
-#ifndef HAVE_REGISTER_T
-typedef int register_t;
-#endif
+#include <afssysdefs.h>
 
 struct sys_pioctl_args {
     syscallarg(int) operation;
@@ -89,7 +66,7 @@ struct xfs_fh_args {
 int xfs_install_syscalls(void);
 int xfs_uninstall_syscalls(void);
 int xfs_stat_syscalls(void);
-pag_t xfs_get_pag(struct ucred *);
+xfs_pag_t xfs_get_pag(struct ucred *);
 
 int xfs_setpag_call(struct ucred **ret_cred);
 int xfs_pioctl_call(struct proc *proc,
@@ -102,5 +79,15 @@ int xfs_setgroups (struct proc *p,
 		   void *varg);
 
 extern int (*old_setgroups_func)(struct proc *, void *);
+extern int xfs_syscall_num; /* The old syscall number */
+
+
+#ifndef HAVE_KERNEL_SYS_LKMNOSYS
+#define sys_lkmnosys nosys
+#endif
+
+#ifndef SYS_MAXSYSCALL
+#define SYS_MAXSYSCALL nsysent
+#endif
 
 #endif				       /* __xfs_syscalls */
