@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -41,7 +36,7 @@
 #include <xfs/xfs_common.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_common-osf.c,v 1.8 1999/05/15 16:14:33 assar Exp $");
+RCSID("$Id: xfs_common-osf.c,v 1.11 2000/10/02 23:31:42 lha Exp $");
 
 static u_int xfs_allocs;
 static u_int xfs_frees;
@@ -52,14 +47,14 @@ xfs_alloc(u_int size)
     xfs_allocs++;
     XFSDEB(XDEBMEM, ("xfs_alloc: xfs_allocs-xfs_frees %d\n", 
 		     xfs_allocs - xfs_frees));
-    return malloc(size, BUCKETINDEX(size), M_TEMP, M_WAITOK); /* What kind? */
+    return malloc(size, BUCKETINDEX(size), M_XFS, M_WAITOK);
 }
 
 void
 xfs_free(void *ptr, u_int size)
 {
     xfs_frees++;
-    free(ptr, M_TEMP);
+    free(ptr, M_XFS);
 }
 #endif
 
@@ -68,3 +63,9 @@ xfs_suser(struct proc *p)
 {
     return suser (xfs_proc_to_cred(p), NULL);
 }
+
+const char *
+xfs_devtoname_r (dev_t dev, char *buf, size_t sz)
+{
+    return "<unknown device>";
+}
