@@ -38,7 +38,7 @@
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_dev-bsd.c,v 1.28 2000/10/31 16:48:36 assar Exp $");
+RCSID("$Id: xfs_dev-bsd.c,v 1.28.2.1 2001/09/30 16:13:55 ahltorp Exp $");
 
 int
 xfs_devopen(dev_t dev, int flag, int devtype, struct proc *proc)
@@ -80,14 +80,18 @@ xfs_devioctl(dev_t dev,
 }
 
 static int
-xfs_realselect(dev_t dev, struct proc *p)
+xfs_realselect(dev_t dev, struct proc *p, void *wql)
 {
     struct xfs_channel *chan = &xfs_channel[minor(dev)];
 
     if (!xfs_emptyq(&chan->messageq))
 	return 1;		       /* Something to read */
 
+#ifdef HAVE_THREE_ARGUMENT_SELRECORD
+    selrecord (p, &chan->selinfo, wql);
+#else
     selrecord (p, &chan->selinfo);
+#endif
     return 0;
 }
 
@@ -105,12 +109,24 @@ xfs_devpoll(dev_t dev, int events, struct proc * p)
     if (!(events & POLLRDNORM))
 	return 0;
 
-    return xfs_realselect(dev, p);
+    return xfs_realselect(dev, p, NULL);
 }
 
 #endif
 
 #ifdef HAVE_VOP_SELECT
+#ifdef HAVE_THREE_ARGUMENT_SELRECORD
+int
+xfs_devselect(dev_t dev, int which, void *wql, struct proc * p)
+{
+    XFSDEB(XDEBDEV, ("xfs_devselect dev = %d, which = %d\n", dev, which));
+
+    if (which != FREAD)
+	return 0;
+
+    return xfs_realselect(dev, p, wql);
+}
+#else
 int
 xfs_devselect(dev_t dev, int which, struct proc * p)
 {
@@ -119,9 +135,9 @@ xfs_devselect(dev_t dev, int which, struct proc * p)
     if (which != FREAD)
 	return 0;
 
-    return xfs_realselect(dev, p);
+    return xfs_realselect(dev, p, NULL);
 }
-
+#endif
 #endif
 
 void
