@@ -39,7 +39,7 @@
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_dev-common.c,v 1.44 2001/01/07 03:06:53 lha Exp $");
+RCSID("$Id: xfs_dev-common.c,v 1.44.2.1 2001/07/15 03:55:37 assar Exp $");
 
 struct xfs_channel xfs_channel[NXFS];
 
@@ -399,7 +399,7 @@ xfs_message_rpc(int fd, struct xfs_message_header * message, u_int size,
     struct xfs_link *this_message;
     struct xfs_link *this_process;
     struct xfs_message_header *msg;
-#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX)
+#if defined(HAVE_STRUCT_PROC_P_SIGMASK) || defined(HAVE_STRUCT_PROC_P_SIGCTX) || defined(__osf__)
     sigset_t oldsigmask;
 #endif
     int catch;
@@ -451,16 +451,24 @@ xfs_message_rpc(int fd, struct xfs_message_header * message, u_int size,
      */
 
 #ifdef HAVE_STRUCT_PROC_P_SIGMASK
+    /* NetBSD 1.5, Darwin 1.3, FreeBSD 4.3, 5.0, OpenBSD 2.8 */
     oldsigmask = proc->p_sigmask;
     xfs_block_sigset (&proc->p_sigmask);
 #elif defined(HAVE_STRUCT_PROC_P_SIGCTX)
+    /* NetBSD 1.6 */
     oldsigmask = proc->p_sigctx.ps_sigmask;
     xfs_block_sigset (&proc->p_sigctx.ps_sigmask);
 #elif defined(HAVE_STRUCT_PROC_P_SIGWAITMASK)
+    /* OSF 4.0 */
     oldsigmask = proc->p_sigwaitmask;
     sigaddset(&proc->p_sigwaitmask, SIGIO);
     sigaddset(&proc->p_sigwaitmask, SIGALRM);
     sigaddset(&proc->p_sigwaitmask, SIGVTALRM);
+#elif defined(__osf__)
+    oldsigmask = u.u_sigmask;
+    sigaddset(&u.u_sigmask, SIGIO);
+    sigaddset(&u.u_sigmask, SIGALRM);
+    sigaddset(&u.u_sigmask, SIGVTALRM);
 #endif
 
     /*
@@ -495,6 +503,8 @@ xfs_message_rpc(int fd, struct xfs_message_header * message, u_int size,
     proc->p_sigctx.ps_sigmask = oldsigmask;
 #elif defined(HAVE_STRUCT_PROC_P_SIGWAITMASK)
     proc->p_sigwaitmask = oldsigmask;
+#elif defined(__osf__)
+    u.u_sigmask = oldsigmask;
 #endif
 
     /*
