@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -44,14 +39,14 @@
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_dev-osf.c,v 1.6 1999/06/03 00:11:50 assar Exp $");
+RCSID("$Id: xfs_dev-osf.c,v 1.9 2000/10/02 23:32:03 lha Exp $");
 
 static int
 xfs_devopen(dev_t dev, int flag, int format)
 {
     XFSDEB(XDEBDEV, ("xfsopen dev = %d.%d, flag = %d, format = %d\n", 
 		     major(dev), minor(dev), flag, format));
-    return xfs_devopen_common(dev);
+    return xfs_devopen_common(dev, u.u_procp);
 }
 
 static int
@@ -71,7 +66,7 @@ xfs_devioctl(dev_t dev,
 	     unsigned int cmd, caddr_t data, int flags)
 {
     XFSDEB(XDEBDEV, ("xfs_devioctl dev = %d.%d, cmd = %lu, "
-		     "data = %p, flags = %x\n", 
+		     "data = %lx, flags = %x\n", 
 		     major(dev), minor(dev), (unsigned long)cmd, data, flags));
     return EINVAL;
 }
