@@ -33,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-common.c,v 1.59 2001/01/07 00:19:07 lha Exp $");
+RCSID("$Id: xfs_syscalls-common.c,v 1.59.2.2 2001/06/04 19:27:12 mattiasa Exp $");
 
 /*
  * XFS system calls.
@@ -389,7 +389,7 @@ fhget_call (struct proc *p,
     if (vp == NULL)
 	return EBADF;
 
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__osf__)
     error = EINVAL; /* XXX: Leaks vnodes if fhget/fhopen is used */
     goto out;
 #endif
@@ -427,7 +427,7 @@ fhopen_call (struct proc *p,
 	return EINVAL;
     }
 
-#ifdef __APPLE__
+#if defined(__APPLE__) || defined(__osf__)
     return EINVAL; /* XXX: Leaks vnodes if fhget/fhopen is used */
 #endif
 
@@ -485,7 +485,7 @@ remote_pioctl (struct proc *p,
 
     msg.insize = vice_ioctl->in_size;
     msg.outsize = vice_ioctl->out_size;
-    msg.cred.uid = xfs_proc_to_ruid(p);
+    msg.cred.uid = xfs_proc_to_euid(p);
     msg.cred.pag = xfs_get_pag(xfs_proc_to_cred(p));
 
     error = xfs_message_rpc(0, &msg.header, sizeof(msg), p); /* XXX */
