@@ -33,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.11 2000/12/12 11:01:09 lha Exp $");
+RCSID("$Id: xfs_syscalls-wrap-bsd.c,v 1.11.2.1 2001/03/04 04:39:27 lha Exp $");
 
 /*
  * XFS system calls.
@@ -55,6 +55,9 @@ static struct sysent old_syscallent;
 static struct sysent xfs_syscallent = {
     5,				       /* number of args */
     sizeof(struct sys_pioctl_args),    /* size of args */
+#if HAVE_STRUCT_SYSENT_SY_FLAGS
+    0,
+#endif
     xfspioctl			       /* function pointer */
 };
 
