@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls-wrap-freebsd.c,v 1.3 1999/05/27 13:51:51 assar Exp $");
+RCSID("$Id: xfs_syscalls-wrap-freebsd.c,v 1.7 2000/10/02 23:32:40 lha Exp $");
 
 /*
  * XFS system calls.
@@ -117,7 +112,7 @@ xfs_stat_syscalls(void)
  * Where it's stored.
  */
 
-static int syscall_offset;
+int xfs_syscall_num;
 
 static struct sysent old_syscallent;
 
@@ -150,7 +145,7 @@ install_first_free_syscall (int *offset,
     int i;
 
     for (i = 1; i < SYS_MAXSYSCALL; ++i)
-	if (try_install_syscall (i, new_sysent, old_sysent)) {
+	if (try_install_syscall (i, new_sysent, old_sysent) == 0) {
 	    *offset = i;
 	    return 0;
 	}
@@ -177,15 +172,15 @@ xfs_install_syscalls(void)
 				  xfs_syscallent,
 				  &old_syscallent);
 	if (ret == 0)
-	    syscall_offset = AFS_SYSCALL;
+	    xfs_syscall_num = AFS_SYSCALL;
     }
 #endif
     if (ret != 0)
-	ret = install_first_free_syscall (&syscall_offset,
+	ret = install_first_free_syscall (&xfs_syscall_num,
 					  xfs_syscallent,
 					  &old_syscallent);
     if (ret == 0)
-	XFSDEB(XDEBSYS, ("xfs_syscall installed at %d\n", syscall_offset));
+	XFSDEB(XDEBSYS, ("xfs_syscall installed at %d\n", xfs_syscall_num));
     else
 	XFSDEB(XDEBSYS, ("failed installing xfs_syscall\n"));
 
@@ -200,8 +195,8 @@ xfs_install_syscalls(void)
 int
 xfs_uninstall_syscalls(void)
 {
-    if (syscall_offset)
-	sysent[syscall_offset] = old_syscallent;
+    if (xfs_syscall_num)
+	sysent[xfs_syscall_num] = old_syscallent;
     sysent[SYS_setgroups] = old_setgroups;
     return 0;
 }
