@@ -33,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_vfsops-common.c,v 1.30 2000/10/02 23:32:57 lha Exp $");
+RCSID("$Id: xfs_vfsops-common.c,v 1.30.2.1 2001/04/27 23:43:12 mattiasa Exp $");
 
 /*
  * XFS vfs operations.
@@ -145,11 +145,13 @@ xfs_mount_common(struct mount *mp,
     xfs[minor(dev)].fd = minor(dev);
 
     VFS_TO_XFS(mp) = &xfs[minor(dev)];
+#if defined(HAVE_KERNEL_VFS_GETNEWFSID)
 #if defined(HAVE_TWO_ARGUMENT_VFS_GETNEWFSID)
     vfs_getnewfsid(mp, MOUNT_AFS);
 #else
     vfs_getnewfsid(mp);
-#endif
+#endif HAVE_TWO_ARGUMENT_VFS_GETNEWFSID
+#endif HAVE_KERNEL_VFS_GETNEWFSID
 
     mp->mnt_stat.f_bsize = DEV_BSIZE;
 #ifndef __osf__
