@@ -33,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_vfsops-osf.c,v 1.22 2000/10/02 23:33:23 lha Exp $");
+RCSID("$Id: xfs_vfsops-osf.c,v 1.22.2.2 2001/06/04 19:40:03 mattiasa Exp $");
 
 /*
  * XFS vfs operations.
@@ -75,7 +75,8 @@ xfs_unmount(struct mount * mp, int mntflags)
 static int
 xfs_root(struct mount * mp, struct vnode ** vpp)
 {
-    XFSDEB(XDEBVFOPS, ("xfs_root mp = %lx\n", (unsigned long)mp));
+    XFSDEB(XDEBVFOPS, ("xfs_root mp = %lx vnode = %lx\n", 
+		       (unsigned long)mp,(unsigned long)vpp));
 
     return xfs_root_common (mp, vpp, u.u_procp, u.u_procp->p_rcred);
 }
@@ -132,6 +133,34 @@ xfs_vptofh(struct vnode * vp,
     return EOPNOTSUPP;
 }
 
+#ifdef HAVE_STRUCT_VFSOPS_VFS_MOUNTROOT
+static int
+xfs_mountroot(struct mount * mp,
+          struct vnode ** vp)
+{
+    XFSDEB(XDEBVFOPS, ("xfs_mountroot\n"));
+    return EOPNOTSUPP;
+}
+#endif /* HAVE_STRUCT_VFSOPS_VFS_MOUNTROOT */
+
+#ifdef HAVE_STRUCT_VFSOPS_VFS_SWAPVP
+static int
+xfs_swapvp(void)
+{
+    XFSDEB(XDEBVFOPS, ("xfs_swapvp\n"));
+    return EOPNOTSUPP;
+}
+#endif /* HAVE_STRUCT_VFSOPS_VFS_SWAPVP */
+
+#ifdef HAVE_STRUCT_VFSOPS_VFS_SMOOTHSYNC
+static int
+xfs_smoothsync(struct mount * mp,
+          u_int int1, u_int int2)
+{
+    return EOPNOTSUPP;
+}
+#endif /* HAVE_STRUCT_VFSOPS_VFS_SMOOTHSYNC */
+
 extern struct vnodeops dead_vnodeops;
 extern struct vfs_ubcops xfs_ubcops;
 
@@ -169,7 +198,16 @@ struct vfsops xfs_vfsops = {
         xfs_sync,
         xfs_fhtovp,
         xfs_vptofh,
-        xfs_init
+        xfs_init,
+#ifdef HAVE_STRUCT_VFSOPS_VFS_MOUNTROOT
+	xfs_mountroot,
+#endif
+#ifdef HAVE_STRUCT_VFSOPS_VFS_SWAPVP
+	xfs_swapvp,
+#endif
+#ifdef HAVE_STRUCT_VFSOPS_VFS_SMOOTHSYNC
+	xfs_smoothsync,
+#endif
 };
 
 
