@@ -51,7 +51,7 @@
 #include <vm/vnode_pager.h>
 #endif
 
-RCSID("$Id: xfs_vnodeops-bsd.c,v 1.87 2001/01/07 00:18:46 lha Exp $");
+RCSID("$Id: xfs_vnodeops-bsd.c,v 1.87.2.4 2001/07/26 19:51:44 lha Exp $");
 
 /*
  * vnode functions
@@ -361,19 +361,32 @@ xfs_remove(struct vop_remove_args * ap)
    struct componentname *cnp */
 {
     struct componentname *cnp = ap->a_cnp;
-    int error = xfs_remove_common(ap->a_dvp, ap->a_vp, cnp->cn_nameptr, 
+    struct vnode *dvp = ap->a_dvp;
+    struct vnode *vp  = ap->a_vp;
+
+    int error = xfs_remove_common(dvp, vp, cnp->cn_nameptr, 
 				  cnp->cn_cred, xfs_cnp_to_proc(cnp));
 
+    cleanup_cnp (cnp, error);
+
+#if !defined(__FreeBSD__) || __FreeBSD_version < 300000
+    if (dvp == vp)
+	vrele(vp);
+    else
+	vput(vp);
+    vput(dvp);
+#endif
+    
 #ifdef __APPLE__
     if (error == 0) {
-	if (UBCINFOEXISTS(ap->a_vp)) {
-	    ubc_setsize(ap->a_vp, 0);
-	    ubc_release(ap->a_vp);
-	    ubc_uncache(ap->a_vp);
+	if (UBCINFOEXISTS(vp)) {
+	    ubc_setsize(vp, 0);
+	    ubc_release(vp);
+	    ubc_uncache(vp);
 	}
     }
 #endif
-    cleanup_cnp (cnp, error);
+
     return error;
 }
 #endif /* HAVE_VOP_REMOVE */
@@ -456,12 +469,22 @@ xfs_rmdir(struct vop_rmdir_args * ap)
    struct componentname *cnp */
 {
     struct componentname *cnp = ap->a_cnp;
+    struct vnode *dvp = ap->a_dvp;
+    struct vnode *vp  = ap->a_vp;
     int error = xfs_rmdir_common(ap->a_dvp, ap->a_vp, 
 				 cnp->cn_nameptr,
 				 cnp->cn_cred,
 				 xfs_cnp_to_proc(cnp));
 
     cleanup_cnp (cnp, error);
+#if !defined(__FreeBSD__) || __FreeBSD_version < 300000
+    if (dvp == vp)
+	vrele(vp);
+    else
+	vput(vp);
+    vput(dvp);
+#endif
+
     return error;
 }
 #endif /* HAVE_VOP_RMDIR */
@@ -505,14 +528,19 @@ xfs_readdir(struct vop_readdir_args * ap)
 	dp_end = (const struct dirent *) uio->uio_iov->iov_base;
 	for (dp_start = dp, ncookies = 0;
 	     dp < dp_end;
-	     dp = (const struct dirent *)((const char *) dp + dp->d_reclen))
+	     dp = (const struct dirent *)((const char *) dp + dp->d_reclen)) {
+	    if (dp->d_reclen <= 0)
+		break;
 	    ncookies++;
+	}
 
 	MALLOC(cookies, xfs_cookie_t *, ncookies * sizeof(xfs_cookie_t),
 	       M_TEMP, M_WAITOK);
 	for (dp = dp_start, cookiep = cookies;
 	     dp < dp_end;
 	     dp = (const struct dirent *)((const char *) dp + dp->d_reclen)) {
+	    if (dp->d_reclen <= 0)
+		break;
 	    off += dp->d_reclen;
 	    *cookiep++ = off;
 	}
@@ -600,13 +628,26 @@ xfs_symlink(struct vop_symlink_args * ap)
   */
 {
     struct componentname *cnp = ap->a_cnp;
-    int error = xfs_symlink_common(ap->a_dvp,
-				   ap->a_vpp,
+    struct vnode *dvp  = ap->a_dvp;
+    struct vnode **vpp = ap->a_vpp;
+    
+    int error = xfs_symlink_common(dvp,
+				   vpp,
 				   cnp,
 				   ap->a_vap,
 				   ap->a_target);
 
+    if (error == 0) {
+	error = xfs_lookup_common(dvp, cnp, vpp);
+#if (!defined(__FreeBSD__) || __FreeBSD_version < 400012) && (!defined(__NetBSD__) || __NetBSD_Version__ < 105240000) && (!defined(__OpenBSD__) || OpenBSD < 200107)
+	if (error == 0)
+	    vput (*vpp);
+#endif
+    }
     cleanup_cnp (cnp, error);
+#if !defined(__FreeBSD__)
+    vput(dvp);
+#endif
     return error;
 }
 #endif /* HAVE_VOP_SYMLINK */
