@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -44,7 +44,7 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_vnodeops.h>
 
-RCSID("$Id: xfs_vnodeops-common.c,v 1.66 2000/11/16 06:16:41 lha Exp $");
+RCSID("$Id: xfs_vnodeops-common.c,v 1.66.2.2 2001/07/16 10:10:44 lha Exp $");
 
 int
 xfs_open_valid(struct vnode *vp, struct ucred *cred, struct proc *p,
@@ -389,7 +389,7 @@ xfs_setattr_common(struct vnode *vp, struct vattr *vap,
 #define CHECK_XFSATTR(A, cast) (vap->A == cast VNOVAL || vap->A == xn->attr.A)
 	if (CHECK_XFSATTR(va_mode,(mode_t)) &&
 	    CHECK_XFSATTR(va_nlink,(short)) &&
-	    CHECK_XFSATTR(va_size,(u_quad_t)) &&
+	    CHECK_XFSATTR(va_size,(va_size_t)) &&
 	    CHECK_XFSATTR(va_uid,(uid_t)) &&
 	    CHECK_XFSATTR(va_gid,(gid_t)) &&
 	    CHECK_XFSATTR(va_mtime.tv_sec,(unsigned int)) &&
@@ -418,7 +418,7 @@ xfs_setattr_common(struct vnode *vp, struct vattr *vap,
 	vattr2xfs_attr(vap, &msg.attr);
 	if (XFS_TOKEN_GOT(xn, XFS_DATA_R)) {
 	    if (vp->v_type == VREG) {
-		if (vap->va_size != (u_quad_t)VNOVAL)
+		if (vap->va_size != (va_size_t)VNOVAL)
 		    XA_SET_SIZE(&msg.attr, vap->va_size);
 		else
 		    XA_SET_SIZE(&msg.attr, xn->attr.va_size);
@@ -510,6 +510,8 @@ xfs_lookup_common(struct vnode *dvp,
     struct proc *proc  = xfs_cnp_to_proc(cnp);
     struct ucred *cred = xfs_proc_to_cred(proc);
 
+    XFSDEB(XDEBVNOPS, ("xfs_lookup_common: enter\n"));
+
     *vpp = NULL;
 
     if (cnp->cn_namelen >= XFS_MAX_NAME)
@@ -529,6 +531,9 @@ xfs_lookup_common(struct vnode *dvp,
 	if (error != 0)
 	    goto done;
 
+	XFSDEB(XDEBVNOPS, ("xfs_lookup_common: dvp = %lx\n", (unsigned long) dvp));
+	
+
 	error = xfs_dnlc_lookup(dvp, cnp, vpp);
 	if (error == 0) {
 
@@ -572,6 +577,7 @@ xfs_lookup_common(struct vnode *dvp,
     } while (error == 0);
 
 done:
+    XFSDEB(XDEBVNOPS, ("xfs_lookup_common: return\n"));
     return error;
 }
 
@@ -650,14 +656,6 @@ xfs_remove_common(struct vnode *dvp,
     if (error == 0)
 	xfs_dnlc_purge (vp);
 
-#if !defined(__FreeBSD__) || __FreeBSD_version < 300000
-    if (dvp == vp)
-	vrele(vp);
-    else
-	vput(vp);
-    vput(dvp);
-#endif
-    
     return error;
 }
 
@@ -767,14 +765,6 @@ xfs_rmdir_common(struct vnode *dvp,
     if (error == 0)
 	xfs_dnlc_purge (vp);
 
-#if !defined(__FreeBSD__) || __FreeBSD_version < 300000
-    if (dvp == vp)
-	vrele(vp);
-    else
-	vput(vp);
-    vput(dvp);
-#endif
-
     XFSDEB(XDEBVNOPS, ("xfs_rmdir error: %d\n", error));
 
     return error;
@@ -876,19 +866,7 @@ xfs_symlink_common(struct vnode *dvp,
     if (error == 0)
 	error = ((struct xfs_message_wakeup *) & msg)->error;
 
-    if (error == 0) {
-	error = xfs_lookup_common(dvp, cnp, vpp);
-#if !defined(__FreeBSD__) || __FreeBSD_version < 400012
-	if (error == 0)
-	    vput (*vpp);
-#endif
-    }
-
  done:
-#if !defined(__FreeBSD__)
-    vput(dvp);
-#endif
-
     return error;
 }
 
@@ -945,7 +923,9 @@ xfs_inactive_common(struct vnode *vp, struct proc *p)
         vrecycle(vp, 0, p);
     }
 #else
-    /* XXX ? */
+
+    XFSDEB(XDEBVNOPS, ("xfs_inactive: vp = %lx vp->v_usecount= %d\n",
+		       (unsigned long)vp, vp?vp->v_usecount:0));
 #endif
     xn->flags &= ~XFS_STALE;
 
@@ -961,7 +941,7 @@ xfs_reclaim_common(struct vnode *vp)
     struct xfs *xfsp = XFS_FROM_VNODE(vp);
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
 
-    XFSDEB(XDEBVNOPS, ("xfs_reclaim: %lx",
+    XFSDEB(XDEBVNOPS, ("xfs_reclaim: %lx\n",
 		       (unsigned long)vp));
 
     XFS_TOKEN_CLEAR(xn,
