@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -47,7 +47,7 @@
 
 #include <sys/buf.h>
 
-RCSID("$Id: xfs_vnodeops-osf.c,v 1.23 2000/10/02 23:33:41 lha Exp $");
+RCSID("$Id: xfs_vnodeops-osf.c,v 1.23.2.3 2001/07/16 10:11:35 lha Exp $");
 
 /*
  * vnode functions
@@ -115,6 +115,7 @@ xfs_access(struct vnode *vp, int mode, struct ucred *cred)
 static int
 xfs_lookup(struct vnode *dvp, struct nameidata *ndp)
 {
+    ndp->ni_dvp = dvp;
     return xfs_lookup_common(dvp, ndp, &ndp->ni_vp);
 }
 
@@ -134,9 +135,11 @@ xfs_create(struct nameidata *ndp, struct vattr *vap)
 			      vap,
 			      cred,
 			      xfs_curproc());
-    if(error == 0)
+    if(error == 0) {
+	ndp->cn_nameptr = ndp->ni_dent.d_name;
 	error = xfs_lookup_common (dvp, ndp, &ndp->ni_vp);
-
+    }
+    vrele(dvp);
     XFSDEB(XDEBVNOPS, ("xfs_create: error = %d\n", error));
     
     return error;
@@ -145,11 +148,14 @@ xfs_create(struct nameidata *ndp, struct vattr *vap)
 static int
 xfs_remove(struct nameidata *ndp)
 {
-    return xfs_remove_common(ndp->ni_dvp, 
+    int ret = xfs_remove_common(ndp->ni_dvp, 
 			     ndp->ni_vp, 
 			     ndp->ni_dent.d_name, 
 			     ndp->ni_cred,
 			     xfs_curproc());
+    vrele(ndp->ni_dvp);
+    vrele(ndp->ni_vp);
+    return ret;
 }
 
 static int
@@ -188,9 +194,11 @@ xfs_mkdir(struct nameidata *ndp,
 			     cred,
 			     xfs_curproc());
 
-    if(error == 0)
+    if(error == 0) {
+	ndp->cn_nameptr = ndp->ni_dent.d_name;
 	error = xfs_lookup_common (dvp, ndp, &ndp->ni_vp);
-
+    }
+    vrele(dvp);
     XFSDEB(XDEBVNOPS, ("xfs_mkdir: error = %d\n", error));
     
     return error;
@@ -199,11 +207,14 @@ xfs_mkdir(struct nameidata *ndp,
 static int
 xfs_rmdir(struct nameidata *ndp)
 {
-    return xfs_rmdir_common(ndp->ni_dvp, 
+    int ret = xfs_rmdir_common(ndp->ni_dvp, 
 			    ndp->ni_vp, 
 			    ndp->ni_dent.d_name, 
 			    ndp->ni_cred,
 			    xfs_curproc());
+    vrele(ndp->ni_dvp);
+    vrele(ndp->ni_vp);
+    return ret;
 }
 
 static int
@@ -218,15 +229,22 @@ xfs_readdir(struct vnode *vp,
 static int
 xfs_link(struct vnode* vp, struct nameidata *ndp)
 {
-    return xfs_link_common(ndp->ni_dvp, vp, ndp->ni_dent.d_name,
+    int ret = xfs_link_common(ndp->ni_dvp, vp, ndp->ni_dent.d_name,
 			   ndp->ni_cred, xfs_curproc());
+    vrele(ndp->ni_dvp);
+    return ret;
 }
 
 static int
 xfs_symlink(struct nameidata *ndp, struct vattr *vap, char *target)
 {
-    return xfs_symlink_common(ndp->ni_dvp, &ndp->ni_vp,
+    int ret;
+
+    ndp->cn_nameptr = ndp->ni_dent.d_name;
+    ret = xfs_symlink_common(ndp->ni_dvp, &ndp->ni_vp,
 			      ndp, vap, target);
+    vrele(ndp->ni_dvp);
+    return ret;
 }
 
 static int
@@ -451,135 +469,28 @@ xfs_getpage(vm_ubc_object_t vop,
 	    struct ucred *cred)
 #endif
 {
-#if 1
-    vm_page_t app, pp;
-    int uflags;
-    vm_offset_t off;
-    int length;
-    
-    int error = 0;
-    
+    int error;
     struct vnode *vp = vop->vu_vp;
     struct vnode *t = DATA_FROM_VNODE(vp);
-    struct vattr attr;
-	    
-#if 0
-    XFSDEB(XDEBVNOPS, ("xfs_getpage: %lx, %lx, %ld\n", 
-		       (unsigned long)vop, offset, len));
-    XFSDEB(XDEBVNOPS, ("xfs_getpage: %lx, %lx, %lx, %d\n", 
-		       (unsigned long)protp, ep, addr, rwflg));
-#endif
-    error = xfs_data_valid(vp, cred, xfs_curproc(), XFS_DATA_R);
-    if(error)
-	return error;
 
-    xfs_vop_getattr(t, &attr, cred, proc, error);
-    if(error)
-	return error;
-	    
-    if(offset + len > attr.va_size) {
-	if(offset >= attr.va_size) {
-	    *pl = VM_PAGE_NULL;
+    XFSDEB(XDEBVNOPS, ("xfs_getpage: enter\n"));
+
+    error = xfs_data_valid(vp, cred, xfs_curproc(), XFS_DATA_R);
+
+    XFSDEB(XDEBVNOPS, ("xfs_getpage = %d\n", error));
+
+    if (error == 0) {
 #ifdef HAVE_SIX_ARGUMENT_UBC_LOOKUP
-	    if(ep) {
-		error = KERN_INVALID_ADDRESS;
-	    } else
-#endif
-		error = 0;
-	    return error;
-	}
-	len = attr.va_size - offset;
-    }
-    length = len + (offset & page_mask);
-    off = trunc_page(offset);
-    error = 0;
-    for(; length > 0; off += PAGE_SIZE, pl++, length -= PAGE_SIZE) {
-	if(off >= attr.va_size) {
-	    *pl = VM_PAGE_NULL;
-	    break;
-	}
-	XFSDEB(XDEBVNOPS, ("get page [%lx,%lx)\n", off, length + off));
-#ifdef HAVE_SIX_ARGUMENT_UBC_LOOKUP
-	error = ubc_lookup(vop, off, PAGE_SIZE, length + off, &app, &uflags);
+	FSOP_GETPAGE(t->v_object,offset,len,protp,pl,plsz,ep,
+		     addr,rwflg,cred,error);
 #else
-	error = ubc_lookup(vop, off, PAGE_SIZE, length, &app, &uflags, vmp);
+	FSOP_GETPAGE(t->v_object,offset,len,protp,pl,plsz,
+		     vmp,rwflg,cred,error);
 #endif
-	if(error) {
-	    XFSDEB(XDEBVNOPS, ("ubc_lookup = %d\n", error));
-	    *pl = VM_PAGE_NULL;
-	    break;
-	}
-	if(uflags & B_NOCACHE) {
-	    vm_page_t page, pp;
-	    int pagecnt;
-	    XFSDEB(XDEBVNOPS, ("cache miss\n"));
-	    page = ubc_kluster(vop, app, off, length, 
-			       B_CACHE | B_WANTFREE | B_BUSY, 
-			       UBC_HNONE, &pagecnt);
-	    t = DATA_FROM_VNODE(vp);
-		
-	    xfs_vfs_readlock(t, NULL);
-	    if(pagecnt > 8) {
-		XFSDEB(XDEBVNOPS, ("========== pagecnt > 8 ==========\n"));
-	    }
-	    /* XXX this is broken */
-	    for(pp = page; pp; ) {
-		struct uio uio;
-		struct iovec iov[8];
-		int num_iov;
-		bzero(&uio, sizeof(uio));
-		for(num_iov = 0; 
-		    pp && num_iov < 8; 
-		    pp = pp->pg_pnext, num_iov++) {
-		    iov[num_iov].iov_base = (caddr_t)page_to_phys(pp);
-		    iov[num_iov].iov_len = PAGE_SIZE;
-		}
-		uio.uio_iov = iov;
-		uio.uio_offset = offset;
-		uio.uio_iovcnt = num_iov;
-		uio.uio_resid = len;
-		uio.uio_segflg = UIO_PHYSSPACE;
-		uio.uio_rw = UIO_READ;
-		if(xfsdeb & XDEBVNOPS) {
-		    printf("uio.uio_iov = %lx\n", uio.uio_iov);
-		    printf("uio.uio_offset = %lx\n", uio.uio_offset);
-		    printf("uio.uio_iovcnt = %d\n", uio.uio_iovcnt);
-		    printf("uio.uio_resid = %lx\n", uio.uio_resid);
-		    printf("uio.uio_segflg = %d\n", uio.uio_segflg);
-		    printf("uio.uio_rw = %d\n", uio.uio_rw);
-		}
-		xfs_vop_read(t, &uio, 0, cred, error);
-		XFSDEB(XDEBVNOPS, ("xfs_vop_read = %d\n", error));
-	    }
-	    xfs_vfs_unlock(t, NULL);
-	    for(pp = page; pp; pp = pp->pg_pnext) {
-		*pl++ = pp;
-		if(protp)
-		    *protp++ = VM_PROT_WRITE;
-		XFSDEB(XDEBVNOPS, ("pp->pg_busy = %d\n", pp->pg_busy));
-		pp->pg_busy = 0;
-	    }
-	    *pl = VM_PAGE_NULL;
-	    XFSDEB(XDEBVNOPS, ("xfs_getpage: done\n"));
-	    return 0;
-	} else {
-	    *pl = app;
-	    *(pl + 1) = VM_PAGE_NULL;
-	    if(protp)
-		*protp++ = VM_PROT_WRITE;
-	}
     }
-    XFSDEB(XDEBVNOPS, ("xfs_getpage: done\n"));
+
+    XFSDEB(XDEBVNOPS, ("xfs_getpage = %d\n", error));
     return error;
-    
-#else
-    printf("xfs_getpage: vop = %lx, offset = %lx, len = %ld\n", 
-	   (unsigned long)vop, offset, len);
-    printf("xfs_getpage: protp = %lx, ep = %lx, addr = %lx, rwflg = %d\n", 
-	   (unsigned long)protp, ep, addr, rwflg);
-    *pl = VM_PAGE_NULL;
-    return EINVAL;
-#endif
 }
 
 int
@@ -589,13 +500,21 @@ xfs_putpage(vm_ubc_object_t vop,
 	    int flags,
 	    struct ucred *cred)
 {
-#if 1
-    panic("xfs_putpage");
-    return EOPNOTSUPP;
-#else
-    printf("xfs_putpage\n");
-    return EINVAL;
-#endif
+    int error;
+    struct vnode *vp = vop->vu_vp;
+    struct vnode *t = DATA_FROM_VNODE(vp);
+
+    XFSDEB(XDEBVNOPS, ("xfs_putpage: enter\n"));
+
+    error = xfs_data_valid(vp, cred, xfs_curproc(), XFS_DATA_R);
+
+    XFSDEB(XDEBVNOPS, ("xfs_putpage = %d\n", error));
+
+    if (error == 0)
+	FSOP_PUTPAGE(t->v_object,pl,pcnt,flags,cred,error);
+
+    XFSDEB(XDEBVNOPS, ("xfs_putpage = %d\n", error));
+    return error;
 }
 
 int
