@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -42,10 +37,11 @@
 #include <xfs/xfs_dev.h>
 #include <xfs/xfs_syscalls.h>
 #include <xfs/xfs_deb.h>
+#include <xfs/xfs_wrap.h>
 
-RCSID("$Id: xfs_wrap-bsd.c,v 1.22 1999/06/08 08:21:10 lha Exp $");
+RCSID("$Id: xfs_wrap-bsd.c,v 1.33 2000/10/02 23:33:47 lha Exp $");
 
-static int xfs_dev_major;
+int xfs_dev_major;
 
 /*
  * Iff `dev' represents a valid xfs device.
@@ -107,6 +103,28 @@ extern struct cdevsw xfs_dev;
 
 #if KLD_MODULE
 
+static void
+make_devices (struct cdevsw *devsw)
+{
+#ifdef HAVE_KERNEL_MAKE_DEV
+    int i;
+
+    for (i = 0; i < NXFS; ++i)
+	make_dev (devsw, i, UID_ROOT, GID_WHEEL, 0600, "xfs%d", i);
+#endif /* HAVE_KERNEL_MAKE_DEV */
+}
+
+static void
+remove_devices (struct cdevsw *devsw)
+{
+#ifdef HAVE_KERNEL_MAKE_DEV
+    int i;
+
+    for (i = 0; i < NXFS; ++i)
+	destroy_dev (makedev (devsw->d_maj, i));
+#endif /* HAVE_KERNEL_MAKE_DEV */
+}
+
 /*
  *
  */
@@ -122,12 +140,17 @@ xfs_load(struct module *mod, int cmd, void *arg)
     case MOD_LOAD :
 	ret = xfs_install ();
 	if (ret == 0) {
+	    make_devices (&xfs_dev);
 	    xfs_dev_major = xfs_dev.d_maj;
-	    printf ("xfs cdev loaded at %d\n", xfs_dev_major);
+	    printf ("xfs: cdev: %d, syscall: %d\n",
+		    xfs_dev_major, xfs_syscall_num);
 	}
 	break;
     case MOD_UNLOAD :
 	ret = xfs_uninstall ();
+	if (ret == 0) {
+	    remove_devices (&xfs_dev);
+	}
 	break;
     default :
 	ret = EINVAL;
@@ -141,10 +164,6 @@ extern struct vfsops xfs_vfsops;
 
 extern struct sysent xfs_syscallent;
 
-/* XXX */
-
-#define AFS_SYSCALL 210
-
 VFS_SET(xfs_vfsops, xfs, 0);
 
 /*
@@ -154,9 +173,7 @@ VFS_SET(xfs_vfsops, xfs, 0);
 
 #ifdef DEV_MODULE
 
-#define XFS_DEV_NO NOMAJ
-
-DEV_MODULE(xfsdev, XFS_DEV_NO, NOMAJ, xfs_dev, xfs_load, NULL);
+DEV_MODULE(xfsdev, xfs_load, NULL);
 
 #else /* DEV_MODULE */
 
@@ -164,14 +181,12 @@ CDEV_MODULE(xfs_dev, NODEV, xfs_dev, xfs_load, NULL);
 
 #endif /* DECLARE_MODULE */
 
-
 #ifdef SYSCALL_MODULE
-static int offset = NO_SYSCALL;
-SYSCALL_MODULE(xfs_syscall, &offset, &xfs_syscallent, NULL, NULL);
+#define AFS_SYSCALL 210 /* XXX */
+int xfs_syscall_num = AFS_SYSCALL;
+SYSCALL_MODULE(xfs_syscall, &xfs_syscall_num, &xfs_syscallent, NULL, NULL);
 #endif /* SYSCALL_MODULE */
 
-
-
 #else /* KLD_MODULE */
 
 /*
@@ -190,15 +205,22 @@ static struct lkm_dev _module = {
 #elif defined(__FreeBSD__)
 MOD_DEV(xfs,LM_DT_CHAR,-1,&xfs_dev);
 #else
+#if !defined(__APPLE__)
 MOD_DEV("xfs_mod",LM_DT_CHAR,-1,&xfs_dev)
 #endif
+#endif
 
 /*
  *
  */
 
+#if defined(__APPLE__)
+__private_extern__ kern_return_t
+xfs_modload(kmod_info_t *ki, void *data)
+#else
 static int
 xfs_modload(struct lkm_table *lkmtp, int cmd)
+#endif
 {
     int error = 0;
 
@@ -215,8 +237,13 @@ xfs_modload(struct lkm_table *lkmtp, int cmd)
  *
  */
 
+#if defined(__APPLE__)
+__private_extern__ kern_return_t
+xfs_modunload(kmod_info_t *ki, void *data)
+#else
 static int
 xfs_modunload(struct lkm_table * lkmtp, int cmd)
+#endif
 {
     int error = 0;
 
@@ -234,6 +261,7 @@ xfs_modunload(struct lkm_table * lkmtp, int cmd)
  *
  */
 
+#if !defined(__APPLE__)
 static int
 xfs_modstat(struct lkm_table * lkmtp, int cmd)
 {
@@ -245,6 +273,9 @@ xfs_modstat(struct lkm_table * lkmtp, int cmd)
     return error;
 }
 
+int
+xfs_mod(struct lkm_table * lkmtp, int cmd, int ver);
+
 int
 xfs_mod(struct lkm_table * lkmtp,
 	int cmd,
@@ -270,10 +301,14 @@ xfs_mod(struct lkm_table * lkmtp,
 	break;
     }
     ret = lkmdispatch(lkmtp, cmd);
-    if(cmd == LKM_E_LOAD)
+    if(cmd == LKM_E_LOAD) {
 	xfs_dev_major = _module.lkm_offset;
+	printf ("xfs: cdev: %d, syscall: %d\n",
+		xfs_dev_major, xfs_syscall_num);
+    }
     return ret;
 }
+#endif
     
 #endif /* KLD_MODULE */
 
