@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,13 +31,29 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_message.h,v 1.30 1999/07/24 13:36:10 lha Exp $ */
+/* $Id: xfs_message.h,v 1.44 2000/11/15 08:36:45 lha Exp $ */
 
 #ifndef _xmsg_h
 #define _xmsg_h
 
-#if !defined(__LINUX__) && !defined(HAVE_GLIBC)
+/* bump this for any incompatible changes */
+
+#define XFS_VERSION 17
+
+#if defined(WIN32)
+#ifdef i386
+#ifndef __CYGWIN__
+typedef int int32_t;
+typedef unsigned int u_int32_t;
+typedef short int16_t;
+typedef unsigned char u_char;
+#endif
+#else
+#error not a i386
+#endif
+#elif !defined(__LINUX__) && !defined(HAVE_GLIBC)
 #include <sys/types.h>
+#include <atypes.h>
 #include <sys/param.h>
 #else
 #include <linux/types.h>
@@ -54,18 +65,18 @@
 /* Temporary hack? */
 #define MAX_XMSG_SIZE (1024*64)
 
-typedef u_int32_t pag_t;
+typedef u_int32_t xfs_pag_t;
 
 /*
  * The xfs_cred, if pag == 0, use uid 
  */
 typedef struct xfs_cred {
-    __kernel_uid_t uid;
-    pag_t pag;
+    u_int32_t uid;
+    xfs_pag_t pag;
 } xfs_cred;
 
-typedef unsigned long xfs_locktype_t;
-typedef unsigned long xfs_lockid_t;
+typedef u_int32_t xfs_locktype_t;
+typedef u_int32_t xfs_lockid_t;
 
 
 #define MAXHANDLE (4*4)
@@ -74,7 +85,7 @@ typedef unsigned long xfs_lockid_t;
 #define XFS_ANONYMOUSID 32766
 
 typedef struct xfs_handle {
-    u_int a, b, c, d;
+    u_int32_t a, b, c, d;
 } xfs_handle;
 
 #define xfs_handle_eq(p, q) \
@@ -120,6 +131,8 @@ typedef struct xfs_cache_handle {
 /* xfs_node.flags */
 #define XFS_DATA_DIRTY	0x0001
 #define XFS_ATTR_DIRTY	0x0002
+#define XFS_AFSDIR	0x0004
+#define XFS_STALE	0x0008
 
 /* Are necessary tokens available? */
 #define XFS_TOKEN_GOT(xn, tok)		((xn)->tokens & (tok))
@@ -131,28 +144,41 @@ typedef struct xfs_cache_handle {
 #define XFS_RIGHT_W	0x02		/* may write? */
 #define XFS_RIGHT_X	0x04		/* may execute? */
 
+/* Max name length passed in xfs messages */
+
+#define XFS_MAX_NAME 256
+#define XFS_MAX_SYMLINK_CONTENT 2048
+
 struct xfs_msg_node {
     xfs_handle handle;
-    u_int tokens;
+    u_int32_t tokens;
+    u_int32_t pad1;
     struct xfs_attr attr;
-    pag_t id[MAXRIGHTS];
+    xfs_pag_t id[MAXRIGHTS];
     u_char rights[MAXRIGHTS];
     u_char anonrights;
+    u_int16_t pad2;
+    u_int32_t pad3;
 };
 
 /*
  * Messages passed through the  xfs_dev.
  */
 struct xfs_message_header {
-  u_int size;
-  u_int opcode;
-  u_int sequence_num;		/* Private */
+  u_int32_t size;
+  u_int32_t opcode;
+  u_int32_t sequence_num;		/* Private */
+  u_int32_t pad1;
 };
 
 /*
  * Used by putdata flag
  */
-enum { XFS_READ = 1, XFS_WRITE = 2, XFS_NONBLOCK = 4, XFS_APPEND = 8};
+enum { XFS_READ     = 0x01,
+       XFS_WRITE    = 0x02,
+       XFS_NONBLOCK = 0x04,
+       XFS_APPEND   = 0x08,
+       XFS_FSYNC    = 0x10};
 
 /*
  * Flags for inactivenode
@@ -163,7 +189,8 @@ enum { XFS_NOREFS = 1, XFS_DELETE = 2 };
  * Flags for installdata
  */
 
-enum { XFS_INVALID_DNLC = 1 };
+enum { XFS_ID_INVALID_DNLC = 0x01, XFS_ID_AFSDIR = 0x02,
+       XFS_ID_HANDLE_VALID = 0x04 };
 
 /*
  * Defined message types and their opcodes.
@@ -217,23 +244,14 @@ enum { XFS_INVALID_DNLC = 1 };
 /* XFS_MESSAGE_VERSION */
 struct xfs_message_version {
   struct xfs_message_header header;
-  int probe;			/* What to probe */
-#define XFS_VERSION_YES		0x1000
-#define XFS_VERSION_NO		0x1001
-
-#define XFS_VERSION_BASE	4711	
-#define XFS_VERSION_VERSION	(XFS_VERSION_BASE+1)
-#define XFS_VERSION_FHGET	(XFS_VERSION_BASE+2)
-#define XFS_VERSION_FHGET_NATIVE (XFS_VERSION_BASE+3)
-#define XFS_VERSION_GC_NODES 	(XFS_VERSION_BASE+4)
-    int ret;
+  u_int32_t ret;
 };
 
 /* XFS_MESSAGE_WAKEUP */
 struct xfs_message_wakeup {
   struct xfs_message_header header;
-  int sleepers_sequence_num;	/* Where to send wakeup */
-  int error;			/* Return value */
+  u_int32_t sleepers_sequence_num;	/* Where to send wakeup */
+  u_int32_t error;			/* Return value */
 };
 
 /* XFS_MESSAGE_GETROOT */
@@ -253,14 +271,14 @@ struct xfs_message_getnode {
   struct xfs_message_header header;
   struct xfs_cred cred;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
 };
 
 /* XFS_MESSAGE_INSTALLNODE */
 struct xfs_message_installnode {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   struct xfs_msg_node node;
 };
 
@@ -282,23 +300,26 @@ struct xfs_message_getdata {
   struct xfs_message_header header;
   struct xfs_cred cred;
   xfs_handle handle;
-  u_int tokens;
+  u_int32_t tokens;
+  u_int32_t pad1;
 };
 
 /* XFS_MESSAGE_INSTALLDATA */
 struct xfs_message_installdata {
   struct xfs_message_header header;
   struct xfs_msg_node node;
-  char cache_name[256];		/* XXX */
+  char cache_name[XFS_MAX_NAME];
   struct xfs_cache_handle cache_handle;
-  u_int flag;
+  u_int32_t flag;
+  u_int32_t pad1;
 };
 
 /* XFS_MSG_INACTIVENODE */
 struct xfs_message_inactivenode {
   struct xfs_message_header header;
   xfs_handle handle;
-  u_int flag;
+  u_int32_t flag;
+  u_int32_t pad1;
 };
 
 /* XFS_MSG_INVALIDNODE */
@@ -312,7 +333,8 @@ struct xfs_message_open {
   struct xfs_message_header header;
   struct xfs_cred cred;
   xfs_handle handle;
-  u_int tokens;
+  u_int32_t tokens;
+  u_int32_t pad1;
 };
 
 /* XFS_MSG_PUTDATA */
@@ -321,7 +343,8 @@ struct xfs_message_putdata {
   xfs_handle handle;
   struct xfs_attr attr;		/* XXX ??? */
   struct xfs_cred cred;
-  u_int flag;
+  u_int32_t flag;
+  u_int32_t pad1;
 };
 
 /* XFS_MSG_PUTATTR */
@@ -336,9 +359,10 @@ struct xfs_message_putattr {
 struct xfs_message_create {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   struct xfs_attr attr;
-  int mode;
+  u_int32_t mode;
+  u_int32_t pad1;
   struct xfs_cred cred;
 };
 
@@ -346,7 +370,7 @@ struct xfs_message_create {
 struct xfs_message_mkdir {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   struct xfs_attr attr;
   struct xfs_cred cred;
 };
@@ -355,7 +379,7 @@ struct xfs_message_mkdir {
 struct xfs_message_link {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   xfs_handle from_handle;
   struct xfs_cred cred;
 };
@@ -364,8 +388,8 @@ struct xfs_message_link {
 struct xfs_message_symlink {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
-  char contents[2048];		/* XXX */
+  char name[XFS_MAX_NAME];
+  char contents[XFS_MAX_SYMLINK_CONTENT];
   struct xfs_attr attr;
   struct xfs_cred cred;
 };
@@ -374,7 +398,7 @@ struct xfs_message_symlink {
 struct xfs_message_remove {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   struct xfs_cred cred;
 };
 
@@ -382,7 +406,7 @@ struct xfs_message_remove {
 struct xfs_message_rmdir {
   struct xfs_message_header header;
   xfs_handle parent_handle;
-  char name[256];		/* XXX */
+  char name[XFS_MAX_NAME];
   struct xfs_cred cred;
 };
 
@@ -390,19 +414,20 @@ struct xfs_message_rmdir {
 struct xfs_message_rename {
   struct xfs_message_header header;
   xfs_handle old_parent_handle;
-  char old_name[256];		/* XXX */
+  char old_name[XFS_MAX_NAME];
   xfs_handle new_parent_handle;
-  char new_name[256];		/* XXX */
+  char new_name[XFS_MAX_NAME];
   struct xfs_cred cred;
 };
 
 /* XFS_MSG_PIOCTL */
 struct xfs_message_pioctl {
   struct xfs_message_header header;
-  int opcode ;
+  u_int32_t opcode ;
+  u_int32_t pad1;
   xfs_cred cred;
-  int insize;
-  int outsize;
+  u_int32_t insize;
+  u_int32_t outsize;
   char msg[2048] ;    /* XXX */
   xfs_handle handle;
 };
@@ -411,9 +436,10 @@ struct xfs_message_pioctl {
 /* XFS_MESSAGE_WAKEUP_DATA */
 struct xfs_message_wakeup_data {
   struct xfs_message_header header;
-  int sleepers_sequence_num;	/* Where to send wakeup */
-  int error;			/* Return value */
-  int len;
+  u_int32_t sleepers_sequence_num;	/* Where to send wakeup */
+  u_int32_t error;			/* Return value */
+  u_int32_t len;
+  u_int32_t pad1;
   char msg[2048] ;    /* XXX */
 };
 
@@ -441,7 +467,8 @@ struct xfs_message_advlock {
 struct xfs_message_gc_nodes {
   struct xfs_message_header header;
 #define XFS_GC_NODES_MAX_HANDLE 50
-  int len;
+  u_int32_t len;
+  u_int32_t pad1;
   xfs_handle handle[XFS_GC_NODES_MAX_HANDLE];
 };
 #endif /* _xmsg_h */
