@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.5 1999/05/30 09:21:25 assar Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.8 2000/10/02 23:42:33 lha Exp $");
 
 /*
  * XFS system calls.
@@ -82,7 +77,7 @@ xfs_is_pag(struct cred *cred)
 	return 0;
 }
 
-pag_t
+xfs_pag_t
 xfs_get_pag(struct cred * cred)
 {
     if (xfs_is_pag(cred)) {
@@ -205,8 +200,8 @@ xfs_pioctl_call(char *a_pathP,
 
     if (error == 0)
 	error = msg2->error;
-    else
-	error = EINVAL; /* return EINVAL to not confuse applications */
+    if (error == ENODEV)
+	error = EINVAL;
     
     if (error == 0 && vice_ioctl.out_size)
 	error = copyout((caddr_t)msg2->msg,
