@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -48,7 +43,7 @@
 #include <xfs/xfs_deb.h>
 #include <xfs/xfs_syscalls.h>
 
-RCSID("$Id: xfs_vnodeops.c,v 1.6 1999/02/01 00:13:52 assar Exp $");
+RCSID("$Id: xfs_vnodeops.c,v 1.11 2000/11/15 12:24:19 lha Exp $");
 
 static int
 xfs_open_valid(struct vnode *vp, struct cred *cred, u_int tok)
@@ -90,7 +85,7 @@ xfs_attr_valid(struct vnode *vp, struct cred *cred, u_int tok)
   struct xfs_node *xn = VNODE_TO_XNODE(vp);
   int error = 0;
   
-  pag_t pag = xfs_get_pag(cred);
+  xfs_pag_t pag = xfs_get_pag(cred);
 
   do {
     if (!XFS_TOKEN_GOT(xn, tok))
@@ -121,7 +116,7 @@ xfs_fetch_rights(struct vnode *vp, struct cred *cred)
     struct xfs_node *xn = VNODE_TO_XNODE(vp);
     int error = 0;
 
-    pag_t pag = xfs_get_pag(cred);
+    xfs_pag_t pag = xfs_get_pag(cred);
 
     struct xfs_message_getattr msg;
 
@@ -358,10 +353,10 @@ xfs_setattr_common(struct vnode *vp,
       msg.cred.pag = xfs_get_pag(cred);
       msg.handle = xn->handle;
       vattr2xfs_attr (vap, &msg.attr);
+      XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
       error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
       if (error == 0)
 	error = ((struct xfs_message_wakeup *) &msg)->error;
-      XFS_TOKEN_CLEAR(xn, XFS_ATTR_VALID, XFS_ATTR_MASK);
   }
 
  done:
@@ -392,7 +387,7 @@ xfs_access_common(struct vnode *vp,
 		  struct cred *cred)     
 {
   int error = 0;
-  pag_t pag = xfs_get_pag(cred);
+  xfs_pag_t pag = xfs_get_pag(cred);
 
   XFSDEB(XDEBVNOPS, ("xfs_access mode = 0%o\n", mode));
 
@@ -457,8 +452,10 @@ xfs_lookup_common(struct vnode *dvp,
 	msg.cred.uid = cred->cr_uid;
 	msg.cred.pag = xfs_get_pag(cred);
 	msg.parent_handle = d->handle;
-	strncpy(msg.name, nm, sizeof(msg.name));
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_LEN)
+	    error = ENAMETOOLONG;
+	else
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
 	if (error == 0)
 	  error = ((struct xfs_message_wakeup *) &msg)->error;
       }
@@ -494,7 +491,8 @@ xfs_create_common(struct vnode *dvp,
     struct xfs_message_create msg;
     msg.header.opcode = XFS_MSG_CREATE;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     vattr2xfs_attr (va, &msg.attr);
 #if 0
     msg.exclusive = exclusive;
@@ -536,7 +534,8 @@ xfs_remove_common(struct vnode *dvp,
 
   msg.header.opcode = XFS_MSG_REMOVE;
   msg.parent_handle = xn->handle;
-  strncpy(msg.name, nm, 256);
+  if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+      return ENAMETOOLONG;
   msg.cred.uid = cred->cr_uid;
   msg.cred.pag = xfs_get_pag(cred);
   error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
@@ -563,7 +562,8 @@ xfs_link_common(struct vnode *vp,
     msg.header.opcode = XFS_MSG_LINK;
     msg.parent_handle = xn->handle;
     msg.from_handle   = xn2->handle;
-    strncpy(msg.name, tnm, 256);
+    if (strlcpy(msg.name, tnm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
 
@@ -588,6 +588,9 @@ xfs_rename_common(struct vnode *sdvp,
 
   XFSDEB(XDEBVNOPS, ("xfs_rename\n"));
 
+  if (strlen(onm >= XFS_MAX_NAME)
+      return ENAMETOOLONG;
+
   error = xfs_lookup_common(tdvp, nnm, vpp, /*pnp*/ NULL, /*flags*/ 0,
 			    /*rdir*/ NULL, cred);
   if (error != ENOENT) {
@@ -596,7 +599,8 @@ xfs_rename_common(struct vnode *sdvp,
       
       remmsg.header.opcode = XFS_MSG_REMOVE;
       remmsg.parent_handle = VNODE_TO_XNODE(tdvp)->handle;
-      strncpy(remmsg.name, nnm, 256);
+      if (strlcpy(remmsg.name, nnm, sizeof(remmsg.name)) >= XFS_MAX_NAME)
+	  return ENAMETOOLONG;
       remmsg.cred.uid = cred->cr_uid;
       remmsg.cred.pag = xfs_get_pag(cred);
       error = xfs_message_rpc(XFS_FROM_VNODE(tdvp)->fd, &remmsg.header,
@@ -609,9 +613,11 @@ xfs_rename_common(struct vnode *sdvp,
 
   msg.header.opcode = XFS_MSG_RENAME;
   msg.old_parent_handle = VNODE_TO_XNODE(sdvp)->handle;
-  strncpy(msg.old_name, onm, 256);
+  if (strlcpy(msg.old_name, onm, sizeof(msg.old_name)) >= XFS_MAX_NAME)
+      return ENAMETOOLONG;
   msg.new_parent_handle = VNODE_TO_XNODE(tdvp)->handle;
-  strncpy(msg.new_name, nnm, 256);
+  if (strlcpy(msg.new_name, nnm, sizeof(msg.new_name)) >= XFS_MAX_NAME)
+      return ENAMETOOLONG;
   msg.cred.uid = cred->cr_uid;
   msg.cred.pag = xfs_get_pag(cred);
   error = xfs_message_rpc(XFS_FROM_VNODE(sdvp)->fd, &msg.header,
@@ -638,7 +644,8 @@ xfs_mkdir_common(struct vnode *dvp,
     struct xfs_message_mkdir msg;
     msg.header.opcode = XFS_MSG_MKDIR;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     vattr2xfs_attr (va, &msg.attr);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
@@ -667,7 +674,8 @@ xfs_rmdir_common(struct vnode *dvp,
 
     msg.header.opcode = XFS_MSG_RMDIR;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    if (strlcpy(msg.name, nm, sizeof(msg.name))) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
     error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
@@ -774,8 +782,10 @@ xfs_symlink_common(struct vnode *dvp,
    
     msg.header.opcode = XFS_MSG_SYMLINK;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, lnm, 256);
-    strncpy(msg.contents, tnm, sizeof(msg.contents));
+    if (strlcpy(msg.name, lnm, sizeof(msg.name))) >= XFS_MAX_NAME)
+	return ENAMETOOLONG;
+    if (strlcpy(msg.contents, tnm, sizeof(msg.contents))) >= XFS_MAX_SYMLINK_CONENT)
+	return ENAMETOOLONG;
     vattr2xfs_attr (tva, &msg.attr);
     msg.cred.uid = cred->cr_uid;
     msg.cred.pag = xfs_get_pag(cred);
@@ -825,7 +835,7 @@ xfs_fsync_common(struct vnode *vp,
   XFSDEB(XDEBVNOPS, ("xfs_fsync\n"));
 
   if (xn->flags & XFS_DATA_DIRTY)
-      error = do_fsync (xfsp, xn, cred, XFS_WRITE);
+      error = do_fsync (xfsp, xn, cred, XFS_WRITE | XFS_FSYNC);
   return error;
 }
 
