@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_dev.h,v 1.5 1998/03/25 03:41:29 assar Exp $ */
+/* $Id: xfs_dev.h,v 1.10 2000/10/02 23:48:22 lha Exp $ */
 /* $Source: /afs/stacken.kth.se/src/SourceRepository/arla/xfs/linux/xfs/xfs_dev.h,v $ */
 
 #ifndef _xfs_dev_h
@@ -64,4 +59,6 @@ int xfs_message_wakeup_data(int fd,
 			    struct xfs_message_wakeup_data *message,
 			    u_int size);
 
+void xfs_queue_inactive (struct xfs_node *xn);
+
 #endif
