@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,48 +31,71 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_locl.h,v 1.31.2.1 1999/08/27 11:36:07 lha Exp $ */
-/* $Source: /afs/stacken.kth.se/src/SourceRepository/arla/xfs/linux/xfs/xfs_locl.h,v $ */
+/* $Id: xfs_locl.h,v 1.51 2000/12/10 23:05:26 lha Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifdef HAVE_ASM_CURRENT_H
 #include <asm/current.h>
-#endif
 #include <linux/config.h>
 #ifdef CONFIG_MODVERSIONS
 #define MODVERSIONS
-#ifdef HAVE_LINUX_MODVERSIONS_H
 #include <linux/modversions.h>
 #endif
-#endif
 #include <linux/version.h>
 #include <linux/module.h>
 #include <linux/errno.h>
 #include <linux/fs.h>
 #include <linux/malloc.h>
+#include <linux/time.h>
 #include <linux/sched.h>
 #include <linux/stat.h>
 #include <linux/string.h>
 #include <linux/locks.h>
-#ifdef HAVE_LINUX_DEVFS_FS_H
-#include <linux/devfs_fs.h>
-#ifdef CONFIG_DEVFS_FS
-#define DEVFS_SUPPORT
-#endif
-#endif
-
-#if LINUX_VERSION_CODE > 131238
-#define LINUX2_1 1
-#endif
-
-#ifdef LINUX2_1
-#include <asm/uaccess.h>
+#include <linux/smp_lock.h>
+#include <linux/wait.h>
+#ifdef HAVE_LINUX_DEVFS_FS_KERNEL_H
+# include <linux/devfs_fs_kernel.h>
 #else
-#include <asm/segment.h>
+# ifdef HAVE_LINUX_DEVFS_FS_H
+#  include <linux/devfs_fs.h>
+# endif
 #endif
+#ifdef CONFIG_DEVFS_FS
+# define DEVFS_SUPPORT
+#endif
+
+#if LINUX_VERSION_CODE >= 0x20300
+#define LINUX2_3 1 
+#endif
+
+/*
+ * Linux 2.3.39 and above has a new setgroups() system call for
+ * 32-bit UIDs, on arm, i386, m68k, sh, and sparc32. The old system call
+ * also has to be handled properly, with a different type for gid_t.
+ * See xfs_syscalls.c for more information.
+ */
+
+#include <asm/unistd.h>
+
+#ifdef __NR_setgroups32
+#define ARLA_NR_setgroups	__NR_setgroups32
+
+/*
+ * For the time being Linux 2.3/2.4 will call the old 16-bit uid
+ * setgroups() __NR_setgroups; at some point in the future it may go away,
+ * so we put this ifdef here
+ */
+#ifdef __NR_setgroups
+#define ARLA_NR_setgroups16	__NR_setgroups
+#endif
+
+#else
+#define ARLA_NR_setgroups	__NR_setgroups
+#endif
+
+#include <asm/uaccess.h>
 
 /* 
  * The people at FSF seems to think that user program
@@ -126,55 +144,100 @@ typedef __u32  u_int32_t;
 #include <xfs/xfs_deb.h>
 
 int
-#ifndef LINUX2_1
-xfs_root(struct super_block *sb, struct inode **vpp);
-#else
-xfs_root(struct super_block *sb, struct dentry **dp);
-#endif /* LINUX2_1 */
+xfs_replace_root(struct super_block *sb);
 
-
-#ifndef LINUX2_1
-extern inline int signal_pending(struct task_struct *p)
-{
-        return (p->signal &~ p->blocked) != 0;
-}
-#endif /* LINUX2_1 */
-
-#ifdef LINUX2_1
 extern struct dentry_operations xfs_dentry_operations;
 
 struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle);
-#endif
-
-#ifdef LINUX2_1
-
-#if HAVE_FULL_NAME_HASH_8BIT
-
-static __inline__ unsigned int
-xfs_full_name_hash(const unsigned char * name, unsigned int len)
-{
-    return full_name_hash (name, len);
-}
-
-#else
-
-/* this macro is broken in linux 2.1.131, see dcache.h */
-
-static __inline__ unsigned int
-xfs_full_name_hash(const unsigned char * name, unsigned int len)
-{
-  unsigned long hash = init_name_hash();
-  while (len--)
-    hash = partial_name_hash(*name++, hash);
-  return end_name_hash(hash);
-}
-
-#endif
-
-#endif
 
 /*
  * i_blocks should apparently always be returned in 512-bytes units
  */
 
 #define I_BLOCKS_UNIT 512
+
+#define I_BLOCKS_BITS 9
+
+/*
+ * Help function to read the inode->i_count a portable way
+ */
+
+static inline int
+xfs_icount (struct inode *inode)
+{
+#ifndef LINUX2_3
+    return inode->i_count;
+#else
+    return atomic_read(&inode->i_count);
+#endif
+}
+
+/*
+ * Help function to increment the inode->i_count a portable way
+ */
+
+static inline void
+xfs_iref (struct inode *inode)
+{
+#ifndef LINUX2_3
+    ++inode->i_count;
+#else
+    atomic_inc(&inode->i_count);
+#endif
+}
+
+/*
+ * Help function to read the inode->i_writecount a portable way
+ */
+
+static inline int
+xfs_iwritecount (struct inode *inode)
+{
+#ifndef LINUX2_3
+    return inode->i_writecount;
+#else
+    return atomic_read(&inode->i_writecount);
+#endif
+}
+
+/*
+ * Help function to read the dentry->d_count a portable way
+ */
+
+static inline int
+xfs_dcount (struct dentry *dentry)
+{
+#ifndef LINUX2_3
+    return dentry->d_count;
+#else
+    return atomic_read(&dentry->d_count);
+#endif
+}
+
+#ifndef LINUX2_3
+
+/*
+ * Compat macro
+ */
+
+#define DECLARE_FSTYPE(var,type,read,flags)	\
+struct file_system_type var = {			\
+	name:		type,			\
+        read_super:	read,			\
+	fs_flags:	flags,			\
+}
+
+
+#define XFS_RESET_I_MAPPING(node) while (0) { }
+
+#else /* LINUX2_3 */
+
+#define XFS_RESET_I_MAPPING(node) (node)->i_mapping = &((node)->i_data)
+
+extern struct address_space_operations xfs_aops;
+
+#endif /* LINUX2_3 */
+
+#ifndef list_for_each
+#define list_for_each(pos, head) for (pos = (head)->next; pos != (head); pos = pos->next)
+#endif /* list_for_each */
