@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_msg_locl.h,v 1.2 1998/11/02 10:11:01 map Exp $ */
+/* $Id: xfs_msg_locl.h,v 1.4 2000/10/02 23:48:43 lha Exp $ */
 
 #ifndef _xfs_msg_locl_h
 #define _xfs_msg_locl_h
@@ -70,4 +65,14 @@ int
 xfs_message_updatefid(int fd,
 		      struct xfs_message_updatefid * message,
 		      u_int size);
+
+int
+xfs_message_gc_nodes(int fd,
+		     struct xfs_message_gc_nodes * message,
+		     u_int size);
+
+int
+xfs_message_version(int fd,
+		    struct xfs_message_version *message,
+		    u_int size);
 #endif				       /* _xfs_msg_locl_h */
