@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,11 +34,9 @@
 #define __NO_VERSION__
 
 #include <xfs/xfs_locl.h>
-#ifdef LINUX2_1
 #include <linux/poll.h>
-#endif /* LINUX2_1 */
 
-RCSID("$Id: xfs_dev.c,v 1.48 1999/07/25 03:47:07 map Exp $");
+RCSID("$Id: xfs_dev.c,v 1.77.2.2 2001/07/26 02:06:20 assar Exp $");
 
 #include <xfs/xfs_message.h>
 #include <xfs/xfs_msg_locl.h>
@@ -86,7 +79,8 @@ struct xfs_channel {
     struct xfs_message_header *message_buffer;
     int status;
 #define CHANNEL_OPENED	0x1
-    struct semaphore channel_sem;
+    struct semaphore channel_sem; /* protects messageq, sleepq, nsequence, inactive_list */
+    struct list_head inactive_list;
 };
 
 static struct xfs_channel xfs_channel[NXFS];
@@ -124,19 +118,153 @@ xfs_outq(struct xfs_link *p)
     p->next = p->prev = 0;
 }
 
+/*
+ * Enqueue the `message' of `size' to the `chan' for later processing.
+ */
+
+static int
+enqueue_message (struct xfs_channel *chan,
+		 struct xfs_message_header *message,
+		 u_int size)
+{
+    struct {
+	struct xfs_link this_message;
+	struct xfs_message_header msg;
+    } *t;
+    /* Prepare message and copy it later */
+
+    message->size = size;
+
+    t = xfs_alloc(sizeof(t->this_message) + size, XFS_MEM_SENDRPC);
+    if (t == NULL)
+	return -ENOMEM;
+    t->this_message.error_or_size = sizeof(t->this_message) + size;
+
+    memmove(&t->msg, message, size);
+    t->this_message.message = &t->msg;
+
+    down(&chan->channel_sem);
+    
+    t->msg.sequence_num = chan->nsequence++;
+    XFSDEB(XDEBMSG, ("enqueue_message seq = %d\n", t->msg.sequence_num));
+
+    xfs_appendq(&chan->messageq, &t->this_message);
+    init_waitqueue_head(&(t->this_message.wait_queue));
+    up(&chan->channel_sem);
+
+    return 0;
+}
+
+/*
+ * add `xn' to the list of inactive nodes.
+ */
+
+void
+xfs_queue_inactive (struct xfs_node *xn)
+{
+    struct xfs *xfsp = XFS_FROM_XNODE(xn);
+    struct xfs_channel *chan = &xfs_channel[xfsp->fd];
+
+    XFSDEB(XDEBDEV, ("xfs_queue_inactive\n"));
+
+    down(&chan->channel_sem);
+    list_add (&xn->inactive_list, &chan->inactive_list);
+    up(&chan->channel_sem);
+    wake_up(&chan->wait_queue);
+}
+
+/*
+ * send of inactive messages for all queued nodes on `fd'
+ */
+
+static void
+xfs_process_inactive_queue (struct xfs_channel *chan)
+{
+    int i = 0;
+    int ret;
+
+    down(&chan->channel_sem);
+    for (;;) {
+	struct list_head *lh = chan->inactive_list.next;
+	struct xfs_node *xn;
+	struct xfs_message_inactivenode msg;
+
+	if (lh == &chan->inactive_list)
+	    break;
+
+	++i;
+
+	xn = list_entry (lh, struct xfs_node, inactive_list);
+
+	msg.header.opcode = XFS_MSG_INACTIVENODE;
+	msg.handle        = xn->handle;
+	msg.flag          = XFS_NOREFS | XFS_DELETE;
+	
+	list_del (&xn->inactive_list);
+
+	up(&chan->channel_sem);
+	ret = enqueue_message (chan, &msg.header, sizeof(msg));
+	down(&chan->channel_sem);
+ 	if (ret == 0) {
+	    xfs_free(xn, XFS_MEM_XNODE);
+	} else {
+	    list_add (&xn->inactive_list, &chan->inactive_list);
+	    XFSDEB(XDEBDEV, ("xfs_process_inactive_queue: "
+			     "failed enqueue-ing msg %d\n", ret));
+	}
+    }
+    up(&chan->channel_sem);
+    if (i)
+	XFSDEB(XDEBDEV, ("xfs_process_inactive_queue: done (%d)\n", i));
+}
+
+/*
+ * throw away all inactive nodes on `fd'
+ *
+ * Need to be called with the chan->channel_sem held
+ */
+
+static void
+xfs_empty_inactive_queue (struct xfs_channel *chan)
+{
+    int i = 0;
+
+    for (;;) {
+	struct list_head *lh = chan->inactive_list.next;
+	struct xfs_node *xn;
+
+	if (lh == &chan->inactive_list)
+	    break;
+
+	++i;
+
+	xn = list_entry (lh, struct xfs_node, inactive_list);
+	list_del (&xn->inactive_list);
+	xfs_free(xn, XFS_MEM_XNODE);
+    }
+    if (i)
+	XFSDEB(XDEBDEV, ("xfs_empty_inactive_queue: done (%d)\n", i));
+}
 
 /*
  * Only allow one open.
  */
-static int xfs_devopen(struct inode *inode, struct file *file)
+static int
+xfs_devopen(struct inode *inode, struct file *file)
 {
     struct xfs_channel *chan;
     struct xfs *xfsp = &xfs[MINOR(inode->i_rdev)];
+    int ret = 0;
     
-    XFSDEB(XDEBDEV, ("xfs_devopen dev = %d, flags = %d\n", inode->i_rdev, file->f_flags));
+    lock_kernel();
+
+    XFSDEB(XDEBDEV, ("xfs_devopen dev = %d, flags = %d\n",
+		     inode->i_rdev, file->f_flags));
     
-    if (MINOR(inode->i_rdev) >=NXFS)
-	return -ENXIO;
+    if (MINOR(inode->i_rdev) >=NXFS) {
+	ret = -ENXIO;
+	goto out;
+    }
 
     MOD_INC_USE_COUNT;
     
@@ -145,28 +273,37 @@ static int xfs_devopen(struct inode *inode, struct file *file)
     /* Only allow one reader/writer */
     if (chan->status & CHANNEL_OPENED) {
 	MOD_DEC_USE_COUNT;
-	return -EBUSY;
-    } else
-	chan->status |= CHANNEL_OPENED;
-    
-    xfsp->status |= XFS_DEVOPEN;
-    
+	ret = -EBUSY;
+	goto out;
+    }
+
     chan->message_buffer = xfs_alloc(MAX_XMSG_SIZE, XFS_MEM_MSGBUF);
+    if (chan->message_buffer == NULL) {
+	MOD_DEC_USE_COUNT;
+	ret = -ENOMEM;
+	goto out;
+    }
     
-    return 0;
+    chan->status |= CHANNEL_OPENED;
+    xfsp->status |= XFS_DEVOPEN;
+
+    INIT_LIST_HEAD(&chan->inactive_list);
+
+ out:
+    unlock_kernel();
+    return ret;
 }
 
-#ifndef LINUX2_1
-static void
-#else
 static int
-#endif /* LINUX2_1 */
 xfs_devclose(struct inode * inode, struct file * file)
 {
     struct xfs_channel *chan = &xfs_channel[MINOR(inode->i_rdev)];
     struct xfs_link *first;
 
-    XFSDEB(XDEBDEV, ("xfs_devclose dev = %d, flags = %d\n", inode->i_rdev, file->f_flags));
+    lock_kernel ();
+
+    XFSDEB(XDEBDEV, ("xfs_devclose dev = %d, flags = %d\n",
+		     inode->i_rdev, file->f_flags));
     
     /* Sanity check, paranoia? */
     if (!(chan->status & CHANNEL_OPENED))
@@ -175,8 +312,9 @@ xfs_devclose(struct inode * inode, struct file * file)
     chan->status &= ~CHANNEL_OPENED;
     xfs[MINOR(inode->i_rdev)].status &= ~XFS_DEVOPEN;
     
-    /* No one is going to read those messages so empty queue! */
     down(&chan->channel_sem);
+
+    /* No one is going to read those messages so empty queue! */
     while (!xfs_emptyq(&chan->messageq)) {
 	XFSDEB(XDEBDEV, ("before outq(messageq)\n"));
 	first = chan->messageq.next;
@@ -185,9 +323,7 @@ xfs_devclose(struct inode * inode, struct file * file)
 	    xfs_free(first, XFS_MEM_SENDRPC);
 	XFSDEB(XDEBDEV, ("after outq(messageq)\n"));
     }
-    up(&chan->channel_sem);
-    
-    down(&chan->channel_sem);
+
     /* Wakeup those waiting for replies that will never arrive. */
     while (!xfs_emptyq(&chan->sleepq)) {
 	XFSDEB(XDEBDEV, ("before outq(sleepq)\n"));
@@ -200,6 +336,8 @@ xfs_devclose(struct inode * inode, struct file * file)
 	XFSDEB(XDEBDEV, ("after outq(sleepq)\n"));
 	down(&chan->channel_sem);
     }
+    xfs_empty_inactive_queue (chan);
+
     up(&chan->channel_sem);
     
     if (chan->message_buffer) {
@@ -211,33 +349,30 @@ xfs_devclose(struct inode * inode, struct file * file)
     free_all_xfs_nodes(&xfs[MINOR(inode->i_rdev)]);
     
     MOD_DEC_USE_COUNT;
-#ifdef LINUX2_1
+    unlock_kernel ();
     return 0;
-#endif /* LINUX2_1 */
 }
 
 /*
  * Move messages from kernel to user space.
  */
-#ifndef LINUX2_1
-static int
-xfs_devread(struct inode *inode, struct file *file, char *buf, int count)
-#else
+
 static ssize_t
 xfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
-#endif /* LINUX2_1 */
 {
-#ifndef LINUX2_1
-    struct xfs_channel *chan = &xfs_channel[MINOR(inode->i_rdev)];
-#else
-    struct xfs_channel *chan = &xfs_channel[0]; /* XXXX */
-#endif /* LINUX2_1 */
+    kdev_t dev = file->f_dentry->d_inode->i_rdev;
+    struct xfs_channel *chan = &xfs_channel[MINOR(dev)];
     struct xfs_link *first;
     int ret = 0;
     
+    lock_kernel ();
+
     XFSDEB(XDEBDEV, ("xfs_devread: m = %p, m->prev = %p, m->next = %p\n",
-		     &chan->messageq, chan->messageq.prev, chan->messageq.next));
+		     &chan->messageq, chan->messageq.prev,
+		     chan->messageq.next));
     
+    xfs_process_inactive_queue (chan);
+
     down(&chan->channel_sem);
     while (!xfs_emptyq (&chan->messageq)) {
 	first = chan->messageq.next;
@@ -247,7 +382,7 @@ xfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
 	
 	if (first->message->size > count)
 	    break;
-	
+
 	XFSDEB(XDEBDEV, ("xfs_devread: message->size = %u\n",
 			 first->message->size));
 	
@@ -263,10 +398,9 @@ xfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
     }
     up(&chan->channel_sem);
     
-#ifdef LINUX2_1
     *ppos += ret;
-#endif /* LINUX2_1 */
     
+    unlock_kernel ();
     return ret;
 }
 
@@ -275,23 +409,17 @@ xfs_devread(struct file *file, char *buf, size_t count, loff_t *ppos)
  * wakeup sleepers, insert new data in VFS.
  */
 
-#ifndef LINUX2_1
-static int
-xfs_devwrite(struct inode *inode, struct file *file, const char *buf, int count)
-#else
 static ssize_t
 xfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
-#endif /* LINUX2_1 */
 {
-#ifndef LINUX2_1
-    struct xfs_channel *chan = &xfs_channel[MINOR(inode->i_rdev)];
-#else
-    struct xfs_channel *chan = &xfs_channel[0];
-#endif /* LINUX2_1 */
+    kdev_t dev = file->f_dentry->d_inode->i_rdev;
+    struct xfs_channel *chan = &xfs_channel[MINOR(dev)];
     char *p;
     struct xfs_message_header *msg_buf;
     int ret = 0;
     
+    lock_kernel ();
+
     XFSDEB(XDEBDEV, ("xfs_devwrite\n"));
     
     if (count > MAX_XMSG_SIZE)
@@ -308,15 +436,9 @@ xfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
 	int error;
 	msg_buf = (struct xfs_message_header *)p;
 	
-#ifndef LINUX2_1
-	error = xfs_message_receive (MINOR(inode->i_rdev),
+	error = xfs_message_receive (MINOR(dev),
 				     msg_buf,
 				     msg_buf->size);
-#else
-	error = xfs_message_receive (0,
-				     msg_buf,
-				     msg_buf->size);
-#endif /* LINUX2_1 */
 	if (error) {
 	    ret = error;
 	    break;
@@ -330,11 +452,10 @@ xfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
     }
     XFSDEB(XDEBDEV, ("xfs_devwrite ret = %d\n", ret));
     
-#ifdef LINUX2_1
     if (ret >= 0)
 	*ppos += ret;
-#endif /* LINUX2_1 */
     
+    unlock_kernel ();
     return ret;
 }
 
@@ -342,98 +463,90 @@ xfs_devwrite(struct file *file, const char *buf, size_t count, loff_t *ppos)
  * Not used.
  */
 static int
-xfs_devioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
+xfs_devioctl(struct inode *inode, struct file *file,
+	     unsigned int cmd, unsigned long arg)
 {
-    XFSDEB(XDEBDEV, ("xfs_devioctl dev = %d, flags = %d\n", inode->i_rdev, file->f_flags));
+    XFSDEB(XDEBDEV, ("xfs_devioctl dev = %d, flags = %d\n",
+		     inode->i_rdev, file->f_flags));
     return -EINVAL;
 }
 
-/*
- * Are there any messages on this filesystem?
- */
-#ifndef LINUX2_1
-static int
-xfs_devselect(struct inode *inode, struct file *file, int sel_type, select_table *wait)
-{
-    struct xfs_channel *chan = &xfs_channel[MINOR(inode->i_rdev)];
-    
-    if (sel_type != SEL_IN)
-	return 0;
-    
-    down(&chan->channel_sem);
-    if (!xfs_emptyq(&chan->messageq)) {
-	up(&chan->channel_sem);
-	return 1;                   /* Something to read */
-    }
-    up(&chan->channel_sem);
-    
-    /*
-     * No need to handle a "collision" since we only allow one
-     * concurrent open.
-     */
-    select_wait(&chan->wait_queue, wait);
-    
-    return 0;
-}
-#else
 static unsigned int
-xfs_devselect(struct file *file, poll_table *wait)
+xfs_devpoll(struct file *file, poll_table *wait)
 {
-    struct xfs_channel *chan = &xfs_channel[0];
+    kdev_t dev = file->f_dentry->d_inode->i_rdev;
+    struct xfs_channel *chan = &xfs_channel[MINOR(dev)];
+    int ret = 0;
     
+    lock_kernel ();
+
     poll_wait(file, &chan->wait_queue, wait);
     
+    xfs_process_inactive_queue (chan);
+
     down(&chan->channel_sem);
-    if (!xfs_emptyq(&chan->messageq)) {
-	up(&chan->channel_sem);
-	return POLLIN;                    /* Something to read */
-    }
+    if (!xfs_emptyq(&chan->messageq))
+	ret = POLLIN;
     up(&chan->channel_sem);
-    return 0;
+    unlock_kernel ();
+    return ret;
 }
-#endif /* LINUX2_1 */
 
 /*
  * Send a message to user space.
  */
+
 int
 xfs_message_send(int fd, struct xfs_message_header *message, u_int size)
 {
     struct xfs_channel *chan = &xfs_channel[fd];
-    struct {
-	struct xfs_link this_message;
-	struct xfs_message_header msg;
-    } *t;
+    int ret;
     
     XFSDEB(XDEBMSG, ("xfs_message_send opcode = %d\n", message->opcode));
     
     if (!(chan->status & CHANNEL_OPENED))	/* No receiver? */
 	return -ENODEV;
     
-    /* Prepare message and copy it later */
-    message->size = size;
-    message->sequence_num = chan->nsequence++;
-    XFSDEB(XDEBMSG, ("xfs_message_send seq = %d\n", message->sequence_num));
-    
-    t = xfs_alloc(sizeof(t->this_message) + size, XFS_MEM_SENDRPC);
-    t->this_message.error_or_size = sizeof(t->this_message) + size;
-    memmove(&t->msg, message, size);
-    
-    t->this_message.message = &t->msg;
-    down(&chan->channel_sem);
-    xfs_appendq(&chan->messageq, &t->this_message);
-    init_waitqueue_head(&(t->this_message.wait_queue));
-    up(&chan->channel_sem);
+    xfs_process_inactive_queue (chan);
+
+    ret = enqueue_message (chan, message, size);
+
     wake_up(&chan->wait_queue);
-    return 0;
+    return ret;
 }
 
-static inline int sigisactive(struct task_struct *t, int signal)
+#ifdef HAVE_STRUCT_TASK_STRUCT_PENDING
+
+static inline int
+sigisactive(struct task_struct *t, int signal)
+{
+    return (sigismember(&t->pending.signal, signal) &&
+	    !sigismember(&t->blocked, signal));
+}
+
+static inline int
+sigissignaled(struct task_struct *t, int signal)
+{
+    return (sigismember(&current->pending.signal, signal));
+}
+
+#else
+
+static inline int
+sigisactive(struct task_struct *t, int signal)
 {
     return (sigismember(&t->signal, signal) &&
 	    !sigismember(&t->blocked, signal));
 }
 
+static inline int
+sigissignaled(struct task_struct *t, int signal)
+{
+    return (sigismember(&current->signal, signal));
+}
+
+#endif
+
 /*
  * Send a message to user space and wait for reply.
  */
@@ -452,58 +565,70 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
 	return -ENODEV;
     
     if (size < sizeof(struct xfs_message_wakeup)) {
-	printk(KERN_EMERG "XFS Panic: Message to small to receive wakeup, opcode = %d\n", message->opcode);
+	printk(KERN_EMERG "XFS Panic: "
+	       "Message too small to receive wakeup, opcode = %d\n",
+	       message->opcode);
 	return -ENOMEM;
     }
     
+    xfs_process_inactive_queue(chan);
+
     this_message = xfs_alloc(sizeof(struct xfs_link), XFS_MEM_SENDRPC);
+    if (this_message == NULL)
+	return -ENOMEM;
     this_process = xfs_alloc(sizeof(struct xfs_link), XFS_MEM_SENDRPC);
+    if (this_process == NULL) {
+	xfs_free (this_message, XFS_MEM_SENDRPC);
+	return -ENOMEM;
+    }
+    
     msg = xfs_alloc(size, XFS_MEM_SENDRPC);
+    if (msg == NULL) {
+	xfs_free (this_message, XFS_MEM_SENDRPC);
+	xfs_free (this_process, XFS_MEM_SENDRPC);
+	return -ENOMEM;
+    }
     memcpy(msg, message, size);
     
     msg->size = size;
-    msg->sequence_num = chan->nsequence++;
-    XFSDEB(XDEBMSG, ("xfs_message_rpc seq = %d\n", msg->sequence_num));
     this_message->error_or_size = 0;
-    
+
     init_waitqueue_head(&(this_message->wait_queue));
     init_waitqueue_head(&(this_process->wait_queue));
-    
-    this_message->message = msg; /* message;*/
-    this_process->message = msg; /* message; */
+
+    this_message->message = msg;
+    this_process->message = msg;
+
     down(&chan->channel_sem);
+    
+    msg->sequence_num = chan->nsequence++;
+    XFSDEB(XDEBMSG, ("xfs_message_rpc seq = %d\n", msg->sequence_num));
+    
     xfs_appendq(&chan->messageq, this_message);
     xfs_appendq(&chan->sleepq, this_process);
     
     this_process->error_or_size = 0;
-    
+
     up(&chan->channel_sem);
     
     wake_up(&chan->wait_queue);
     
-#ifdef LINUX2_1
     this_process->woken = 0;
     while (!sigisactive(current, SIGINT) &&
 	   !sigisactive(current, SIGTERM) &&
-	   !sigisactive(current, SIGSTOP) &&
 	   !sigisactive(current, SIGKILL) &&
 	   !this_process->woken) {
 	interruptible_sleep_on(&this_process->wait_queue);
     }
-#else
-    /* 
-     * XXX
-     *
-     * Use uninterruptible sleep in 2.0 until code for signal blocking
-     * has been written.
-     */
-    
-    this_process->woken = 0;
-    sleep_on(&this_process->wait_queue);
-#endif
     
     if (!this_process->woken) {
-	XFSDEB(XDEBMSG, ("caught signal\n"));
+	int i;
+
+	XFSDEB(XDEBMSG, ("caught signal:"));
+	for (i = 1; i <= _NSIG; ++i)
+	    if (sigissignaled(current, i))
+		XFSDEB(XDEBMSG, ("%d ", i));
+	XFSDEB(XDEBMSG, ("\n"));
 	this_process->error_or_size = -EINTR;
     }
     
@@ -522,8 +647,13 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
     }
     up(&chan->channel_sem);
     ret = this_process->error_or_size;
-    XFSDEB(XDEBMSG, ("xfs_message_rpc opcode this_process->error_or_size = %d\n", this_process->error_or_size));
-    XFSDEB(XDEBMSG, ("xfs_message_rpc opcode ((xfs_message_wakeup*)(this_process->message))->error = %d\n", ((struct xfs_message_wakeup*)(this_process->message))->error ));
+    XFSDEB(XDEBMSG, ("xfs_message_rpc opcode this_process"
+		     "->error_or_size = %d\n", this_process->error_or_size));
+    XFSDEB(XDEBMSG, ("xfs_message_rpc opcode "
+		     "((xfs_message_wakeup*)"
+		     "(this_process->message))->error = %d\n",
+		     ((struct xfs_message_wakeup*)
+		      (this_process->message))->error ));
     
     memcpy(message, msg, size);
     
@@ -533,7 +663,8 @@ xfs_message_rpc(int fd, struct xfs_message_header *message, u_int size)
 
     if (ret < -1000 || ret > 0) {
 	printk(KERN_EMERG
-	       "XFS Panic: xfs_message_rpc was about to return error %d\n", ret);
+	       "XFS Panic: xfs_message_rpc was about to return error %d\n",
+	       ret);
 	return -EINVAL;
     }
 
@@ -554,46 +685,57 @@ xfs_message_receive(int fd, struct xfs_message_header *message, u_int size)
     switch (message->opcode) {
     case XFS_MSG_WAKEUP:
 	return xfs_message_wakeup(fd,
-				  (struct xfs_message_wakeup *) message,
+				  (struct xfs_message_wakeup *)
+				  message,
 				  message->size);
     case XFS_MSG_WAKEUP_DATA:
 	return xfs_message_wakeup_data(fd,
-				       (struct xfs_message_wakeup_data *) message,
+				       (struct xfs_message_wakeup_data *)
+				       message,
 				       message->size);
     case XFS_MSG_INSTALLROOT:
 	return xfs_message_installroot(fd,
-				       (struct xfs_message_installroot *) message,
+				       (struct xfs_message_installroot *)
+				       message,
 				       message->size);
     case XFS_MSG_INSTALLNODE:
 	return xfs_message_installnode(fd,
-				       (struct xfs_message_installnode *) message, 
+				       (struct xfs_message_installnode *)
+				       message, 
 				       message->size);
     case XFS_MSG_INSTALLATTR:
 	return xfs_message_installattr(fd,
-				       (struct xfs_message_installattr *) message, 
+				       (struct xfs_message_installattr *)
+				       message, 
 				       message->size);
     case XFS_MSG_INSTALLDATA:
 	return xfs_message_installdata(fd,
-				       (struct xfs_message_installdata *) message, 
+				       (struct xfs_message_installdata *)
+				       message, 
 				       message->size);
     case XFS_MSG_INVALIDNODE:
 	return xfs_message_invalidnode(fd,
-				       (struct xfs_message_invalidnode *) message, 
+				       (struct xfs_message_invalidnode *)
+				       message, 
 				       message->size);
     case XFS_MSG_UPDATEFID:
 	return xfs_message_updatefid(fd,
-				     (struct xfs_message_updatefid *)message,
+				     (struct xfs_message_updatefid *)
+				     message,
 				     message->size);
     case XFS_MSG_GC_NODES:
 	return xfs_message_gc_nodes(fd,
-				    (struct xfs_message_gc_nodes *)message,
+				    (struct xfs_message_gc_nodes *)
+				    message,
 				    message->size);
     case XFS_MSG_VERSION:
 	return xfs_message_version(fd,
-				   (struct xfs_message_version *)message,
+				   (struct xfs_message_version *)
+				   message,
 				   message->size);
     default:
-	printk(KERN_EMERG "XFS Panic: xfs_dev: Unknown message opcode == %d\n",
+	printk(KERN_EMERG 
+	       "XFS Panic: xfs_dev: Unknown message opcode == %d\n",
 	       message->opcode);
 	return -EINVAL;
     }
@@ -613,7 +755,10 @@ xfs_message_wakeup(int fd, struct xfs_message_wakeup *message, u_int size)
     for (; t != sleepq; t = t->next)
 	if (t->message->sequence_num == message->sleepers_sequence_num) {
 	    if (t->message->size < size) {
-		printk(KERN_EMERG "XFS Panic: Could not wakeup requestor with opcode = %d properly, too small receive buffer.\n", t->message->opcode);
+		printk(KERN_EMERG 
+		       "XFS Panic: Could not wakeup requestor with "
+		       "opcode = %d properly, too small receive buffer.\n",
+		       t->message->opcode);
 		t->error_or_size = -ENOMEM;
 	    } else {
 		memmove(t->message, message, size);
@@ -639,14 +784,17 @@ xfs_message_wakeup_data(int fd, struct xfs_message_wakeup_data *message,
     struct xfs_link *sleepq = &chan->sleepq;
     struct xfs_link *t = chan->sleepq.next; /* Really first in q */
     
-    XFSDEB(XDEBMSG, ("xfs_message_wakeup error: %d seq = %d\n",
+    XFSDEB(XDEBMSG, ("xfs_message_wakeup_data error: %d seq = %d\n",
 		     message->error, message->sleepers_sequence_num));
     
     down(&chan->channel_sem);
     for (; t != sleepq; t = t->next)
 	if (t->message->sequence_num == message->sleepers_sequence_num) {
 	    if (t->message->size < size) {
-		printk(KERN_EMERG "XFS Panic: Could not wakeup requestor with opcode = %d properly, to small receive buffer.\n", t->message->opcode);
+		printk(KERN_EMERG 
+		       "XFS Panic: Could not wakeup requestor with "
+		       "opcode = %d properly, too small receive buffer.\n",
+		       t->message->opcode);
 		t->error_or_size = -ENOMEM;
 	    } else {
 		memmove(t->message, message, size);
@@ -665,22 +813,12 @@ xfs_message_wakeup_data(int fd, struct xfs_message_wakeup_data *message,
 }
 
 struct file_operations xfs_fops = {
-    NULL,			/* lseek */
-    xfs_devread,		/* read */
-    xfs_devwrite,		/* write */
-    NULL,			/* readdir */
-    xfs_devselect,		/* select */
-    xfs_devioctl,		/* ioctl */
-    NULL,			/* mmap */
-    xfs_devopen,		/* open */
-#ifdef HAVE_STRUCT_FILE_OPERATIONS_FLUSH
-    NULL,			/* flush */
-#endif
-    xfs_devclose,		/* release */
-    NULL,			/* fsync */
-    NULL,			/* fasync */
-    NULL,			/* check_media_change */
-    NULL			/* revalidate */
+    read:		xfs_devread,
+    write:		xfs_devwrite,
+    poll:		xfs_devpoll,
+    ioctl:		xfs_devioctl,
+    release:		xfs_devclose,
+    open:		xfs_devopen,
 };
 
 int
@@ -703,14 +841,31 @@ xfs_init_device(void)
 }
 
 void
-xfs_print_sleep_queue()
+xfs_print_sleep_queue(void)
 {
-    struct xfs_channel *chan = &xfs_channel[0];
-    struct xfs_link *sleepq = &chan->sleepq;
-    struct xfs_link *t = chan->sleepq.next; /* Really first in q */
+    int i;
 
-    printk("Sleeping:");
-    for (; t != sleepq; t = t->next) {
-	printk(" %d", t->message->sequence_num);
+    for (i = 0; i < NXFS; ++i) {
+	struct xfs_channel *chan = &xfs_channel[i];
+	struct xfs_link *sleepq = &chan->sleepq;
+	struct xfs_link *t = chan->sleepq.next; /* Really first in q */
+
+	printk("Sleeping queue %d :", i);
+	for (; t != sleepq; t = t->next) {
+	    printk(" %d", t->message->sequence_num);
+
+#ifdef HAVE_WAIT_QUEUE_TASK_LIST
+	    if (!list_empty(&t->wait_queue.task_list)) {
+		const wait_queue_t *w = (const wait_queue_t *)
+		    list_entry(t->wait_queue.task_list.next,
+			       wait_queue_t, task_list);
+
+		printk(" (pid %d)", w->task->pid);
+	    }
+#else /* !HAVE_WAIT_QUEUE_TASK_LIST */
+	    if (t->wait_queue && t->wait_queue->task)
+		printk(" (pid %d)", t->wait_queue->task->pid);
+#endif /* !HAVE_WAIT_QUEUE_TASK_LIST */
+	}
     }
 }
