@@ -38,7 +38,7 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node.c,v 1.51.2.1 2001/01/30 01:48:11 lha Exp $");
+RCSID("$Id: xfs_node.c,v 1.51.2.3 2001/09/06 23:40:45 ahltorp Exp $");
 
 /*
  * Copy the attributes from `attr' into `inode', setting the fields
@@ -202,7 +202,7 @@ new_xfs_node(struct xfs *xfsp, struct xfs_msg_node *node)
     /* Init other fields */
     result->attr = node->attr;
     xfs_attr2inode (&result->attr, result->vn, 1);
-    XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
+    result->tokens = node->tokens;
     memmove(result->id, node->id, sizeof(result->id));
     memmove(result->rights, node->rights, sizeof(result->rights));
     
