@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -42,7 +42,7 @@
 #include <linux/smp_lock.h>
 #include <linux/smp.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.64 2000/10/02 23:47:54 lha Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.64.2.3 2001/08/01 01:03:44 lha Exp $");
 
 /* 
  * Def pag:
@@ -279,8 +279,13 @@ struct file_handle {
     __u32 gen;
 };
 
+/*
+ * copy a `file_handle' of `dentry' to the buffer pointed to in
+ * `vice_ioctl'.  return 0 or -errno.
+ */
+
 static int
-fhget_call(struct ViceIoctl *vice_ioctl, struct dentry *dentry)
+fhget_call(struct ViceIoctl *vice_ioctl, const struct dentry *dentry)
 {
     struct file_handle fh;
     struct xfs_cache_handle temp_fh;
@@ -293,10 +298,8 @@ fhget_call(struct ViceIoctl *vice_ioctl, struct dentry *dentry)
 		       (unsigned int) fh.dev, (unsigned int) fh.inode,
 		       (unsigned int) fh.gen));
 
-    if (vice_ioctl->out_size < sizeof(struct xfs_cache_handle)) {
-	dput(dentry);
+    if (vice_ioctl->out_size < sizeof(struct xfs_cache_handle))
 	return -EINVAL;
-    }
     memset(&temp_fh, 0, sizeof(temp_fh));
     
     if (sizeof(temp_fh) < sizeof(fh))
@@ -306,12 +309,8 @@ fhget_call(struct ViceIoctl *vice_ioctl, struct dentry *dentry)
 
     if (copy_to_user (vice_ioctl->out,
 		      &temp_fh,
-		      sizeof(struct xfs_cache_handle)) != 0) {
-	dput(dentry);
+		      sizeof(struct xfs_cache_handle)) != 0)
 	return -EFAULT;
-    }
-
-    dput(dentry);
 
     return 0;
 }
@@ -320,7 +319,8 @@ fhget_call(struct ViceIoctl *vice_ioctl, struct dentry *dentry)
  * lookup file identified by `handle' and return a dentry or an error.
  */
 
-struct dentry * xfs_fh_to_dentry(struct xfs_cache_handle *handle)
+struct dentry *
+xfs_fh_to_dentry(struct xfs_cache_handle *handle)
 {
     struct file_handle fh;
     struct dentry *dentry;
@@ -509,11 +509,10 @@ xfs_debug_print (struct ViceIoctl *vice_ioctl, struct dentry *node)
 #ifdef LINUX2_3
 
 static struct dentry *
-user_path2dentry (char *user_path, int follow)
+user_path2dentry (struct nameidata *nd, char *user_path, int follow)
 {
     char *kname;
     int flags;
-    struct nameidata nd;
     int error;
 
     kname = getname (user_path);
@@ -524,12 +523,12 @@ user_path2dentry (char *user_path, int follow)
 	flags |= LOOKUP_FOLLOW;
 
     error = 0;
-    if (path_init (kname, flags, &nd))
-	error = path_walk(kname, &nd);
+    if (path_init (kname, flags, nd))
+	error = path_walk(kname, nd);
     putname(kname);
     if (error)
 	return ERR_PTR(error);
-    return nd.dentry;
+    return nd->dentry;
 }
 
 #else /* !LINUX2_3 */
@@ -559,14 +558,15 @@ sys_afs_int (int operation,
     struct xfs_message_pioctl msg;
     struct xfs_message_wakeup_data *msg2;
     struct dentry *dentry = NULL;
+#ifdef LINUX2_3
+    struct nameidata nd;
+#endif
     
-#ifdef __SMP__
     XFSDEB(XDEBSYS, ("sys_afs locking kernel; cpu: %d\n", smp_processor_id()));
 
     lock_kernel();
 
     XFSDEB(XDEBSYS, ("sys_afs kernel locked; cpu: %d\n", smp_processor_id()));
-#endif
 
     XFSDEB(XDEBSYS, ("sys_afs operation: %d a_pathP: %p "
 		     "a_opcode: %d a_paramsP: %p "
@@ -595,7 +595,12 @@ sys_afs_int (int operation,
 	}
 	if (a_pathP != NULL) {
 	    XFSDEB(XDEBMSG, ("xfs_syscall: looking up: %p\n", a_pathP));
+#ifdef LINUX2_3
+	    dentry = user_path2dentry (&nd, a_pathP, a_followSymlinks);
+#else
+	    XFSDEB(XDEBMSG, ("xfs_syscall: looking up: %p\n", a_pathP));
 	    dentry = user_path2dentry (a_pathP, a_followSymlinks);
+#endif
 	    if (!dentry) {
 		error = -EINVAL;
 		goto unlock;
@@ -604,6 +609,7 @@ sys_afs_int (int operation,
 		XFSDEB(XDEBMSG, ("xfs_syscall: error during namei: %ld\n",
 				 PTR_ERR(dentry)));
 		error = PTR_ERR(dentry);
+		dentry = NULL;
 		goto unlock;
 	    }
 	    XFSDEB(XDEBMSG,("xfs_syscall: inode: %p inodenum: %lx\n",
@@ -616,15 +622,11 @@ sys_afs_int (int operation,
 	case VIOC_FHGET_32:
 #endif
 	    error = fhget_call(&vice_ioctl, dentry);
-	    dentry = NULL;
 	    goto unlock;
 	case VIOC_FHOPEN:
 #ifdef VIOC_FHOPEN_32
 	case VIOC_FHOPEN_32:
 #endif
-	    if (dentry)
-		dput(dentry);
-	    dentry = NULL;
 	    error = fhopen_call(&vice_ioctl, a_followSymlinks);
 	    goto unlock;
 	case VIOC_XFSDEBUG:
@@ -632,18 +634,12 @@ sys_afs_int (int operation,
 	case VIOC_XFSDEBUG_32:
 #endif
 	    error = xfs_debug (&vice_ioctl);
-	    if (dentry)
-		dput(dentry);
-	    dentry = NULL;
 	    goto unlock;
 	case VIOC_XFSDEBUG_PRINT:
 #ifdef VIOC_XFSDEBUG_PRINT_32
 	case VIOC_XFSDEBUG_PRINT_32:
 #endif
 	    error = xfs_debug_print (&vice_ioctl, dentry);
-	    if (dentry)
-		dput(dentry);
-	    dentry = NULL;
 	    goto unlock;
 	}
 
@@ -652,22 +648,16 @@ sys_afs_int (int operation,
 	    if (strcmp(DENTRY_TO_INODE(dentry)->i_sb->s_type->name,
 		       "xfs") != 0) {
 		XFSDEB(XDEBMSG, ("xfs_syscall: not in afs\n"));
-		dput(dentry);
-		dentry = NULL;
 		error = -EINVAL;
 		goto unlock;
 	    }
 	    xn = VNODE_TO_XNODE(DENTRY_TO_INODE(dentry));
 	    if (xn == NULL) {
 		XFSDEB(XDEBMSG, ("xfs_syscall: is an xfs dentry, but has no xnode\n"));
-		dput(dentry);
-		dentry = NULL;
 		error = -EINVAL;
 		goto unlock;
 	    }
 	    msg.handle = xn->handle;
-	    dput(dentry);
-	    dentry = NULL;
 	}
 
 	msg.header.opcode = XFS_MSG_PIOCTL;
@@ -709,11 +699,17 @@ sys_afs_int (int operation,
     }
     
  unlock:
+#ifdef LINUX2_3
+    if (dentry)
+	path_release(&nd);
+#else
+    if (dentry)
+	dput(dentry);
+#endif
+
     XFSDEB(XDEBSYS, ("xfs_syscall returns error: %d\n", error));
-#ifdef __SMP__
     XFSDEB(XDEBSYS, ("sys_afs kernel unlock; cpu: %d\n", smp_processor_id()));
     unlock_kernel();
-#endif
     return error;
 }
 
