@@ -38,7 +38,7 @@
 #include <xfs/xfs_syscalls.h>
 #include <xfs/nxfs.h>
 
-RCSID("$Id: xfs_vfsops.c,v 1.71 2000/11/16 06:09:41 lha Exp $");
+RCSID("$Id: xfs_vfsops.c,v 1.71.2.1 2001/01/30 15:45:00 lha Exp $");
 
 struct xfs xfs[NXFS];
 
@@ -119,7 +119,7 @@ xfs_replace_root (struct super_block *sb)
 	free_xfs_node (xn);
 	iput (inode);
 
-	xfs_attr2inode (&xfsp->root->attr, root);
+	xfs_attr2inode (&xfsp->root->attr, root, 1);
 	root->i_op = &xfs_dir_inode_operations;
 #ifndef HAVE_STRUCT_INODE_OPERATIONS_DEFAULT_FILE_OPS
 	root->i_fop = &xfs_dir_operations;
