@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_locl.h,v 1.1 1998/03/22 18:26:38 assar Exp $ */
+/* $Id: xfs_locl.h,v 1.5 2000/11/04 20:48:39 lha Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -59,6 +54,7 @@
 #include <sys/sunddi.h>
 #include <sys/cmn_err.h>
 #include <sys/systm.h>
+#include <sys/syscall.h>
 #include <sys/modctl.h>
 #include <sys/cred.h>
 #include <sys/param.h>
@@ -66,3 +62,5 @@
 #include <sys/debug.h>
 #include <sys/pathname.h>
 #include <sys/kmem.h>
+#include <sys/signal.h>
+#include <vm/seg_vn.h>
