@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_syscalls.h,v 1.5 1999/05/27 15:15:26 assar Exp $ */
+/* $Id: xfs_syscalls.h,v 1.7 2000/10/02 23:54:43 lha Exp $ */
 
 #ifndef  __xfs_syscalls
 #define  __xfs_syscalls
@@ -47,7 +42,7 @@
 int xfs_install_syscalls(void);
 int xfs_uninstall_syscalls(void);
 int xfs_stat_syscalls(void);
-pag_t xfs_get_pag(struct cred *);
+xfs_pag_t xfs_get_pag(struct cred *);
 
 void xfs_install_setgroups(void);
 void xfs_uninstall_setgroups(void);
