@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  *
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  *
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- *
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_vfsops.h,v 1.1 1998/09/08 09:10:27 assar Exp $ */
+/* $Id: xfs_vfsops.h,v 1.3 2000/10/02 23:54:49 lha Exp $ */
 
 #ifndef _xfs_vfsops_h
 #define _xfs_vfsops_h
@@ -51,4 +46,7 @@ xfs_fhopen (fsid_t fsid,
 	    fid_t fid,
 	    int flags);
 
+int
+xfs_uprintf_filsys(void);
+
 #endif				       /* _xfs_vfsops_h */
