@@ -36,7 +36,7 @@
 #include <xfs/xfs_fs.h>
 #include <xfs/xfs_deb.h>
 
-RCSID("$Id: xfs_node.c,v 1.14.2.1 2001/01/30 02:14:01 lha Exp $");
+RCSID("$Id: xfs_node.c,v 1.14.2.3 2001/09/06 10:15:24 ahltorp Exp $");
 
 /*
  * Create a new xfs_node and make a VN_HOLD()!
@@ -102,8 +102,8 @@ new_xfs_node(struct xfs *xfsp, struct xfs_node *dp, struct xfs_msg_node *node)
     }
     
     /* Init other fields */
-    xfs_attr2vattr (&node->attr, &result->attr);
-    XFS_TOKEN_SET(result, XFS_ATTR_R, XFS_ATTR_MASK);
+    xfs_attr2vattr (&node->attr, &result->attr, 1);
+    result->tokens = node->tokens;
     bcopy((caddr_t)node->id,
 	  (caddr_t)result->id, sizeof(result->id));
     bcopy((caddr_t)node->rights,
@@ -397,7 +397,7 @@ vattr2xfs_attr(const struct vattr *va, struct xfs_attr *xa)
 }
 
 void
-xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va, , int clear_node)
+xfs_attr2vattr(const struct xfs_attr *xa, struct vattr *va, int clear_node)
 {
     if (clear_node)
 	bzero((caddr_t)va, sizeof(*va));
