@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2001 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include <xfs/xfs_locl.h>
 
-RCSID("$Id: xfs_syscalls.c,v 1.29 2000/10/02 23:53:27 lha Exp $");
+RCSID("$Id: xfs_syscalls.c,v 1.29.2.1 2001/05/11 17:54:26 assar Exp $");
 
 /*
  * XFS system calls.
@@ -263,11 +263,9 @@ fhget_call (struct vnode *vp,
 	return set_errno(EINVAL);
     }
 
-    error = copyout ((caddr_t)&fh_args,
-		     (caddr_t)vice_ioctl->out,
-		     sizeof(fh_args));
-
-    return set_errno(error);
+    if(copyout ((caddr_t)&fh_args, (caddr_t)vice_ioctl->out, sizeof(fh_args)))
+	return set_errno(EFAULT);
+    return 0;
 }
 
 /*
@@ -292,10 +290,8 @@ fhopen_call (struct vnode *vp,
     if (vice_ioctl->in_size < sizeof(fh_args))
 	return set_errno(EINVAL);
 
-    error = copyin ((caddr_t)vice_ioctl->in,
-		    (caddr_t)&fh_args, sizeof(fh_args));
-    if (error)
-	return set_errno(error);
+    if (copyin ((caddr_t)vice_ioctl->in, (caddr_t)&fh_args, sizeof(fh_args)))
+	return set_errno(EFAULT);
     return xfs_fhopen (fh_args.fsid, fh_args.fid, flags);
 }
 
@@ -313,17 +309,14 @@ remote_pioctl (int a_opcode,
     int error;
 
     if (vice_ioctl->in_size > 2048) {
-	printf("xfs_pioctl_call: got a humongous in packet: opcode: %d",
+	printf("remote_pioctl: got a humongous in packet: opcode: %d",
 	       a_opcode);
 	return set_errno(EINVAL);
     }
     if (vice_ioctl->in_size != 0) {
-	error = copyin((caddr_t)vice_ioctl->in,
-		       (caddr_t)&msg.msg,
-		       vice_ioctl->in_size);
-
-	if (error)
-	    return set_errno(error);
+	if(copyin((caddr_t)vice_ioctl->in, (caddr_t)&msg.msg,
+		  vice_ioctl->in_size))
+	    return set_errno(EFAULT);
     }
 
     if (vp != NULL) {
@@ -350,10 +343,13 @@ remote_pioctl (int a_opcode,
 	error = EINVAL;
     
     if (error == 0 && vice_ioctl->out_size)
-	error = copyout((caddr_t)msg2->msg,
-			(caddr_t)vice_ioctl->out, 
-			min(msg2->len, vice_ioctl->out_size));
-    return set_errno(error);
+	if(copyout((caddr_t)msg2->msg, (caddr_t)vice_ioctl->out, 
+		   min(msg2->len, vice_ioctl->out_size)))
+	   return set_errno(EFAULT);
+    if (error)
+	return set_errno(error);
+    else
+	return 0;
 }
 
 /*
@@ -373,11 +369,8 @@ xfs_debug(struct ViceIoctl *vice_ioctl)
 	if (vice_ioctl->in_size < sizeof(int32_t))
 	    return EINVAL;
 	
-	error = copyin (vice_ioctl->in,
-			&flags,
-			sizeof(flags));
-	if (error)
-	    return error;
+	if(copyin (vice_ioctl->in, &flags, sizeof(flags)))
+	    return EFAULT;
 	
 	xfsdeb = flags;
     }
@@ -386,11 +379,8 @@ xfs_debug(struct ViceIoctl *vice_ioctl)
 	if (vice_ioctl->out_size < sizeof(int32_t))
 	    return EINVAL;
 	
-	error = copyout (&xfsdeb,
-			 vice_ioctl->out,
-			 sizeof(int32_t));
-	if (error)
-	    return error;
+	if(copyout (&xfsdeb, vice_ioctl->out, sizeof(int32_t)))
+	    return EFAULT;
     }
     return 0;
 }
@@ -414,11 +404,8 @@ xfs_debug_print (struct ViceIoctl *vice_ioctl, struct vnode *vp)
 	if (vice_ioctl->in_size < sizeof(int32_t))
 	    return EINVAL;
 	
-	error = copyin (vice_ioctl->in,
-			&tmp,
-			sizeof(tmp));
-	if (error)
-	    return error;
+	if(copyin (vice_ioctl->in, &tmp, sizeof(tmp)))
+	    return EFAULT;
 	switch (tmp) {
 	case XDEBNODE :
 	    xfs_uprintf_filsys ();
@@ -435,10 +422,10 @@ xfs_debug_print (struct ViceIoctl *vice_ioctl, struct vnode *vp)
  */
 
 static int
-xfs_pioctl_call(char *a_pathP,
-		int a_opcode,
-		struct ViceIoctl *a_paramsP,
-		int a_followSymlinks)
+xfs_pioctl_int(char *a_pathP,
+	       int a_opcode,
+	       struct ViceIoctl *a_paramsP,
+	       int a_followSymlinks)
 {
     int error;
     struct vnode *vp = NULL;
@@ -450,9 +437,8 @@ xfs_pioctl_call(char *a_pathP,
     if (a_pathP != NULL) {
 	char path[MAXPATHLEN];
 
-	error = copyinstr (a_pathP, path, sizeof(path), NULL);
-	if (error)
-	    return set_errno(error);
+	if(copyinstr (a_pathP, path, sizeof(path), NULL))
+	    return set_errno(EFAULT);
 
 	XFSDEB(XDEBSYS, ("xfs_syscall: looking up: %s\n", path));
 
@@ -496,11 +482,28 @@ xfs_pioctl_call(char *a_pathP,
 }
 
 static int
-xfs_syscall_int(int operation,
-		char *a_pathP,
+xfs_pioctl_call(char *a_pathP,
 		int a_opcode,
 		struct ViceIoctl *a_paramsP,
 		int a_followSymlinks)
+{
+    int error;
+    struct ViceIoctl vice_ioctl;
+
+    XFSDEB(XDEBSYS, ("xfs_syscall\n"));
+
+    if (copyin ((caddr_t)a_paramsP, (caddr_t)&vice_ioctl, sizeof(vice_ioctl)))
+	return set_errno(EFAULT);
+
+    return xfs_pioctl_int (a_pathP, a_opcode, &vice_ioctl, a_followSymlinks);
+}
+
+static int
+xfs_syscall(int operation,
+	    char *a_pathP,
+	    int a_opcode,
+	    struct ViceIoctl *a_paramsP,
+	    int a_followSymlinks)
 {
     int ret;
 
@@ -519,28 +522,6 @@ xfs_syscall_int(int operation,
     return ret;
 }
 
-static int
-xfs_syscall(int operation,
-	    char *a_pathP,
-	    int a_opcode,
-	    struct ViceIoctl *a_paramsP,
-	    int a_followSymlinks)
-{
-    int error;
-    struct ViceIoctl vice_ioctl;
-
-    XFSDEB(XDEBSYS, ("xfs_syscall\n"));
-
-    error = copyin ((caddr_t)a_paramsP,
-		    (caddr_t)&vice_ioctl,
-		    sizeof(vice_ioctl));
-    if (error)
-	return set_errno (error);
-
-    return xfs_syscall_int (operation, a_pathP, a_opcode, &vice_ioctl,
-			    a_followSymlinks);
-}
-
 static struct sysent xfs_sysent = {
     5,
     SE_ARGC | SE_LOADABLE,
@@ -556,25 +537,20 @@ struct modlsys xfs_modlsys = {
 #ifdef _SYSCALL32_IMPL
 
 static int
-xfs_syscall32(int operation,
-	      char *a_pathP,
-	      int a_opcode,
-	      struct ViceIoctl32 *a_paramsP32,
-	      int a_followSymlinks)
+xfs_pioctl_call32(char *a_pathP,
+		  int a_opcode,
+		  struct ViceIoctl32 *a_paramsP32,
+		  int a_followSymlinks)
+
 {
     struct ViceIoctl vice_ioctl;
     struct ViceIoctl32 vice_ioctl32;
     int error;
 
-    XFSDEB(XDEBSYS, ("xfs_syscall32\n"));
+    if (copyin ((caddr_t)a_paramsP32, &vice_ioctl32, sizeof(vice_ioctl32)))
+	return set_errno (EFAULT);
 
-    error = copyin ((caddr_t)a_paramsP32,
-		    &vice_ioctl32,
-		    sizeof(vice_ioctl32));
-    if (error)
-	return set_errno (error);
-
-    XFSDEB(XDEBSYS, ("xfs_syscall32: params = (%x, %x, %d, %d)\n",
+    XFSDEB(XDEBSYS, ("xfs_pioctl_call32: params = (%x, %x, %d, %d)\n",
 		     vice_ioctl32.in, vice_ioctl32.out,
 		     vice_ioctl32.in_size, vice_ioctl32.out_size));
 
@@ -583,8 +559,34 @@ xfs_syscall32(int operation,
     vice_ioctl.in_size  = vice_ioctl32.in_size;
     vice_ioctl.out_size = vice_ioctl32.out_size;
 
-    return xfs_syscall_int (operation, a_pathP, a_opcode, &vice_ioctl,
-			    a_followSymlinks);
+    return xfs_pioctl_int (a_pathP, a_opcode, &vice_ioctl,
+			   a_followSymlinks);
+}
+
+static int
+xfs_syscall32(int operation,
+	      char *a_pathP,
+	      int a_opcode,
+	      struct ViceIoctl32 *a_paramsP32,
+	      int a_followSymlinks)
+{
+    int ret;
+
+    XFSDEB(XDEBSYS, ("xfs_syscall32\n"));
+
+    switch (operation) {
+    case AFSCALL_PIOCTL:
+	ret = xfs_pioctl_call32(a_pathP, a_opcode, a_paramsP32,
+				a_followSymlinks);
+	break;
+    case AFSCALL_SETPAG:
+	ret = xfs_setpag_call(curproc);
+	break;
+    default:
+	uprintf("Unimplemeted call: %d\n", operation);
+	ret = set_errno(EINVAL);
+    }
+    return ret;
 }
 
 static struct sysent xfs_sysent32 = {
