@@ -42,7 +42,7 @@ xfs_dnlc_lookup _PARAMS((struct vnode *, char *));
 extern void xfs_dnlc_purge _PARAMS((void));
 
 void
-xfs_attr2vattr _PARAMS((const struct xfs_attr *xa, struct vattr *va));
+xfs_attr2vattr _PARAMS((const struct xfs_attr *xa, struct vattr *va, int clear_node));
 
 void
 vattr2xfs_attr _PARAMS((const struct vattr *va, struct xfs_attr *xa));
