@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -484,8 +479,10 @@ xfs_lookup(dvp, nm, vpp, cred, pnp, flags)
 	msg.cred.uid = u.u_cred->cr_uid;
 	msg.cred.pag = 0;		/* XXX */
 	msg.parent_handle = d->handle;
-	strncpy(msg.name, nm, sizeof(msg.name));
-	error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
+	if (strlcpy(msg.name, nm, sizeof(msg.name)) >= XFS_MAX_LEN)
+	    error = ENAMETOOLONG;
+	else
+	    error = xfs_message_rpc(xfsp->fd, &msg.header, sizeof(msg));
 	if (error == 0)
 	  error = ((struct xfs_message_wakeup *) &msg)->error;
       }
@@ -532,7 +529,8 @@ xfs_create(dvp, nm, va, exclusive, mode, vpp, cred)
     struct xfs_message_create msg;
     msg.header.opcode = XFS_MSG_CREATE;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    strncpy(msg.name, nm, sizeof(msg.name));
+    msg.name[sizeof(msg.name)-1] = '\0';
     vattr2xfs_attr (va, &msg.attr);
 #if 0
     msg.exclusive = exclusive;
@@ -631,7 +629,8 @@ xfs_mkdir(dvp, nm, va, vpp, cred)
     struct xfs_message_mkdir msg;
     msg.header.opcode = XFS_MSG_CREATE;
     msg.parent_handle = xn->handle;
-    strncpy(msg.name, nm, 256);
+    strncpy(msg.name, nm, sizeof(msg.name));
+    msg.name[sizeof(msg.name)-1] = '\0';
     vattr2xfs_attr (va, &msg.attr);
     msg.cred.uid = u.u_cred->cr_uid;
     msg.cred.pag = 0;		/* XXX */
