@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,6 +31,8 @@
  * SUCH DAMAGE.
  */
 
+/* xfs_vops.c */
+
 
 #define DEFINE_IRP(x) NTSTATUS x (PDEVICE_OBJECT device, PIRP irp)
 
@@ -49,23 +46,62 @@ DEFINE_IRP(xfs_dirctl);
 DEFINE_IRP(xfs_devctl);
 DEFINE_IRP(xfs_shutdown);
 DEFINE_IRP(xfs_cleanup);
+DEFINE_IRP(xfs_queryvol);
+DEFINE_IRP(xfs_fscontrol);
 
 #undef DEFINE_IRP
 
+/* xfs_deb.c */
+
 LONG
 xfs_log_new_seq (void);
 
 void
-xfs_log (PDEVICE_OBJECT dev,
-	 UCHAR MajorFunctionCode,
-	 UCHAR RetryCount,
-	 USHORT EventCategory,
-	 NTSTATUS ErrorCode,
-	 ULONG UniqueErrorValue,
-	 LONG SequenceNumber,
-	 ULONG IoControlCode,
-	 USHORT DumpDataSize,
-	 ULONG DumpData[],
-	 USHORT NumStrings,
-	 CHAR *Strings[]);
+xfs_log(PDEVICE_OBJECT device, ULONG UniqueId,
+	NTSTATUS ErrorCode, NTSTATUS Status);
+
+void
+xfs_debug (unsigned long level, char *fmt, ...);
+
+XFS_CCB *
+xfs_get_ccb (void);
+
+void
+xfs_release_ccb (XFS_CCB *ccb);
+
+struct xfs_link *
+xfs_alloc_link (struct xfs_channel *chan);
+
+void
+xfs_free_link (struct xfs_channel *chan, struct xfs_link *link);
+
+void *
+xfs_alloc (size_t size);
+
+void
+xfs_free (void *ptr, size_t size);
+
+void
+xfs_initq(struct xfs_link *q);
+
+int
+xfs_emptyq(const struct xfs_link *q);
+
+int
+xfs_onq(const struct xfs_link *link);
+
+void
+xfs_appendq(struct xfs_link *q, struct xfs_link *p);
+
+void
+xfs_outq(struct xfs_link *p);
+
+struct xfs_link *
+xfs_firstlink (struct xfs_link *head);
+
+struct xfs_link *
+xfs_nextlink (struct xfs_link *head, struct xfs_link *prev);
+
+struct xfs_node *
+xfs_dnlc_lookup (struct xfs_node *node, char *filename);
 
