@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -40,7 +35,7 @@
 #ifndef __XFS_TYPE_H
 #define __XFS_TYPE_H
 
-#ifdef	_CPU_X86_
+#ifdef	_X86_
 typedef	char		int8_t;
 typedef	short		int16_t;
 typedef	int		int32_t;
@@ -48,7 +43,11 @@ typedef	unsigned char	u_int8_t;
 typedef	unsigned short	u_int16_t;
 typedef	unsigned int	u_int32_t;
 #else
-#error what ?
-#endif	/* _CPU_X86_ */
+#error what cpu ?
+#endif	/* _X86_ */
+
+typedef unsigned int u_int;
+
+#define	AlignPointer(val) ((((u_int32_t)(val)) + 7) & 0xfffffff8)
 
 #endif /* __XFS_TYPE_H */
