@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1999, 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: xfs_deb.c,v 1.1 1999/07/12 11:25:55 lha Exp $ */
+/* $Id: xfs_deb.c,v 1.5 2000/10/02 23:59:32 lha Exp $ */
 
 #include <xfs_locl.h>
 
@@ -50,51 +45,54 @@ xfs_log_new_seq (void)
 
 
 void
-xfs_log (PDEVICE_OBJECT dev,
-	 UCHAR MajorFunctionCode,
-	 UCHAR RetryCount,
-	 USHORT EventCategory,
-	 NTSTATUS ErrorCode,
-	 ULONG UniqueErrorValue,
-	 LONG SequenceNumber,
-	 ULONG IoControlCode,
-	 USHORT DataDumpSize,
-	 ULONG DumpData[],
-	 USHORT NumStrings,
-	 CHAR *Strings[])
+xfs_log(PDEVICE_OBJECT device, ULONG UniqueId,
+	NTSTATUS ErrorCode, NTSTATUS Status)
 {
-    PIO_ERROR_LOG_PACKET pkt;
-    UCHAR size = sizeof (IO_ERROR_LOG_PACKET);
-    int i;
-
-    size += DataDumpSize * sizeof(ULONG);
+    PIO_ERROR_LOG_PACKET errorLogEntry;
     
-    ASSERT (StringsSize == 0);
-    /* XXX Add strings to size */
-
-    pkt = IoAllocateErrorLogEntry (dev, size);
-
-    if (pkt == NULL)
-	return;
+    errorLogEntry = (PIO_ERROR_LOG_PACKET)
+	IoAllocateErrorLogEntry(device,
+				(UCHAR)(sizeof(IO_ERROR_LOG_PACKET) + 
+					sizeof(DEVICE_OBJECT)));
     
-    pkt->MajorFunctionCode	= MajorFunctionCode;
-    pkt->RetryCount		= RetryCount;
-    pkt->DumpDataSize		= DataDumpSize;
-    pkt->NumberOfStrings	= NumStrings;
-    pkt->StringOffset		= 0;
-    pkt->EventCategory		= EventCategory;
-    pkt->ErrorCode		= ErrorCode;
-    pkt->UniqueErrorValue	= UniqueErrorValue;
-    pkt->SequenceNumber	        = SequenceNumber;
-    pkt->IoControlCode		= IoControlCode;
-    pkt->DeviceOffset.LowPart	= 0;
-    pkt->DeviceOffset.HighPart	= 0;
-    for (i = 0; i < DataDumpSize; i++)
-	pkt->DumpData[i]	= DumpData[i];
-    
-    /* XXX fill in strings here */
+    if (errorLogEntry != NULL) {
+	errorLogEntry->SequenceNumber	= 0;
+	errorLogEntry->MajorFunctionCode = 0;
+	errorLogEntry->RetryCount	= 0;
+        errorLogEntry->ErrorCode 	= ErrorCode;
+        errorLogEntry->UniqueErrorValue	= UniqueId;
+        errorLogEntry->FinalStatus 	= Status;
+	errorLogEntry->DumpDataSize	= 1;
+	errorLogEntry->StringOffset	= sizeof(errorLogEntry);
+	errorLogEntry->NumberOfStrings	= 0;
 
-    IoWriteErrorLogEntry (pkt);
+        /*
+	 * The following is necessary because DumpData is of type ULONG
+	 * and DeviceObject can be more than that
+	 */
+        RtlCopyMemory(
+            &errorLogEntry->DumpData[0],
+            &device,
+            sizeof(DEVICE_OBJECT));
+        errorLogEntry->DumpDataSize = sizeof(DEVICE_OBJECT);
+        IoWriteErrorLogEntry(errorLogEntry);
+    }
 }
 
 
+static unsigned long debug_mask = -1;
+
+void
+xfs_debug (unsigned long level, char *fmt, ...)
+{
+    va_list args;
+    char str[300];
+    
+    if ((debug_mask & level) == 0)
+	return;
+    
+    va_start (args, fmt);
+    vsprintf (str, fmt, args);
+    va_end (args);
+    DbgPrint (str);    
+}
