@@ -33,7 +33,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $");
+RCSID("$Id: output.c,v 1.73.2.1 2001/03/04 04:48:48 lha Exp $");
 #endif
 
 #include <stdio.h>
@@ -1478,23 +1478,21 @@ genfree (List *list, Listitem *item, void *arg)
 static Bool
 genencodein (List *list, Listitem *item, void *arg)
 {
-     Argument *a = (Argument *)listdata (item);
-     FILE *f = (FILE *)arg;
-
-     if (a->argtype != TIN && a->argtype != TINOUT)
-	  return TRUE;
-     else {
-	  if (a->type->type == TPOINTER) {
-	       char *tmp = (char *)emalloc (strlen (a->name) + 4);
-	       
-	       sprintf (tmp, "(*%s)", a->name);
-
-	       encode_type (tmp, a->type->subtype, f, ENCODE_RX, CLIENT);
-	       free (tmp);
-	  } else
-	       encode_type (a->name, a->type, f, ENCODE_RX, CLIENT);
-	  return FALSE;
-     }
+    Argument *a = (Argument *)listdata (item);
+    FILE *f = (FILE *)arg;
+    
+    if (a->argtype == TIN || a->argtype == TINOUT) {
+	if (a->type->type == TPOINTER) {
+	    char *tmp = (char *)emalloc (strlen (a->name) + 4);
+	    
+	    sprintf (tmp, "(*%s)", a->name);
+	    
+	    encode_type (tmp, a->type->subtype, f, ENCODE_RX, CLIENT);
+	    free (tmp);
+	} else
+	    encode_type (a->name, a->type, f, ENCODE_RX, CLIENT);
+    }
+    return FALSE;
 }
 
 static Bool
@@ -1768,7 +1766,7 @@ generate_standard_c_prologue (FILE *f,
 	      "#define bcopy(a,b,c) memcpy((b),(a),(c))\n"
 	      "#endif /* !HAVE_BCOPY */\n\n");
      fprintf (f, "#ifdef RCSID\n"
-	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $\");\n"
+	      "RCSID(\"%s generated from %s.xg with $Id: output.c,v 1.73.2.1 2001/03/04 04:48:48 lha Exp $\");\n"
 	      "#endif\n\n", filename, basename);
 }
 
@@ -2085,7 +2083,7 @@ generate_tcpdump_patches(FILE *td_file, const char *filename)
 {
     Type optype = {TULONG};
 
-    fprintf(td_file, "/* generated by $Id: output.c,v 1.73 2000/11/25 22:38:20 lha Exp $ */\n\n");
+    fprintf(td_file, "/* generated by $Id: output.c,v 1.73.2.1 2001/03/04 04:48:48 lha Exp $ */\n\n");
 
     fprintf (td_file, 
 	     "#include <stdio.h>\n"
