@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -36,7 +31,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: sym.h,v 1.6 1999/03/04 06:43:03 assar Exp $ */
+/* $Id: sym.h,v 1.8 2000/10/02 22:37:05 lha Exp $ */
 
 #ifndef _SYM_
 #define _SYM_
@@ -66,6 +61,7 @@ typedef struct {
 	      unsigned id;
 	      List *arguments;
 	      unsigned flags;
+	      char *package;
 	  } proc;
      } u;
 } Symbol;
